@@ -14,6 +14,7 @@ MODULE cp_restart
   !
   !
   USE iotk_module
+#if defined (__OLDXML)
   USE qexml_module,     ONLY : qexml_init,qexml_openfile, qexml_closefile, &
                           qexml_write_header,qexml_write_control, qexml_write_cell, &
                           qexml_write_ions, qexml_write_planewaves, qexml_write_spin, &
@@ -22,12 +23,15 @@ MODULE cp_restart
                           qexml_kpoint_dirname, qexml_read_header, qexml_read_status_cp, &
                           qexml_read_ions, qexml_read_spin, qexml_read_occ, &
                           qexml_read_bands_info, qexml_read_bands_cp, &
-                          fmt_version => qexml_default_version, qexml_wfc_filename, qexml_restart_dirname
-  USE xml_io_base,     ONLY  : write_wfc, read_wfc, write_rho_xml,read_print_counter, create_directory
+                          qexml_wfc_filename, qexml_restart_dirname
+  USE io_files,  ONLY : xmlpun, qexml_version, qexml_version_init
+  USE xml_io_base, ONLY  : write_wfc
+#endif
+  USE xml_io_base,     ONLY  : read_wfc, write_rho, read_print_counter
   !
   USE kinds,     ONLY : DP
   USE io_global, ONLY : ionode, ionode_id, stdout
-  USE io_files,  ONLY : prefix, iunpun, xmlpun, qexml_version, qexml_version_init
+  USE io_files,  ONLY : prefix, iunpun, create_directory
   USE mp,        ONLY : mp_bcast
   USE parser,    ONLY : version_compare
   USE matrix_inversion
@@ -36,11 +40,12 @@ MODULE cp_restart
   !
   SAVE
   !
+#if defined (__OLDXML)
   PRIVATE :: read_cell
+#endif
   !
   INTEGER, PRIVATE :: iunout
   !
-  !
   ! variables to describe qexml current version
   ! and back compatibility
   !
@@ -56,14 +61,16 @@ MODULE cp_restart
                              vnhp, xnhp0, xnhpm, nhpcl, nhpdim, occ0, occm,  &
                              lambda0,lambdam, xnhe0, xnhem, vnhe, ekincm,    &
                              et, rho, c02, cm2, ctot, iupdwn, nupdwn,        &
-                             iupdwn_tot, nupdwn_tot, wfc, mat_z ) ! BS added wfc
+                             iupdwn_tot, nupdwn_tot, wfc ) ! BS added wfc
       !------------------------------------------------------------------------
       !
       USE control_flags,            ONLY : gamma_only, force_pairing, trhow, &
                                            tksw, twfcollect, do_makov_payne, &
-                                           smallmem, llondon, lxdm, ts_vdw
+                                           smallmem, llondon, lxdm, ts_vdw,  &
+                                           tfor, tpre
       USE control_flags,            ONLY : lwfpbe0nscf, lwfnscf, lwf ! Lingzhu Kong
       USE constants,                ONLY : e2
+      USE dener,                    ONLY : detot
       USE io_files,                 ONLY : psfile, pseudo_dir, iunwfc, &
                                            nwordwfc, tmp_dir, diropn
       USE mp_images,                ONLY : intra_image_comm, me_image, &
@@ -84,20 +91,23 @@ MODULE cp_restart
       USE cell_base,                ONLY : ibrav, alat, celldm, s_to_r, ainv ! BS added ainv
       USE ions_base,                ONLY : nsp, nat, na, atm, zv, &
                                            amass, iforce, ind_bck
-      USE funct,                    ONLY : get_dft_name, get_inlc
+      USE funct,                    ONLY : get_dft_name, get_inlc, &
+           dft_is_hybrid, get_exx_fraction, get_screening_parameter, &
+           dft_is_nonlocc, get_nonlocc_name
       USE ldaU_cp,                  ONLY : lda_plus_U, ns, ldmx,Hubbard_l, &
                                            Hubbard_lmax, Hubbard_U
       USE energies,                 ONLY : enthal, ekin, eht, esr, eself, &
                                            epseu, enl, exc, vave
       USE mp,                       ONLY : mp_sum, mp_barrier
       USE fft_base,                 ONLY : dfftp, dffts, dfftb
-      USE uspp_param,               ONLY : n_atom_wfc
+      USE uspp_param,               ONLY : n_atom_wfc, upf
       USE global_version,           ONLY : version_number
       USE cp_main_variables,        ONLY : descla
-      USE cp_interfaces,            ONLY : collect_lambda, collect_zmat
-      USE kernel_table,             ONLY : vdw_table_name
-      USE london_module,            ONLY : scal6, lon_rcut
-      USE tsvdw_module,             ONLY : vdw_isolated
+      USE cp_interfaces,            ONLY : collect_lambda
+      USE kernel_table,             ONLY : vdw_table_name, kernel_file_name
+      USE london_module,            ONLY : scal6, lon_rcut, in_c6
+      USE tsvdw_module,             ONLY : vdw_isolated, vdw_econv_thr
+      USE mytime,                   ONLY : f_wall
       !
       IMPLICIT NONE
       !
@@ -146,11 +156,11 @@ MODULE cp_restart
       INTEGER,               INTENT(IN) :: iupdwn_tot(:)! 
       INTEGER,               INTENT(IN) :: nupdwn_tot(:)! 
       REAL(DP),              INTENT(IN) :: wfc(:,:)     ! BS 
-      REAL(DP),    OPTIONAL, INTENT(IN) :: mat_z(:,:,:) ! 
       !
       LOGICAL               :: write_charge_density
       CHARACTER(LEN=20)     :: dft_name
-      CHARACTER(LEN=256)    :: dirname, filename, rho_file_base
+      CHARACTER(LEN=256)    :: dirname
+      CHARACTER(LEN=320)    :: filename, sourcefile
       CHARACTER(LEN=4)      :: cspin
       INTEGER               :: kunit, ib, ik_eff
       INTEGER               :: k1, k2, k3
@@ -167,7 +177,7 @@ MODULE cp_restart
       REAL(DP)              :: nelec
       REAL(DP)              :: scalef
       LOGICAL               :: lsda
-      REAL(DP)              :: s0, s1, cclock
+      REAL(DP)              :: s0, s1
       INTEGER               :: nbnd_tot
       INTEGER               :: natomwfc, nbnd_
       REAL(DP), ALLOCATABLE :: mrepl(:,:)
@@ -177,6 +187,14 @@ MODULE cp_restart
       CHARACTER(iotk_attlenx)  :: attr
       REAL(DP), ALLOCATABLE :: temp_vec(:), wfc_temp(:,:) ! BS 
       !
+      !
+      k1  = 0
+      k2  = 0
+      k3  = 0
+      nk1 = 0
+      nk2 = 0
+      nk3 = 0
+      !
       ! ... subroutine body
       !
       write_charge_density = trhow
@@ -195,6 +213,68 @@ MODULE cp_restart
       !
       nbnd_    = nupdwn(1) 
       nbnd_tot = MAX( nupdwn(1), nupdwn_tot(1) )
+      nelec = nelt
+      !
+      ! ... Cell related variables
+      ! ... Dirty trick to avoid bogus complaints because ht in intent(in)
+      !
+      h = ht
+      CALL invmat( 3, h, htm1, omega )
+      h = TRANSPOSE( ht )
+      !
+      a1 = ht(1,:)/alat
+      a2 = ht(2,:)/alat
+      a3 = ht(3,:)/alat
+      !
+      CALL recips( a1, a2, a3, b1, b2, b3 )
+      !
+      ! ... Compute array ityp, and tau
+      !
+      ALLOCATE( ityp( nat ) )
+      ALLOCATE( tau( 3, nat ) )
+      !
+      isa = 0
+      !
+      DO is = 1, nsp
+         !
+         DO ia = 1, na(is)
+            !
+            isa = isa + 1
+            ityp(isa) = is
+            !
+         END DO
+         !
+      END DO
+      !
+      natomwfc =  n_atom_wfc ( nat, ityp ) 
+      !
+      CALL s_to_r( stau0, tau, na, nsp, h )
+      !   
+      lsda = ( nspin == 2 )
+      !
+      ALLOCATE( ftmp( nbnd_tot , nspin ) )
+      !
+      ftmp = 0.0d0
+      !
+      DO iss = 1, nspin
+         !
+         ftmp( 1:nupdwn(iss), iss ) = occ0( iupdwn(iss) : iupdwn(iss) + nupdwn(iss) - 1 )
+         !
+      END DO
+      !
+#if !defined (__OLDXML)
+      !
+      CALL errore('cp_writefile','called in wrong case',1)
+      !
+#else
+      !
+      ! ... Some ( CP/FPMD ) default values
+      !
+      IF ( nspin == 2 ) THEN
+         kunit = 2
+      ELSE
+         kunit = 1
+      END IF
       !
       IF ( ionode ) THEN
          !
@@ -223,73 +303,6 @@ MODULE cp_restart
          CALL create_directory( qexml_kpoint_dirname( dirname, i ) )
          !
       END DO
-      !
-      ! ... Some ( CP/FPMD ) default values
-      !
-      IF ( nspin == 2 ) THEN
-         !
-         kunit = 2
-         !
-      ELSE
-         !
-         kunit = 1
-         !
-      END IF
-      !
-      k1  = 0
-      k2  = 0
-      k3  = 0
-      nk1 = 0
-      nk2 = 0
-      nk3 = 0
-      !
-      ! ... Compute Cell related variables
-      ! ... Dirty trick to avoid bogus complaints because ht in intent(in)
-      h = ht
-      CALL invmat( 3, h, htm1, omega )
-      h = TRANSPOSE( ht )
-      !
-      !
-      a1 = ht(1,:)
-      a2 = ht(2,:)
-      a3 = ht(3,:)
-      !
-      ! ... Beware: omega may be negative if axis are left-handed!
-      !
-      scalef = 1.D0 / SQRT( ABS (omega) )
-      !
-      ! ... Compute array ityp, and tau
-      !
-      ALLOCATE( ityp( nat ) )
-      ALLOCATE( tau( 3, nat ) )
-      !
-      isa = 0
-      !
-      DO is = 1, nsp
-         !
-         DO ia = 1, na(is)
-            !
-            isa = isa + 1
-            ityp(isa) = is
-            !
-         END DO
-         !
-      END DO
-      !
-      CALL s_to_r( stau0, tau, na, nsp, h )
-      !   
-      lsda = ( nspin == 2 )
-      !
-      ALLOCATE( ftmp( nbnd_tot , nspin ) )
-      !
-      ftmp = 0.0d0
-      !
-      DO iss = 1, nspin
-         !
-         ftmp( 1:nupdwn(iss), iss ) = occ0( iupdwn(iss) : iupdwn(iss) + nupdwn(iss) - 1 )
-         !
-      END DO
-      !
       IF ( ionode ) THEN
          !
          ! ... Open XML descriptor
@@ -297,7 +310,7 @@ MODULE cp_restart
          WRITE( stdout, '(/,3X,"writing restart file: ",A)' ) TRIM( dirname )
          !
          CALL qexml_init( iunpun )
-         CALL qexml_openfile( TRIM( dirname ) // '/' // TRIM( xmlpun ), &
+         CALL qexml_openfile( TRIM( dirname ) // TRIM( xmlpun ), &
                              & 'write', BINARY = .FALSE., IERR = ierr  )
          !
       END IF
@@ -306,7 +319,7 @@ MODULE cp_restart
       !
       CALL errore( 'cp_writefile ', 'cannot open restart file for writing', ierr )
       !
-      s0 = cclock()
+      s0 = f_wall()
       !
       IF ( ionode ) THEN
 
@@ -333,16 +346,10 @@ MODULE cp_restart
 !-------------------------------------------------------------------------------
 ! ... CELL
 !-------------------------------------------------------------------------------
-         !
-         a1 = a1 / alat
-         a2 = a2 / alat
-         a3 = a3 / alat
-         !
-         CALL recips( a1, a2, a3, b1, b2, b3 )
          !
          CALL qexml_write_cell( ibrav, celldm, alat, a1, a2, a3, b1, b2, b3, &
                           "Bohr","Bohr","2 pi / a", &
-                          do_makov_payne, .FALSE., .FALSE. )
+                          do_makov_payne, .FALSE., .FALSE., .FALSE. )
          !
 !-------------------------------------------------------------------------------
 ! ... IONS
@@ -412,52 +419,7 @@ MODULE cp_restart
 ! ... CHARGE-DENSITY
 !-------------------------------------------------------------------------------
       !
-      IF (write_charge_density) then
-         !
-         rho_file_base = 'charge-density'
-         !
-         IF ( ionode ) THEN
-              CALL iotk_link( iunpun, "CHARGE-DENSITY", rho_file_base, &
-              CREATE = .FALSE., BINARY = .TRUE. )
-         END IF
-         !
-         rho_file_base = TRIM( dirname ) // '/' // TRIM( rho_file_base )
-         !
-         IF ( nspin == 1 ) THEN
-            !
-            CALL write_rho_xml( rho_file_base, rho(:,1), &
-                                dfftp%nr1, dfftp%nr2, dfftp%nr3, dfftp%nr1x, dfftp%nr2x, &
-                                dfftp%ipp, dfftp%npp, ionode, intra_bgrp_comm, inter_bgrp_comm )
-            !
-         ELSE IF ( nspin == 2 ) THEN
-            !
-            ALLOCATE( rhoaux( SIZE( rho, 1 ) ) )
-            !
-            rhoaux = rho(:,1) + rho(:,2) 
-            !
-            CALL write_rho_xml( rho_file_base, rhoaux, &
-                                dfftp%nr1, dfftp%nr2, dfftp%nr3, dfftp%nr1x, dfftp%nr2x, &
-                                dfftp%ipp, dfftp%npp, ionode, intra_bgrp_comm, inter_bgrp_comm )
-            !
-            rho_file_base = 'spin-polarization'
-            !
-            IF ( ionode ) &
-                 CALL iotk_link( iunpun, "SPIN-POLARIZATION", rho_file_base, &
-                 CREATE = .FALSE., BINARY = .TRUE. )
-            !
-            rho_file_base = TRIM( dirname ) // '/' // TRIM( rho_file_base )
-            !
-            rhoaux = rho(:,1) - rho(:,2) 
-            !
-            CALL write_rho_xml( rho_file_base, rhoaux, &
-                                dfftp%nr1, dfftp%nr2, dfftp%nr3, dfftp%nr1x, dfftp%nr2x, &
-                                dfftp%ipp, dfftp%npp, ionode, intra_bgrp_comm, inter_bgrp_comm )
-            !
-            DEALLOCATE( rhoaux )
-            !
-         END IF
-         !
-      END IF ! write_charge_density
+      IF (write_charge_density) CALL write_rho ( dirname, rho, nspin )
       !
 !-------------------------------------------------------------------------------
 ! ... LDA+U OCCUPATIONS (compatibility with PWscf)
@@ -466,9 +428,7 @@ MODULE cp_restart
       IF ( lda_plus_u ) THEN
          !
          IF ( ionode ) THEN
-            i = LEN_TRIM( dirname )
-            ! ugly hack to remove .save from dirname
-            filename = dirname (1:i-4) // 'occup'
+            filename = dirname // 'occup'
             OPEN (UNIT=iunout,FILE=filename,FORM ='formatted',STATUS='unknown')
             WRITE( iunout, * , iostat = ierr) ns
          END IF
@@ -571,10 +531,6 @@ MODULE cp_restart
       !
       IF ( ionode ) THEN
          ! 
-         natomwfc =  n_atom_wfc ( nat, ityp ) 
-         !
-         nelec = nelt
-         !
          CALL qexml_write_bands_info(  nk, natomwfc, &
                                        nbnd_tot, nupdwn_tot(1),nupdwn_tot(2),&
                                        nspin, nelec, nel(1), nel(2), &
@@ -597,6 +553,10 @@ MODULE cp_restart
 !-------------------------------------------------------------------------------
 ! ... EIGENVECTORS
 !-------------------------------------------------------------------------------
+      !
+      ! ... Beware: omega may be negative if axis are left-handed!
+      !
+      scalef = 1.D0 / SQRT( ABS (omega) )
       !
       k_points_loop2: DO ik = 1, nk
 
@@ -794,23 +754,6 @@ MODULE cp_restart
                !
             END IF
             !
-            IF( PRESENT( mat_z ) ) THEN
-               !
-               CALL collect_zmat( mrepl, mat_z(:,:,iss), descla(iss) )
-               !
-               IF ( ionode ) THEN
-                  !
-                  filename = TRIM( qexml_wfc_filename( ".", 'mat_z', ik, iss ) )
-                  !
-                  CALL iotk_link( iunpun, "MAT_Z" // TRIM( cspin ), &
-                                  filename, CREATE = .TRUE., BINARY = .TRUE. )
-                  !
-                  CALL iotk_write_dat( iunpun, "MAT_Z" // TRIM( cspin ), mrepl )
-                  !
-               END IF
-               !
-            END IF
-            !
             DEALLOCATE( mrepl )
             !
          END DO
@@ -895,7 +838,7 @@ MODULE cp_restart
          tmp_dir = tmp_dir_save
          !
       END IF
-
+#endif 
 !-------------------------------------------------------------------------------
 ! ... END RESTART SECTIONS
 !-------------------------------------------------------------------------------
@@ -904,7 +847,7 @@ MODULE cp_restart
       DEALLOCATE( tau  )
       DEALLOCATE( ityp )
       !
-      s1 = cclock() 
+      s1 = f_wall() 
       !
       IF ( ionode ) THEN
          !
@@ -923,7 +866,7 @@ MODULE cp_restart
                             taui, cdmi, stau0, svel0, staum, svelm, force,    &
                             vnhp, xnhp0, xnhpm, nhpcl,nhpdim,occ0, occm,      &
                             lambda0, lambdam, b1, b2, b3, xnhe0, xnhem, vnhe, &
-                            ekincm, c02, cm2, wfc, mat_z ) ! added wfc
+                            ekincm, c02, cm2, wfc ) ! added wfc
       !------------------------------------------------------------------------
       !
       USE control_flags,            ONLY : gamma_only, force_pairing, iverbosity, twfcollect, lwf ! BS added lwf
@@ -939,12 +882,13 @@ MODULE cp_restart
                                            sort_tau, ityp, ions_cofmass
       USE gvect,       ONLY : ig_l2g, mill
       USE cp_main_variables,        ONLY : nprint_nfi, descla
-      USE cp_interfaces,            ONLY : distribute_lambda, distribute_zmat
+      USE cp_interfaces,            ONLY : distribute_lambda
       USE mp,                       ONLY : mp_sum, mp_bcast
       USE mp_global,                ONLY : intra_image_comm, my_bgrp_id
       USE mp_global,                ONLY : root_bgrp, intra_bgrp_comm, inter_bgrp_comm, intra_pool_comm
       USE parameters,               ONLY : ntypx
       USE constants,                ONLY : eps8, angstrom_au, pi
+      USE mytime,                   ONLY : f_wall
       !
       IMPLICIT NONE
       !
@@ -989,7 +933,6 @@ MODULE cp_restart
       COMPLEX(DP),           INTENT(INOUT) :: c02(:,:)     ! 
       COMPLEX(DP),           INTENT(INOUT) :: cm2(:,:)     ! 
       REAL(DP),              INTENT(INOUT) :: wfc(:,:)     ! BS 
-      REAL(DP),    OPTIONAL, INTENT(INOUT) :: mat_z(:,:,:) ! 
       !
       CHARACTER(LEN=256)   :: dirname, kdirname, filename
       CHARACTER(LEN=5)     :: kindex
@@ -1028,7 +971,7 @@ MODULE cp_restart
       INTEGER,  ALLOCATABLE :: if_pos_(:,:) 
       CHARACTER(LEN=256)    :: psfile_(ntypx)
       CHARACTER(LEN=80)     :: pos_unit
-      REAL(DP)              :: s1, s0, cclock
+      REAL(DP)              :: s1, s0
       REAL(DP), ALLOCATABLE :: mrepl(:,:) 
       LOGICAL               :: exst, exist_wfc 
       CHARACTER(LEN=256)    :: tmp_dir_save
@@ -1038,10 +981,12 @@ MODULE cp_restart
       ! ... look for an empty unit
       !
       CALL iotk_free_unit( iunout, ierr )
-      !
       CALL errore( 'cp_readfile', &
                    'no free units to read wavefunctions', ierr )
       !
+#if !defined (__OLDXML)
+      CALL errore('cp_readfile','called in wrong case',1)
+#else
       kunit = 1
       found = .FALSE.
       exist_wfc = .FALSE.
@@ -1051,16 +996,12 @@ MODULE cp_restart
       ! ... Open XML descriptor
       !
       IF ( ionode ) THEN
-         !
-         !filename = TRIM( dirname ) // '/' // TRIM( xmlpun )
          !
          WRITE( stdout, '(/,3X,"reading restart file: ",A)' ) TRIM( dirname )
          !
-         !CALL iotk_open_read( iunpun, FILE = TRIM( filename ), IERR = ierr )
-         !
          CALL qexml_init( iunpun )
          !
-         CALL qexml_openfile( TRIM( dirname ) // '/' // TRIM( xmlpun ), &
+         CALL qexml_openfile( TRIM( dirname ) // TRIM( xmlpun ), &
                               'read', BINARY = .FALSE., IERR = ierr  )
          !
       END IF
@@ -1070,7 +1011,7 @@ MODULE cp_restart
       CALL errore( 'cp_readfile', &
                    'cannot open restart file for reading', ierr )
       !
-      s0 = cclock()
+      s0 = f_wall()
       !
       IF ( ionode ) THEN
          !
@@ -1120,7 +1061,7 @@ MODULE cp_restart
       !
       IF ( ionode ) THEN
          !
-         filename = TRIM( dirname ) // '/' // TRIM( xmlpun )
+         filename = TRIM( dirname ) // TRIM( xmlpun )
          !
          CALL iotk_open_read( iunpun, FILE = TRIM( filename ), IERR = ierr )
          !
@@ -1630,22 +1571,6 @@ MODULE cp_restart
 
             CALL distribute_lambda( mrepl, lambdam(:,:,iss), descla(iss) )
             !
-            IF ( PRESENT( mat_z ) ) THEN
-               !
-               IF( ionode ) THEN
-                  CALL iotk_scan_dat( iunpun, "MAT_Z" // TRIM( iotk_index( iss ) ), mrepl, FOUND = found )
-                  IF( .NOT. found ) THEN
-                     WRITE( stdout, * ) 'WARNING mat_z not read from restart file'
-                     mrepl = 0.0d0
-                  END IF
-               END IF
-
-               CALL mp_bcast( mrepl, ionode_id, intra_image_comm )
-
-               CALL distribute_zmat( mrepl, mat_z(:,:,iss), descla(iss) )
-               !
-            END IF
-            !
             DEALLOCATE( mrepl )
             !
          END DO
@@ -1676,12 +1601,12 @@ MODULE cp_restart
                                                    &wfc(1:3, iupdwn(iss):iupdwn(iss) + nupdwn(iss) -1 ) )
                   !
                ENDDO
+               CALL iotk_scan_end(   iunpun, "WANNIER_CENTERS" )
                !
             ELSE
                WRITE( stdout, * ) 'WARNING wannier centers not read from restart file:'
             ENDIF
             !
-            CALL iotk_scan_end(   iunpun, "WANNIER_CENTERS" )
          END IF
          !
       END IF
@@ -1732,9 +1657,6 @@ MODULE cp_restart
       IF ( lwf ) CALL mp_bcast( wfc, ionode_id, intra_image_comm )
       !-------------------------------------------------------------------------
       !
-      IF ( PRESENT( mat_z ) ) &
-         CALL mp_bcast( mat_z(:,:,:), ionode_id, intra_image_comm )
-      !
       IF ( ionode ) &
          CALL iotk_close_read( iunpun )
       !
@@ -1769,7 +1691,7 @@ MODULE cp_restart
 
       END IF
       !
-      s1 = cclock()
+      s1 = f_wall()
       !
       IF ( ionode ) THEN
          !
@@ -1782,6 +1704,7 @@ MODULE cp_restart
          CALL read_print_counter( nprint_nfi, tmp_dir, ndr )
          IF( iverbosity > 1 ) write( stdout,*) 'nprint_nfi= ',nprint_nfi
       endif
+#endif
       !
       RETURN
       !
@@ -1811,6 +1734,9 @@ MODULE cp_restart
       INTEGER            :: io_bgrp_id
       REAL(DP)           :: scalef
       !
+#if !defined (__OLDXML)
+      CALL errore('cp_read_wfc','called in the wrong case',1)
+#else
       kunit = 1
       !
       ik_eff = ik + ( iss - 1 ) * nk
@@ -1859,6 +1785,7 @@ MODULE cp_restart
       CALL mp_sum( io_bgrp_id, inter_bgrp_comm )
       CALL mp_sum( io_bgrp_id, intra_bgrp_comm )
       CALL mp_bcast( c2, io_bgrp_id, inter_bgrp_comm )
+#endif
       !
       RETURN
       !
@@ -1889,6 +1816,9 @@ MODULE cp_restart
       INTEGER            :: ik_eff, ib, nb, kunit, iss_, nspin_, ngwt_, nbnd_
       REAL(DP)           :: scalef
       !
+#if !defined (__OLDXML)
+      CALL errore('cp_read_wfc_Kong','XSD under development',1)
+#else
       kunit = 1
       !
       ik_eff = ik + ( iss - 1 ) * nk
@@ -1932,6 +1862,7 @@ MODULE cp_restart
                      inter_bgrp_comm, intra_pool_comm )
 
       !
+#endif
       RETURN
       !
     END SUBROUTINE cp_read_wfc_Kong
@@ -1972,10 +1903,13 @@ MODULE cp_restart
       REAL(DP)         :: b1_(3), b2_(3), b3_(3)
       CHARACTER(iotk_attlenx)  :: attr
       !
+#if !defined (__OLDXML)
+      CALL errore('cp_read_cell','XSD under development',1)
+#else
       !
       dirname = qexml_restart_dirname( tmp_dir, prefix, ndr ) 
       !
-      filename = TRIM( dirname ) // '/' // TRIM( xmlpun )
+      filename = TRIM( dirname ) // TRIM( xmlpun )
       !
       IF ( ionode ) &
          CALL iotk_open_read( iunpun, FILE = TRIM( filename ), &
@@ -2079,189 +2013,13 @@ MODULE cp_restart
       CALL mp_bcast( vnhh,  ionode_id, intra_image_comm )
       !
       IF ( ionode ) CALL iotk_close_read( iunpun )
+#endif
       !
       RETURN
       !
     END SUBROUTINE cp_read_cell
     !
-    !------------------------------------------------------------------------    !------------------------------------------------------------------------
-    !SUBROUTINE read_ions( nsp, nat, atm, ityp, psfile, &
-    !                      amass, tau, if_pos, pos_unit, ierr )
-      !------------------------------------------------------------------------
-      !
-    !  INTEGER,            INTENT(OUT) :: nsp, nat
-    !  CHARACTER(LEN=3),   INTENT(OUT) :: atm(:)
-    !  INTEGER,            INTENT(OUT) :: ityp(:)
-    !  CHARACTER(LEN=256), INTENT(OUT) :: psfile(:)
-    !  REAL(DP),           INTENT(OUT) :: amass(:)
-    !  REAL(DP),           INTENT(OUT) :: tau(:,:)
-    !  INTEGER,            INTENT(OUT) :: if_pos(:,:)
-    !  INTEGER,            INTENT(OUT) :: ierr
-    !  CHARACTER(LEN=*),   INTENT(OUT) :: pos_unit
-    !  !
-    !  LOGICAL          :: found, back_compat
-    !  INTEGER          :: i
-    !  CHARACTER(LEN=3) :: lab
-    !  CHARACTER(iotk_attlenx)  :: attr
-    !  !
-    !  ierr = 0
-    !  !
-    !  CALL iotk_scan_begin( iunpun, "IONS", FOUND = found )
-    !  !
-    !  IF ( .NOT. found ) THEN
-    !     !
-    !     ierr = 1
-    !     !
-    !     RETURN
-    !     !
-    !  END IF
-    !  !
-    !  CALL iotk_scan_dat( iunpun, "NUMBER_OF_ATOMS",   nat )
-    !  CALL iotk_scan_dat( iunpun, "NUMBER_OF_SPECIES", nsp )
-    !  !
-    !  IF ( nsp > SIZE( atm ) .OR. nat > SIZE( ityp ) ) THEN
-    !     !
-    !     ierr = 10
-    !     !
-    !     CALL iotk_scan_end( iunpun, "IONS" )
-    !     !
-    !     RETURN
-    !     !
-    !  END IF
-    !  !
-    !  !
-    !  DO i = 1, nsp
-    !     !
-    !     IF ( qexml_version_before_1_4_0 ) THEN
-    !        !
-    !        CALL iotk_scan_dat( iunpun, "ATOM_TYPE", atm(i) )
-    !        CALL iotk_scan_dat( iunpun, TRIM( atm(i) )//"_MASS", amass(i) )
-    !        CALL iotk_scan_dat( iunpun, "PSEUDO_FOR_" // TRIM( atm(i) ), psfile(i) )
-    !        !
-    !     ELSE
-    !        !
-    !        ! current format
-    !        !
-    !        CALL iotk_scan_begin( iunpun, "SPECIE"//TRIM(iotk_index(i)) )
-    !        !
-    !        CALL iotk_scan_dat( iunpun, "ATOM_TYPE", atm(i) )
-    !        CALL iotk_scan_dat( iunpun, "MASS", amass(i) )
-    !        CALL iotk_scan_dat( iunpun, "PSEUDO", psfile(i) )
-    !        !
-    !        CALL iotk_scan_end( iunpun, "SPECIE"//TRIM(iotk_index(i)) )
-    !        !
-    !     ENDIF
-    !     !
-    !  ENDDO
-    !  !
-    !  CALL iotk_scan_empty( iunpun, "UNITS_FOR_ATOMIC_POSITIONS", attr )
-    !  CALL iotk_scan_attr( attr, "UNITS", pos_unit  )
-    !  !
-    !  DO i = 1, nat
-    !     !
-    !     CALL iotk_scan_empty( iunpun, "ATOM" // TRIM( iotk_index( i ) ), attr )
-    !     CALL iotk_scan_attr( attr, "SPECIES", lab )
-    !     CALL iotk_scan_attr( attr, "INDEX",   ityp(i) )
-    !     CALL iotk_scan_attr( attr, "tau",     tau(:,i) )
-    !     CALL iotk_scan_attr( attr, "if_pos",  if_pos(:,i) )
-    !     !
-    !  END DO
-    !  !
-    !  CALL iotk_scan_end( iunpun, "IONS" )
-    !  !
-    !  RETURN
-    !  !
-    !END SUBROUTINE read_ions
-    !
-    !------------------------------------------------------------------------
-    !SUBROUTINE read_ions( nsp, nat, atm, ityp, psfile, &
-    !                      amass, tau, if_pos, pos_unit, ierr )
-      !------------------------------------------------------------------------
-      !
-    !  INTEGER,            INTENT(OUT) :: nsp, nat
-    !  CHARACTER(LEN=3),   INTENT(OUT) :: atm(:)
-    !  INTEGER,            INTENT(OUT) :: ityp(:)
-    !  CHARACTER(LEN=256), INTENT(OUT) :: psfile(:)
-    !  REAL(DP),           INTENT(OUT) :: amass(:)
-    !  REAL(DP),           INTENT(OUT) :: tau(:,:)
-    !  INTEGER,            INTENT(OUT) :: if_pos(:,:)
-    !  INTEGER,            INTENT(OUT) :: ierr
-    !  CHARACTER(LEN=*),   INTENT(OUT) :: pos_unit
-    !  !
-    !  LOGICAL          :: found, back_compat
-    !  INTEGER          :: i
-    !  CHARACTER(LEN=3) :: lab
-    !  CHARACTER(iotk_attlenx)  :: attr
-    !  !
-    !  ierr = 0
-    !  !
-    !  CALL iotk_scan_begin( iunpun, "IONS", FOUND = found )
-    !  !
-    !  IF ( .NOT. found ) THEN
-    !     !
-    !     ierr = 1
-    !     !
-    !     RETURN
-    !     !
-    !  END IF
-    !  !
-    !  CALL iotk_scan_dat( iunpun, "NUMBER_OF_ATOMS",   nat )
-    !  CALL iotk_scan_dat( iunpun, "NUMBER_OF_SPECIES", nsp )
-    !  !
-    !  IF ( nsp > SIZE( atm ) .OR. nat > SIZE( ityp ) ) THEN
-    !     !
-    !     ierr = 10
-    !     !
-    !     CALL iotk_scan_end( iunpun, "IONS" )
-    !     !
-    !     RETURN
-    !     !
-    !  END IF
-    !  !
-    !  !
-    !  DO i = 1, nsp
-    !     !
-    !     IF ( qexml_version_before_1_4_0 ) THEN
-    !        !
-    !        CALL iotk_scan_dat( iunpun, "ATOM_TYPE", atm(i) )
-    !        CALL iotk_scan_dat( iunpun, TRIM( atm(i) )//"_MASS", amass(i) )
-    !        CALL iotk_scan_dat( iunpun, "PSEUDO_FOR_" // TRIM( atm(i) ), psfile(i) )
-    !        !
-    !     ELSE
-    !        !
-    !        ! current format
-    !        !
-    !        CALL iotk_scan_begin( iunpun, "SPECIE"//TRIM(iotk_index(i)) )
-    !        !
-    !        CALL iotk_scan_dat( iunpun, "ATOM_TYPE", atm(i) )
-    !        CALL iotk_scan_dat( iunpun, "MASS", amass(i) )
-    !        CALL iotk_scan_dat( iunpun, "PSEUDO", psfile(i) )
-    !        !
-    !        CALL iotk_scan_end( iunpun, "SPECIE"//TRIM(iotk_index(i)) )
-    !        !
-    !     ENDIF
-    !     !
-    !  ENDDO
-    !  !
-    !  CALL iotk_scan_empty( iunpun, "UNITS_FOR_ATOMIC_POSITIONS", attr )
-    !  CALL iotk_scan_attr( attr, "UNITS", pos_unit  )
-    !  !
-    !  DO i = 1, nat
-    !     !
-    !     CALL iotk_scan_empty( iunpun, "ATOM" // TRIM( iotk_index( i ) ), attr )
-    !     CALL iotk_scan_attr( attr, "SPECIES", lab )
-    !     CALL iotk_scan_attr( attr, "INDEX",   ityp(i) )
-    !     CALL iotk_scan_attr( attr, "tau",     tau(:,i) )
-    !     CALL iotk_scan_attr( attr, "if_pos",  if_pos(:,i) )
-    !     !
-    !  END DO
-    !  !
-    !  CALL iotk_scan_end( iunpun, "IONS" )
-    !  !
-    !  RETURN
-    !  !
-    !END SUBROUTINE read_ions
-    !
+#if defined (__OLDXML)
     !----------------------------------------------------------------------------
     SUBROUTINE read_cell( ibrav, celldm, alat, a1, a2, a3, b1, b2, b3 )
       !------------------------------------------------------------------------
@@ -2430,6 +2188,6 @@ MODULE cp_restart
        RETURN
 
     END SUBROUTINE write_gk
-    !
+#endif
     !
 END MODULE cp_restart
