@@ -39,7 +39,7 @@
              !
           END IF
 
-          doublegrid = ( dual > 4.D0 )
+          doublegrid = ( dual > 4.0_dp + eps8 )
           IF ( doublegrid .AND. .NOT. okvan ) &
              CALL errore( 'setup', 'No USPP: set ecutrho=4*ecutwfc', 1 )
           ecutrho = dual * ecutwfc
@@ -235,13 +235,13 @@
 
 6         FORMAT(/,3X,'Electronic states',/  &
                   ,3X,'-----------------',/  &
-                  ,3X,'Number of Electron = ',I5,', of States = ',I5,/ &
+                  ,3X,'Number of Electrons= ',I5,', of States = ',I5,/ &
                   ,3X,'Occupation numbers :')
 7         FORMAT(2X,10F5.2)
 8         FORMAT(/,3X,'Electronic states',/  &
                   ,3X,'-----------------',/  &
                   ,3X,'Local Spin Density calculation',/ &
-                  ,3X,'Number of Electron = ',I5)
+                  ,3X,'Number of Electrons= ',I5)
 9         FORMAT(  3X,'Spins up   = ', I5, ', occupations: ')
 10        FORMAT(  3X,'Spins down = ', I5, ', occupations: ')
 11        FORMAT(/,3X,'WARNING: system charge = ',F12.6)
@@ -514,6 +514,7 @@ SUBROUTINE gmeshinfo( )
    USE gvecw,     only: ngw_g, ngw, ngwx
    USE gvecs,     only: ngms_g, ngms, ngsx
    USE gvect,     only: ngm, ngm_g, ngmx
+   USE fft_base,  ONLY: dfftp, dffts
 
    IMPLICIT NONE
 
@@ -521,6 +522,16 @@ SUBROUTINE gmeshinfo( )
    INTEGER :: ierr, min_val, max_val, i
    REAL(DP) :: avg_val
 
+   IF( ngm /= dfftp%ngm ) THEN
+      CALL errore( " gmeshinfo ", " number of G-vectors in module gvect not consistent with FFT descriptor ", 1 )
+   END IF
+   IF( ngms /= dffts%ngm ) THEN
+      CALL errore( " gmeshinfo ", " number of G-vectors in module gvecs not consistent with FFT descriptor ", 2 )
+   END IF
+   IF( ngw /= dffts%ngw ) THEN
+      CALL errore( " gmeshinfo ", " number of G-vectors in module gvecw not consistent with FFT descriptor ", 2 )
+   END IF
+
    IF(ionode) THEN
       WRITE( stdout,*)
       WRITE( stdout,*) '  Reciprocal Space Mesh'
@@ -701,7 +712,7 @@ subroutine formf( tfirst, eself )
   use control_flags,   ONLY : iprint, tpre, iverbosity
   use io_global,       ONLY : stdout
   use mp_global,       ONLY : intra_bgrp_comm
-  use gvecs,           ONLY : ngms
+  USE fft_base,        ONLY : dffts
   use cell_base,       ONLY : omega, tpiba2, tpiba
   use ions_base,       ONLY : rcmax, zv, nsp, na
   use local_pseudo,    ONLY : vps, vps0, rhops, dvps, drhops
@@ -751,7 +762,7 @@ subroutine formf( tfirst, eself )
            dvps(1,is) = dvps_sp(is)%y(1) * cost1
         END IF
         !
-        DO ig = gstart, ngms
+        DO ig = gstart, dffts%ngm
            xg = SQRT( gg(ig) ) * tpiba
            vps (ig,is) = spline(  vps_sp(is), xg ) * cost1
            dvps(ig,is) = spline( dvps_sp(is), xg ) * cost1
@@ -761,27 +772,27 @@ subroutine formf( tfirst, eself )
 
         call formfn( rgrid(is)%r, rgrid(is)%rab, &
                      upf(is)%vloc(1:rgrid(is)%mesh), zv(is), rcmax(is), gg, &
-                     omega, tpiba2, rgrid(is)%mesh, ngms, oldvan(is), tpre, &
+                     omega, tpiba2, rgrid(is)%mesh, dffts%ngm, oldvan(is), tpre, &
                      vps(:,is), vps0(is), dvps(:,is) )
 
 ! obsolete BHS form
 ! call formfa( vps(:,is), dvps(:,is), rc1(is), rc2(is), wrc1(is), wrc2(is), &
 !              rcl(:,is,lloc(is)), al(:,is,lloc(is)), bl(:,is,lloc(is)),    &
-!              zv(is), rcmax(is), g, omega, tpiba2, ngms, gstart, tpre )
+!              zv(is), rcmax(is), g, omega, tpiba2, dffts%ngm, gstart, tpre )
 
      END IF
      !
      !     fourier transform of local pp and gaussian nuclear charge
      !
      call compute_rhops( rhops(:,is), drhops(:,is), zv(is), rcmax(is), gg, &
-                         omega, tpiba2, ngms, tpre )
+                         omega, tpiba2, dffts%ngm, tpre )
 
      if( tfirst .or. ( iverbosity > 2 ) )then
-        vpsum = SUM( vps( 1:ngms, is ) )
-        rhopsum = SUM( rhops( 1:ngms, is ) )
+        vpsum = SUM( vps( 1:dffts%ngm, is ) )
+        rhopsum = SUM( rhops( 1:dffts%ngm, is ) )
         call mp_sum( vpsum, intra_bgrp_comm )
         call mp_sum( rhopsum, intra_bgrp_comm )
-        WRITE( stdout,1250) vps(1,is),rhops(1,is)
+        WRITE( stdout,1250) (vps(ig,is),rhops(ig,is),ig=1,5)
         WRITE( stdout,1300) vpsum,rhopsum
      endif
      !
@@ -891,7 +902,7 @@ subroutine nlfh_x( stress, bec_bgrp, dbec, lambda, descla )
   !     contribution to the internal stress tensor due to the constraints
   !
   USE kinds,             ONLY : DP
-  use uspp,              ONLY : nkb, qq
+  use uspp,              ONLY : nkb, qq_nt
   use uspp_param,        ONLY : nh, nhm, nvb, ish
   use ions_base,         ONLY : na
   use electrons_base,    ONLY : nbspx, nbsp, nudx, nspin, nupdwn, iupdwn, ibgrp_g2l
@@ -980,9 +991,9 @@ subroutine nlfh_x( stress, bec_bgrp, dbec, lambda, descla )
                     do iv=1,nh(is)
                        do jv=1,nh(is)
                           inl=ish(is)+(jv-1)*na(is)+ia
-                          if(abs(qq(iv,jv,is)).gt.1.e-5) then
+                          if(abs(qq_nt(iv,jv,is)).gt.1.e-5) then
                              do i = 1, nc
-                                tmpbec(iv,i) = tmpbec(iv,i) +  qq(iv,jv,is) * bec( inl, i, iss  )
+                                tmpbec(iv,i) = tmpbec(iv,i) +  qq_nt(iv,jv,is) * bec( inl, i, iss  )
                              end do
                           endif
                        end do
@@ -1078,13 +1089,12 @@ subroutine nlinit
       use core,            ONLY : rhocb, allocate_core
       use constants,       ONLY : pi, fpi
       use ions_base,       ONLY : na, nsp
-      use uspp,            ONLY : aainit, beta, qq, dvan, nhtol, nhtolm, indv,&
+      use uspp,            ONLY : aainit, beta, qq_nt, dvan, nhtol, nhtolm, indv,&
                                   dbeta
       use uspp_param,      ONLY : upf, lmaxq, nbetam, lmaxkb, nhm, nh, ish, nvb
       use atom,            ONLY : rgrid
       use qgb_mod,         ONLY : qgb, dqgb
       use smallbox_gvec,   ONLY : ngb
-      use gvect,           ONLY : ngm
       use cp_interfaces,   ONLY : pseudopotential_indexes, compute_dvan, &
                                   compute_betagx, compute_qradx, build_pstab, build_cctab
       USE fft_base,        ONLY : dfftp
@@ -1120,13 +1130,13 @@ subroutine nlinit
       !
       call aainit( lmaxkb + 1 )
       !
-      CALL allocate_core( dfftp%nnr, ngm, ngb, nsp )
+      CALL allocate_core( dfftp%nnr, dfftp%ngm, ngb, nsp )
       !
       !
       allocate( beta( ngw, nhm, nsp ) )
       allocate( qgb( ngb, nhm*(nhm+1)/2, nsp ) )
-      allocate( qq( nhm, nhm, nsp ) )
-      qq  (:,:,:) =0.d0
+      allocate( qq_nt( nhm, nhm, nsp ) )
+      qq_nt  (:,:,:) =0.d0
       IF (tpre) THEN
          allocate( dqgb( ngb, nhm*(nhm+1)/2, nsp, 3, 3 ) )
          allocate( dbeta( ngw, nhm, nsp, 3, 3 ) )
@@ -1462,14 +1472,13 @@ END SUBROUTINE print_lambda_x
       USE kinds,              ONLY: DP
       USE ions_base,          ONLY: nsp
       USE gvect, ONLY: gstart, g, gg
-      USE gvecs,              ONLY: ngms
-      USE gvect,              ONLY: ngm, nl
       USE cell_base,          ONLY: omega, ainv, tpiba2
       USE mp,                 ONLY: mp_sum
       USE mp_global,          ONLY: intra_bgrp_comm
       USE uspp_param,         ONLY: upf
       USE fft_interfaces,     ONLY: fwfft
-      USE fft_base,           ONLY: dfftp
+      USE fft_base,           ONLY: dfftp, dffts
+      USE fft_helper_subroutines, ONLY: fftx_threed2oned
 
       IMPLICIT NONE
 
@@ -1490,26 +1499,28 @@ END SUBROUTINE print_lambda_x
       COMPLEX(DP) :: srhoc
       REAL(DP)    :: vxcc
       !
-      COMPLEX(DP), ALLOCATABLE :: vxc( : )
+      COMPLEX(DP), ALLOCATABLE :: vxc( : ), vxg(:)
 !
       dcc = 0.0d0
       !
       ALLOCATE( vxc( nnr ) )
+      ALLOCATE( vxg( dfftp%ngm ) )
       !
       vxc(:) = vxcr(:,1)
       !
       IF( nspin > 1 ) vxc(:) = vxc(:) + vxcr(:,2)
       !
-      CALL fwfft( 'Dense', vxc, dfftp )
+      CALL fwfft( 'Rho', vxc, dfftp )
+      CALL fftx_threed2oned( dfftp, vxc, vxg )
       !
       DO i=1,3
          DO j=1,3
-            DO ig = gstart, ngms
+            DO ig = gstart, dffts%ngm
                srhoc = 0.0d0
                DO is = 1, nsp
                  IF( upf(is)%nlcc ) srhoc = srhoc + sfac( ig, is ) * drhocg( ig, is )
                ENDDO
-               vxcc = DBLE( CONJG( vxc( nl( ig ) ) ) * srhoc ) / SQRT( gg(ig) * tpiba2 )
+               vxcc = DBLE( CONJG( vxg( ig ) ) * srhoc ) / SQRT( gg(ig) * tpiba2 )
                dcc(i,j) = dcc(i,j) + vxcc * &
      &                      2.d0 * tpiba2 * g(i,ig) *                  &
      &                    (g(1,ig)*ainv(j,1) +                         &
@@ -1519,6 +1530,7 @@ END SUBROUTINE print_lambda_x
          ENDDO
       ENDDO
 
+      DEALLOCATE( vxg )
       DEALLOCATE( vxc )
 
       dcc = dcc * omega
@@ -1538,7 +1550,7 @@ END SUBROUTINE print_lambda_x
       USE ions_base,          ONLY: na, nsp, nat
       USE io_global,          ONLY: stdout
       USE gvect, ONLY: gstart
-      USE uspp,               ONLY: nkb, qq
+      USE uspp,               ONLY: nkb, qq_nt
       USE uspp_param,         ONLY: nh, ish, nvb
       USE mp,                 ONLY: mp_sum
       USE mp_global,          ONLY: intra_bgrp_comm, nbgrp, inter_bgrp_comm
@@ -1614,7 +1626,7 @@ END SUBROUTINE print_lambda_x
                         DO ia=1,na(is)
                            inl=ish(is)+(iv-1)*na(is)+ia
                            jnl=ish(is)+(jv-1)*na(is)+ia
-                           rsum = rsum + qq(iv,jv,is)*becp_tmp(inl)*becp(jnl,ibgrp_k)
+                           rsum = rsum + qq_nt(iv,jv,is)*becp_tmp(inl)*becp(jnl,ibgrp_k)
                         END DO
                      END DO
                   END DO
@@ -1699,7 +1711,7 @@ END SUBROUTINE print_lambda_x
       USE kinds,             ONLY: DP
       USE io_global,         ONLY: stdout
       USE ions_base,         ONLY: na, nsp, nat
-      USE uspp,              ONLY: nhsa=>nkb, qq
+      USE uspp,              ONLY: nhsa=>nkb, qq_nt
       USE uspp_param,        ONLY: nhm, nh, ish, nvb
       USE electrons_base,    ONLY: nspin, iupdwn, nupdwn, nbspx_bgrp, ibgrp_g2l, i2gupdwn_bgrp, nbspx, &
                                    iupdwn_bgrp, nupdwn_bgrp
@@ -1798,9 +1810,9 @@ END SUBROUTINE print_lambda_x
                      DO iv=1,nh(is)
                         DO jv=1,nh(is)
                            inl=ish(is)+(jv-1)*na(is)+ia
-                           IF(ABS(qq(iv,jv,is)).GT.1.e-5) THEN
+                           IF(ABS(qq_nt(iv,jv,is)).GT.1.e-5) THEN
                               DO i=1,nc
-                                 tmpbec(iv,i)=tmpbec(iv,i) + qq(iv,jv,is)*bec(inl,i,iss)
+                                 tmpbec(iv,i)=tmpbec(iv,i) + qq_nt(iv,jv,is)*bec(inl,i,iss)
                               END DO
                            ENDIF
                         END DO
