@@ -18,7 +18,9 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
                                        ndr, ndw, nomore, tsde, textfor,        &
                                        tortho, tnosee, tnosep, trane, tranp,   &
                                        tsdp, tcp, tcap, ampre, amprp, tnoseh,  &
-                                       tolp, ortho_eps, ortho_max
+                                       tolp, ortho_eps, ortho_max,             &
+                                       tfirst, tlast !moved here to make
+                                                     !autopilot work
   USE core,                     ONLY : rhoc
   USE uspp_param,               ONLY : nhm, nh, nvb, ish
   USE uspp,                     ONLY : nkb, vkb, becsum, deeq, okvan, nlcc_any
@@ -37,10 +39,8 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
                                        berry_energy2, pberryel2, pberryion2
   USE ensemble_dft,             ONLY : tens, z0t, gibbsfe
   USE cg_module,                ONLY : tcg,  cg_update, c0old
-  USE gvect,                    ONLY : ngm, ngm_g
-  USE gvecs,                    ONLY : ngms
-  USE smallbox_gvec,                    ONLY : ngb
-  USE gvecw,                    ONLY : ngw, ngw_g
+  USE smallbox_gvec,            ONLY : ngb
+  USE gvecw,                    ONLY : ngw
   USE gvect,       ONLY : gstart, mill, eigts1, eigts2, eigts3
   USE ions_base,                ONLY : na, nat, amass, nax, nsp, rcmax
   USE ions_base,                ONLY : ind_srt, ions_cofmass, ions_kinene, &
@@ -108,16 +108,17 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   USE cp_autopilot,             ONLY : pilot
   USE ions_nose,                ONLY : ions_nose_allocate, ions_nose_shiftvar
   USE orthogonalize_base,       ONLY : updatc
-  USE control_flags,            ONLY : force_pairing
+  USE control_flags,            ONLY : force_pairing, tprint
   USE mp,                       ONLY : mp_bcast, mp_sum
   USE mp_global,                ONLY : root_bgrp, intra_bgrp_comm, np_ortho, &
                                        me_ortho, ortho_comm, &
                                        me_bgrp, inter_bgrp_comm, nbgrp, me_image
   USE ldaU_cp,                  ONLY : lda_plus_u, vupsi
-  USE fft_base,                 ONLY : dfftp
+  USE fft_base,                 ONLY : dfftp, dffts
   USE london_module,            ONLY : energy_london, force_london, stres_london
   USE input_parameters,         ONLY : tcpbo
   USE funct,                    ONLY : dft_is_hybrid, start_exx, exx_is_active
+  USE funct,                    ONLY : dft_is_meta
   !
   IMPLICIT NONE
   !
@@ -129,8 +130,8 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   !
   ! ... control variables
   !
-  LOGICAL :: tfirst, tlast, tstop, tconv
-  LOGICAL :: tprint, tfile, tstdout
+  LOGICAL :: tstop, tconv
+  LOGICAL :: tfile, tstdout
     !  logical variable used to control printout
   !
   ! ... forces on ions
@@ -163,17 +164,25 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   REAL(DP), ALLOCATABLE :: pmass(:)
   REAL(DP), ALLOCATABLE :: forceh(:,:)
   !
+  REAL(DP) :: exx_start_thr
   CALL start_clock( 'cpr_total' )
   !
   etot_out = 0.D0
   enow     = 1.D9
   stress   = 0.0D0
   thstress   = 0.0D0
-  !
-  tfirst = .TRUE.
-  tlast  = .FALSE.
+  !  moved to control_flags.f90 (Modules)
+  !  tfirst = .TRUE.
+  !  tlast  = .FALSE.
   nacc   = 5
   !
+  if (dft_is_meta()) then
+    !HK/MCA : for SCAN0 calculation the initial SCAN has to converge better than the PBE -> PBE0 case
+    exx_start_thr = 1.E+1_DP
+  else
+    exx_start_thr = 1.E+2_DP
+  end if ! dft_is_meta
+  !
   ALLOCATE ( pmass (nsp) )
   pmass(1:nsp) = amass(1:nsp) * amu_au
   nspin_sub = nspin
@@ -206,10 +215,11 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      dt2bye   = dt2 / emass
      nfi     = nfi + 1
      tlast   = ( nfi == nomore ) .OR. tlast
-     tprint  = ( MOD( nfi, iprint ) == 0 ) .OR. tlast 
+     tprint  = ( MOD( nfi, iprint ) == 0 ) .OR. tlast !this can be set to .true. also by cp_autopilot in 'call pilot(nfi)', to compute velocities of the wfc in the last step of CG
      tfile   = ( MOD( nfi, iprint ) == 0 )
      tstdout = ( MOD( nfi, iprint_stdout ) == 0 ) .OR. tlast
      !
+
      IF ( abivol ) THEN
         IF ( pvar ) THEN
            IF ( nfi .EQ. 1 ) THEN
@@ -268,7 +278,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
         !
         ! ... strucf calculates the structure factor sfac
         !
-        CALL strucf( sfac, eigts1, eigts2, eigts3, mill, ngms )
+        CALL strucf( sfac, eigts1, eigts2, eigts3, mill, dffts%ngm )
         !
      END IF
      !
@@ -369,6 +379,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
         !
      END IF
      !
+     !
      !=======================================================================
      !
      !              verlet algorithm
@@ -795,7 +806,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
            END IF
            CALL r_to_s( tau0, taus, na, nsp, ainv )
            CALL phfacs( eigts1,eigts2,eigts3, eigr, mill, taus, dfftp%nr1,dfftp%nr2,dfftp%nr3, nat )
-           CALL strucf( sfac, eigts1, eigts2, eigts3, mill, ngms )
+           CALL strucf( sfac, eigts1, eigts2, eigts3, mill, dffts%ngm )
            !
            IF ( thdyn )    CALL formf( tfirst, eself )
            IF ( tefield )  CALL efield_update( eigr )
@@ -814,6 +825,8 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      END IF
      !
      ! ... now:  cm=c(t) c0=c(t+dt)
+     ! ... and, if tcg == .true.  :
+     ! ...    c0old=c(t),c0=c(t+dt)
      !
      tfirst = .FALSE.
      !
@@ -858,7 +871,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      !
      IF( .NOT.exx_is_active().AND.dft_is_hybrid().AND.tconvthrs%active ) THEN
        !
-       IF(delta_etot.LT.tconvthrs%derho*1.E+2_DP) THEN
+       IF(delta_etot.LT.tconvthrs%derho*exx_start_thr) THEN
          !
          WRITE(stdout,'(/,3X,"Exact Exchange is turned on ...")')
          ! 
@@ -1096,7 +1109,9 @@ SUBROUTINE terminate_run()
   CALL print_clock( 'fft' )
   CALL print_clock( 'ffts' )
   CALL print_clock( 'fftw' )
-  CALL print_clock( 'fft_scatter' )
+  CALL print_clock( 'fft_scatt_xy' )
+  CALL print_clock( 'fft_scatt_yz' )
+  CALL print_clock( 'fft_scatt_tg' )
   CALL print_clock( 'betagx' )
   CALL print_clock( 'qradx' )
   CALL print_clock( 'tmp_clk1' )
@@ -1128,8 +1143,6 @@ SUBROUTINE terminate_run()
   !
   IF (tcg) call print_clock_tcg()
   !
-  CALL print_clock( 'ALLTOALL' )
-  !
   CALL plugin_clock()
   !
   CALL mp_report()
