@@ -26,15 +26,16 @@
       use cell_base,            only: ainv, at, omega, alat
       use small_box,            only: small_box_set
       use smallbox_grid_dim,    only: smallbox_grid_init,smallbox_grid_info
-      USE fft_types,            ONLY: fft_type_allocate, fft_type_init
+      USE fft_types,            ONLY: fft_type_init
       use ions_base,            only: nat
-      USE recvec_subs,          ONLY: ggen
-      USE gvect,                ONLY: mill_g, eigts1,eigts2,eigts3, gg, &
-                                      ecutrho, gcutm, gvect_init
-      use gvecs,                only: gcutms, gvecs_init
+      USE recvec_subs,          ONLY: ggen, ggens
+      USE gvect,                ONLY: mill_g, eigts1,eigts2,eigts3, g, gg, &
+                                      ecutrho, gcutm, gvect_init, mill, &
+                                      ig_l2g, gstart, ngm, ngm_g
+      use gvecs,                only: gcutms, gvecs_init, ngms
       use gvecw,                only: gkcut, gvecw_init, g2kin_init
       USE smallbox_subs,        ONLY: ggenb
-      USE fft_base,             ONLY: dfftp, dffts, dfftb, dfft3d, dtgs, fft_base_info
+      USE fft_base,             ONLY: dfftp, dffts, dfftb, fft_base_info
       USE fft_smallbox,         ONLY: cft_b_omp_init
       USE fft_base,             ONLY: smap
       USE control_flags,        ONLY: gamma_only, smallmem
@@ -47,14 +48,13 @@
       USE input_parameters,     ONLY: ref_cell, ref_alat
       use cell_base,            ONLY: ref_at, ref_bg
       USE exx_module,           ONLY: h_init
-      USE task_groups,          ONLY: task_groups_init
 
       implicit none
 ! 
       integer  :: i
       real(dp) :: rat1, rat2, rat3
       real(dp) :: bg(3,3), tpiba2 
-      integer :: ng_, ngs_, ngm_ , ngw_ 
+      integer :: ng_, ngs_, ngm_ , ngw_, nyfft_
 #if defined(__MPI)
       LOGICAL :: lpara = .true.
 #else
@@ -89,6 +89,10 @@
       h_init=at*alat
 
       ! ... Initialize FFT real-space grids and small box grid
+      nyfft_ = ntask_groups
+      dffts%has_task_groups = (ntask_groups >1)
+      dfftp%has_task_groups = .FALSE.
+      lpara = ( nproc_bgrp > 1 )
       !
       IF ( ref_cell ) THEN
         !
@@ -100,18 +104,18 @@
         WRITE( stdout,'(3X,"ref_cell_a2 =",1X,3f14.8,3x,"ref_cell_b2 =",3f14.8)') ref_at(:,2)*ref_alat,ref_bg(:,2)/ref_alat
         WRITE( stdout,'(3X,"ref_cell_a3 =",1X,3f14.8,3x,"ref_cell_b3 =",3f14.8)') ref_at(:,3)*ref_alat,ref_bg(:,3)/ref_alat
         !
-        !
-        CALL fft_type_init( dffts, smap, "wave", gamma_only, lpara, intra_bgrp_comm, ref_at, ref_bg, gkcut )
-        CALL fft_type_init( dfftp, smap, "rho", gamma_only, lpara, intra_bgrp_comm, ref_at, ref_bg,  gcutm )
-        CALL fft_type_init( dfft3d, smap, "wave", gamma_only, .false., intra_bgrp_comm, ref_at, ref_bg, gkcut)
+        CALL fft_type_init( dffts, smap, "wave", gamma_only, lpara, intra_bgrp_comm, ref_at, ref_bg, gkcut, nyfft=nyfft_ )
+        CALL fft_type_init( dfftp, smap, "rho", gamma_only, lpara, intra_bgrp_comm, ref_at, ref_bg,  gcutm, nyfft=nyfft_ )
         !
       ELSE
         !
-        CALL fft_type_init( dffts, smap, "wave", gamma_only, lpara, intra_bgrp_comm, at, bg, gkcut )
-        CALL fft_type_init( dfftp, smap, "rho", gamma_only, lpara, intra_bgrp_comm, at, bg,  gcutm )
-        CALL fft_type_init( dfft3d, smap, "wave", gamma_only, .false., intra_bgrp_comm, at, bg, gkcut)
+        CALL fft_type_init( dffts, smap, "wave", gamma_only, lpara, intra_bgrp_comm, at, bg, gkcut, nyfft=nyfft_ )
+        CALL fft_type_init( dfftp, smap, "rho", gamma_only, lpara, intra_bgrp_comm, at, bg,  gcutm, nyfft=nyfft_ )
         !
       END IF
+      ! define the clock labels ( this enables the corresponding fft too ! )
+      dffts%rho_clock_label = 'ffts' ; dffts%wave_clock_label = 'fftw'
+      dfftp%rho_clock_label = 'fft' 
       !
       !
       CALL smallbox_grid_init( dfftp, dfftb )
@@ -140,16 +144,10 @@
       !       (but only if the axis triplet is right-handed, otherwise
       !        for a left-handed triplet, ainv is minus the inverse of a)
       !
-      CALL task_groups_init( dffts, dtgs, ntask_groups )
       CALL fft_base_info( ionode, stdout )
-      ngw_ = dffts%nwl( dffts%mype + 1 )
-      ngs_ = dffts%ngl( dffts%mype + 1 )
-      ngm_ = dfftp%ngl( dfftp%mype + 1 )
-      IF( gamma_only ) THEN
-         ngw_ = (ngw_ + 1)/2
-         ngs_ = (ngs_ + 1)/2
-         ngm_ = (ngm_ + 1)/2
-      END IF
+      ngw_ = dffts%ngw
+      ngs_ = dffts%ngm
+      ngm_ = dfftp%ngm
 
       !
       ! ... Initialize reciprocal space local and global dimensions
@@ -173,18 +171,24 @@
         WRITE( stdout,'(3X,"Reference Cell alat  =",F14.8,1X,"A.U.")' ) ref_alat
         !
         IF( smallmem ) THEN
-           CALL ggen( gamma_only, ref_at, ref_bg, intra_bgrp_comm, no_global_sort = .TRUE. )
+           CALL ggen( dfftp, gamma_only, ref_at, ref_bg, gcutm, ngm_g, ngm, &
+                g, gg, mill, ig_l2g, gstart, no_global_sort = .TRUE. )
         ELSE
-           CALL ggen( gamma_only, ref_at, ref_bg )
+           CALL ggen( dfftp, gamma_only, ref_at, ref_bg, gcutm, ngm_g, ngm, &
+                g, gg, mill, ig_l2g, gstart )
         END IF
+        CALL ggens( dffts, gamma_only, ref_at, g, gg, mill, gcutms, ngms )
         !
       ELSE
         !
         IF( smallmem ) THEN
-           CALL ggen( gamma_only, at, bg, intra_bgrp_comm, no_global_sort = .TRUE. )
+           CALL ggen( dfftp, gamma_only, at, bg, gcutm, ngm_g, ngm, &
+                g, gg, mill, ig_l2g, gstart, no_global_sort = .TRUE. )
         ELSE
-           CALL ggen( gamma_only, at, bg )
+           CALL ggen( dfftp, gamma_only, at, bg, gcutm, ngm_g, ngm, &
+                g, gg, mill, ig_l2g, gstart )
         END IF
+        CALL ggens( dffts, gamma_only, at, g, gg, mill, gcutms, ngms )
         !
       END IF
 
@@ -221,7 +225,7 @@
          !
          CALL ggenb ( ecutrho, iverbosity )
          !
-#if defined __OPENMP
+#if defined _OPENMP
          CALL cft_b_omp_init( dfftb%nr1, dfftb%nr2, dfftb%nr3 )
 #endif
       ELSE IF( okvan .OR. nlcc_any ) THEN
@@ -247,7 +251,24 @@
       CALL stop_clock( 'init_dim' )
       !
       return
-      end subroutine init_dimensions
+
+   CONTAINS
+
+      SUBROUTINE fft_extra_info()
+         INTEGER :: ir
+         IF(ionode) THEN
+            WRITE(stdout,*) 'FFT parallelization for potentials'
+            WRITE(stdout,*) dfftp%nproc, dfftp%nproc2, dfftp%nproc3
+            WRITE(stdout,*) 'FFT parallelization for smooth grid'
+            WRITE(stdout,*) dffts%nproc, dffts%nproc2, dffts%nproc3
+         END IF
+         WRITE(1000+dfftp%mype,*) dfftp%nr1x, dfftp%nr2x
+         DO ir = 1, dfftp%nr1x * dfftp%nr2x
+            WRITE(1000+dfftp%mype,*) dfftp%isind( ir )
+         END DO
+      END SUBROUTINE fft_extra_info
+
+   END SUBROUTINE init_dimensions
 
 
 
@@ -265,7 +286,11 @@
       use cell_base,        only: at, alat, r_to_s, cell_init, deth
       use cell_base,        only: ibrav, ainv, h, hold, tcell_base_init
       USE ions_positions,   ONLY: allocate_ions_positions, tau0, taus
+#if defined (__OLDXML)
       use cp_restart,       only: cp_read_cell
+#else
+      use cp_restart_new,   only: cp_read_cell
+#endif
       USE fft_base,         ONLY: dfftb
       USE fft_smallbox_type,      ONLY: fft_box_allocate
       USE cp_main_variables,ONLY: ht0, htm, taub
@@ -350,12 +375,8 @@
         ! geometry is set to the cell parameters read from stdin
         !
         WRITE(stdout, '(3X,"ibrav = ",i4,"       cell parameters read from input file")') ibrav
-        do i = 1, 3
-            h(i,1) = at(i,1)*alat
-            h(i,2) = at(i,2)*alat
-            h(i,3) = at(i,3)*alat
-        enddo
 
+        h    = at * alat
         hold = h
 
       end if
@@ -399,7 +420,7 @@
       INTEGER,  INTENT(IN) :: iverbosity
       !
       REAL(DP) :: rat1, rat2, rat3
-      INTEGER :: ig, i1, i2, i3
+      INTEGER :: ig
       !
       !WRITE( stdout, "(4x,'h from newinit')" )
       !do i=1,3
@@ -412,11 +433,8 @@
       !
       !  re-calculate G-vectors and kinetic energy
       !
-      do ig=1,ngm
-         i1=mill(1,ig)
-         i2=mill(2,ig)
-         i3=mill(3,ig)
-         g(:,ig)=i1*bg(:,1)+i2*bg(:,2)+i3*bg(:,3)
+      do ig = 1, dfftp%ngm
+         g(:,ig)= mill(1,ig)*bg(:,1) + mill(2,ig)*bg(:,2) + mill(3,ig)*bg(:,3)
          gg(ig)=g(1,ig)**2 + g(2,ig)**2 + g(3,ig)**2
       enddo
       !
@@ -446,45 +464,23 @@
 
       USE fft_types, ONLY: fft_type_descriptor
       use io_global, only: stdout, ionode
+      USE fft_helper_subroutines, ONLY: fft_dist_info
 
       IMPLICIT NONE
 
-
       TYPE(fft_type_descriptor), INTENT(IN) :: dfftp, dffts
 
-      INTEGER :: i
-
       IF(ionode) THEN
-
         WRITE( stdout,*)
         WRITE( stdout,*) '  Real Mesh'
         WRITE( stdout,*) '  ---------'
-        WRITE( stdout,1000) dfftp%nr1, dfftp%nr2, dfftp%nr3, dfftp%nr1, dfftp%nr2, dfftp%npl, 1, 1, dfftp%nproc
-        WRITE( stdout,1010) dfftp%nr1x, dfftp%nr2x, dfftp%nr3x
-        WRITE( stdout,1020) dfftp%nnr
-        WRITE( stdout,*) '  Number of x-y planes for each processors: '
-        WRITE( stdout, fmt = '( 3X, "nr3l = ", 10I5 )' ) &
-           ( dfftp%npp( i ), i = 1, dfftp%nproc )
-
+        CALL fft_dist_info( dfftp, stdout )
         WRITE( stdout,*)
         WRITE( stdout,*) '  Smooth Real Mesh'
         WRITE( stdout,*) '  ----------------'
-        WRITE( stdout,1000) dffts%nr1, dffts%nr2, dffts%nr3, dffts%nr1, dffts%nr2, dffts%npl,1,1, dfftp%nproc
-        WRITE( stdout,1010) dffts%nr1x, dffts%nr2x, dffts%nr3x
-        WRITE( stdout,1020) dffts%nnr
-        WRITE( stdout,*) '  Number of x-y planes for each processors: '
-        WRITE( stdout, fmt = '( 3X, "nr3sl = ", 10I5 )' ) &
-           ( dffts%npp( i ), i = 1, dfftp%nproc )
-
+        CALL fft_dist_info( dffts, stdout )
       END IF
 
-1000  FORMAT(3X, &
-         'Global Dimensions   Local  Dimensions   Processor Grid',/,3X, &
-         '.X.   .Y.   .Z.     .X.   .Y.   .Z.     .X.   .Y.   .Z.',/, &
-         3(1X,I5),2X,3(1X,I5),2X,3(1X,I5) )
-1010  FORMAT(3X, 'Array leading dimensions ( nr1x, nr2x, nr3x )   = ', 3(1X,I5))
-1020  FORMAT(3X, 'Local number of cell to store the grid ( nrxx ) = ', 1X, I9 )
-
       RETURN
       END SUBROUTINE realspace_grids_info
 
