@@ -30,8 +30,6 @@
                                 e0, psihpsi, compute_entropy2, &
                                 compute_entropy_der, compute_entropy, &
                                 niter_cold_restart, lambda_cold
-      USE gvect,          ONLY: ngm
-      USE gvecs,          ONLY: ngms
       USE smallbox_gvec,  ONLY: ngb
       USE gvecw,          ONLY: ngw
       USE gvect,          ONLY: gstart
@@ -75,13 +73,13 @@
       COMPLEX (kind=DP)           :: eigrb( ngb, nat )
       REAL(kind=DP)               :: rhor( dfftp%nnr, nspin )
       REAL(kind=DP)               :: vpot( dfftp%nnr, nspin )
-      COMPLEX(kind=DP)            :: rhog( ngm, nspin )
+      COMPLEX(kind=DP)            :: rhog( dfftp%ngm, nspin )
       REAL(kind=DP)               :: rhos( dffts%nnr, nspin )
       REAL(kind=DP)               :: rhoc( dfftp%nnr )
       COMPLEX(kind=DP)            :: ei1( dfftp%nr1:dfftp%nr1, nat )
       COMPLEX(kind=DP)            :: ei2( dfftp%nr2:dfftp%nr2, nat )
       COMPLEX(kind=DP)            :: ei3( dfftp%nr3:dfftp%nr3, nat )
-      COMPLEX(kind=DP)            :: sfac( ngms, nsp )
+      COMPLEX(kind=DP)            :: sfac( dffts%ngm, nsp )
   
 
 !local variables
@@ -329,8 +327,6 @@
 
       USE ensemble_dft,   ONLY: tens,  ninner, ismear, etemp, &
                                  c0diag, becdiag, z0t, nrcx, nrlx
-      USE gvect,          ONLY: ngm
-      USE gvecs,          ONLY: ngms
       USE smallbox_gvec,  ONLY: ngb
       USE gvecw,          ONLY: ngw
       USE gvect,          ONLY: gstart
@@ -367,13 +363,13 @@
       COMPLEX (kind=DP)           :: eigrb( ngb, nat )
       REAL(kind=DP)               :: rhor( dfftp%nnr, nspin )
       REAL(kind=DP)               :: vpot( dfftp%nnr, nspin )
-      COMPLEX(kind=DP)            :: rhog( ngm, nspin )
+      COMPLEX(kind=DP)            :: rhog( dfftp%ngm, nspin )
       REAL(kind=DP)               :: rhos( dffts%nnr, nspin )
       REAL(kind=DP)               :: rhoc( dfftp%nnr )
       COMPLEX(kind=DP)            :: ei1( dfftp%nr1:dfftp%nr1, nat )
       COMPLEX(kind=DP)            :: ei2( dfftp%nr2:dfftp%nr2, nat )
       COMPLEX(kind=DP)            :: ei3( dfftp%nr3:dfftp%nr3, nat )
-      COMPLEX(kind=DP)            :: sfac( ngms, nsp )
+      COMPLEX(kind=DP)            :: sfac( dffts%ngm, nsp )
   
       REAL(kind=DP), INTENT(in)   :: c0hc0(nrcx,nrcx,nspin)
       REAL(kind=DP), INTENT(in)   :: c1hc1(nrcx,nrcx,nspin)
@@ -513,8 +509,6 @@
                                 compute_entropy2, nrlx, nrcx, &
                                 compute_entropy_der, compute_entropy, &
                                 niter_cold_restart, lambda_cold
-      USE gvect,          ONLY: ngm
-      USE gvecs,          ONLY: ngms
       USE smallbox_gvec,          ONLY: ngb
       USE gvecw,          ONLY: ngw
       USE gvect, &
