@@ -141,6 +141,7 @@ MODULE input
                                nstep_      => nstep
      USE control_flags, ONLY : tsde_          => tsde, &
                                tzeroe_        => tzeroe, &
+                               trescalee_     => trescalee, &
                                trhor_         => trhor, &
                                trhow_         => trhow, &
                                tksw_          => tksw,  &
@@ -182,7 +183,8 @@ MODULE input
      !
      ! ...  Other modules
      !
-     USE cp_main_variables,        ONLY : nprint_nfi
+     USE check_stop,         ONLY : max_seconds_ => max_seconds
+     USE cp_main_variables,  ONLY : nprint_nfi
      USE wave_base,          ONLY : frice_ => frice
      USE ions_base,          ONLY : fricp_ => fricp
      USE cell_base,          ONLY : frich_ => frich
@@ -209,10 +211,10 @@ MODULE input
         ortho_eps, ortho_max, ntyp, tolp, calculation, disk_io, dt,            &
         tcg, ndr, ndw, iprint, isave, tstress, k_points, tprnfor, verbosity,   &
         ampre, nstep, restart_mode, ion_positions, startingwfc,                &
-        orthogonalization, electron_velocities, nat, if_pos,                   &
+        orthogonalization, electron_velocities, nat, rd_if_pos,                &
         tefield, epol, efield, tefield2, epol2, efield2, remove_rigid_rot,     &
         iesr, saverho, rd_for, assume_isolated, wf_collect,                    &
-        memory, ref_cell, tcpbo
+        memory, ref_cell, tcpbo, max_seconds
      USE funct,              ONLY : dft_is_hybrid
      !
      IMPLICIT NONE
@@ -340,6 +342,8 @@ MODULE input
      ampre_  = ampre
      taurdr_ = .FALSE.
      !
+     max_seconds_ = max_seconds
+     !
      SELECT CASE ( TRIM( restart_mode ) )
        !
        CASE( 'from_scratch' )
@@ -465,6 +469,9 @@ MODULE input
           tzeroe_ = .FALSE.
         CASE ('zero')
           tzeroe_ = .TRUE.
+        CASE ('change_step')
+          tzeroe_=.FALSE.
+          trescalee_ = .TRUE.
         CASE DEFAULT
           CALL errore(' control_flags ',' unknown electron_velocities '//TRIM(electron_velocities), 1 )
       END SELECT
@@ -527,7 +534,7 @@ MODULE input
           tfor_  = .TRUE.
           fricp_ = 0.d0
         CASE ('cg')       ! Conjugate Gradient minimization for ions
-          CALL errore( "iosys ", " ion_dynamics = '//TRIM(ion_dynamics)//' not yet implemented ", 1 )
+          CALL errore( 'iosys ', ' ion_dynamics = '//TRIM(ion_dynamics)//' not yet implemented ', 1 )
         CASE ('damp')
           tsdp_      = .FALSE.
           tfor_      = .TRUE.
@@ -716,7 +723,7 @@ MODULE input
      !
      USE input_parameters, ONLY: ibrav , celldm , trd_ht, dt,                 &
            rd_ht, a, b, c, cosab, cosac, cosbc, ntyp , nat ,                  &
-           na_inp , sp_pos , rd_pos , rd_vel, atom_mass, atom_label, if_pos,  &
+           na_inp , sp_pos , rd_pos , rd_vel, atom_mass, atom_label,rd_if_pos,&
            atomic_positions, sic, sic_epsilon, ecutwfc,                       &
            ecutrho, ecfixed, qcutz, q2sigma, tk_inp, wmass,                   &
            ion_radius, emass, emass_cutoff, temph, fnoseh, nr1b, nr2b, nr3b,  &
@@ -819,7 +826,7 @@ MODULE input
      ! ...  Set ions base module
 
      CALL ions_base_init( ntyp , nat , na_inp , sp_pos , rd_pos , rd_vel,  &
-                          atom_mass, atom_label, if_pos, atomic_positions, &
+                          atom_mass, atom_label, rd_if_pos, atomic_positions, &
                           alat_ , at, ion_radius, rd_for )
 
      ! ...   Set Values for the cutoff
@@ -974,8 +981,6 @@ MODULE input
      ! ... initialize kernel table for nonlocal functionals
      !
      IF ( dft_is_nonlocc( ) ) THEN
-        IF( tpre .or. thdyn ) CALL errore( ' module setup ', &
-          ' Stress is not properly implemented with vdw-DF ', 1 )
         vdw_table_name_ = vdw_table_name
         inlc = get_inlc()
         call initialize_kernel_table(inlc)
