@@ -12,10 +12,8 @@ SUBROUTINE fftx_error__( calling_routine, message, ierr )
   !
   ! ... This is a simple routine which writes an error message to output: 
   !
+  USE fft_param
   IMPLICIT NONE
-#if defined(__MPI)
-  INCLUDE 'mpif.h'
-#endif
   !
   CHARACTER(LEN=*), INTENT(IN) :: calling_routine, message
     ! the name of the calling calling_routine
