@@ -6,25 +6,14 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-!--------------------------------------------------------------------------!
-! FFT scalar drivers Module - contains machine-dependent routines for      !
-! FFTW, FFTW3, ESSL (both 3d for serial execution and 1d+2d FFTs for       !
-! parallel execution; NEC ASL libraries (3d only, no parallel execution)   !
-! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
-! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
-!--------------------------------------------------------------------------!
-
 #if defined(__DFTI)
-
 #include "mkl_dfti.f90"
-
 !=----------------------------------------------------------------------=!
-   MODULE fft_scalar
+   MODULE fft_scalar_dfti
 !=----------------------------------------------------------------------=!
 
-       USE, intrinsic ::  iso_c_binding
        USE MKL_DFTI ! -- this can be found in the MKL include directory
+       USE fft_param
 
        IMPLICIT NONE
         SAVE
@@ -32,10 +21,6 @@
         PRIVATE
         PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
 
-! ...   Local Parameter
-
-#include "fft_param.f90"
-
         TYPE dfti_descriptor_array
            TYPE(DFTI_DESCRIPTOR), POINTER :: desc
         END TYPE
@@ -80,7 +65,7 @@
 
      INTEGER :: tid
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset, ldz_t
      INTEGER :: omp_get_max_threads
      EXTERNAL :: omp_get_max_threads
@@ -246,7 +231,7 @@
      LOGICAL :: dofft( nfftx ), found
      INTEGER, PARAMETER  :: stdout = 6
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
      INTEGER  :: itid, mytid, ntids
@@ -587,9 +572,9 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, do_fft_z, do_f
   CALL cfft3d (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign)
 
 END SUBROUTINE cfft3ds
-
-!=----------------------------------------------------------------------=!
-   END MODULE fft_scalar
-!=----------------------------------------------------------------------=!
-
+#else
+   MODULE fft_scalar_dfti
 #endif
+!=----------------------------------------------------------------------=!
+END MODULE fft_scalar_dfti
+!=----------------------------------------------------------------------=!
