@@ -189,6 +189,7 @@ CONTAINS
   INTEGER :: i
   ! counters on G space
   !
+  INTEGER :: nyfft=1
 
   REAL(DP) :: amod
   ! modulus of G vectors
@@ -216,15 +217,19 @@ CONTAINS
   ! compute gkcut calling an internal procedure
   !
   CALL calculate_gkcut()
-  
-  CALL fft_type_init( fc%dfftt, fc%smapt, "rho", .not. tk, .true., intra_pool_comm, fc%at_t, fc%bg_t, fc%gcutmt/gkcut )
+  CALL fft_type_init( fc%dfftt, fc%smapt, "rho", .not. tk, .true., intra_pool_comm, fc%at_t, fc%bg_t, fc%gcutmt,fc%dual_t, &
+                      nyfft=nyfft)
+  ! define the clock labels ( this enables the corresponding fft too ! )
+  fc%dfftt%rho_clock_label = 'fftc' ; fc%dfftt%wave_clock_label = 'fftcw'
+  !CALL fft_type_init( fc%dfftt, fc%smapt, "rho", .not. tk, .true., intra_pool_comm, fc%at_t, fc%bg_t, fc%gcutmt/gkcut )
   !
   ! set the values of fft arrays
   !
   fc%nrx1t  = fc%dfftt%nr1x
   fc%nrx2t  = fc%dfftt%nr2x
   fc%nrx3t  = fc%dfftt%nr3x
-
+  write(stdout,*) fc%dfftt%nr1x,fc%dfftt%nr2x,fc%dfftt%nr3x
+  write(stdout,*) fc%dfftt%nr1,fc%dfftt%nr2,fc%dfftt%nr3
   ! compute number of points per plane
   ncplane  = fc%nrx1t * fc%nrx2t
   ncplanes = fc%nrx1t * fc%nrx2t
@@ -245,17 +250,17 @@ CONTAINS
      &    CALL errore('data_structure','inconsistent plane dimension on dense grid', abs(fc%dfftt%nnp-ncplane) )
   
   WRITE( stdout, '(/5x,"Planes per process (custom) : nr3t =", &
-       &        i4," npp = ",i4," ncplane =",i6)') fc%nr3t, fc%dfftt%npp (me_pool + 1) , ncplane
+       &        i4," nr3p = ",i4," ncplane =",i6)') fc%nr3t, fc%dfftt%my_nr3p , ncplane
 
   WRITE( stdout,*)
   WRITE( stdout,'(5X,                                                     &
     & "Proc/  planes cols     G    "/5X)')
   DO i=1,nproc_pool
-    WRITE( stdout,'(5x,i4,1x,i5,i7,i9)') i, fc%dfftt%npp(i), fc%dfftt%nsp(i), fc%dfftt%ngl(i)
+    WRITE( stdout,'(5x,i4,1x,i5,i7,i9)') i, fc%dfftt%nr3p(i), fc%dfftt%nsp(i), fc%dfftt%ngl(i)
   ENDDO
   IF ( nproc_pool > 1 ) THEN
       WRITE( stdout,'(5x,"tot",2x,i5,i7,i9)') &
-      sum(fc%dfftt%npp(1:nproc_pool)), sum(fc%dfftt%nsp(1:nproc_pool)), sum(fc%dfftt%ngl(1:nproc_pool))
+      sum(fc%dfftt%nr3p(1:nproc_pool)), sum(fc%dfftt%nsp(1:nproc_pool)), sum(fc%dfftt%ngl(1:nproc_pool))
   ENDIF
   WRITE( stdout,*)
 
@@ -271,7 +276,9 @@ CONTAINS
 
   CALL calculate_gkcut()
 
-  CALL fft_type_init( fc%dfftt, fc%smapt, "rho", .not. tk, .false., intra_pool_comm, fc%at_t, fc%bg_t, fc%gcutmt/gkcut )
+  CALL fft_type_init( fc%dfftt, fc%smapt, "rho", .not. tk, .false., intra_pool_comm, fc%at_t, fc%bg_t, fc%gcutmt/gkcut, &
+                      nyfft=nyfft )
+  fc%dfftt%rho_clock_label = 'fftc' ; fc%dfftt%wave_clock_label = 'fftcw'
 
   fc%nrx1t  = fc%dfftt%nr1x
   fc%nrx2t  = fc%dfftt%nr2x
@@ -284,7 +291,10 @@ CONTAINS
   nxx   = fc%nrxxt
   nxxs  = fc%nrxxt
 
-  fc%ngmt  = fc%dfftt%ngl ( 1 )
+  call errore ('data_structure_custom','serial version not working',1)
+  !!! The following line is obsolete and breaks compilation
+  !!! fc%ngmt  = fc%dfftt%ngl (dffts%mype + 1 )
+  !!!
   IF( .not. tk ) THEN
      fc%ngmt = ( fc%ngmt + 1 ) / 2 
   ENDIF
@@ -328,6 +338,7 @@ CONTAINS
        ! if k-points are automatically generated (which happens later)
        ! use max(bg)/2 as an estimate of the largest k-point
        !
+      
        gkcut = 0.5d0 * max ( &
           sqrt (sum(fc%bg_t (1:3, 1)**2) ), &
           sqrt (sum(fc%bg_t (1:3, 2)**2) ), &
@@ -390,7 +401,7 @@ SUBROUTINE initialize_fft_custom(fc)
 
   
   call ggent(fc)
-  
+
   return
 END SUBROUTINE initialize_fft_custom
 
@@ -721,6 +732,7 @@ SUBROUTINE cft3t( fc, f, n1, n2, n3, nx1, nx2, nx3, sign )
   call stop_clock('cft3t')
   !
 #else
+   
   !
   ! ... serial case
   !
