@@ -28,7 +28,7 @@ subroutine pola_basis_lanczos(n_set,nstates,numpw, nsteps,ispin)
    USE mp, ONLY : mp_sum, mp_barrier, mp_bcast
    USE mp_world, ONLY : world_comm, mpime, nproc
    USE mp_pools, ONLY : intra_pool_comm
-   USE gvecs,              ONLY : nls, nlsm, doublegrid
+   USE gvecs,              ONLY : doublegrid
    USE fft_custom_gwl
    USE mp_wave, ONLY : mergewf,splitwf
    USE fft_base,             ONLY : dfftp, dffts
@@ -120,10 +120,10 @@ subroutine pola_basis_lanczos(n_set,nstates,numpw, nsteps,ispin)
          !trasform to r-space
          psic(:)=(0.d0,0.d0)
          do ig=1,max_ngm
-            psic(nl(ig))=tmp_g(ig)
-            psic(nlm(ig))=CONJG(tmp_g(ig))
+            psic(dfftp%nl(ig))=tmp_g(ig)
+            psic(dfftp%nlm(ig))=CONJG(tmp_g(ig))
          enddo
-         CALL invfft ('Dense', psic, dfftp)
+         CALL invfft ('Rho', psic, dfftp)
          tmp_r(:)=dble(psic(:))
          call davcio(tmp_r,dfftp%nnr,iunrprod,iw,1)
       enddo
@@ -644,7 +644,7 @@ subroutine lanczos_state(zstates, nstates, itype, nsteps,istate,ispin)
   USE wavefunctions_module, ONLY : evc, psic
   USE mp, ONLY : mp_sum, mp_barrier, mp_bcast
   USE mp_world, ONLY : mpime, nproc, world_comm
-  USE gvecs,              ONLY : nls, nlsm, doublegrid
+  USE gvecs,              ONLY : doublegrid
   USE g_psi_mod,            ONLY : h_diag, s_diag
   USE becmod,           ONLY : becp,allocate_bec_type,deallocate_bec_type
   USE uspp,                 ONLY : vkb, nkb, okvan
@@ -1292,7 +1292,7 @@ subroutine global_pola_lanczos(nstates,nstates_eff,threshold,nglobal,nsteps,nump
   USE mp_world, ONLY : world_comm
   USE wavefunctions_module, ONLY : evc, psic
   USE gvect
-  USE gvecs,              ONLY : nls, nlsm, doublegrid
+  USE gvecs,              ONLY : doublegrid
   USE fft_base,             ONLY : dfftp, dffts
   USE fft_interfaces,       ONLY : fwfft, invfft
   USE wannier_gw, ONLY : l_verbose
@@ -1532,8 +1532,8 @@ subroutine global_pola_lanczos(nstates,nstates_eff,threshold,nglobal,nsteps,nump
      do iv=1,num_nbndv(ispin),num_nbndv(ispin)-1!ATTENZIONE
 !put iv on real space
         psic(:)=(0.d0,0.d0)
-        psic(nls(igk_k(1:npw,1)))  = evc(1:npw,iv)
-        psic(nlsm(igk_k(1:npw,1))) = CONJG( evc(1:npw,iv) )
+        psic(dffts%nl(igk_k(1:npw,1)))  = evc(1:npw,iv)
+        psic(dffts%nlm(igk_k(1:npw,1))) = CONJG( evc(1:npw,iv) )
         CALL invfft ('Wave', psic, dffts)
         wv_real(:)= DBLE(psic(:))
 !loop on wannier_products
@@ -1542,15 +1542,15 @@ subroutine global_pola_lanczos(nstates,nstates_eff,threshold,nglobal,nsteps,nump
      !trasform to r-space
            psic(:)=(0.d0,0.d0)
            do ig=1,max_ngm
-              psic(nl(ig))=tmp_g(ig)
-              psic(nlm(ig))=CONJG(tmp_g(ig))
+              psic(dfftp%nl(ig))=tmp_g(ig)
+              psic(dfftp%nlm(ig))=CONJG(tmp_g(ig))
            enddo
-           CALL invfft ('Dense', psic, dfftp)
+           CALL invfft ('Rho', psic, dfftp)
            tmp_r(:)=dble(psic(:))
 !!form products with w_v and trasfrom in G space
            psic(:)=cmplx(tmp_r(:)*wv_real(:),0.d0)
            CALL fwfft ('Wave', psic, dffts)
-           wp_prod(1:npw) = psic(nls(igk_k(1:npw,1)))
+           wp_prod(1:npw) = psic(dffts%nl(igk_k(1:npw,1)))
 !!project on conduction subspace
            call pc_operator(wp_prod(:),ispin, .false.)
 !!do scalar product
@@ -1598,7 +1598,6 @@ end subroutine global_pola_lanczos
 subroutine orthonormalize_two_manifolds_scalapack( state1, n1,state2, n2, threshold, state_out, n_out)
 !this subroutine form am orthormal basis set from 2 manifold (with orthonormal basis sets)
 !ONLY FOR NORM_CONSERVING CASE
-#if defined(__SCALAPACK)
 
    USE io_global,            ONLY : stdout, ionode, ionode_id
    USE kinds,    ONLY : DP
@@ -1630,6 +1629,7 @@ subroutine orthonormalize_two_manifolds_scalapack( state1, n1,state2, n2, thresh
   REAL(kind=DP) :: sca
   INTEGER :: desc_a(9),desc_b(9)
  
+#if defined(__SCALAPACK)
 !buid overlap matrix
   n=n1+n2
   n_r=ceiling(real(n)/real(max(nprow,npcol)))
@@ -2199,7 +2199,7 @@ subroutine pola_basis_lanczos_real(n_set,nstates,numpw, nsteps,ispin)
    USE mp, ONLY : mp_sum, mp_barrier, mp_bcast
    USE mp_pools, ONLY : intra_pool_comm
    USE mp_world, ONLY : world_comm, mpime, nproc
-   USE gvecs,              ONLY : nls, nlsm, doublegrid
+   USE gvecs,              ONLY : doublegrid
    USE fft_custom_gwl
    USE mp_wave, ONLY : mergewf,splitwf
    USE fft_base,             ONLY : dfftp, dffts
