@@ -228,7 +228,15 @@ CONTAINS
       ! ... convergence is checked here
       !
       energy_error = ABS( energy_p - energy )
+      !
+      ! ... obscure PGI bug as of v.17.4
+      !
+#if defined (__PGI)
+      grad_in = MATMUL( TRANSPOSE(hinv_block), grad(1:n-9) )
+      grad_error = MAXVAL( ABS( grad_in ) )
+#else
       grad_error = MAXVAL( ABS( MATMUL( TRANSPOSE(hinv_block), grad(1:n-9)) ) )
+#endif
       conv_bfgs = energy_error < energy_thr
       conv_bfgs = conv_bfgs .AND. ( grad_error < grad_thr )
       !
@@ -237,7 +245,7 @@ CONTAINS
                                              TRANSPOSE(h) ) ) ) / omega
           conv_bfgs = conv_bfgs .AND. ( cell_error < cell_thr ) 
 #undef DEBUG
-#ifdef DEBUG
+#if defined(DEBUG)
            write (*,'(3f15.10)') TRANSPOSE ( RESHAPE( grad(n-8:n), (/ 3, 3 /) ) )
            write (*,*)
            write (*,'(3f15.10)') TRANSPOSE(h)
@@ -503,6 +511,10 @@ CONTAINS
          overlap(k_m, : ) = 1.0_DP
          overlap(k_m,k_m) = 0.0_DP
          !
+         ! make sure the overlap matrix is not singular 
+         !
+         FORALL( i = 1:k ) overlap(i,i) = overlap(i,i) * ( 1.0_DP + eps8 ) ; overlap(k_m,k_m) = eps8
+         !
          ! ... overlap is inverted via Bunch-Kaufman diagonal pivoting method
          !
          CALL DSYTRF( 'U', k_m, overlap, k_m, iwork, work, k_m, info )
@@ -916,12 +928,12 @@ CONTAINS
               &         I3," bfgs steps")' ) scf_iter, bfgs_iter
          IF ( lmovecell ) THEN
             WRITE( UNIT = stdout, &
-              & FMT = '(5X,"(criteria: energy < ",ES8.1,", force < ",ES8.1, &
-              &       ", cell < ",ES8.1,")")') energy_thr, grad_thr, cell_thr
+              & FMT = '(5X,"(criteria: energy < ",ES8.1," Ry, force < ",ES8.1,&
+              &       "Ry/Bohr, cell < ",ES8.1,"kbar)")') energy_thr, grad_thr, cell_thr
          ELSE
             WRITE( UNIT = stdout, &
-              & FMT = '(5X,"(criteria: energy < ",ES8.1,", force < ",ES8.1, &
-              &                        ")")') energy_thr, grad_thr
+              & FMT = '(5X,"(criteria: energy < ",ES8.1," Ry, force < ",ES8.1,&
+              &            " Ry/Bohr)")') energy_thr, grad_thr
          END IF
          WRITE( UNIT = stdout, &
               & FMT = '(/,5X,"End of BFGS Geometry Optimization")' )
