@@ -566,7 +566,7 @@ subroutine readgth (iunps, np, upf)
      ! pseudo with NLCC
      upf%nlcc=.true.
      upf%generated="New Soft-Accurate NLCC pseudopotentials, generated by Santanu Saha"
-     upf%author=upf%author//"/Saha"
+     upf%author=TRIM(upf%author)//"/Saha"
   ENDIF
   IF ( lmax-1 > lmaxx ) call errore ('readgth', 'strange lmax', lmax-1)
   IF ( lmax == lloc) THEN
