@@ -7,7 +7,7 @@
 !
 !---------------------------------------------
 ! TB
-! included monopole related stuff, search 'TB'
+! included gate related stuff, search 'TB'
 !---------------------------------------------
 !
 !=----------------------------------------------------------------------------=!
@@ -197,9 +197,9 @@ MODULE input_parameters
           ! if .TRUE. a sawtooth potential simulating a finite electric field
           ! is added to the local potential = only used in PW
 
-          ! TB - added monopole also below to the namelist
-        LOGICAL :: monopole = .FALSE.
-          ! if .TRUE. a monopole plane in charged systems is added with a
+          ! TB - added gate also below to the namelist
+        LOGICAL :: gate = .FALSE.
+          ! if .TRUE. a charged plate in charged systems is added with a
           ! total charge which is opposite to the charge of the system
 
           LOGICAL :: tefield2  = .false.
@@ -285,7 +285,7 @@ MODULE input_parameters
           gdir, nppstr, wf_collect, lelfield, nberrycyc, refg,            &
           tefield2, saverho, tabps, lkpoint_dir, use_wannier, lecrpa,     &
           tqmmm, vdw_table_name, lorbm, memory, point_label_type,         &
-          lfcpopt, lfcpdyn, input_xml_schema_file, monopole                                        
+          lfcpopt, lfcpdyn, input_xml_schema_file, gate                                        
 !
 !=----------------------------------------------------------------------------=!
 !  SYSTEM Namelist Input Parameters
@@ -373,6 +373,7 @@ MODULE input_parameters
           ! (do not) use symmetry, q => -q symmetry in k-point generation
         LOGICAL :: nosym_evc = .false.
           ! if .true. use symmetry only to symmetrize k points
+
         LOGICAL :: force_symmorphic = .false.
           ! if .true. disable fractionary translations (nonsymmorphic groups)
         LOGICAL :: use_all_frac = .false.
@@ -388,6 +389,9 @@ MODULE input_parameters
           ! pseudopotential files; 'none' means DFT is read from pseudos.
           ! Only used in PW - allowed values: any legal DFT value
 
+        REAL(DP) :: starting_charge( nsx ) = 0.0_DP
+          ! ONLY PW
+
         REAL(DP) :: starting_magnetization( nsx ) = 0.0_DP
           ! ONLY PW
 
@@ -436,6 +440,11 @@ MODULE input_parameters
         REAL(DP) :: conv_thr_init = 0.001_DP
         REAL(DP) :: conv_thr_multi = 0.1_DP
         REAL(DP) :: ecutfock = -1.d0
+          ! variables used in Lin Lin's ACE and SCDM
+        REAL(DP) :: localization_thr = 0.0_dp, scdmden=1.0d0, scdmgrd=1.0d0
+        INTEGER  :: n_proj  = 0
+        LOGICAL  :: scdm=.FALSE.
+        LOGICAL  :: ace=.TRUE.
 
           ! parameters for external electric field
         INTEGER  :: edir = 0
@@ -443,10 +452,10 @@ MODULE input_parameters
         REAL(DP) :: eopreg = 0.0_DP
         REAL(DP) :: eamp = 0.0_DP
 
-          ! TB parameters for monopole representing the gate
+          ! TB parameters for charged plate representing the gate
           ! and a possible potential barrier
           ! added also below to the namelist
-        REAL(DP) :: zmon  = 0.5
+        REAL(DP) :: zgate = 0.5
         LOGICAL  :: relaxz = .false.
         LOGICAL  :: block = .false.
         REAL(DP) :: block_1 = 0.45
@@ -507,6 +516,13 @@ MODULE input_parameters
                        london_c6( nsx ) = -1.0_DP, &
                        london_rvdw( nsx ) = -1.0_DP
 
+          ! Grimme-D3 (DFT-D3) dispersion correction.
+          ! version=3 is Grimme-D3, version=2 is Grimme-D2,
+          ! version=3 dftd3_threebody = .false. similar to grimme d2 but with
+          ! the two_body parameters of d3
+        integer  ::    dftd3_version = 3
+        logical ::     dftd3_threebody = .true.
+
         LOGICAL   :: ts_vdw = .false.
           ! OBSOLESCENT: same as vdw_corr='Tkatchenko-Scheffler'
         LOGICAL :: ts_vdw_isolated = .FALSE.
@@ -518,10 +534,8 @@ MODULE input_parameters
 
         LOGICAL :: xdm = .FALSE.
           ! OBSOLESCENT: same as vdw_corr='xdm'
-        REAL(DP) :: xdm_a1 = 0.6836_DP
-        REAL(DP) :: xdm_a2 = 1.5045_DP
-          ! xdm_a1 and xdm_a2 -- parameters for the BJ damping function
-          ! The default values are for the b86bpbe functional.
+        REAL(DP) :: xdm_a1 = 0.0_DP
+        REAL(DP) :: xdm_a2 = 0.0_DP
           !
         CHARACTER(LEN=3) :: esm_bc = 'pbc'
           ! 'pbc': ordinary calculation with periodic boundary conditions
@@ -549,20 +563,37 @@ MODULE input_parameters
           ! used to enable debug mode (output v_hartree and v_local)
 
         INTEGER :: esm_debug_gpmax = 0
-          ! if esm_debug is .TRUE., calcualte v_hartree and v_local
+          ! if esm_debug is .TRUE., calculate v_hartree and v_local
           ! for abs(gp)<=esm_debug_gpmax (gp is integer and has tpiba unit)
 
         REAL(DP) :: fcp_mu         = 0.0_DP
           ! target Fermi energy
+
         REAL(DP) :: fcp_mass       = 10000.0_DP
           ! mass for the FCP
+
         REAL(DP) :: fcp_tempw      = 300.0_DP
           ! target temperature for the FCP dynamics
+
+        CHARACTER(LEN=8) :: fcp_relax = 'lm'
+          ! 'lm':    line minimisation
+          ! 'mdiis': MDIIS algorithm
+
+        CHARACTER(len=8) :: fcp_relax_allowed(2)
+        DATA fcp_relax_allowed / 'lm', 'mdiis' /
+
         REAL(DP) :: fcp_relax_step = 0.5_DP
           ! step size for steepest descent
+
         REAL(DP) :: fcp_relax_crit = 0.001_DP
           ! threshold for force acting on FCP
 
+        INTEGER :: fcp_mdiis_size = 4
+          ! size of MDIIS algorism
+
+        REAL(DP) :: fcp_mdiis_step = 0.2_DP
+          ! step width of MDIIS algorism
+
         INTEGER :: space_group = 0
           ! space group number for coordinates given in crystallographic form
           !
@@ -585,14 +616,15 @@ MODULE input_parameters
         NAMELIST / system / ibrav, celldm, a, b, c, cosab, cosac, cosbc, nat, &
              ntyp, nbnd, ecutwfc, ecutrho, nr1, nr2, nr3, nr1s, nr2s,         &
              nr3s, nr1b, nr2b, nr3b, nosym, nosym_evc, noinv, use_all_frac,   &
-             force_symmorphic, starting_magnetization,                        &
+             force_symmorphic, starting_charge, starting_magnetization,       &
              occupations, degauss, nspin, ecfixed,                            &
              qcutz, q2sigma, lda_plus_U, lda_plus_u_kind,                     &
              Hubbard_U, Hubbard_J, Hubbard_alpha,                             &
              Hubbard_J0, Hubbard_beta,                                        &
              edir, emaxpos, eopreg, eamp, smearing, starting_ns_eigenvalue,   &
              U_projection_type, input_dft, la2F, assume_isolated,             &
-             nqx1, nqx2, nqx3, ecutfock,                                      &
+             nqx1, nqx2, nqx3, ecutfock, localization_thr, scdm, ace,         &
+             scdmden, scdmgrd, n_proj,                                        &
              exxdiv_treatment, x_gamma_extrapolation, yukawa, ecutvcut,       &
              exx_fraction, screening_parameter, ref_alat,                     &
              noncolin, lspinorb, starting_spin_angle, lambda, angle1, angle2, &
@@ -601,14 +633,15 @@ MODULE input_parameters
              sic, sic_epsilon, force_pairing, sic_alpha,                      &
              tot_charge, tot_magnetization, spline_ps, one_atom_occupations,  &
              vdw_corr, london, london_s6, london_rcut, london_c6, london_rvdw,&
+             dftd3_version, dftd3_threebody,                                  &
              ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,                       &
              xdm, xdm_a1, xdm_a2,                                             &
              step_pen, A_pen, sigma_pen, alpha_pen, no_t_rev,                 &
              esm_bc, esm_efield, esm_w, esm_nfit, esm_debug, esm_debug_gpmax, &
-             esm_a, esm_zb, fcp_mu, fcp_mass, fcp_tempw, fcp_relax_step,      &
-             fcp_relax_crit,                                                  &
+             esm_a, esm_zb, fcp_mu, fcp_mass, fcp_tempw, fcp_relax,           &
+             fcp_relax_step, fcp_relax_crit, fcp_mdiis_size, fcp_mdiis_step,  &
              space_group, uniqueb, origin_choice, rhombohedral,               &
-             zmon, relaxz, block, block_1, block_2, block_height
+             zgate, relaxz, block, block_1, block_2, block_height
 
 !=----------------------------------------------------------------------------=!
 !  ELECTRONS Namelist Input Parameters
@@ -1375,7 +1408,7 @@ MODULE input_parameters
 !
         REAL(DP), ALLOCATABLE :: rd_pos(:,:)  ! unsorted positions from input
         INTEGER,  ALLOCATABLE :: sp_pos(:)
-        INTEGER,  ALLOCATABLE :: if_pos(:,:)
+        INTEGER,  ALLOCATABLE :: rd_if_pos(:,:)
         INTEGER,  ALLOCATABLE :: na_inp(:)
         LOGICAL  :: tapos = .false.
         LOGICAL  :: lsg   = .false.
@@ -1509,7 +1542,7 @@ SUBROUTINE reset_input_checks()
     !
     IF ( allocated( rd_pos ) ) DEALLOCATE( rd_pos )
     IF ( allocated( sp_pos ) ) DEALLOCATE( sp_pos )
-    IF ( allocated( if_pos ) ) DEALLOCATE( if_pos )
+    IF ( allocated( rd_if_pos ) ) DEALLOCATE( rd_if_pos )
     IF ( allocated( na_inp ) ) DEALLOCATE( na_inp )
     IF ( allocated( rd_vel ) ) DEALLOCATE( rd_vel )
     IF ( allocated( sp_vel ) ) DEALLOCATE( sp_vel )
@@ -1517,7 +1550,7 @@ SUBROUTINE reset_input_checks()
     !
     ALLOCATE( rd_pos( 3, nat ) )
     ALLOCATE( sp_pos( nat)   )
-    ALLOCATE( if_pos( 3, nat ) )
+    ALLOCATE( rd_if_pos( 3, nat ) )
     ALLOCATE( na_inp( ntyp)  )
     ALLOCATE( rd_vel( 3, nat ) )
     ALLOCATE( sp_vel( nat)   )
@@ -1525,7 +1558,7 @@ SUBROUTINE reset_input_checks()
     !
     rd_pos = 0.0_DP
     sp_pos = 0
-    if_pos = 1
+    rd_if_pos = 1
     na_inp = 0
     rd_vel = 0.0_DP
     sp_vel = 0
@@ -1583,7 +1616,7 @@ SUBROUTINE reset_input_checks()
     IF ( allocated( wk ) ) DEALLOCATE( wk )
     IF ( allocated( rd_pos ) ) DEALLOCATE( rd_pos )
     IF ( allocated( sp_pos ) ) DEALLOCATE( sp_pos )
-    IF ( allocated( if_pos ) ) DEALLOCATE( if_pos )
+    IF ( allocated( rd_if_pos ) ) DEALLOCATE( rd_if_pos )
     IF ( allocated( na_inp ) ) DEALLOCATE( na_inp )
     IF ( allocated( rd_vel ) ) DEALLOCATE( rd_vel )
     IF ( allocated( sp_vel ) ) DEALLOCATE( sp_vel )
