@@ -36,7 +36,8 @@ MODULE london_module
   ! r     ( 3 , mxr )     : ordered distance vectors
   ! dist2 ( mxr )         : ordered distances
   !
-  REAL ( DP ) , PUBLIC :: scal6=0._dp , lon_rcut=0._dp , in_C6 ( nsx ), in_rvdw( nsx )
+  REAL ( DP ) , PUBLIC :: scal6=0._dp , lon_rcut=0._dp , &
+       in_C6 ( nsx ) = -1.0_dp, in_rvdw( nsx ) = -1.0_dp
   !
   ! scal6    : global scaling factor
   ! lon_rcut : public cut-off radius
@@ -67,6 +68,8 @@ MODULE london_module
       !
       USE cell_base ,          ONLY : alat, omega
       !
+      USE constants,           ONLY : eps16
+      !
       USE io_global,           ONLY : ionode, ionode_id, stdout
       !
 #if defined __MPI
@@ -207,7 +210,7 @@ MODULE london_module
            !
            i = atomic_number ( atom_label ( ilab ) )
            IF ( i > 0 .AND. i < 87 ) THEN
-              IF ( in_C6 (ilab) > 0.0_DP ) THEN
+              IF ( in_C6 (ilab) > -eps16 ) THEN
                  C6_i  ( ilab )  = in_C6 (ilab)
               ELSE
                  C6_i  ( ilab )  = vdw_coeffs(1,i)
@@ -228,8 +231,8 @@ MODULE london_module
          !
          DO ilab = 1 , ntyp
            !
-           IF ( ( C6_i  ( ilab ) < 0.d0 ) .or. &
-                ( R_vdw ( ilab ) < 0.d0 ) ) THEN
+           IF ( ( C6_i  ( ilab ) < -eps16 ) .or. &
+                ( R_vdw ( ilab ) < 0.0_DP ) ) THEN
              !
              CALL errore ( ' init_london ' ,&
                            ' one or more parameters not found ' , 4 )
@@ -253,9 +256,9 @@ MODULE london_module
            !
          END DO
          !
-         WRITE ( stdout ,'( /, 5X, "-------------------------------------" , &
-                          & /, 5X, "Parameters for Dispersion Correction:" , &
-                          & /, 5X, "-------------------------------------" , &
+         WRITE ( stdout ,'( /, 5X, "-------------------------------------------------" , &
+                          & /, 5X, "Parameters for Dispersion (Grimme-D2) Correction:" , &
+                          & /, 5X, "-------------------------------------------------" , &
                           & /, 5X, "  atom      VdW radius       C_6     " , / )' )
          DO ata = 1 , ntyp
             !
