@@ -5,7 +5,14 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-#if defined(__XSD)
+#if defined(__OLDXML)
+! 
+MODULE qexsd_input
+  IMPLICIT NONE
+  INTEGER :: dummy__
+END MODULE qexsd_input
+!
+#else
 !---------------------------------------------------------
 MODULE qexsd_input
 !--------------------------------------------------------
@@ -17,11 +24,9 @@ MODULE qexsd_input
   USE kinds,            ONLY : DP
   USE input_parameters, ONLY : input_xml_schema_file
   !
-  USE iotk_base,        ONLY : iotk_indent, iotk_maxindent
   USE constants,        ONLY : e2,bohr_radius_angs
-  USE iotk_module
-  USE qes_module
-  USE qexsd_module,     ONLY : qexsd_init_atomic_species,qexsd_init_atomic_structure,qexsd_init_dft      
+  USE qes_types_module
+  USE qes_libs_module
   !
   IMPLICIT NONE
   !
@@ -138,7 +143,7 @@ MODULE qexsd_input
            smearing_local="gaussian"
        CASE ('methfessel-paxton', 'm-p', 'mp')
            smearing_local="mp"
-       CASE ( 'marzari-vanderbilt', 'cold', 'm-v', 'mv') 
+       CASE ( 'marzari-vanderbilt', 'cold', 'm-v', 'mv','Marzari-Vanderbilt') 
            smearing_local="mv"
        CASE ('fermi-dirac', 'f-d', 'fd') 
            smearing_local="fd"
@@ -164,22 +169,24 @@ MODULE qexsd_input
      ALLOCATE (inpOcc_objs(inpOcc_size))
      IF ( inpOcc_size .GT. 1) THEN 
         CALL qes_init_inputOccupations( inpOcc_objs(1),"input_occupations", 1, &
-                  REAL(spin_degeneracy,KIND=DP),nbnd-1, input_occupations(2:nbnd) ) 
+                  REAL(spin_degeneracy,KIND=DP),input_occupations(2:nbnd) ) 
         CALL qes_init_inputOccupations( inpOcc_objs(2),"input_occupations", 2, & 
-                  REAL(spin_degeneracy,KIND=DP) , nbnd-1,input_occupations_minority(2:nbnd))
+                  REAL(spin_degeneracy,KIND=DP) , input_occupations_minority(2:nbnd))
      ELSE 
         CALL qes_init_inputOccupations( inpOcc_objs(1),"input_occupations", 1,            &
-                                        REAL(spin_degeneracy,KIND=DP) , nbnd-1, input_occupations(2:nbnd) )   
+                                        REAL(spin_degeneracy,KIND=DP) , input_occupations(2:nbnd) )   
      END IF
+  ELSE 
+     ALLOCATE (inpOcc_objs(0))
+     inpOcc_size = 0
   END IF
   !
   IF (PRESENT ( tot_mag)) tot_mag_ispresent = .TRUE.
         
-  CALL qes_init_bands(obj,TAGNAME,NBND_ISPRESENT=(nbnd .GT. 0), NBND = nbnd,&
-                      SMEARING_ISPRESENT = smearing_obj%lread, SMEARING = smearing_obj,& 
-                      TOT_CHARGE_ISPRESENT=.TRUE., TOT_CHARGE = tot_charge,  &
-                      TOT_MAGNETIZATION_ISPRESENT = tot_mag_ispresent, TOT_MAGNETIZATION = tot_mag, &
-                      OCCUPATIONS=occup_obj, INPUTOCCUPATIONS_ISPRESENT=inp_occ_arepresent, &
+  CALL qes_init_bands(obj,TAGNAME,NBND_ISPRESENT=(nbnd .GT. 0), NBND = nbnd, SMEARING_ISPRESENT = smearing_obj%lread,&
+                      SMEARING = smearing_obj, TOT_CHARGE_ISPRESENT=.TRUE., TOT_CHARGE = tot_charge,                 &
+                      TOT_MAGNETIZATION_ISPRESENT = tot_mag_ispresent, TOT_MAGNETIZATION = tot_mag,                  & 
+                      OCCUPATIONS=occup_obj, INPUTOCCUPATIONS_ISPRESENT=inp_occ_arepresent,                          &
                       NDIM_INPUTOCCUPATIONS= inpOcc_size, INPUTOCCUPATIONS = inpOcc_objs)
   CALL qes_reset_smearing(smearing_obj)
   CALL qes_reset_occupations(occup_obj)
@@ -284,6 +291,7 @@ MODULE qexsd_input
    CHARACTER(LEN=*),PARAMETER           :: TAGNAME="k_points_IBZ"
    TYPE(monkhorst_pack_type)            :: mpack_obj
    TYPE(k_point_type),ALLOCATABLE       :: kp_obj(:)
+   TYPE (k_point_type)                  :: dummy_kpobj(1)
    LOGICAL                              :: mpack_ispresent,kp_ispresent
    CHARACTER(LEN=100)                   :: kind_of_grid
    INTEGER                              :: kdim,ik,jk,kcount
@@ -302,7 +310,7 @@ MODULE qexsd_input
                                    s1,s2,s3,kind_of_grid)
       CALL qes_init_k_points_IBZ(obj,TAGNAME,monkhorst_pack_ispresent=.TRUE.,&
                                  monkhorst_pack=mpack_obj,nk_ispresent=.FALSE.,&
-                                 nk=0,k_point_ispresent=.FALSE.,ndim_k_point=0,k_point=kp_obj)
+                                 nk=0,k_point_ispresent=.FALSE.,ndim_k_point=0,k_point=dummy_kpobj)
       CALL qes_reset_monkhorst_pack(mpack_obj)
    ELSE
       IF ( ibrav_lattice ) THEN 
@@ -441,7 +449,7 @@ MODULE qexsd_input
       isotropic=.TRUE.
       isotropic_ispresent=.TRUE.  
    END IF
-   IF (free_cell_ispresent) CALL  qes_init_integerMatrix(free_cell_obj,"free_cell",3,3,my_forceh)
+   IF (free_cell_ispresent) CALL  qes_init_integerMatrix(free_cell_obj,"free_cell",[3,3],my_forceh )
    !
    CALL qes_init_cell_control(obj,TAGNAME, PRESSURE = pressure, CELL_DYNAMICS=cell_dynamics, WMASS_ISPRESENT=.TRUE.,&
                               WMASS=wmass, CELL_FACTOR_ISPRESENT=.TRUE., CELL_FACTOR=cell_factor,&
@@ -532,7 +540,7 @@ MODULE qexsd_input
    ! 
    CHARACTER(LEN=*),PARAMETER                  :: TAGNAME="external_atomic_forces"
    !
-   CALL qes_init_matrix(obj,TAGNAME,ndim1_mat=3,ndim2_mat=nat,mat=extfor)
+   CALL qes_init_matrix(obj,TAGNAME,[3,nat],mat=extfor )
    END SUBROUTINE qexsd_init_external_atomic_forces
    !
    !     
@@ -548,7 +556,7 @@ MODULE qexsd_input
    CHARACTER(LEN=*),PARAMETER           :: TAGNAME = "free_positions" 
    REAL(DP),DIMENSION(:,:),ALLOCATABLE  :: free_positions
    ! 
-   CALL qes_init_integerMatrix(obj,TAGNAME,ndim1_int_mat=3,ndim2_int_mat=nat,int_mat=if_pos(:,1:nat))
+   CALL qes_init_integerMatrix(obj,TAGNAME,[3,nat], int_mat=if_pos )
    END SUBROUTINE qexsd_init_free_positions
    ! 
    !----------------------------------------------------------------------------------
@@ -568,7 +576,7 @@ MODULE qexsd_input
       xdim=3
       ydim=nat
    END IF
-   CALL qes_init_matrix(obj,TAGNAME,xdim,ydim,rd_vel)
+   CALL qes_init_matrix(obj,TAGNAME,[xdim,ydim],rd_vel )
    END SUBROUTINE qexsd_init_starting_atomic_velocities
    ! 
    !-------------------------------------------------------------------------------------
@@ -595,7 +603,7 @@ MODULE qexsd_input
    ! 
    !-------------------------------------------------------------------------------------------------
    SUBROUTINE qexsd_init_electric_field_input (obj,tefield,dipfield,lelfield,lberry,edir,gdir,emaxpos,eopreg,eamp,    &
-                                               efield,efield_cart,nberrycyc,nppstr)
+         efield,efield_cart,nberrycyc,nppstr, gate, zgate, relaxz, block, block_1, block_2, block_height )
    !---------------------------------------------------------------------------------------------------
    ! 
    IMPLICIT NONE
@@ -606,6 +614,8 @@ MODULE qexsd_input
    REAL(DP),INTENT(IN),OPTIONAL                 :: emaxpos,eopreg,eamp
    REAL(DP),INTENT(IN),OPTIONAL                 :: efield
    REAL(DP),INTENT(IN),OPTIONAL,DIMENSION(3)    :: efield_cart
+   LOGICAL,INTENT(IN),OPTIONAL                  :: gate, block,relaxz 
+   REAL(DP),INTENT(IN),OPTIONAL                 :: zgate,block_1, block_2, block_height
    ! 
    CHARACTER(LEN=*),PARAMETER                   :: TAGNAME="electric_field",&
                                                    SAWTOOTH="sawtooth_potential",&
@@ -618,7 +628,12 @@ MODULE qexsd_input
    LOGICAL                                      :: dir_ispresent=.FALSE., amp_ispresent= .FALSE.,&
                                                    nberrycyc_ispresent=.FALSE.,nppstr_ispresent=.FALSE., &
                                                    electric_field_ispresent = .FALSE.
+   LOGICAL                                      :: gate_, block_
+   REAL(DP)                                     :: block_1_, block_2_, block_3_
+   TYPE(gate_settings_type),TARGET              :: gata_settings_obj
+   TYPE(gate_settings_type),POINTER             :: gata_settings_ptr
    ! 
+   electric_potential = "none"
    IF (tefield) THEN  
       electric_potential=SAWTOOTH
       emaxpos_loc=emaxpos
@@ -655,7 +670,11 @@ MODULE qexsd_input
          electric_field_direction = gdir
       END IF
    END IF  
-      
+   IF (PRESENT (gate)) THEN 
+      gata_settings_ptr => gata_settings_obj 
+      CALL qes_init_gate_settings(gata_settings_obj, "gate_settings", gate, zgate, relaxz,&
+         block, block_1, block_2, block_height ) 
+   END IF 
    CALL  qes_init_electric_field( obj, TAGNAME, electric_potential=electric_potential,        &
                                 dipole_correction_ispresent=dipfield, dipole_correction = dipfield, &
                                 electric_field_direction_ispresent= dir_ispresent, &
@@ -667,7 +686,8 @@ MODULE qexsd_input
                                 electric_field_vector = efield_cart_loc,                                     &
                                 electric_field_vector_ispresent= electric_field_ispresent, &
                                 n_berry_cycles_ispresent=nberrycyc_ispresent,n_berry_cycles=nberrycyc_loc,&
-                                nk_per_string_ispresent=nppstr_ispresent,nk_per_string=nppstr_loc  )
+                                nk_per_string_ispresent=nppstr_ispresent,nk_per_string=nppstr_loc, &
+                                gate_settings = gata_settings_obj)
    END SUBROUTINE qexsd_init_electric_field_input
    !
    !----------------------------------------------------------------------------------------------------------
@@ -704,14 +724,54 @@ MODULE qexsd_input
    END SUBROUTINE qexsd_init_atomic_constraints
    !
    !------------------------------------------------------------------------------------------------------------ 
+   SUBROUTINE qexsd_init_occupations(obj, occupations, nspin) 
+   !------------------------------------------------------------------------------------------------------------
+     !
+     IMPLICIT NONE
+     TYPE(occupations_type),INTENT(OUT)        :: obj
+     CHARACTER(LEN=*),INTENT(IN)               :: occupations
+     INTEGER,INTENT(IN)                        :: nspin
+     ! 
+     INTEGER                                   :: spin_degeneracy
+     !   
+     IF (nspin .GT. 1) THEN 
+        spin_degeneracy = 1
+     ELSE 
+        spin_degeneracy = 2
+     END IF
+     CALL  qes_init_occupations(obj, "occupations", spin= spin_degeneracy, & 
+                              spin_ispresent =.FALSE., occupations = TRIM(occupations))
+   END SUBROUTINE qexsd_init_occupations
+   !
+   !---------------------------------------------------------
+   SUBROUTINE qexsd_init_smearing(obj, smearing, degauss)
+   !---------------------------------------------------------
+      !
+      IMPLICIT NONE
+      TYPE(smearing_type),INTENT(OUT)      :: obj
+      CHARACTER(LEN = * ), INTENT(IN)      :: smearing
+      REAL(DP),INTENT(IN)                  :: degauss 
+      ! 
+      CHARACTER(LEN=256)                   :: smearing_local 
+
+      SELECT CASE (TRIM  (smearing))
+        CASE ("gaussian", "gauss")
+            smearing_local="gaussian"
+        CASE ('methfessel-paxton', 'm-p', 'mp')
+            smearing_local="mp"
+        CASE ( 'marzari-vanderbilt', 'cold', 'm-v', 'mv', 'Marzari-Vanderbilt') 
+            smearing_local="mv"
+        CASE ('fermi-dirac', 'f-d', 'fd') 
+            smearing_local="fd"
+        CASE default
+            smearing_local='not set'
+      END SELECT 
+      CALL qes_init_smearing(obj,"smearing",degauss=degauss,smearing=smearing_local)
+      !
+      END SUBROUTINE qexsd_init_smearing
+      !--------------------------------------------------------------------------------------------
+      !
 END MODULE qexsd_input          
-         
-#else
-! 
-MODULE qexsd_input
-  IMPLICIT NONE
-  INTEGER :: dummy__
-END MODULE qexsd_input
 ! 
 #endif
   
