@@ -7,7 +7,7 @@
 !
 !---------------------------------------------
 ! TB
-! included monopole related stuff, search 'TB'
+! included gate related stuff, search 'TB'
 !---------------------------------------------
 !
 !----------------------------------------------------------------------------
@@ -112,12 +112,11 @@ MODULE read_namelists_module
        forc_conv_thr = 1.E-3_DP
        disk_io  = 'default'
        dipfield = .FALSE.
-       monopole = .FALSE. !TB
+       gate     = .FALSE. !TB
        lberry   = .FALSE.
        gdir     = 0
        nppstr   = 0
-       wf_collect = .FALSE.
-       IF( prog == 'CP' ) wf_collect = .TRUE.  ! default for CP is true
+       wf_collect = .TRUE.
        lelfield = .FALSE.
        lorbm = .FALSE.
        nberrycyc  = 1
@@ -190,6 +189,7 @@ MODULE read_namelists_module
        q2sigma = 0.01_DP
        input_dft = 'none'
        ecutfock  = -1.0_DP
+       starting_charge = 0.0_DP
 !
 ! ... set starting_magnetization to an invalid value:
 ! ... in PW starting_magnetization MUST be set for at least one atomic type
@@ -199,11 +199,12 @@ MODULE read_namelists_module
        starting_magnetization = sm_not_set
 
        IF ( prog == 'PW' ) THEN
-          !
           starting_ns_eigenvalue = -1.0_DP
           U_projection_type = 'atomic'
-          !
        END IF
+       !
+       ! .. DFT + U
+       !
        lda_plus_U = .FALSE.
        lda_plus_u_kind = 0
        Hubbard_U = 0.0_DP
@@ -215,20 +216,32 @@ MODULE read_namelists_module
        A_pen=0.0_DP
        sigma_pen=0.01_DP
        alpha_pen=0.0_DP
+       !
+       ! ... EXX
+       !
+       ace=.TRUE.
+       n_proj = 0    
+       localization_thr = 0.0_dp
+       scdm=.FALSE.
+       scdmden=1.0d0
+       scdmgrd=1.0d0
+       !
+       ! ... electric fields
+       !
        edir = 1
        emaxpos = 0.5_DP
        eopreg = 0.1_DP
        eamp = 0.0_DP
-       ! TB monopole related variables
-       zmon = 0.5
+       ! TB gate related variables
+       zgate = 0.5
        relaxz = .false.
        block = .false.
        block_1 = 0.45
        block_2 = 0.55
        block_height = 0.0
-
        !
        !  ... postprocessing of DOS & phonons & el-ph
+       !
        la2F = .FALSE.
        !
        ! ... non collinear program variables
@@ -267,8 +280,8 @@ MODULE read_namelists_module
        ts_vdw_isolated = .FALSE.
        ts_vdw_econv_thr = 1.E-6_DP
        xdm = .FALSE.
-       xdm_a1 = 0.6836_DP
-       xdm_a2 = 1.5045_DP
+       xdm_a1 = 0.0_DP
+       xdm_a2 = 0.0_DP
        !
        ! ... ESM
        !
@@ -286,8 +299,13 @@ MODULE read_namelists_module
        fcp_mu          = 0.0_DP
        fcp_mass        = 10000.0_DP
        fcp_tempw       = 0.0_DP
+       fcp_relax       = 'lm'
        fcp_relax_step  = 0.5_DP
        fcp_relax_crit  = 0.001_DP
+       fcp_mdiis_size  = 4
+       fcp_mdiis_step  = 0.2_DP
+       !
+       ! ... Wyckoff
        !
        space_group=0
        uniqueb = .FALSE.
@@ -721,7 +739,7 @@ MODULE read_namelists_module
        CALL mp_bcast( lfcpopt,       ionode_id, intra_image_comm )
        CALL mp_bcast( lfcpdyn,       ionode_id, intra_image_comm )
        CALL mp_bcast( input_xml_schema_file, ionode_id, intra_image_comm )
-       CALL mp_bcast( monopole,      ionode_id, intra_image_comm ) !TB
+       CALL mp_bcast( gate,          ionode_id, intra_image_comm ) !TB
        !
        RETURN
        !
@@ -780,19 +798,28 @@ MODULE read_namelists_module
        CALL mp_bcast( qcutz,             ionode_id, intra_image_comm )
        CALL mp_bcast( q2sigma,           ionode_id, intra_image_comm )
        CALL mp_bcast( input_dft,         ionode_id, intra_image_comm )
+
+       ! ... EXX
+
+       CALL mp_bcast( ace,                 ionode_id, intra_image_comm )
+       CALL mp_bcast( localization_thr,    ionode_id, intra_image_comm )
+       CALL mp_bcast( scdm,                ionode_id, intra_image_comm )
+       CALL mp_bcast( scdmden,             ionode_id, intra_image_comm )
+       CALL mp_bcast( scdmgrd,             ionode_id, intra_image_comm )
+       CALL mp_bcast( n_proj,              ionode_id, intra_image_comm )
        CALL mp_bcast( nqx1,                   ionode_id, intra_image_comm )
        CALL mp_bcast( nqx2,                   ionode_id, intra_image_comm )
        CALL mp_bcast( nqx3,                   ionode_id, intra_image_comm )
        CALL mp_bcast( exx_fraction,           ionode_id, intra_image_comm )
        CALL mp_bcast( screening_parameter,    ionode_id, intra_image_comm ) 
-       !gau-pbe in
        CALL mp_bcast( gau_parameter,          ionode_id, intra_image_comm )
-       !gau-pbe out
        CALL mp_bcast( exxdiv_treatment,       ionode_id, intra_image_comm )
        CALL mp_bcast( x_gamma_extrapolation,  ionode_id, intra_image_comm )
        CALL mp_bcast( yukawa,                 ionode_id, intra_image_comm )
        CALL mp_bcast( ecutvcut,               ionode_id, intra_image_comm )
        CALL mp_bcast( ecutfock,               ionode_id, intra_image_comm )
+       !
+       CALL mp_bcast( starting_charge,        ionode_id, intra_image_comm )
        CALL mp_bcast( starting_magnetization, ionode_id, intra_image_comm )
        CALL mp_bcast( starting_ns_eigenvalue, ionode_id, intra_image_comm )
        CALL mp_bcast( U_projection_type,      ionode_id, intra_image_comm )
@@ -812,7 +839,6 @@ MODULE read_namelists_module
        CALL mp_bcast( eopreg,                 ionode_id, intra_image_comm )
        CALL mp_bcast( eamp,                   ionode_id, intra_image_comm )
        CALL mp_bcast( la2F,                   ionode_id, intra_image_comm )
-
        !
        ! ... non collinear broadcast
        !
@@ -863,8 +889,11 @@ MODULE read_namelists_module
        CALL mp_bcast( fcp_mu,          ionode_id, intra_image_comm )
        CALL mp_bcast( fcp_mass,        ionode_id, intra_image_comm )
        CALL mp_bcast( fcp_tempw,       ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_relax,       ionode_id, intra_image_comm )
        CALL mp_bcast( fcp_relax_step,  ionode_id, intra_image_comm )
        CALL mp_bcast( fcp_relax_crit,  ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_mdiis_size,  ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_mdiis_step,  ionode_id, intra_image_comm )
        !
        !
        ! ... space group information
@@ -874,9 +903,9 @@ MODULE read_namelists_module
        CALL mp_bcast( origin_choice,      ionode_id, intra_image_comm )
        CALL mp_bcast( rhombohedral,       ionode_id, intra_image_comm )
        !
-       ! TB - monopole broadcast
+       ! TB - gate broadcast
        !
-       CALL mp_bcast( zmon,               ionode_id, intra_image_comm )
+       CALL mp_bcast( zgate,              ionode_id, intra_image_comm )
        CALL mp_bcast( relaxz,             ionode_id, intra_image_comm )
        CALL mp_bcast( block,              ionode_id, intra_image_comm )
        CALL mp_bcast( block_1,            ionode_id, intra_image_comm )
@@ -1254,8 +1283,8 @@ MODULE read_namelists_module
           IF( TRIM(calculation) == calculation_allowed(i) ) allowed = .TRUE.
        END DO
        IF( .NOT. allowed ) &
-          CALL errore( sub_name, ' calculation '''// &
-                       & TRIM(calculation)//''' not allowed ',1)
+          CALL errore( sub_name, ' calculation "'// &
+                       & TRIM(calculation)//'" not allowed ',1)
        IF( ndr < 50 ) &
           CALL errore( sub_name,' ndr out of range ', 1 )
        IF( ndw > 0 .AND. ndw < 50 ) &
@@ -1317,12 +1346,11 @@ MODULE read_namelists_module
           IF( TRIM(memory) == memory_allowed(i) ) allowed = .TRUE.
        END DO
        IF( .NOT. allowed ) &
-          CALL errore( sub_name, ' memory '''// &
-                       & TRIM(memory)//''' not allowed ',1)
+          CALL errore(sub_name, ' memory "' // TRIM(memory)//'" not allowed',1)
        ! TB
-       IF ( monopole .and. tefield .and. (.not. dipfield) ) &
-          CALL errore(sub_name, ' monopole cannot be used with tefield if dipole correction is not active', 1)
-       IF ( monopole .and. dipfield .and. (.not. tefield) ) &
+       IF ( gate .and. tefield .and. (.not. dipfield) ) &
+          CALL errore(sub_name, ' gate cannot be used with tefield if dipole correction is not active', 1)
+       IF ( gate .and. dipfield .and. (.not. tefield) ) &
           CALL errore(sub_name, ' dipole correction is not active if tefield = .false.', 1)
 
        RETURN
@@ -1442,12 +1470,21 @@ MODULE read_namelists_module
                                           TRIM(exxdiv_treatment) == "vcut_spherical" ) ) &
           CALL errore(sub_name, ' x_gamma_extrapolation cannot be used with vcut', 1 )
        !
-       ! TB - monopole check
+       ! TB - gate check
        !
-       IF ( monopole .and. tot_charge == 0 ) &
-          CALL errore(sub_name, ' charged plane (monopole) to compensate tot_charge of 0', 1)
+       IF ( gate .and. tot_charge == 0 ) &
+          CALL errore(sub_name, ' charged plane (gate) to compensate tot_charge of 0', 1)
        RETURN
        !
+       ! ... control on FCP variables
+       !
+       allowed = .FALSE.
+       DO i = 1, SIZE(fcp_relax_allowed)
+          IF( TRIM(fcp_relax) == fcp_relax_allowed(i) ) allowed = .TRUE.
+       END DO
+       IF( .NOT. allowed ) &
+          CALL errore(sub_name, ' fcp_relax '''//TRIM(fcp_relax)//''' not allowed ', 1)
+       !
      END SUBROUTINE
      !
      !=----------------------------------------------------------------------=!
@@ -1473,8 +1510,8 @@ MODULE read_namelists_module
               electron_dynamics_allowed(i) ) allowed = .TRUE.
        END DO
        IF( .NOT. allowed ) &
-          CALL errore( sub_name, ' electron_dynamics '''//&
-                       & TRIM(electron_dynamics)//''' not allowed ',1)
+          CALL errore( sub_name, ' electron_dynamics "'//&
+                       & TRIM(electron_dynamics)//'" not allowed ',1)
        IF( emass <= 0.0_DP ) &
           CALL errore( sub_name, ' emass less or equal 0 ',1)
        IF( emass_cutoff <= 0.0_DP ) &
@@ -1517,8 +1554,8 @@ MODULE read_namelists_module
           IF( TRIM(ion_dynamics) == ion_dynamics_allowed(i) ) allowed = .TRUE.
        END DO
        IF( .NOT. allowed ) &
-          CALL errore( sub_name, ' ion_dynamics '''// &
-                       & TRIM(ion_dynamics)//''' not allowed ',1)
+          CALL errore( sub_name, ' ion_dynamics "'// &
+                       & TRIM(ion_dynamics)//'" not allowed ',1)
        IF( tempw <= 0.0_DP ) &
           CALL errore( sub_name,' tempw out of range ',1)
        IF( fnosep( 1 ) <= 0.0_DP ) &
@@ -1561,8 +1598,8 @@ MODULE read_namelists_module
               cell_dynamics_allowed(i) ) allowed = .TRUE.
        END DO
        IF( .NOT. allowed ) &
-          CALL errore( sub_name, ' cell_dynamics '''// &
-                       TRIM(cell_dynamics)//''' not allowed ',1)
+          CALL errore( sub_name, ' cell_dynamics "'// &
+                       TRIM(cell_dynamics)//'" not allowed ',1)
        IF( wmass < 0.0_DP ) &
           CALL errore( sub_name,' wmass out of range ',1)
        IF( prog == 'CP' ) THEN
@@ -1769,7 +1806,7 @@ MODULE read_namelists_module
      !=----------------------------------------------------------------------=!
      !
      !-----------------------------------------------------------------------
-     SUBROUTINE read_namelists( prog, unit )
+     SUBROUTINE read_namelists( prog_, unit )
        !-----------------------------------------------------------------------
        !
        !  this routine reads data from standard input and puts them into
@@ -1787,17 +1824,17 @@ MODULE read_namelists_module
        !
        ! ... declare variables
        !
-       CHARACTER(LEN=2) :: prog   ! ... specify the calling program
-                                  !     prog = 'PW'  pwscf
-                                  !     prog = 'CP'  cpr
-       !
+       CHARACTER(LEN=*) :: prog_  ! specifies the calling program, allowed:
+                                  !     prog = 'PW'     pwscf
+                                  !     prog = 'CP'     cp
+                                  !     prog = 'PW+iPi' pwscf + i-Pi
        !
        INTEGER, INTENT(IN), optional :: unit
        !
        ! ... declare other variables
        !
+       CHARACTER(LEN=2) :: prog
        INTEGER :: ios
-       !
        INTEGER :: unit_loc=5
        !
        ! ... end of declarations
@@ -1806,34 +1843,28 @@ MODULE read_namelists_module
        !
        IF(PRESENT(unit)) unit_loc = unit
        !
+       prog = prog_(1:2) ! Allowed: 'PW' or 'CP'
        IF( prog /= 'PW' .AND. prog /= 'CP' ) &
           CALL errore( ' read_namelists ', ' unknown calling program ', 1 )
        !
        ! ... default settings for all namelists
        !
-       IF( prog == 'PW' .OR. prog == 'CP') THEN
-         CALL control_defaults( prog )
-         CALL system_defaults( prog )
-         CALL electrons_defaults( prog )
-         CALL ions_defaults( prog )
-         CALL cell_defaults( prog )
-       ENDIF
+       CALL control_defaults( prog )
+       CALL system_defaults( prog )
+       CALL electrons_defaults( prog )
+       CALL ions_defaults( prog )
+       CALL cell_defaults( prog )
        !
        ! ... Here start reading standard input file
        !
        !
        ! ... CONTROL namelist
        !
-       IF(prog == 'PW' .OR. prog == 'CP' ) THEN
        ios = 0
        IF( ionode ) THEN
           READ( unit_loc, control, iostat = ios )
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist control ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "control")
        !
        CALL control_bcast( )
        CALL control_checkin( prog )
@@ -1849,11 +1880,7 @@ MODULE read_namelists_module
        IF( ionode ) THEN
           READ( unit_loc, system, iostat = ios )
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist system ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "system")
        !
        CALL system_bcast( )
        !
@@ -1865,37 +1892,24 @@ MODULE read_namelists_module
        IF( ionode ) THEN
           READ( unit_loc, electrons, iostat = ios )
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist electrons ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "electrons")
        !
        CALL electrons_bcast( )
        CALL electrons_checkin( prog )
        !
-       ! ... IONS namelist
+       ! ... IONS namelist - must be read only if ionic motion is expected,
+       ! ...                 or if code called by i-Pi via run_driver
        !
        ios = 0
        IF ( ionode ) THEN
-          !
-          IF ( TRIM( calculation ) == 'relax'    .OR. &
-               TRIM( calculation ) == 'md'       .OR. &
-               TRIM( calculation ) == 'vc-relax' .OR. &
-               TRIM( calculation ) == 'vc-md'    .OR. &
-               TRIM( calculation ) == 'cp'       .OR. &
-               TRIM( calculation ) == 'vc-cp'    .OR. &
-               TRIM( calculation ) == 'smd'      .OR. &
-               TRIM( calculation ) == 'cp-wf-nscf' .OR. &
-               TRIM( calculation ) == 'vc-cp-wf'   .OR. &
-               TRIM( calculation ) == 'cp-wf' ) READ( unit_loc, ions, iostat = ios )
-  
-       END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist ions ', ABS(ios) )
+          IF ( ( TRIM( calculation ) /= 'scf'   .AND. &
+                 TRIM( calculation ) /= 'nscf'  .AND. &
+                 TRIM( calculation ) /= 'bands' ) .OR. &
+               ( TRIM( prog_ ) == 'PW+iPi' ) ) THEN
+             READ( unit_loc, ions, iostat = ios )
+          END IF
        END IF
+       CALL check_namelist_read(ios, unit_loc, "ions")
        !
        CALL ions_bcast( )
        CALL ions_checkin( prog )
@@ -1912,11 +1926,7 @@ MODULE read_namelists_module
              READ( unit_loc, cell, iostat = ios )
           END IF
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist cell ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "cell")
        !
        CALL cell_bcast()
        CALL cell_checkin( prog )
@@ -1927,11 +1937,7 @@ MODULE read_namelists_module
              READ( unit_loc, press_ai, iostat = ios )
           end if
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist press_ai ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "press_ai")
        !
        CALL press_ai_bcast()
        !
@@ -1946,11 +1952,7 @@ MODULE read_namelists_module
              READ( unit_loc, wannier, iostat = ios )
           END IF
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist wannier ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "wannier")
        !
        CALL wannier_bcast()
        CALL wannier_checkin( prog )
@@ -1964,20 +1966,48 @@ MODULE read_namelists_module
              READ( unit_loc, wannier_ac, iostat = ios )
           END IF
        END IF
-       CALL mp_bcast( ios, ionode_id, intra_image_comm )
-       IF( ios /= 0 ) THEN
-          CALL errore( ' read_namelists ', &
-                     & ' reading namelist wannier_new ', ABS(ios) )
-       END IF
+       CALL check_namelist_read(ios, unit_loc, "wannier_ac")
        !
        CALL wannier_ac_bcast()
        CALL wannier_ac_checkin( prog )
        !
-       ENDIF
-       !
        RETURN
        !
      END SUBROUTINE read_namelists
      !
+     SUBROUTINE check_namelist_read(ios, unit_loc, nl_name)
+       USE io_global, ONLY : ionode, ionode_id
+       USE mp,        ONLY : mp_bcast
+       USE mp_images, ONLY : intra_image_comm
+       !
+       IMPLICIT NONE
+       INTEGER,INTENT(in) :: ios, unit_loc
+       CHARACTER(LEN=*) :: nl_name
+       CHARACTER(len=512) :: line
+       INTEGER :: ios2
+       !
+       IF( ionode ) THEN
+         ios2=0
+         IF (ios /=0) THEN
+           BACKSPACE(unit_loc)
+           READ(unit_loc,'(A512)', iostat=ios2) line
+         END IF
+       END IF
+
+       CALL mp_bcast( ios2, ionode_id, intra_image_comm )
+       IF( ios2 /= 0 ) THEN
+          CALL errore( ' read_namelists ', ' could not find namelist &'//TRIM(nl_name), 2)
+       ENDIF
+       !
+       CALL mp_bcast( ios, ionode_id, intra_image_comm )
+       CALL mp_bcast( line, ionode_id, intra_image_comm )
+       IF( ios /= 0 ) THEN
+          CALL errore( ' read_namelists ', &
+                       ' bad line in namelist &'//TRIM(nl_name)//&
+                       ': "'//TRIM(line)//'" (error could be in the previous line)',&
+                       1 )
+       END IF
+       !
+     END SUBROUTINE check_namelist_read
      !
 END MODULE read_namelists_module
