@@ -1004,7 +1004,7 @@ PRIVATE :: GetVdWParam
   !
   DO iproc=1,nproc_bgrp
     !
-    recvcount(iproc)=dfftp%npp(iproc)*nr1*nr2
+    recvcount(iproc)=dfftp%nr3p(iproc)*nr1*nr2
     !
   END DO
   !
@@ -1016,13 +1016,13 @@ PRIVATE :: GetVdWParam
     !
   END DO
   !
-  CALL MPI_ALLGATHERV(rhor(1,1),dfftp%npp(me_bgrp+1)*nr1*nr2,&
+  CALL MPI_ALLGATHERV(rhor(1,1),dfftp%nr3p(me_bgrp+1)*nr1*nr2,&
       MPI_DOUBLE_PRECISION,rhor_tmp1(1),recvcount,rdispls,&
       MPI_DOUBLE_PRECISION,intra_bgrp_comm,ierr)
   !
   IF (nspin.EQ.2) THEN
     !
-    CALL MPI_ALLGATHERV(rhor(1,2),dfftp%npp(me_bgrp+1)*nr1*nr2,&
+    CALL MPI_ALLGATHERV(rhor(1,2),dfftp%nr3p(me_bgrp+1)*nr1*nr2,&
         MPI_DOUBLE_PRECISION,rhor_tmp2(1),recvcount,rdispls,&
         MPI_DOUBLE_PRECISION,intra_bgrp_comm,ierr)
     !
@@ -2216,10 +2216,10 @@ PRIVATE :: GetVdWParam
   !
   ! Partition out dispersion potential consistent with slabs of the charge density...
   !
-  IF (dfftp%npp(me_bgrp+1).NE.0) THEN
+  IF (dfftp%nr3p(me_bgrp+1).NE.0) THEN
     !
 !$omp parallel do 
-    DO ip=1,dfftp%npp(me_bgrp+1)*nr1*nr2
+    DO ip=1,dfftp%nr3p(me_bgrp+1)*nr1*nr2
       !
       UtsvdW(ip)=UtsvdWA(ip+rdispls(me_bgrp+1)) 
       !
