@@ -31,7 +31,6 @@ MODULE rVV10
   public :: xc_rVV10,  &
             interpolate_kernel, &
             initialize_spline_interpolation, &
-            numerical_gradient, &
             stress_rVV10, b_value
 
 CONTAINS
@@ -41,12 +40,12 @@ CONTAINS
 !!                                       |  xc_rVV10   |
 !!                                       |_____________|
 
-  SUBROUTINE xc_rVV10(rho_valence, rho_core, nspin, etxc, vtxc, v)
+  SUBROUTINE xc_rVV10(rho_valence, rho_core, nspin, etxc, vtxc, v, b_value_)
     
     !! Modules to include
     !! -------------------------------------------------------------------------
     
-    use gvect,           ONLY : ngm, nl, g, nlm
+    use gvect,           ONLY : ngm, g
     USE fft_base,        ONLY : dfftp
     USE cell_base,       ONLY : omega, tpiba
     !! -------------------------------------------------------------------------
@@ -58,6 +57,7 @@ CONTAINS
     real(dp), intent(IN) :: rho_core(:)            !  PWSCF input variables 
     INTEGER,  INTENT(IN) :: nspin                  !
     real(dp), intent(inout) :: etxc, vtxc, v(:,:)  !_  
+    real(DP),optional,intent(in) :: b_value_
    
     
     integer :: i_grid, theta_i, i_proc, I      
@@ -85,6 +85,8 @@ CONTAINS
     !call errore('xc_rVV10','rVV10 functional not implemented for spin polarized runs', size(rho_valence,2)-1)
     if (nspin>2) call errore('xc_vdW_DF','vdW functional not implemented for nspin > 2', nspin)
 
+    if(present(b_value_)) b_value = b_value_
+    
     !! --------------------------------------------------------------------------------------------------------
 
     call start_clock( 'rVV10' )
@@ -120,7 +122,7 @@ CONTAINS
     !! ---------------------------------------------------------------------------------------
 
     allocate( q0(dfftp%nnr) )
-    allocate( gradient_rho(dfftp%nnr, 3) )
+    allocate( gradient_rho(3,dfftp%nnr) )
     allocate( dq0_drho(dfftp%nnr), dq0_dgradrho(dfftp%nnr) )
     allocate( total_rho(dfftp%nnr) )
    
@@ -137,7 +139,7 @@ CONTAINS
     !! -------------------------------------------------------------------------
     !! Here we calculate the gradient in reciprocal space using FFT
     !! -------------------------------------------------------------------------
-    call numerical_gradient(total_rho,gradient_rho)
+    call fft_gradient_r2r( dfftp, total_rho, g, gradient_rho)
 
     !! -------------------------------------------------------------------------
     !! Get Q and all the derivatives
@@ -176,7 +178,7 @@ CONTAINS
     call start_clock( 'rVV10_ffts')
     
     do theta_i = 1, Nqs
-       CALL invfft('Dense', thetas(:,theta_i), dfftp) 
+       CALL invfft('Rho', thetas(:,theta_i), dfftp) 
     end do
 
     call stop_clock( 'rVV10_ffts')
@@ -233,7 +235,7 @@ CONTAINS
   SUBROUTINE stress_rVV10(rho_valence, rho_core, nspin, sigma)
 
       USE fft_base,        ONLY : dfftp
-      use gvect,           ONLY : ngm, nl, g, nlm
+      use gvect,           ONLY : ngm, g
       USE cell_base,       ONLY : tpiba
 
       implicit none
@@ -271,7 +273,7 @@ CONTAINS
       !! Allocations
       !! ---------------------------------------------------------------------------------------
 
-      allocate( gradient_rho(dfftp%nnr, 3) )
+      allocate( gradient_rho(3,dfftp%nnr) )
       allocate( total_rho(dfftp%nnr) )
       allocate( q0(dfftp%nnr) )
       allocate( dq0_drho(dfftp%nnr), dq0_dgradrho(dfftp%nnr) )
@@ -291,7 +293,7 @@ CONTAINS
       !! -------------------------------------------------------------------------
       !! Here we calculate the gradient in reciprocal space using FFT
       !! -------------------------------------------------------------------------
-      call numerical_gradient(total_rho,gradient_rho)
+      call fft_gradient_r2r( dfftp, total_rho, g, gradient_rho)
       
       !! -------------------------------------------------------------------------------------------------------------
       !! Get q0.
@@ -335,7 +337,7 @@ CONTAINS
       !!-----------------------------------------------------------------------------------
       !! Modules to include
       !! ----------------------------------------------------------------------------------
-      use gvect,                 ONLY : ngm, nl, g, nlm, nl, gg, igtongl, &
+      use gvect,                 ONLY : ngm, g, gg, igtongl, &
                                         gl, ngl, gstart
       USE fft_base,              ONLY : dfftp
       USE cell_base,             ONLY : omega, tpiba, alat, at, tpiba2
@@ -391,7 +393,7 @@ CONTAINS
       call start_clock( 'rVV10_ffts')
 
       do theta_i = 1, Nqs
-         CALL invfft('Dense', u_vdW(:,theta_i), dfftp) 
+         CALL invfft('Rho', u_vdW(:,theta_i), dfftp) 
       end do
 
       call stop_clock( 'rVV10_ffts')
@@ -460,7 +462,7 @@ CONTAINS
                           do m = 1, l
                                         
                               sigma (l, m) = sigma (l, m) -  prefactor * &
-                                             (gradient_rho(i_grid,l) * gradient_rho(i_grid,m))
+                                             (gradient_rho(l,i_grid) * gradient_rho(m,i_grid))
                            enddo
                         enddo
                      endif
@@ -490,7 +492,7 @@ CONTAINS
 
       !! Modules to include
       !! ----------------------------------------------------------------------------------
-      use gvect,                 ONLY : ngm, nl, g, nl, gg, igtongl, gl, ngl, gstart 
+      use gvect,                 ONLY : ngm, g, gg, igtongl, gl, ngl, gstart 
       USE fft_base,              ONLY : dfftp
       USE cell_base,             ONLY : omega, tpiba, tpiba2
       USE constants, ONLY: pi
@@ -539,7 +541,7 @@ CONTAINS
                      do m = 1, l
 
                      sigma (l, m) = sigma (l, m) - G_multiplier * 0.5 * &
-                                     thetas(nl(g_i),q1_i)*dkernel_of_dk(q1_i,q2_i)*conjg(thetas(nl(g_i),q2_i))* &
+                                     thetas(dfftp%nl(g_i),q1_i)*dkernel_of_dk(q1_i,q2_i)*conjg(thetas(dfftp%nl(g_i),q2_i))* &
                                      (g (l, g_i) * g (m, g_i) * tpiba2) / g_kernel 
                      end do
                  end do 
@@ -586,9 +588,9 @@ CONTAINS
           
      if (total_rho(i_grid) > epsr) then
 
-      gmod2 = gradient_rho(i_grid,1)**2 + &
-              gradient_rho(i_grid,2)**2 + &
-              gradient_rho(i_grid,3)**2
+      gmod2 = gradient_rho(1,i_grid)**2 + &
+              gradient_rho(2,i_grid)**2 + &
+              gradient_rho(3,i_grid)**2
  
        !! Calculate some intermediate values needed to find q
        !! ------------------------------------------------------------------------------------
@@ -682,7 +684,7 @@ CONTAINS
 
     do theta_i = 1, Nqs
 
-     CALL fwfft ('Dense', thetas(:,theta_i), dfftp)
+     CALL fwfft ('Rho', thetas(:,theta_i), dfftp)
     end do
 
     call stop_clock( 'rVV10_ffts')
@@ -1011,59 +1013,6 @@ subroutine interpolate_Dkernel_Dk(k, dkernel_of_dk)
   
 end subroutine interpolate_Dkernel_Dk 
 
-
-
-!! ###############################################################################################################
-!!                                       |                       |
-!!                                       |   NUMERICAL_GRADIENT  |
-!!                                       |_______________________|
-
-
-!! Calculates the gradient of the charge density numerically on the grid.  We use
-!! the PWSCF gradient style.
-
-subroutine numerical_gradient(total_rho, gradient_rho)
-
-   use gvect,             ONLY : ngm, nl, g, nlm
-   USE cell_base,         ONLY : tpiba
-   USE fft_base,          ONLY : dfftp
-   USE fft_interfaces,    ONLY : fwfft, invfft 
-   !
-   ! I/O variables
-   !
-   real(dp), intent(in) :: total_rho(:)        !! Input array holding total charge density.
- 
-   real(dp), intent(out) :: gradient_rho(:,:) !! Output array that will holds the gradient
-   !                                          !! of the charge density.
-   ! local variables
-   !
-   integer :: icar                            !! counter on cartesian components
-   complex(dp), allocatable :: c_rho(:)       !! auxiliary complex array for rho
-   complex(dp), allocatable :: c_grho(:)      !! auxiliary complex array for grad rho
- 
-   ! rho in G space
-   allocate ( c_rho(dfftp%nnr), c_grho(dfftp%nnr) )
-   c_rho(1:dfftp%nnr) = CMPLX(total_rho(1:dfftp%nnr),0.0_DP)
-   CALL fwfft ('Dense', c_rho, dfftp) 
- 
-   do icar=1,3
-      ! compute gradient in G space
-      c_grho(:) =CMPLX(0.0_DP,0.0_DP)
-      c_grho(nl(:)) = CMPLX (0.0_DP,1.0_DP) * tpiba * g(icar,:) * c_rho(nl(:))
-      if (gamma_only) c_grho( nlm(:) ) = CONJG( c_grho( nl(:) ) )
- 
-      ! back in real space
-      CALL invfft ('Dense', c_grho, dfftp) 
-      gradient_rho(:,icar) = REAL( c_grho(:) )
-   end do
-   deallocate ( c_rho, c_grho )
-
-   !gradient_rho = 0.0D0
-   return
-
-end subroutine numerical_gradient
-
-
 !! #################################################################################################
 !!                                          |              |
 !!                                          | thetas_to_uk |
@@ -1072,7 +1021,7 @@ end subroutine numerical_gradient
 
 subroutine thetas_to_uk(thetas, u_vdW)
   
-  USE gvect,           ONLY : nl, nlm, gg, ngm, igtongl, gl, ngl, gstart
+  USE gvect,           ONLY : gg, ngm, igtongl, gl, ngl, gstart
   USE fft_base,        ONLY : dfftp
   USE cell_base,       ONLY : tpiba, omega
 
@@ -1090,7 +1039,7 @@ subroutine thetas_to_uk(thetas, u_vdW)
   
   allocate( kernel_of_k(Nqs, Nqs) )
 
-  u_vdW(:,:) = CMPLX(0.0_DP,0.0_DP)
+  u_vdW(:,:) = CMPLX(0.0_DP,0.0_DP, kind=dp)
   
   last_g = -1 
 
@@ -1104,17 +1053,17 @@ subroutine thetas_to_uk(thetas, u_vdW)
         
      end if
      
-     theta = thetas(nl(g_i),:)
+     theta = thetas(dfftp%nl(g_i),:)
      
      do q2_i = 1, Nqs
         do q1_i = 1, Nqs
-           u_vdW(nl(g_i),q2_i) = u_vdW(nl(g_i),q2_i) + kernel_of_k(q2_i,q1_i)*theta(q1_i)
+           u_vdW(dfftp%nl(g_i),q2_i) = u_vdW(dfftp%nl(g_i),q2_i) + kernel_of_k(q2_i,q1_i)*theta(q1_i)
         end do
      end do
 
   end do
 
-  if (gamma_only) u_vdW(nlm(:),:) = CONJG(u_vdW(nl(:),:))
+  if (gamma_only) u_vdW(dfftp%nlm(:),:) = CONJG(u_vdW(dfftp%nl(:),:))
   
   deallocate( kernel_of_k )
      
@@ -1129,7 +1078,7 @@ end subroutine thetas_to_uk
 
 subroutine vdW_energy(thetas, vdW_xc_energy)
   
-  USE gvect,           ONLY : nl, nlm, gg, ngm, igtongl, gl, ngl, gstart
+  USE gvect,           ONLY : gg, ngm, igtongl, gl, ngl, gstart
   USE fft_base,        ONLY : dfftp
   USE cell_base,       ONLY : tpiba, omega
 
@@ -1151,7 +1100,7 @@ subroutine vdW_energy(thetas, vdW_xc_energy)
   vdW_xc_energy = 0.0D0
  
   allocate (u_vdW(dfftp%nnr,Nqs))
-  u_vdW(:,:) = CMPLX(0.0_DP,0.0_DP)
+  u_vdW(:,:) = CMPLX(0.0_DP,0.0_DP, kind=dp)
 
   allocate( kernel_of_k(Nqs, Nqs) )
   
@@ -1173,20 +1122,20 @@ subroutine vdW_energy(thetas, vdW_xc_energy)
         
      end if
      
-     theta = thetas(nl(g_i),:)
+     theta = thetas(dfftp%nl(g_i),:)
 
      do q2_i = 1, Nqs
         do q1_i = 1, Nqs
-           u_vdW(nl(g_i),q2_i)  = u_vdW(nl(g_i),q2_i) + kernel_of_k(q2_i,q1_i)*theta(q1_i)
+           u_vdW(dfftp%nl(g_i),q2_i)  = u_vdW(dfftp%nl(g_i),q2_i) + kernel_of_k(q2_i,q1_i)*theta(q1_i)
         end do
-        vdW_xc_energy = vdW_xc_energy + G_multiplier * (u_vdW(nl(g_i),q2_i)*conjg(theta(q2_i)))
+        vdW_xc_energy = vdW_xc_energy + G_multiplier * (u_vdW(dfftp%nl(g_i),q2_i)*conjg(theta(q2_i)))
      end do
      
      if (g_i < gstart ) vdW_xc_energy = vdW_xc_energy / G_multiplier
 
   end do
 
-  if (gamma_only) u_vdW(nlm(:),:) = CONJG(u_vdW(nl(:),:))
+  if (gamma_only) u_vdW(dfftp%nlm(:),:) = CONJG(u_vdW(dfftp%nl(:),:))
 
   !! Final value 
   vdW_xc_energy = 0.5D0 * omega * vdW_xc_energy   
@@ -1206,7 +1155,7 @@ end subroutine vdW_energy
 
   subroutine get_potential(q0, dq0_drho, dq0_dgradrho, total_rho, gradient_rho, u_vdW, potential)
 
-    use gvect,               ONLY : nl, g, nlm
+    use gvect,               ONLY : g
     USE fft_base,            ONLY : dfftp
     USE cell_base,           ONLY : alat, tpiba
 
@@ -1311,11 +1260,11 @@ end subroutine vdW_energy
     end do
 
     do icar = 1,3
-      h(:) = CMPLX(h_prefactor(:) * gradient_rho(:,icar),0.0_DP)
-      CALL fwfft ('Dense', h, dfftp) 
-      h(nl(:)) = CMPLX(0.0_DP,1.0_DP) * tpiba * g(icar,:) * h(nl(:))
-      if (gamma_only) h(nlm(:)) = CONJG(h(nl(:)))
-      CALL invfft ('Dense', h, dfftp) 
+      h(:) = CMPLX( h_prefactor(:)*gradient_rho(icar,:), 0.0_DP, kind=dp)
+      CALL fwfft ('Rho', h, dfftp) 
+      h(dfftp%nl(:)) = CMPLX(0.0_DP,1.0_DP,kind=dp)*tpiba*g(icar,:)*h(dfftp%nl(:))
+      if (gamma_only) h(dfftp%nlm(:)) = CONJG(h(dfftp%nl(:)))
+      CALL invfft ('Rho', h, dfftp) 
       potential(:) = potential(:) - REAL(h(:))
     end do
 
