@@ -18,6 +18,7 @@ SUBROUTINE elphon()
   USE ions_base, ONLY : nat, ntyp => nsp, ityp, tau, amass
   USE gvecs, ONLY: doublegrid
   USE fft_base, ONLY : dfftp, dffts
+  USE fft_interfaces, ONLY : fft_interpolate
   USE noncollin_module, ONLY : nspin_mag, noncolin, m_loc
   USE lsda_mod, ONLY : nspin
   USE uspp,  ONLY: okvan
@@ -57,9 +58,9 @@ SUBROUTINE elphon()
 
   if(dvscf_star%basis.eq.'cartesian') then
      write(stdout,*) 'Setting patterns to identity'
-     u=CMPLX(0.d0,0.d0)
+     u=(0.d0,0.d0)
      do irr=1,3*nat
-        u(irr,irr)=CMPLX(1.d0,0.d0)
+        u(irr,irr)=(1.d0,0.d0)
      enddo
   endif
   !
@@ -87,7 +88,7 @@ SUBROUTINE elphon()
         ALLOCATE (dvscfins (dffts%nnr, nspin_mag , npert(irr)) )
         DO is = 1, nspin_mag
            DO ipert = 1, npe
-              CALL cinterpolate (dvscfin(1,is,ipert),dvscfins(1,is,ipert),-1)
+              CALL fft_interpolate (dfftp, dvscfin(:,is,ipert), dffts, dvscfins(:,is,ipert))
            ENDDO
         ENDDO
      ELSE
@@ -203,6 +204,12 @@ SUBROUTINE readmat (iudyn, ibrav, celldm, nat, ntyp, ityp, omega, &
   IF ( ntyp.NE.ntyp_ .OR. nat.NE.nat_ .OR.ibrav_.NE.ibrav .OR. &
        ABS ( celldm_ (1) - celldm (1) ) > 1.0d-5) &
           CALL errore ('readmat', 'inconsistent data', 1)
+  IF ( ibrav_ == 0 ) THEN
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+  END IF
   DO nt = 1, ntyp
      READ (iudyn, * ) i, atm, amass_
      IF ( nt.NE.i .OR. ABS (amass_ - amu_ry*amass (nt) ) > 1.0d-5) &
@@ -271,8 +278,7 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   !      Original routine written by Francesco Mauri
   !
   USE kinds, ONLY : DP
-  USE fft_base, ONLY : dffts, dtgs
-  USE fft_parallel, ONLY : tg_cgather
+  USE fft_base, ONLY : dffts
   USE wavefunctions_module,  ONLY: evc
   USE buffers,  ONLY : get_buffer
   USE klist, ONLY: xk, ngk, igk_k
@@ -282,7 +288,7 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   USE buffers, ONLY : get_buffer
   USE uspp, ONLY : vkb
   USE el_phon, ONLY : el_ph_mat, el_ph_mat_rec, el_ph_mat_rec_col, &
-                      comp_elph, done_elph
+                      comp_elph, done_elph, elph_nbnd_min, elph_nbnd_max
   USE modes, ONLY : u
   USE units_ph, ONLY : iubar, lrbar, lrwfc, iuwfc
   USE control_ph, ONLY : trans, current_iq
@@ -291,10 +297,12 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   USE mp_bands,   ONLY: intra_bgrp_comm, ntask_groups
   USE mp_pools,   ONLY: npool
   USE mp,        ONLY: mp_sum
+  USE elph_tetra_mod, ONLY : elph_tetra
 
   USE eqv,        ONLY : dvpsi, evq
   USE qpoint,     ONLY : nksq, ikks, ikqs, nksqtot
   USE control_lr, ONLY : lgamma
+  USE fft_helper_subroutines
 
   IMPLICIT NONE
   !
@@ -308,6 +316,15 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
        tg_psic(:,:), aux2(:,:)
   INTEGER :: v_siz, incr
   COMPLEX(DP), EXTERNAL :: zdotc
+  integer :: ibnd_fst, ibnd_lst
+  !
+  if(elph_tetra == 0) then
+     ibnd_fst = 1
+     ibnd_lst = nbnd
+  else
+     ibnd_fst = elph_nbnd_min
+     ibnd_lst = elph_nbnd_max
+  end if
   !
   IF (.NOT. comp_elph(irr) .OR. done_elph(irr)) RETURN
 
@@ -317,12 +334,12 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   el_ph_mat_rec=(0.0_DP,0.0_DP)
   ALLOCATE (aux2(npwx*npol, nbnd))
   incr=1
-  IF ( dtgs%have_task_groups ) THEN
+  IF ( dffts%has_task_groups ) THEN
      !
-     v_siz =  dtgs%tg_nnr * dtgs%nogrp
+     v_siz =  dffts%nnr_tg
      ALLOCATE( tg_dv   ( v_siz, nspin_mag ) )
      ALLOCATE( tg_psic( v_siz, npol ) )
-     incr = dtgs%nogrp
+     incr = fftx_ntgrp(dffts)
      !
   ENDIF
   !
@@ -369,23 +386,21 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
         !
         ! calculate dvscf_q*psi_k
         !
-        IF ( dtgs%have_task_groups ) THEN
+        IF ( dffts%has_task_groups ) THEN
            IF (noncolin) THEN
-              CALL tg_cgather( dffts, dtgs, dvscfins(:,1,ipert), tg_dv(:,1))
+              CALL tg_cgather( dffts, dvscfins(:,1,ipert), tg_dv(:,1))
               IF (domag) THEN
                  DO ipol=2,4
-                    CALL tg_cgather( dffts, dtgs, dvscfins(:,ipol,ipert), &
-                                                          tg_dv(:,ipol))
+                    CALL tg_cgather( dffts,  dvscfins(:,ipol,ipert), tg_dv(:,ipol))
                  ENDDO
               ENDIF
            ELSE
-              CALL tg_cgather( dffts, dtgs, dvscfins(:,current_spin,ipert), &
-                                                            tg_dv(:,1))
+              CALL tg_cgather( dffts, dvscfins(:,current_spin,ipert), tg_dv(:,1))
            ENDIF
         ENDIF
         aux2=(0.0_DP,0.0_DP)
-        DO ibnd = 1, nbnd, incr
-           IF ( dtgs%have_task_groups ) THEN
+        DO ibnd = ibnd_fst, ibnd_lst, incr
+           IF ( dffts%has_task_groups ) THEN
               CALL cft_wave_tg (ik, evc, tg_psic, 1, v_siz, ibnd, nbnd )
               CALL apply_dpot(v_siz, tg_psic, tg_dv, 1)
               CALL cft_wave_tg (ik, aux2, tg_psic, -1, v_siz, ibnd, nbnd)
@@ -401,8 +416,8 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
         !
         ! calculate elphmat(j,i)=<psi_{k+q,j}|dvscf_q*psi_{k,i}> for this pertur
         !
-        DO ibnd =1, nbnd
-           DO jbnd = 1, nbnd
+        DO ibnd = ibnd_fst, ibnd_lst
+           DO jbnd = ibnd_fst, ibnd_lst
               elphmat (jbnd, ibnd, ipert) = zdotc (npwq, evq (1, jbnd), 1, &
                    dvpsi (1, ibnd), 1)
               IF (noncolin) &
@@ -417,8 +432,8 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
      !  save all e-ph matrix elements into el_ph_mat
      !
      DO ipert = 1, npe
-        DO jbnd = 1, nbnd
-           DO ibnd = 1, nbnd
+        DO jbnd = ibnd_fst, ibnd_lst
+           DO ibnd = ibnd_fst, ibnd_lst
               el_ph_mat (ibnd, jbnd, ik, ipert + imode0) = elphmat (ibnd, jbnd, ipert)
               el_ph_mat_rec (ibnd, jbnd, ik, ipert ) = elphmat (ibnd, jbnd, ipert)
            ENDDO
@@ -427,20 +442,22 @@ SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   ENDDO
   !
   done_elph(irr)=.TRUE.
-  IF (npool>1) THEN
-     ALLOCATE(el_ph_mat_rec_col(nbnd,nbnd,nksqtot,npe))
-     CALL el_ph_collect(npe,el_ph_mat_rec,el_ph_mat_rec_col,nksqtot,nksq)
-   ELSE
-     el_ph_mat_rec_col => el_ph_mat_rec
-  ENDIF
-  CALL ph_writefile('el_phon',current_iq,irr,ierr)
-  IF (npool > 1) DEALLOCATE(el_ph_mat_rec_col)
+  if(elph_tetra == 0) then
+     IF (npool>1) THEN
+        ALLOCATE(el_ph_mat_rec_col(nbnd,nbnd,nksqtot,npe))
+        CALL el_ph_collect(npe,el_ph_mat_rec,el_ph_mat_rec_col,nksqtot,nksq)
+     ELSE
+        el_ph_mat_rec_col => el_ph_mat_rec
+     ENDIF
+     CALL ph_writefile('el_phon',current_iq,irr,ierr)
+     IF (npool > 1) DEALLOCATE(el_ph_mat_rec_col)
+  end if
   DEALLOCATE(el_ph_mat_rec)
   !
   DEALLOCATE (elphmat)
   DEALLOCATE (aux1)
   DEALLOCATE (aux2)
-  IF ( dtgs%have_task_groups ) THEN
+  IF ( dffts%has_task_groups ) THEN
      DEALLOCATE( tg_dv )
      DEALLOCATE( tg_psic )
   ENDIF
@@ -472,14 +489,13 @@ SUBROUTINE elphsum ( )
   USE modes,       ONLY : u, nirr
   USE dynmat,      ONLY : dyn, w2
   USE io_global,   ONLY : stdout, ionode, ionode_id
-  USE xml_io_base, ONLY : create_directory
   USE mp_pools,    ONLY : my_pool_id, npool, kunit
   USE mp_images,   ONLY : intra_image_comm
   USE mp,          ONLY : mp_bcast
   USE control_ph,  ONLY : tmp_dir_phq, xmldyn, current_iq
   USE save_ph,     ONLY : tmp_dir_save
-  USE io_files,    ONLY : prefix, tmp_dir, seqopn
-  
+  USE io_files,    ONLY : prefix, tmp_dir, seqopn, create_directory
+  !
   USE lr_symm_base, ONLY : minus_q, nsymq, rtau
   USE qpoint,       ONLY : xq, nksq
   USE control_lr,   ONLY : lgamma
@@ -1123,6 +1139,300 @@ SUBROUTINE elphsum_simple
 
 END SUBROUTINE elphsum_simple
    
+!-----------------------------------------------------------------------
+SUBROUTINE elphfil_epa(iq)
+  !-----------------------------------------------------------------------
+  !
+  !      EPA (electron-phonon-averaged) approximation
+  !      Writes electron-phonon matrix elements to a file
+  !      Written by Georgy Samsonidze on 2015-01-28
+  !
+  !      Adv. Energy Mater. 2018, 1800246
+  !      doi:10.1002/aenm.201800246
+  !      https://doi.org/10.1002/aenm.201800246
+  !
+  !-----------------------------------------------------------------------
+  USE cell_base, ONLY : ibrav, alat, omega, tpiba, at, bg
+  USE disp, ONLY : nq1, nq2, nq3, nqs, x_q, wq, lgamma_iq
+  USE dynmat, ONLY : dyn, w2
+  USE el_phon, ONLY : el_ph_mat, done_elph
+  USE fft_base, ONLY : dfftp, dffts, dfftb
+  USE gvect, ONLY : ngm_g, ecutrho
+  USE io_global, ONLY : ionode, ionode_id
+  USE ions_base, ONLY : nat, nsp, atm, ityp, tau
+  USE kinds, ONLY : DP
+  USE klist, ONLY : xk, wk, nelec, nks, nkstot, ngk
+  USE lsda_mod, ONLY : nspin, isk
+  USE modes, ONLY : nirr, nmodes, npert, npertx, u, t, tmq, &
+       name_rap_mode, num_rap_mode
+  USE lr_symm_base, ONLY : irgq, nsymq, irotmq, rtau, gi, gimq, &
+       minus_q, invsymq
+  USE mp, ONLY : mp_bcast, mp_sum
+  USE mp_images, ONLY : intra_image_comm
+  USE mp_pools, ONLY : npool, intra_pool_comm
+  USE qpoint, ONLY : nksq, nksqtot, ikks, ikqs, eigqts
+  USE start_k, ONLY : nk1, nk2, nk3, k1, k2, k3
+  USE symm_base, ONLY : s, invs, ftau, nrot, nsym, nsym_ns, &
+       nsym_na, ft, sr, sname, t_rev, irt, time_reversal, &
+       invsym, nofrac, allfrac, nosym, nosym_evc, no_t_rev
+  USE wvfct, ONLY : nbnd, et, wg
+  USE gvecw, ONLY : ecutwfc
+  USE io_files, ONLY : prefix
+
+  IMPLICIT NONE
+
+  INTEGER, INTENT(IN) :: iq
+
+  INTEGER :: iuelph, ios, irr, ii, jj, kk, ll
+  character :: cdate*9, ctime*9, sdate*32, stime*32, &
+       stitle*32, myaccess*10, mystatus*7
+  CHARACTER(LEN=80) :: filelph
+
+  REAL(DP), ALLOCATABLE :: xk_collect(:,:), wk_collect(:)
+  REAL(DP), ALLOCATABLE :: et_collect(:,:), wg_collect(:,:)
+  INTEGER, ALLOCATABLE :: ngk_collect(:)
+  INTEGER, ALLOCATABLE :: ikks_collect(:), ikqs_collect(:)
+  COMPLEX(DP), ALLOCATABLE :: el_ph_mat_collect(:,:,:,:)
+
+  INTEGER, EXTERNAL :: find_free_unit, atomic_number
+
+  filelph = TRIM(prefix) // '.epa.k'
+
+  DO irr = 1, nirr
+     IF (.NOT. done_elph(irr)) RETURN
+  ENDDO
+
+  IF (iq .EQ. 1) THEN
+     myaccess = 'sequential'
+     mystatus = 'replace'
+  ELSE
+     myaccess = 'append'
+     mystatus = 'old'
+  ENDIF
+  IF (ionode) THEN
+     iuelph = find_free_unit()
+     OPEN(unit = iuelph, file = TRIM(filelph), form = 'unformatted', &
+          access = myaccess, status = mystatus, iostat = ios)
+  ELSE
+     iuelph = 0
+  ENDIF
+  CALL mp_bcast(ios, ionode_id, intra_image_comm)
+  CALL errore('elphfil_epa', 'opening file ' // filelph, ABS(ios))
+
+  IF (iq .EQ. 1) THEN
+     CALL date_and_tim(cdate, ctime)
+     WRITE(sdate, '(A2,"-",A3,"-",A4,21X)') cdate(1:2), cdate(3:5), cdate(6:9)
+     WRITE(stime, '(A8,24X)') ctime(1:8)
+     WRITE(stitle, '("EPA-Complex",21X)')
+     CALL cryst_to_cart(nqs, x_q, at, -1)
+     ! write header
+     IF (ionode) THEN
+        WRITE(iuelph) stitle, sdate, stime
+        WRITE(iuelph) ibrav, nat, nsp, nrot, nsym, nsym_ns, nsym_na, &
+             ngm_g, nspin, nbnd, nmodes, nqs
+        WRITE(iuelph) nq1, nq2, nq3, nk1, nk2, nk3, k1, k2, k3
+        WRITE(iuelph) time_reversal, invsym, nofrac, allfrac, nosym, &
+             nosym_evc, no_t_rev
+        WRITE(iuelph) alat, omega, tpiba, nelec, ecutrho, ecutwfc
+        WRITE(iuelph) dfftp%nr1, dfftp%nr2, dfftp%nr3
+        WRITE(iuelph) dffts%nr1, dffts%nr2, dffts%nr3
+        WRITE(iuelph) dfftb%nr1, dfftb%nr2, dfftb%nr3
+        WRITE(iuelph) ((at(ii, jj), ii = 1, 3), jj = 1, 3)
+        WRITE(iuelph) ((bg(ii, jj), ii = 1, 3), jj = 1, 3)
+        WRITE(iuelph) (atomic_number(atm(ii)), ii = 1, nsp)
+        WRITE(iuelph) (ityp(ii), ii = 1, nat)
+        WRITE(iuelph) ((tau(ii, jj), ii = 1, 3), jj = 1, nat)
+        WRITE(iuelph) ((x_q(ii, jj), ii = 1, 3), jj = 1, nqs)
+        WRITE(iuelph) (wq(ii), ii = 1, nqs)
+        WRITE(iuelph) (lgamma_iq(ii), ii = 1, nqs)
+     ENDIF
+     CALL cryst_to_cart(nqs, x_q, bg, 1)
+  ENDIF
+
+  ! collect data for current q-point
+  ALLOCATE(xk_collect(3, nkstot))
+  ALLOCATE(wk_collect(nkstot))
+  ALLOCATE(et_collect(nbnd, nkstot))
+  ALLOCATE(wg_collect(nbnd, nkstot))
+  ALLOCATE(ngk_collect(nkstot))
+  ALLOCATE(ikks_collect(nksqtot))
+  ALLOCATE(ikqs_collect(nksqtot))
+  ALLOCATE(el_ph_mat_collect(nbnd, nbnd, nksqtot, nmodes))
+  IF (npool > 1) THEN
+     CALL poolcollect(3, nks, xk, nkstot, xk_collect)
+     CALL poolcollect(1, nks, wk, nkstot, wk_collect)
+     CALL poolcollect(nbnd, nks, et, nkstot, et_collect)
+     CALL poolcollect(nbnd, nks, wg, nkstot, wg_collect)
+     CALL ipoolcollect(1, nks, ngk, nkstot, ngk_collect)
+     CALL jpoolcollect(1, nksq, ikks, nksqtot, ikks_collect)
+     CALL jpoolcollect(1, nksq, ikqs, nksqtot, ikqs_collect)
+     CALL el_ph_collect(nmodes, el_ph_mat, el_ph_mat_collect, nksqtot, nksq)
+  ELSE
+     xk_collect(1:3, 1:nks) = xk(1:3, 1:nks)
+     wk_collect(1:nks) = wk(1:nks)
+     et_collect(1:nbnd, 1:nks) = et(1:nbnd, 1:nks)
+     wg_collect(1:nbnd, 1:nks) = wg(1:nbnd, 1:nks)
+     ngk_collect(1:nks) = ngk(1:nks)
+     ikks_collect(1:nksq) = ikks(1:nksq)
+     ikqs_collect(1:nksq) = ikqs(1:nksq)
+     el_ph_mat_collect(1:nbnd, 1:nbnd, 1:nksq, 1:nmodes) = &
+          el_ph_mat(1:nbnd, 1:nbnd, 1:nksq, 1:nmodes)
+  ENDIF
+  CALL cryst_to_cart(nkstot, xk_collect, at, -1)
+  ! write data for current q-point
+  IF (ionode) THEN
+     WRITE(iuelph) nsymq, irotmq, nirr, npertx, nkstot, nksqtot
+     WRITE(iuelph) minus_q, invsymq
+     WRITE(iuelph) (irgq(ii), ii = 1, 48)
+     WRITE(iuelph) (npert(ii), ii = 1, nmodes)
+     WRITE(iuelph) (((rtau(ii, jj, kk), ii = 1, 3), jj = 1, 48), &
+          kk = 1, nat)
+     WRITE(iuelph) ((gi(ii, jj), ii = 1, 3), jj = 1, 48)
+     WRITE(iuelph) (gimq(ii), ii = 1, 3)
+     WRITE(iuelph) ((u(ii, jj), ii = 1, nmodes), jj = 1, nmodes)
+     WRITE(iuelph) ((((t(ii, jj, kk, ll), ii = 1, npertx), &
+          jj = 1, npertx), kk = 1, 48), ll = 1, nmodes)
+     WRITE(iuelph) (((tmq(ii, jj, kk), ii = 1, npertx), &
+          jj = 1, npertx), kk = 1, nmodes)
+     WRITE(iuelph) (name_rap_mode(ii), ii = 1, nmodes)
+     WRITE(iuelph) (num_rap_mode(ii), ii = 1, nmodes)
+     WRITE(iuelph) (((s(ii, jj, kk), ii = 1, 3), jj = 1, 3), kk = 1, 48)
+     WRITE(iuelph) (invs(ii), ii = 1, 48)
+     WRITE(iuelph) ((ftau(ii, jj), ii = 1, 3), jj = 1, 48)
+     WRITE(iuelph) ((ft(ii, jj), ii = 1, 3), jj = 1, 48)
+     WRITE(iuelph) (((sr(ii, jj, kk), ii = 1, 3), jj = 1, 3), kk = 1, 48)
+     WRITE(iuelph) (sname(ii), ii = 1, 48)
+     WRITE(iuelph) (t_rev(ii), ii = 1, 48)
+     WRITE(iuelph) ((irt(ii, jj), ii = 1, 48), jj = 1, nat)
+     WRITE(iuelph) ((xk_collect(ii, jj), ii = 1, 3), jj = 1, nkstot)
+     WRITE(iuelph) (wk_collect(ii), ii = 1, nkstot)
+     WRITE(iuelph) ((et_collect(ii, jj), ii = 1, nbnd), jj = 1, nkstot)
+     WRITE(iuelph) ((wg_collect(ii, jj), ii = 1, nbnd), jj = 1, nkstot)
+     WRITE(iuelph) (isk(ii), ii = 1, nkstot)
+     WRITE(iuelph) (ngk_collect(ii), ii = 1, nkstot)
+     WRITE(iuelph) (ikks_collect(ii), ii = 1, nksqtot)
+     WRITE(iuelph) (ikqs_collect(ii), ii = 1, nksqtot)
+     WRITE(iuelph) (eigqts(ii), ii = 1, nat)
+     WRITE(iuelph) (w2(ii), ii = 1, nmodes)
+     WRITE(iuelph) ((dyn(ii, jj), ii = 1, nmodes), jj = 1, nmodes)
+     WRITE(iuelph) ((((el_ph_mat_collect(ii, jj, kk, ll), ii = 1, nbnd), &
+          jj = 1, nbnd), kk = 1, nksqtot), ll = 1, nmodes)
+     CLOSE (unit = iuelph, status = 'keep')
+  ENDIF
+  CALL cryst_to_cart(nkstot, xk_collect, bg, 1)
+  DEALLOCATE(xk_collect)
+  DEALLOCATE(wk_collect)
+  DEALLOCATE(et_collect)
+  DEALLOCATE(wg_collect)
+  DEALLOCATE(ngk_collect)
+  DEALLOCATE(ikks_collect)
+  DEALLOCATE(ikqs_collect)
+  DEALLOCATE(el_ph_mat_collect)
+
+  RETURN
+
+END SUBROUTINE elphfil_epa
+   
+!----------------------------------------------------------------------------
+SUBROUTINE ipoolcollect( length, nks, f_in, nkstot, f_out )
+  !----------------------------------------------------------------------------
+  !
+  ! ... as poolcollect, for an integer vector
+  !
+  USE mp_pools,  ONLY : my_pool_id, npool, kunit, &
+                        inter_pool_comm, intra_pool_comm
+  USE mp,        ONLY : mp_sum
+  !
+  IMPLICIT NONE
+  !
+  INTEGER, INTENT(IN) :: length, nks, nkstot
+  ! first dimension of arrays
+  ! number of k-points per pool
+  ! total number of k-points
+  INTEGER, INTENT(IN)  :: f_in (length,nks)
+  ! pool-distributed function
+  INTEGER, INTENT(OUT) :: f_out(length,nkstot)
+  ! pool-collected function
+  !
+  INTEGER :: nbase, rest, nks1
+  !
+  nks1    = kunit * ( nkstot / kunit / npool )
+  !
+  rest = ( nkstot - nks1 * npool ) / kunit
+  !
+  IF ( ( my_pool_id + 1 ) <= rest ) nks1 = nks1 + kunit
+  !
+  IF (nks1.ne.nks) &
+     call errore('ipoolcollect','inconsistent number of k-points',1)
+  !
+  ! ... calculates nbase = the position in the list of the first point that
+  ! ...                    belong to this npool - 1
+  !
+  nbase = nks * my_pool_id
+  !
+  IF ( ( my_pool_id + 1 ) > rest ) nbase = nbase + rest * kunit
+  !
+  ! copy the original points in the correct position of the list
+  !
+  f_out=0
+  f_out(:,nbase+1:nbase+nks) = f_in(:,1:nks)
+  !
+  CALL mp_sum( f_out, inter_pool_comm )
+  !
+  RETURN
+  !
+END SUBROUTINE ipoolcollect
+
+!----------------------------------------------------------------------------
+SUBROUTINE jpoolcollect( length, nks, f_in, nkstot, f_out )
+  !----------------------------------------------------------------------------
+  !
+  ! ... as ipoolcollect, without kunit and with an index shift
+  !
+  USE mp_pools,  ONLY : my_pool_id, npool, kunit, &
+                        inter_pool_comm, intra_pool_comm
+  USE mp,        ONLY : mp_sum
+  !
+  IMPLICIT NONE
+  !
+  INTEGER, INTENT(IN) :: length, nks, nkstot
+  ! first dimension of arrays
+  ! number of k-points per pool
+  ! total number of k-points
+  INTEGER, INTENT(IN)  :: f_in (length,nks)
+  ! pool-distributed function
+  INTEGER, INTENT(OUT) :: f_out(length,nkstot)
+  ! pool-collected function
+  !
+  INTEGER :: nbase, rest, nks1
+  !
+  nks1    = ( nkstot / npool )
+  !
+  rest = ( nkstot - nks1 * npool )
+  !
+  IF ( ( my_pool_id + 1 ) <= rest ) nks1 = nks1 + 1
+  !
+  IF (nks1.ne.nks) &
+     call errore('jpoolcollect','inconsistent number of k-points',1)
+  !
+  ! ... calculates nbase = the position in the list of the first point that
+  ! ...                    belong to this npool - 1
+  !
+  nbase = nks * my_pool_id
+  !
+  IF ( ( my_pool_id + 1 ) > rest ) nbase = nbase + rest
+  !
+  ! copy the original points in the correct position of the list
+  !
+  f_out=0
+  f_out(:,nbase+1:nbase+nks) = f_in(:,1:nks) + nbase * kunit
+  !
+  CALL mp_sum( f_out, inter_pool_comm )
+  !
+  RETURN
+  !
+END SUBROUTINE jpoolcollect
+   
 !-----------------------------------------------------------------------
 FUNCTION dos_ef (ngauss, degauss, ef, et, wk, nks, nbnd)
   !-----------------------------------------------------------------------
