@@ -17,6 +17,7 @@ SUBROUTINE ep_matrix_element_wannier()
   USE ions_base, ONLY : nat, ntyp => nsp, ityp, tau, amass
   USE gvecs, ONLY: doublegrid
   USE fft_base, ONLY : dfftp, dffts
+  USE fft_interfaces, ONLY : fft_interpolate
   USE noncollin_module, ONLY : nspin_mag, noncolin
   USE dynmat, ONLY : dyn, w2
   USE modes,  ONLY : npert, nirr, u
@@ -54,9 +55,9 @@ SUBROUTINE ep_matrix_element_wannier()
      write(stdout,*) 'Reading dvscf in cartesian coordinates !'
      write(stdout,*)
      
-     u=CMPLX(0.d0,0.d0)
+     u=(0.d0,0.d0)
      do irr=1,3*nat
-        u(irr,irr)=CMPLX(1.d0,0.d0)
+        u(irr,irr)=(1.d0,0.d0)
      enddo
      
 
@@ -64,7 +65,7 @@ SUBROUTINE ep_matrix_element_wannier()
 !        ALLOCATE (dvrot ( nrxx , nspin , 3*nat) )
 !        fildvscf_asc=trim(tmp_dir)//trim(prefix)//"."//trim(fildvscf)//'1'
 !        open(unit=7899,file=fildvscf_asc,status='unknown')
-!        dvrot=CMPLX(0.0,0.0)
+!        dvrot=(0.0,0.0)
 !        do na=1,nat
 !           do ipol=1,3
 !              irr=(na-1)*3+ipol
@@ -72,7 +73,7 @@ SUBROUTINE ep_matrix_element_wannier()
 !                 do j = 1, dfftp%nr2
 !                    do i = 1, dfftp%nr1
 !                       read(7899,*)   n, rep,imp
-!                       dvrot(n,1,irr)=CMPLX(rep,imp)
+!                       dvrot(n,1,irr)=CMPLX(rep,imp,kind=dp)
 !                    enddo
 !                 enddo
 !              enddo
@@ -111,7 +112,7 @@ SUBROUTINE ep_matrix_element_wannier()
         ALLOCATE (dvscfins (dffts%nnr, nspin_mag , npert(irr)) )
         DO is = 1, nspin_mag
            DO ipert = 1, npert(irr)
-              CALL cinterpolate (dvscfin(1,is,ipert),dvscfins(1,is,ipert),-1)
+              CALL fft_interpolate (dfftp, dvscfin(:,is,ipert), dffts, dvscfins(:,is,ipert))
            ENDDO
         ENDDO
      ELSE
@@ -346,7 +347,6 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
   USE mp,        ONLY: mp_sum
   USE ions_base, ONLY : nat
   USE io_global, ONLY : stdout
-  USE gvecs, ONLY : nls
 
   USE eqv,        ONLY : dvpsi!, evq
   USE qpoint,     ONLY : nksq, ikks, ikqs
@@ -401,7 +401,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
      !
      ! read unperturbed wavefuctions psi(k) and psi(k+q)
      !
-     evc=cmplx(0.d0,0.d0)
+     evc=(0.d0,0.d0)
 
 !    Warning error in reading wfc, this could explain.
 !    We read here the wfc at the Gamma point, that is
@@ -620,7 +620,6 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
   USE fft_interfaces,  ONLY : fwfft, invfft
   USE wvfct, ONLY: nbnd, npwx
   USE gvect, ONLY : ngm, g
-  USE gvecs, ONLY : nls
   USE gvecw, ONLY : gcutw
   USE cell_base, ONLY : bg
   USE qpoint, ONLY : nksq
@@ -669,10 +668,10 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
 
   call gk_sort (xkqg, ngm, g_scra, gcutw, npwq_refolded, igkq_, gk)
 
-  phase(:) = CMPLX(0.d0,0.d0)
+  phase(:) = (0.d0,0.d0)
 
   if ( igqg(ik)>0) then
-     phase( nls(igqg(ik)) ) = (1.d0,0.d0)
+     phase( dffts%nl(igqg(ik)) ) = (1.d0,0.d0)
   endif
 
 
@@ -685,28 +684,28 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
   
   do m=1,nbnd
      psi_scratch = (0.d0, 0.d0)
-     psi_scratch(nls (igk_ (1:npw_) ) ) = evq (1:npw_, m)
-!     psi_scratch(nls (igk_ (1:npw) ) ) = evq (1:npw, m)
+     psi_scratch(dffts%nl (igk_ (1:npw_) ) ) = evq (1:npw_, m)
+!     psi_scratch(dffts%nl (igk_ (1:npw) ) ) = evq (1:npw, m)
      CALL invfft ('Wave', psi_scratch, dffts)
      !     call cft3s (psic, nr1s, nr2s, nr3s, nrx1s, nrx2s, nrx3s, +2)
      psi_scratch(1:dffts%nnr) = psi_scratch(1:dffts%nnr) * phase(1:dffts%nnr)
      !     call cft3s (psic, nr1s, nr2s, nr3s, nrx1s, nrx2s, nrx3s, -2)
      CALL fwfft ('Wave', psi_scratch, dffts)
-     evq(1:npwq_refolded,m) = psi_scratch(nls (igkq_(1:npwq_refolded) ) )
+     evq(1:npwq_refolded,m) = psi_scratch(dffts%nl (igkq_(1:npwq_refolded) ) )
   enddo
 
   if(noncolin) then
      do m=1,nbnd
         psi_scratch = (0.d0, 0.d0)
-        psi_scratch(nls (igk_ (1:npw_) ) ) = evq (npwx+1:npwx+npw_, m)
-        !       psi_scratch(nls (igk_ (1:npw) ) ) = evq (1:npw, m)
+        psi_scratch(dffts%nl (igk_ (1:npw_) ) ) = evq (npwx+1:npwx+npw_, m)
+        !       psi_scratch(dffts%nl (igk_ (1:npw) ) ) = evq (1:npw, m)
         CALL invfft ('Wave', psi_scratch, dffts)
         !     call cft3s (psic, nr1s, nr2s, nr3s, nrx1s, nrx2s, nrx3s, +2)
         psi_scratch(1:dffts%nnr) = psi_scratch(1:dffts%nnr) * phase(1:dffts%nnr)
         !     call cft3s (psic, nr1s, nr2s, nr3s, nrx1s, nrx2s, nrx3s, -2)
         CALL fwfft ('Wave', psi_scratch, dffts)
-        !       evq(npwx+1:npwx+npwq_refolded,m) = psi_scratch(nls (igkq_(1:npwq_refolded) ) )
-        evq((npwx+1):(npwx+npwq_refolded),m) = psi_scratch(nls (igkq_(1:npwq_refolded) ) )
+        !       evq(npwx+1:npwx+npwq_refolded,m) = psi_scratch(dffts%nl (igkq_(1:npwq_refolded) ) )
+        evq((npwx+1):(npwx+npwq_refolded),m) = psi_scratch(dffts%nl (igkq_(1:npwq_refolded) ) )
      enddo
   endif
  
