@@ -68,7 +68,7 @@
 PROGRAM bgw2pw
 
   USE environment, ONLY : environment_start, environment_end
-  USE io_files, ONLY : prefix, tmp_dir
+  USE io_files, ONLY : prefix, tmp_dir, create_directory
   USE io_global, ONLY : ionode, ionode_id
   USE kinds, ONLY : DP
   USE mp, ONLY : mp_bcast
@@ -137,8 +137,8 @@ PROGRAM bgw2pw
   CALL openfil_pp ( )
 
   IF ( wfng_flag ) THEN
-    input_file_name = TRIM ( tmp_dir ) // '/' // TRIM ( wfng_file )
-    output_dir_name = TRIM ( tmp_dir ) // '/' // TRIM ( prefix ) // '.save'
+    input_file_name = TRIM ( tmp_dir ) // TRIM ( wfng_file )
+    output_dir_name = TRIM ( tmp_dir ) // TRIM ( prefix ) // '.save'
     IF ( ionode ) WRITE ( 6, '(5x,"call write_evc")' )
     CALL start_clock ( 'write_evc' )
     CALL write_evc ( input_file_name, real_or_complex, wfng_nband, &
@@ -148,8 +148,8 @@ PROGRAM bgw2pw
   ENDIF
 
   IF ( rhog_flag ) THEN
-    input_file_name = TRIM ( tmp_dir ) // '/' // TRIM ( rhog_file )
-    output_dir_name = TRIM ( tmp_dir ) // '/' // TRIM ( prefix ) // '.save'
+    input_file_name = TRIM ( tmp_dir ) // TRIM ( rhog_file )
+    output_dir_name = TRIM ( tmp_dir ) // TRIM ( prefix ) // '.save'
     IF ( ionode ) WRITE ( 6, '(5x,"call write_cd")' )
     CALL start_clock ( 'write_cd' )
     CALL write_cd ( input_file_name, real_or_complex, output_dir_name )
@@ -193,8 +193,9 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
   USE mp_world, ONLY : world_comm, nproc
   USE mp_pools, ONLY : kunit, npool, my_pool_id, intra_pool_comm
   USE symm_base, ONLY : s, nsym
-  USE xml_io_base, ONLY : create_directory
+#if defined (__OLDXML)
   USE qexml_module, ONLY : qexml_kpoint_dirname, qexml_wfc_filename
+#endif
 #if defined(__MPI)
   USE parallel_include, ONLY : MPI_INTEGER, MPI_DOUBLE_COMPLEX
 #endif
@@ -442,7 +443,7 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
         DO is = 1, ns
           IF ( real_or_complex .EQ. 1 ) THEN
             DO ig = 1, ngk_g ( ik )
-              wfng_buf ( ig, is ) = CMPLX ( wfngr ( ig, is ), 0.0D0 )
+              wfng_buf ( ig, is ) = CMPLX ( wfngr ( ig, is ), 0.0D0, KIND=dp )
             ENDDO
           ELSE
             DO ig = 1, ngk_g ( ik )
@@ -465,7 +466,7 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
 #else
       DO is = 1, ns
         DO ig = 1, ngkdist_g
-          wfng_dist ( ig, ib, is, ik ) = wfng_buf ( ig, is )
+         wfng_dist ( ig, ib, is, ik ) = wfng_buf ( ig, is )
         ENDDO
       ENDDO
 #endif
@@ -513,9 +514,13 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
   CALL mp_max ( npw_g, world_comm )
 
   CALL create_directory ( output_dir_name )
+#if defined (__OLDXML)
   DO ik = 1, nk
     CALL create_directory (qexml_kpoint_dirname( output_dir_name, ik ) )
   ENDDO
+#else
+  CALL errore('bgw2pw','XSD implementation pending',1)
+#endif
 
   filename = TRIM ( output_dir_name ) // '/gvectors.dat'
 
@@ -536,8 +541,9 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
 
   DO ik = 1, nk
 
+#if defined (__OLDXML)
     filename = TRIM ( qexml_wfc_filename ( output_dir_name, 'gkvectors', ik ) )
-
+#endif
     IF ( ionode ) THEN
       CALL iotk_open_write ( iu, FILE = TRIM ( filename ), ROOT="GK-VECTORS", BINARY = .TRUE. )
       CALL iotk_write_dat ( iu, "NUMBER_OF_GK-VECTORS", ngk_g ( ik ) )
@@ -564,13 +570,13 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
     ENDIF
 
     DO is = 1, ns
-
+#if defined (__OLDXML)
       IF ( ns .GT. 1 ) THEN
         filename = TRIM ( qexml_wfc_filename ( output_dir_name, 'eigenval', ik, is, EXTENSION = 'xml' ) )
       ELSE
         filename = TRIM ( qexml_wfc_filename ( output_dir_name, 'eigenval', ik, EXTENSION = 'xml' ) )
       ENDIF
-
+#endif
       IF ( ionode ) THEN
         CALL iotk_open_write ( iu, FILE = TRIM ( filename ), BINARY = .FALSE. )
         CALL iotk_write_attr ( attr, "nbnd", nb, FIRST = .TRUE. )
@@ -583,13 +589,13 @@ SUBROUTINE write_evc ( input_file_name, real_or_complex, &
         CALL iotk_write_dat ( iu, "OCCUPATIONS", oc ( :, ik, is ) )
         CALL iotk_close_write ( iu )
       ENDIF
-
+#if defined (__OLDXML)
       IF ( ns .GT. 1 ) THEN
         filename = TRIM ( qexml_wfc_filename ( output_dir_name, 'evc', ik, is ) )
       ELSE
         filename = TRIM ( qexml_wfc_filename ( output_dir_name, 'evc', ik ) )
       ENDIF
-
+#endif
       IF ( ionode ) THEN
         CALL iotk_open_write ( iu, FILE = TRIM ( filename ), ROOT = "WFC", BINARY = .TRUE. )
         CALL iotk_write_attr ( attr, "ngw", npw_g, FIRST = .TRUE. )
@@ -648,10 +654,10 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
   USE constants, ONLY : eps6
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : invfft
-  USE gvect, ONLY : ngm, ngm_g, ig_l2g, nl, mill
+  USE gvect, ONLY : ngm, ngm_g, ig_l2g, mill
   USE io_global, ONLY : ionode, ionode_id
   USE ions_base, ONLY : nat
-  USE io_rho_xml, ONLY : write_rho
+  USE xml_io_base, ONLY : write_rho
   USE iotk_module, ONLY : iotk_attlenx, iotk_free_unit, iotk_open_write, &
     iotk_write_begin, iotk_write_attr, iotk_write_empty, iotk_write_dat, &
     iotk_write_end, iotk_close_write
@@ -663,7 +669,6 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
   USE scf, ONLY : rho
   USE symm_base, ONLY : s, nsym
   USE wavefunctions_module, ONLY : psic
-  USE xml_io_base, ONLY : create_directory
 
   IMPLICIT NONE
 
@@ -784,7 +789,7 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
     DO is = 1, ns
       IF ( real_or_complex .EQ. 1 ) THEN
         DO ig = 1, ng
-          rhog ( ig, is ) = CMPLX ( rhogr ( ig, is ), 0.0D0 )
+          rhog ( ig, is ) = CMPLX ( rhogr ( ig, is ), 0.0D0, KIND=dp )
         ENDDO
       ELSE
         DO ig = 1, ng
@@ -804,7 +809,7 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
   IF ( ionode ) THEN
     DO is = 1, ns
       DO ig = 1, ng
-        rhog ( ig, is ) = rhog ( ig, is ) / CMPLX ( omega, 0.0D0 )
+        rhog ( ig, is ) = rhog ( ig, is ) / CMPLX ( omega, 0.0D0, KIND=dp )
       ENDDO
     ENDDO
   ENDIF
@@ -857,9 +862,9 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
 
   DO is = 1, ns
     DO ig = 1, ngm
-      psic ( nl ( ig ) ) = rho%of_g ( ig, is )
+      psic ( dfftp%nl ( ig ) ) = rho%of_g ( ig, is )
     ENDDO
-    CALL invfft ( 'Dense', psic, dfftp )
+    CALL invfft ( 'Rho', psic, dfftp )
     DO ir = 1, dfftp%nnr
       rho%of_r ( ir, is ) = psic ( ir )
     ENDDO
@@ -886,7 +891,7 @@ SUBROUTINE write_cd ( input_file_name, real_or_complex, output_dir_name )
 
   DEALLOCATE ( gvec )
 
-  CALL write_rho ( rho%of_r, nspin )
+  CALL write_rho ( output_dir_name, rho%of_r, nspin )
 
   RETURN
 
