@@ -148,13 +148,10 @@ PROGRAM epsilon
   USE io_global,   ONLY : stdout, ionode, ionode_id
   USE mp,          ONLY : mp_bcast
   USE mp_world,    ONLY : world_comm
-  USE iotk_module
-  USE xml_io_base
   USE io_files,    ONLY : tmp_dir, prefix
   USE constants,   ONLY : RYTOEV
   USE ener,        ONLY : ef
-  USE klist,       ONLY : lgauss
-  USE ktetra,      ONLY : ltetra
+  USE klist,       ONLY : lgauss, ltetra
   USE wvfct,       ONLY : nbnd
   USE lsda_mod,    ONLY : nspin
   USE mp_global,   ONLY : mp_startup
