@@ -16,10 +16,11 @@ PROGRAM do_projwfc
   ! See files INPUT_PROJWFC.* in Doc/ directory for usage
   ! IMPORTANT: since v.5 namelist name is &projwfc and no longer &inputpp
   !
+  USE parameters, ONLY : npk
   USE io_global,  ONLY : stdout, ionode, ionode_id
   USE constants,  ONLY : rytoev
   USE kinds,      ONLY : DP
-  USE klist,      ONLY : degauss, ngauss, lgauss
+  USE klist,      ONLY : nks, nkstot, xk, degauss, ngauss, lgauss, ltetra
   USE io_files,   ONLY : nd_nmbr, prefix, tmp_dir
   USE noncollin_module, ONLY : noncolin
   USE mp,         ONLY : mp_bcast
@@ -31,14 +32,21 @@ PROGRAM do_projwfc
   USE basis,      ONLY : natomwfc
   USE control_flags, ONLY: twfcollect
   USE paw_variables, ONLY : okpaw
+  ! following modules needed for generation of tetrahedra
+  USE ktetra,     ONLY : tetra, tetra_type, opt_tetra_init
+  USE symm_base,  ONLY : nsym, s, time_reversal, t_rev
+  USE cell_base,  ONLY : at, bg
+  USE start_k,    ONLY : k1, k2, k3, nk1, nk2, nk3
+  USE lsda_mod,   ONLY : lsda
   !
   IMPLICIT NONE
   !
   CHARACTER(LEN=256), EXTERNAL :: trimcheck
   !
   CHARACTER (len=256) :: filpdos, filproj, outdir
-  REAL (DP)      :: Emin, Emax, DeltaE, degauss1, ef_0
-  INTEGER :: ngauss1, ios
+  REAL (DP), allocatable :: xk_collect(:,:)
+  REAL (DP) :: Emin, Emax, DeltaE, degauss1, ef_0
+  INTEGER :: nks2, ngauss1, ios
   LOGICAL :: lwrite_overlaps, lbinary_data
   LOGICAL :: lsym, kresolveddos, tdosinboxes, plotboxes, pawproj
   INTEGER, PARAMETER :: N_MAX_BOXES = 999
@@ -145,15 +153,47 @@ PROGRAM do_projwfc
   !
   CALL openfil_pp ( )
   !
-  !   decide Gaussian broadening
+  !   Tetrahedron method
   !
-  IF (degauss1/=0.d0) THEN
+  IF ( ltetra ) THEN
+     !
+     ! info on tetrahedra is no longer saved to file and must be rebuilt
+     ! workaround for old xml file, to be removed when the old xml file is
+     IF(ALLOCATED(tetra)) DEALLOCATE(tetra)
+     !
+     ! in the lsda case, only the first half of the k points
+     ! are needed in the input of "tetrahedra"
+     !
+     IF ( lsda ) THEN
+        nks2 = nkstot / 2
+     ELSE
+        nks2 = nkstot
+     END IF
+     IF(tetra_type < 2) THEN
+        ! use linear tetrahedron for both tetra_type=0 and tetra_type=1
+        IF ( tetra_type == 0 ) tetra_type = 1
+        WRITE( stdout,'(/5x,"Linear tetrahedron method (read from file) ")')
+     ELSE
+        WRITE( stdout,'(/5x,"Optimized tetrahedron method (read from file) ")')
+     END IF
+     !
+     ! not sure this is needed
+     !
+     ALLOCATE(xk_collect(3,nkstot))
+     CALL poolcollect(3, nks, xk, nkstot, xk_collect)
+     !
+     CALL opt_tetra_init(nsym, s, time_reversal, t_rev, at, bg, npk, k1,k2,k3, &
+          &              nk1, nk2, nk3, nks2, xk_collect, 1)
+     !
+     DEALLOCATE(xk_collect)
+     !
+  ELSE IF (degauss1/=0.d0) THEN
      degauss=degauss1
      ngauss =ngauss1
      WRITE( stdout,'(/5x,"Gaussian broadening (read from input): ",&
           &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
      lgauss=.true.
-  ELSEIF (lgauss) THEN
+  ELSE IF (lgauss) THEN
      WRITE( stdout,'(/5x,"Gaussian broadening (read from file): ",&
           &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
   ELSE
@@ -192,7 +232,7 @@ ELSE
   IF ( ionode ) THEN
      IF ( tdosinboxes ) THEN
         CALL partialdos_boxes (Emin, Emax, DeltaE, kresolveddos, filpdos, n_proj_boxes)
-     ELSE IF ( lsym ) THEN
+     ELSE IF ( lsym .OR. kresolveddos ) THEN
         IF (noncolin) THEN
            CALL partialdos_nc (Emin, Emax, DeltaE, kresolveddos, filpdos)
         ELSE
@@ -247,6 +287,117 @@ SUBROUTINE get_et_from_gww ( nbnd, et )
   ENDIF
 END SUBROUTINE get_et_from_gww
 !
+SUBROUTINE write_lowdin ( filproj, nat, lmax_wfc, nspin, charges, charges_lm )
+  !
+  USE kinds, ONLY : dp
+  USE io_global, ONLY : stdout, ionode
+  USE klist, ONLY: nelec
+  !
+  IMPLICIT NONE
+  !
+  CHARACTER (len=*), INTENT(in) :: filproj
+  INTEGER, INTENT(IN) :: nat, lmax_wfc, nspin
+  REAL(DP), INTENT(in) :: charges (nat, 0:lmax_wfc, nspin )
+  REAL(DP), INTENT(in), OPTIONAL :: charges_lm (nat, 0:lmax_wfc, 1:2*lmax_wfc+1, nspin )
+  INTEGER, EXTERNAL :: find_free_unit
+  !
+  CHARACTER(len=256) :: filename
+  INTEGER :: is, l, m, na, unit
+  REAL(DP) :: totcharge(2), psum
+  CHARACTER (len=1)  :: l_label(0:3)=(/'s','p','d','f'/)
+  CHARACTER (len=7)  :: lm_label(1:7,1:3)=reshape( (/ &
+    'z      ','x      ','y      ','       ','       ','       ','       ', &
+    'z2     ','xz     ','yz     ','x2-y2  ','xy     ','       ','       ', &
+    'z3     ','xz2    ','yz2    ','zx2-zy2','xyz    ','x3-3xy2','3yx2-y3' /), (/7,3/) )
+  !
+  filename = trim(filproj)//'.lowdin'
+
+  IF ( ionode ) THEN
+     unit = find_free_unit()
+     OPEN( unit=unit, file=trim(filename), status='unknown', form='formatted')
+     WRITE( stdout, '(/"Lowdin Charges: "/)')
+     WRITE( unit, '(/"Lowdin Charges: "/)')
+     !
+     DO na = 1, nat
+        DO is = 1, nspin
+          WRITE(stdout, *) is, nspin, charges(na,0:lmax_wfc,is)
+          totcharge(is) = SUM(charges(na,0:lmax_wfc,is))
+        ENDDO
+        IF ( nspin == 1) THEN
+           DO l = 0, lmax_wfc
+              WRITE(stdout, 2000,advance='no') na, totcharge(1), l_label(l), charges(na,l,1)
+              WRITE(unit, 2000,advance='no') na, totcharge(1), l_label(l), charges(na,l,1)
+              IF (l /= 0 .AND. present(charges_lm)) THEN
+                 DO m = 1, 2*l+1
+                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
+                    WRITE( unit,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
+                 ENDDO
+              ENDIF
+              WRITE(stdout,*)
+              WRITE(unit,*)
+           ENDDO
+        ELSEIF ( nspin == 2) THEN
+           WRITE( stdout, 2000) na, totcharge(1) + totcharge(2), &
+                ( l_label(l), charges(na,l,1) + charges(na,l,2), l=0,lmax_wfc)
+          WRITE( unit, 2000) na, totcharge(1) + totcharge(2), &
+                ( l_label(l), charges(na,l,1) + charges(na,l,2), l=0,lmax_wfc)
+           DO l = 0, lmax_wfc
+              WRITE(stdout,2001,advance='no') totcharge(1), l_label(l), charges(na,l,1)
+              WRITE(unit,2001,advance='no') totcharge(1), l_label(l), charges(na,l,1)
+              IF (l /= 0 .AND. present(charges_lm)) THEN
+                 DO m = 1, 2*l+1
+                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
+                    WRITE( unit,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
+                 ENDDO
+              ENDIF
+              WRITE(stdout,*)
+              WRITE(unit,*)
+           ENDDO
+           DO l = 0, lmax_wfc
+              WRITE(stdout,2002,advance='no') totcharge(2), l_label(l), charges(na,l,2)
+              WRITE(unit,2002,advance='no') totcharge(2), l_label(l), charges(na,l,2)
+              IF (l /= 0 .AND. present(charges_lm)) THEN
+                 DO m = 1, 2*l+1
+                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,2)
+                    WRITE( unit,'(A1,A,"=",F8.4,", ")',advance='no') &
+                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,2)
+                 ENDDO
+              ENDIF
+              WRITE(stdout,*)
+              WRITE(unit,*)
+           ENDDO
+           WRITE( stdout, 2003) totcharge(1) - totcharge(2), &
+                ( l_label(l), charges(na,l,1) - charges(na,l,2), l=0,lmax_wfc)
+           WRITE( unit, 2003) totcharge(1) - totcharge(2), &
+                ( l_label(l), charges(na,l,1) - charges(na,l,2), l=0,lmax_wfc)
+        ENDIF
+     ENDDO
+     !
+     psum = SUM(charges(:,:,:)) / nelec
+     WRITE( stdout, '(5x,"Spilling Parameter: ",f8.4)') 1.0d0 - psum
+     WRITE( unit, '(5x,"Spilling Parameter: ",f8.4)') 1.0d0 - psum
+     !
+     ! Sanchez-Portal et al., Sol. State Commun.  95, 685 (1995).
+     ! The spilling parameter measures the ability of the basis provided by
+     ! the pseudo-atomic wfc to represent the PW eigenstates,
+     ! by measuring how much of the subspace of the Hamiltonian
+     ! eigenstates falls outside the subspace spanned by the atomic basis
+     !
+     CLOSE(unit)
+  END IF
+
+2000 FORMAT (5x,"Atom # ",i3,": total charge = ",f8.4,4(", ",a1," =",f8.4))
+2001 FORMAT (15x,"  spin up      = ",f8.4,4(", ",a1," =",f8.4))
+2002 FORMAT (15x,"  spin down    = ",f8.4,4(", ",a1," =",f8.4))
+2003 FORMAT (15x,"  polarization = ",f8.4,4(", ",a1," =",f8.4))
+
+END SUBROUTINE
+!
 !-----------------------------------------------------------------------
 SUBROUTINE projwave( filproj, lsym, lwrite_ovp, lbinary )
   !-----------------------------------------------------------------------
@@ -287,19 +438,24 @@ SUBROUTINE projwave( filproj, lsym, lwrite_ovp, lbinary )
   REAL   (DP), ALLOCATABLE ::roverlap(:,:), rwork1(:),rproj0(:,:)
   ! ... or for gamma-point.
   REAL(DP), ALLOCATABLE :: charges(:,:,:), charges_lm(:,:,:,:), proj1 (:)
-  REAL(DP) :: psum, totcharge(2)
+  REAL(DP) :: psum
   INTEGER  :: nksinit, nkslast
   CHARACTER(len=256) :: filename
-  CHARACTER (len=1)  :: l_label(0:3)=(/'s','p','d','f'/)
-  CHARACTER (len=7)  :: lm_label(1:7,1:3)=reshape( (/ &
-    'z      ','x      ','y      ','       ','       ','       ','       ', &
-    'z2     ','xz     ','yz     ','x2-y2  ','xy     ','       ','       ', &
-    'z3     ','xz2    ','yz2    ','zx2-zy2','xyz    ','x3-3xy2','3yx2-y3' /), (/7,3/) )
   INTEGER, ALLOCATABLE :: idx(:)
   LOGICAL :: lsym
   LOGICAL :: freeswfcatom
   !
   !
+  INTERFACE 
+      SUBROUTINE write_lowdin ( filproj, nat, lmax_wfc, nspin, charges, charges_lm )
+           IMPORT  :: DP
+           CHARACTER (len=*), INTENT(in) :: filproj
+           INTEGER, INTENT(IN) :: nat, lmax_wfc, nspin
+           REAL(DP), INTENT(in) :: charges (nat, 0:lmax_wfc, nspin )
+           REAL(DP), INTENT(in), OPTIONAL :: charges_lm (nat, 0:lmax_wfc, 1:2*lmax_wfc+1, nspin )
+      END SUBROUTINE write_lowdin
+  END INTERFACE
+
   WRITE( stdout, '(/5x,"Calling projwave .... ")')
   IF ( gamma_only ) THEN
      WRITE( stdout, '(5x,"gamma-point specific algorithms are used")')
@@ -555,12 +711,12 @@ SUBROUTINE projwave( filproj, lsym, lwrite_ovp, lbinary )
      IF (filproj/=' ') THEN
         DO is=1,nspin
            IF (nspin==2) THEN
-              IF (is==1) filename=trim(filproj)//'.up'
-              IF (is==2) filename=trim(filproj)//'.down'
+              IF (is==1) filename=trim(filproj)//'.projwfc_up'
+              IF (is==2) filename=trim(filproj)//'.projwfc_down'
               nksinit=(nkstot/2)*(is-1)+1
               nkslast=(nkstot/2)*is
            ELSE
-              filename=trim(filproj)
+              filename=trim(filproj)//'.projwfc_up'
               nksinit=1
               nkslast=nkstot
            ENDIF
@@ -569,9 +725,9 @@ SUBROUTINE projwave( filproj, lsym, lwrite_ovp, lbinary )
                 dfftp%nr1, dfftp%nr2, dfftp%nr3, nat, ntyp, ibrav, celldm, at, gcutm, dual,   &
                 ecutwfc, nkstot/nspin, nbnd, natomwfc)
            DO nwfc = 1, natomwfc
-              WRITE(iunproj,'(2i5,a3,3i5)') &
+              WRITE(iunproj,'(2i5,1x,a4,1x,a2,1x,3i5)') &
                   nwfc, nlmchi(nwfc)%na, atm(ityp(nlmchi(nwfc)%na)), &
-                  nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m
+                  nlmchi(nwfc)%els, nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m
               DO ik=nksinit,nkslast
                  DO ibnd=1,nbnd
                    WRITE(iunproj,'(2i8,f20.10)') ik,ibnd, &
@@ -672,63 +828,7 @@ SUBROUTINE projwave( filproj, lsym, lwrite_ovp, lbinary )
         ENDDO
      ENDDO
      !
-     WRITE( stdout, '(/"Lowdin Charges: "/)')
-     !
-     DO na = 1, nat
-        DO is = 1, nspin
-           totcharge(is) = SUM(charges(na,0:lmax_wfc,is))
-        ENDDO
-        IF ( nspin == 1) THEN
-           DO l = 0, lmax_wfc
-              WRITE(stdout, 2000,advance='no') na, totcharge(1), l_label(l), charges(na,l,1)
-              IF (l /= 0) THEN
-                 DO m = 1, 2*l+1
-                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
-                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
-                 ENDDO
-              ENDIF
-              WRITE(stdout,*)
-           ENDDO
-        ELSEIF ( nspin == 2) THEN
-           WRITE( stdout, 2000) na, totcharge(1) + totcharge(2), &
-                ( l_label(l), charges(na,l,1) + charges(na,l,2), l=0,lmax_wfc)
-           DO l = 0, lmax_wfc
-              WRITE(stdout,2001,advance='no') totcharge(1), l_label(l), charges(na,l,1)
-              IF (l /= 0) THEN
-                 DO m = 1, 2*l+1
-                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
-                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,1)
-                 ENDDO
-              ENDIF
-              WRITE(stdout,*)
-           ENDDO
-           DO l = 0, lmax_wfc
-              WRITE(stdout,2002,advance='no') totcharge(2), l_label(l), charges(na,l,2)
-              IF (l /= 0) THEN
-                 DO m = 1, 2*l+1
-                    WRITE( stdout,'(A1,A,"=",F8.4,", ")',advance='no') &
-                       l_label(l), trim(lm_label(m,l)), charges_lm(na,l,m,2)
-                 ENDDO
-              ENDIF
-              WRITE(stdout,*)
-           ENDDO
-           WRITE( stdout, 2003) totcharge(1) - totcharge(2), &
-                ( l_label(l), charges(na,l,1) - charges(na,l,2), l=0,lmax_wfc)
-        ENDIF
-     ENDDO
-2000 FORMAT (5x,"Atom # ",i3,": total charge = ",f8.4,4(", ",a1," =",f8.4))
-2001 FORMAT (15x,"  spin up      = ",f8.4,4(", ",a1," =",f8.4))
-2002 FORMAT (15x,"  spin down    = ",f8.4,4(", ",a1," =",f8.4))
-2003 FORMAT (15x,"  polarization = ",f8.4,4(", ",a1," =",f8.4))
-     !
-     psum = SUM(charges(:,:,:)) / nelec
-     WRITE( stdout, '(5x,"Spilling Parameter: ",f8.4)') 1.0d0 - psum
-     !
-     ! Sanchez-Portal et al., Sol. State Commun.  95, 685 (1995).
-     ! The spilling parameter measures the ability of the basis provided by
-     ! the pseudo-atomic wfc to represent the PW eigenstates,
-     ! by measuring how much of the subspace of the Hamiltonian
-     ! eigenstates falls outside the subspace spanned by the atomic basis
+     CALL write_lowdin ( filproj, nat, lmax_wfc, nspin, charges, charges_lm )
      !
      DEALLOCATE (charges, charges_lm)
      !
@@ -773,7 +873,7 @@ SUBROUTINE projwave_nc(filproj, lsym, lwrite_ovp, lbinary, ef_0 )
   IMPLICIT NONE
   !
   CHARACTER(len=*) :: filproj
-  CHARACTER(256) :: file_eband
+  CHARACTER(256) :: filename
   LOGICAL :: lwrite_ovp, lbinary
   LOGICAL :: lsym
   LOGICAL :: freeswfcatom
@@ -787,7 +887,7 @@ SUBROUTINE projwave_nc(filproj, lsym, lwrite_ovp, lbinary, ef_0 )
   COMPLEX(DP), ALLOCATABLE :: overlap(:,:), work(:,:), work1(:), proj0(:,:)
   ! Some workspace for k-point calculation ...
   REAL(DP), ALLOCATABLE :: charges(:,:,:), proj1 (:), eband_proj(:)
-  REAL(DP) :: psum, totcharge(2), fact(2), spinor, compute_mj
+  REAL(DP) :: psum, fact(2), spinor, compute_mj
   INTEGER, ALLOCATABLE :: idx(:)
   !
   COMPLEX(DP) :: d12(2, 2, 48), d32(4, 4, 48), d52(6, 6, 48), &
@@ -797,6 +897,15 @@ SUBROUTINE projwave_nc(filproj, lsym, lwrite_ovp, lbinary, ef_0 )
   !
   !
   !
+  INTERFACE 
+      SUBROUTINE write_lowdin ( filproj, nat, lmax_wfc, nspin, charges, charges_lm )
+           IMPORT  :: DP
+           CHARACTER (len=*), INTENT(in) :: filproj
+           INTEGER, INTENT(IN) :: nat, lmax_wfc, nspin
+           REAL(DP), INTENT(in) :: charges (nat, 0:lmax_wfc, nspin )
+           REAL(DP), INTENT(in), OPTIONAL :: charges_lm (nat, 0:lmax_wfc, 1:2*lmax_wfc+1, nspin )
+      END SUBROUTINE write_lowdin
+  END INTERFACE
   IF (.not.noncolin) CALL errore('projwave_nc','called in the wrong case',1)
   IF (gamma_only) CALL errore('projwave_nc','gamma_only not yet implemented',1)
   WRITE( stdout, '(/5x,"Calling projwave_nc .... ")')
@@ -856,6 +965,7 @@ SUBROUTINE projwave_nc(filproj, lsym, lwrite_ovp, lbinary, ef_0 )
       CALL weights()
 !   write(6,*) 'ef_0 = ', ef_0
 !   write(6,*) wg
+      ef_0 = ef_0 / rytoev
       eband_tot = 0.d0
       ALLOCATE (eband_proj(natomwfc))
       eband_proj = 0.d0
@@ -1075,7 +1185,6 @@ SUBROUTINE projwave_nc(filproj, lsym, lwrite_ovp, lbinary, ef_0 )
 
 !-- AlexS
    IF ( lforcet ) THEN
-     ef_0 = ef_0 / rytoev     
      DO i = 1, nbnd
          psum = wg(i,ik) * (et(i,ik)-ef_0)
          eband_tot = eband_tot + psum
@@ -1099,8 +1208,8 @@ IF ( lforcet ) THEN
  CALL mp_sum( eband_proj, inter_pool_comm )
 IF ( ionode ) THEN
 
-       file_eband = trim(filproj)
-       OPEN (4,file=file_eband,form='formatted', status='unknown')
+       filename = trim(filproj)
+       OPEN (4,file=filename,form='formatted', status='unknown')
 
        eband_proj_tot = 0.d0
        DO na = 1, nat
@@ -1177,20 +1286,23 @@ ENDIF
      ! write on the file filproj
      !
      IF (filproj/=' ') THEN
+        filename = trim(filproj)//'.projwfc_up'
         iunproj=33
-        CALL write_io_header(filproj, iunproj, title, dfftp%nr1x, dfftp%nr2x, dfftp%nr3x, &
+        CALL write_io_header(filename, iunproj, title, dfftp%nr1x, dfftp%nr2x, dfftp%nr3x, &
            dfftp%nr1, dfftp%nr2, dfftp%nr3, nat, ntyp, ibrav, celldm, at, gcutm, dual, ecutwfc, &
            nkstot,nbnd,natomwfc)
         DO nwfc = 1, natomwfc
            IF (lspinorb) THEN
-              WRITE(iunproj,1000) &
-                   nwfc, nlmchi(nwfc)%na,atm(ityp(nlmchi(nwfc)%na)), &
-                   nlmchi(nwfc)%n,nlmchi(nwfc)%jj,nlmchi(nwfc)%l, &
-                   compute_mj(nlmchi(nwfc)%jj,nlmchi(nwfc)%l,nlmchi(nwfc)%m)
-           ELSE
-              WRITE(iunproj,1500) &
+              WRITE(iunproj,'(2i5,1x,a4,1x,a2,1x,2i5,f5.1,1x,f5.1)') &
                    nwfc, nlmchi(nwfc)%na, atm(ityp(nlmchi(nwfc)%na)), &
-                   nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m, &
+                   nlmchi(nwfc)%els, nlmchi(nwfc)%n, nlmchi(nwfc)%l, &
+                   nlmchi(nwfc)%jj, &
+                   compute_mj(nlmchi(nwfc)%jj,nlmchi(nwfc)%l, nlmchi(nwfc)%m)
+           ELSE
+              WRITE(iunproj,'(2i5,1x,a4,1x,a2,1x,3i5,1x,f4.1)') &
+                   nwfc, nlmchi(nwfc)%na, atm(ityp(nlmchi(nwfc)%na)), &
+                   nlmchi(nwfc)%els, nlmchi(nwfc)%n, nlmchi(nwfc)%l, &
+                   nlmchi(nwfc)%m, &
                    0.5d0-int(nlmchi(nwfc)%ind/(2*nlmchi(nwfc)%l+2))
            ENDIF
            DO ik=1,nkstot
@@ -1221,7 +1333,7 @@ ENDIF
              nlmchi(nwfc)%n, nlmchi(nwfc)%jj, nlmchi(nwfc)%l,   &
              compute_mj(nlmchi(nwfc)%jj,nlmchi(nwfc)%l,nlmchi(nwfc)%m)
         ENDDO
-1000    FORMAT (5x,"state #",i3,": atom ",i3," (",a3,"), wfc ",i2, &
+1000    FORMAT (5x,"state #",i4,": atom ",i3," (",a3,"), wfc ",i2, &
                    " (j=",f3.1," l=",i1," m_j=",f4.1,")")
      ELSE
         DO nwfc = 1, natomwfc
@@ -1230,7 +1342,7 @@ ENDIF
              nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m, &
              0.5d0-int(nlmchi(nwfc)%ind/(2*nlmchi(nwfc)%l+2))
         ENDDO
-1500    FORMAT (5x,"state #",i3,": atom ",i3," (",a3,"), wfc ",i2, &
+1500    FORMAT (5x,"state #",i4,": atom ",i3," (",a3,"), wfc ",i2, &
                    " (l=",i1," m=",i2," s_z=",f4.1,")")
      ENDIF
      !
@@ -1260,10 +1372,10 @@ ENDIF
            !
            ! fancy (?!?) formatting
            !
-           WRITE( stdout, '(5x,"psi = ",5(f5.3,"*[#",i3,"]+"))') &
+           WRITE( stdout, '(5x,"psi = ",5(f5.3,"*[#",i4,"]+"))') &
                 (proj1 (i), idx(i), i = 1, min(5,nwfc))
            DO j = 1, (nwfc-1)/5
-              WRITE( stdout, '(10x,"+",5(f5.3,"*[#",i3,"]+"))') &
+              WRITE( stdout, '(10x,"+",5(f5.3,"*[#",i4,"]+"))') &
                    (proj1 (i), idx(i), i = 5*j+1, min(5*(j+1),nwfc))
            ENDDO
            psum = SUM ( proj(1:natomwfc, ibnd, ik) )
@@ -1311,43 +1423,7 @@ ENDIF
         ENDDO
      ENDIF
      !
-     WRITE( stdout, '(/"Lowdin Charges: "/)')
-     !
-     DO na = 1, nat
-        DO is = 1, nspin0
-           totcharge(is) = SUM(charges(na,0:lmax_wfc,is))
-        ENDDO
-        IF ( nspin0 == 1) THEN
-           WRITE( stdout, 2000) na, totcharge(1), &
-                ( charges(na,l,1), l= 0,lmax_wfc)
-        ELSEIF ( nspin0 == 2) THEN
-           WRITE( stdout, 2000) na, totcharge(1) + totcharge(2), &
-                ( charges(na,l,1) + charges(na,l,2), l=0,lmax_wfc)
-           WRITE( stdout, 2001) totcharge(1), &
-                ( charges(na,l,1), l= 0,lmax_wfc)
-           WRITE( stdout, 2002) totcharge(2), &
-                ( charges(na,l,2), l= 0,lmax_wfc)
-           WRITE( stdout, 2003) totcharge(1) - totcharge(2), &
-                 ( charges(na,l,1) - charges(na,l,2), l=0,lmax_wfc)
-        ENDIF
-     ENDDO
-2000 FORMAT (5x,"Atom # ",i3,": total charge = ",f8.4 ,&
-          & ", s, p, d, f = ",4f8.4)
-2001 FORMAT (15x,"  spin up      = ",f8.4 , &
-          & ", s, p, d, f = ",4f8.4)
-2002 FORMAT (15x,"  spin down    = ",f8.4 , &
-          & ", s, p, d, f = ",4f8.4)
-2003 FORMAT (15x,"  polarization = ",f8.4 , &
-          & ", s, p, d, f = ",4f8.4)
-     !
-     psum = sum(charges(:,:,:)) / nelec
-     WRITE( stdout, '(5x,"Spilling Parameter: ",f8.4)') 1.0d0 - psum
-     !
-     ! Sanchez-Portal et al., Sol. State Commun.  95, 685 (1995).
-     ! The spilling parameter measures the ability of the basis provided by
-     ! the pseudo-atomic wfc to represent the PW eigenstates,
-     ! by measuring how much of the subspace of the Hamiltonian
-     ! eigenstates falls outside the subspace spanned by the atomic basis
+     CALL write_lowdin ( filproj, nat, lmax_wfc, nspin0, charges )
      !
      DEALLOCATE (charges)
      !
@@ -1401,14 +1477,8 @@ SUBROUTINE projwave_paw( filproj)
   REAL   (DP), ALLOCATABLE ::roverlap(:,:), rwork1(:),rproj0(:,:)
   ! ... or for gamma-point.
   REAL(DP), ALLOCATABLE :: charges(:,:,:), charges_lm(:,:,:,:), proj1 (:)
-  REAL(DP) :: psum, totcharge(2)
+  REAL(DP) :: psum
   INTEGER  :: nksinit, nkslast
-  CHARACTER(len=256) :: filename
-  CHARACTER (len=1)  :: l_label(0:3)=(/'s','p','d','f'/)
-  CHARACTER (len=7)  :: lm_label(1:7,1:3)=reshape( (/ &
-    'z      ','x      ','y      ','       ','       ','       ','       ', &
-    'z2     ','xz     ','yz     ','x2-y2  ','xy     ','       ','       ', &
-    'z3     ','xz2    ','yz2    ','zx2-zy2','xyz    ','x3-3xy2','3yx2-y3' /), (/7,3/) )
   INTEGER, ALLOCATABLE :: idx(:)
   LOGICAL :: lsym
   LOGICAL :: freeswfcatom
@@ -1480,7 +1550,7 @@ SUBROUTINE projwave_paw( filproj)
   !
   DO ik = 1, nks
      CALL davcio (evc, 2*nwordwfc, iunwfc, ik, - 1)
-
+     npw = ngk(ik)
      CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
 
      proj0=0; 
@@ -1549,7 +1619,7 @@ SUBROUTINE  write_proj (filename, lbinary, projs, lwrite_ovp, ovps )
   !-----------------------------------------------------------------------
   !
   USE kinds
-  USE io_files,         ONLY : iun => iunsat, prefix, tmp_dir
+  USE io_files,         ONLY : iun => iunsat, prefix, tmp_dir, postfix
   USE basis,            ONLY : natomwfc
   USE cell_base
   USE klist,            ONLY : wk, xk, nkstot, nelec
@@ -1574,7 +1644,7 @@ SUBROUTINE  write_proj (filename, lbinary, projs, lwrite_ovp, ovps )
 ! subroutine body
 !
 
-  tmp = trim( tmp_dir ) // trim( prefix ) // '.save/' //trim(filename)
+  tmp = trim( tmp_dir ) // trim( prefix ) // postfix //trim(filename)
   !
   IF ( lbinary ) THEN
       tmp = TRIM(tmp) // ".dat"
@@ -1771,11 +1841,10 @@ SUBROUTINE pprojwave( filproj, lsym, lwrite_ovp, lbinary )
   REAL   (DP), ALLOCATABLE ::roverlap_d(:,:)
   ! ... or for gamma-point.
   REAL(DP), ALLOCATABLE :: charges(:,:,:), proj1 (:)
-  REAL(DP) :: psum, totcharge(2)
+  REAL(DP) :: psum
   INTEGER  :: nksinit, nkslast
   CHARACTER(len=256) :: filename
   CHARACTER(len=256) :: auxname
-  CHARACTER (len=1)  :: l_label(0:3)=(/'s','p','d','f'/)
   INTEGER, ALLOCATABLE :: idx(:)
   LOGICAL :: lsym
   LOGICAL :: freeswfcatom
@@ -1791,6 +1860,16 @@ SUBROUTINE pprojwave( filproj, lsym, lwrite_ovp, lbinary )
   INTEGER, ALLOCATABLE :: ic_notcnv( : )
   !
   !
+  INTERFACE 
+      SUBROUTINE write_lowdin ( filproj, nat, lmax_wfc, nspin, charges, charges_lm )
+           IMPORT  :: DP
+           CHARACTER (len=*), INTENT(in) :: filproj
+           INTEGER, INTENT(IN) :: nat, lmax_wfc, nspin
+           REAL(DP), INTENT(in) :: charges (nat, 0:lmax_wfc, nspin )
+           REAL(DP), INTENT(in), OPTIONAL :: charges_lm (nat, 0:lmax_wfc, 1:2*lmax_wfc+1, nspin )
+      END SUBROUTINE write_lowdin
+  END INTERFACE
+  !
   WRITE( stdout, '(/5x,"Calling pprojwave .... ")')
   IF ( gamma_only ) THEN
      WRITE( stdout, '(5x,"gamma-point specific algorithms are used")')
@@ -2129,12 +2208,12 @@ SUBROUTINE pprojwave( filproj, lsym, lwrite_ovp, lbinary )
      IF (filproj/=' ') THEN
         DO is=1,nspin
            IF (nspin==2) THEN
-              IF (is==1) filename=trim(filproj)//'.up'
-              IF (is==2) filename=trim(filproj)//'.down'
+              IF (is==1) filename=trim(filproj)//'.projwfc_up'
+              IF (is==2) filename=trim(filproj)//'.projwfc_down'
               nksinit=(nkstot/2)*(is-1)+1
               nkslast=(nkstot/2)*is
            ELSE
-              filename=trim(filproj)
+              filename=trim(filproj)//'.projwfc_up'
               nksinit=1
               nkslast=nkstot
            ENDIF
@@ -2143,9 +2222,9 @@ SUBROUTINE pprojwave( filproj, lsym, lwrite_ovp, lbinary )
                 dfftp%nr1, dfftp%nr2, dfftp%nr3, nat, ntyp, ibrav, celldm, at, gcutm, dual, &
                 ecutwfc, nkstot/nspin,nbnd,natomwfc)
            DO nwfc = 1, natomwfc
-              WRITE(iunproj,'(2i5,a3,3i5)') &
+              WRITE(iunproj,'(2i5,1x,a4,1x,a2,1x,3i5)') &
                   nwfc, nlmchi(nwfc)%na, atm(ityp(nlmchi(nwfc)%na)), &
-                  nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m
+                  nlmchi(nwfc)%els, nlmchi(nwfc)%n, nlmchi(nwfc)%l, nlmchi(nwfc)%m
               DO ik=nksinit,nkslast
                  DO ibnd=1,nbnd
                    WRITE(iunproj,'(2i8,f20.10)') ik,ibnd, &
@@ -2242,39 +2321,7 @@ SUBROUTINE pprojwave( filproj, lsym, lwrite_ovp, lbinary )
         ENDDO
      ENDDO
      !
-     WRITE( stdout, '(/"Lowdin Charges: "/)')
-     !
-     DO na = 1, nat
-        DO is = 1, nspin
-           totcharge(is) = SUM(charges(na,0:lmax_wfc,is))
-        ENDDO
-        IF ( nspin == 1) THEN
-           WRITE( stdout, 2000) na, totcharge(1), &
-                ( l_label(l), charges(na,l,1), l= 0,lmax_wfc)
-        ELSEIF ( nspin == 2) THEN
-           WRITE( stdout, 2000) na, totcharge(1) + totcharge(2), &
-                ( l_label(l), charges(na,l,1) + charges(na,l,2), l=0,lmax_wfc)
-           WRITE( stdout, 2001) totcharge(1), &
-                ( l_label(l), charges(na,l,1), l= 0,lmax_wfc)
-           WRITE( stdout, 2002) totcharge(2), &
-                ( l_label(l), charges(na,l,2), l= 0,lmax_wfc)
-           WRITE( stdout, 2003) totcharge(1) - totcharge(2), &
-                ( l_label(l), charges(na,l,1) - charges(na,l,2), l=0,lmax_wfc)
-        ENDIF
-     ENDDO
-2000 FORMAT (5x,"Atom # ",i3,": total charge = ",f8.4,4(", ",a1," =",f8.4))
-2001 FORMAT (15x,"  spin up      = ",f8.4,4(", ",a1," =",f8.4))
-2002 FORMAT (15x,"  spin down    = ",f8.4,4(", ",a1," =",f8.4))
-2003 FORMAT (15x,"  polarization = ",f8.4,4(", ",a1," =",f8.4))
-     !
-     psum = SUM(charges(:,:,:)) / nelec
-     WRITE( stdout, '(5x,"Spilling Parameter: ",f8.4)') 1.0d0 - psum
-     !
-     ! Sanchez-Portal et al., Sol. State Commun.  95, 685 (1995).
-     ! The spilling parameter measures the ability of the basis provided by
-     ! the pseudo-atomic wfc to represent the PW eigenstates,
-     ! by measuring how much of the subspace of the Hamiltonian
-     ! eigenstates falls outside the subspace spanned by the atomic basis
+     CALL write_lowdin ( filproj, nat, lmax_wfc, nspin, charges)
      !
      DEALLOCATE (charges)
      !
