@@ -149,7 +149,7 @@ SUBROUTINE c_phase
 !----------------------------------------------------------------------!
 
 !   Geometric phase calculation along a strip of nppstr k-points
-!   averaged over a 2D grid of nkort k-points ortogonal to nppstr 
+!   averaged over a 2D grid of nkort k-points orthogonal to nppstr 
 
 !  --- Make use of the module with common information ---
    USE kinds,                ONLY : DP
@@ -174,7 +174,8 @@ SUBROUTINE c_phase
    USE spin_orb,             ONLY : lspinorb
    USE mp_bands,             ONLY : intra_bgrp_comm, nproc_bgrp
    USE mp,                   ONLY : mp_sum
-   USE qexsd_module,         ONLY : qexsd_init_berryPhaseOutput, qexsd_bp_obj
+   USE qes_libs_module,      ONLY : qes_reset_berryPhaseOutput
+   USE qexsd_module,         ONLY : qexsd_init_berryPhaseOutput,  qexsd_bp_obj
 !  --- Avoid implicit definitions ---
    IMPLICIT NONE
 
@@ -949,8 +950,8 @@ SUBROUTINE c_phase
    WRITE( stdout,"(/,11X,'P = ',F11.7,'  (mod ',F11.7,')  C/m^2')") &
         fac*pdl_tot,fac*DBLE(mod_tot)
 !  --- Write polarization direction ---
-   WRITE( stdout,"(/,8X,'The polarization direction is:  ( ', &
-       &  F7.5,' , ',F7.5,' , ',F7.5,' )')") upol(1),upol(2),upol(3)
+   WRITE( stdout,"(/,8X,'The polarization direction is:  (', &
+       &  F8.5,' ,',F8.5,' ,',F8.5,' )')") upol(1),upol(2),upol(3)
 
 !  --- End of information relative to polarization calculation ---
    WRITE( stdout,"(/,/,15X,50('=')/,/)")
@@ -959,6 +960,7 @@ SUBROUTINE c_phase
 ! Here we write all output information in a berry_phase_type variable to print
 ! them in the XML output  P.D. april 2016
 !------------------------------------------------------------------------------
+  CALL qes_reset_berryPhaseOutput(qexsd_bp_obj )   
   CALL qexsd_init_berryPhaseOutput(qexsd_bp_obj, gpar, gvec, nppstr, nkort, xk, pdl_ion, mod_ion,  &
                                   pdl_ion_tot, mod_ion_tot, nstring, pdl_elec , mod_elec, wstring, &
                                   pdl_elec_up, mod_elec_up, pdl_elec_dw, mod_elec_dw, pdl_elec_tot,&
