@@ -7,7 +7,7 @@
 !
 !----------------------------------------------------------------------------
 ! TB
-! included monopole related energy
+! included gate related energy
 !----------------------------------------------------------------------------
 !
 !----------------------------------------------------------------------------
@@ -26,7 +26,7 @@ SUBROUTINE electrons()
   USE lsda_mod,             ONLY : nspin, magtot, absmag
   USE ener,                 ONLY : etot, hwf_energy, eband, deband, ehart, &
                                    vtxc, etxc, etxcc, ewld, demet, epaw, &
-                                   elondon, ef_up, ef_dw
+                                   elondon, edftd3, ef_up, ef_dw
   USE scf,                  ONLY : rho, rho_core, rhog_core, v, vltot, vrs, &
                                    kedtau, vnew
   USE control_flags,        ONLY : tr2, niter, conv_elec, restart, lmd, &
@@ -40,14 +40,16 @@ SUBROUTINE electrons()
   USE noncollin_module,     ONLY : noncolin, magtot_nc, i_cons,  bfield, &
                                    lambda, report
   USE uspp,                 ONLY : okvan
-  USE exx,                  ONLY : exxinit, exxenergy2, exxbuff, &
-                                   fock0, fock1, fock2, dexx
+  USE exx,                  ONLY : aceinit,exxinit, exxenergy2, exxenergy, exxbuff, &
+                                   fock0, fock1, fock2, fock3, dexx, use_ace, local_thr
   USE funct,                ONLY : dft_is_hybrid, exx_is_active
   USE control_flags,        ONLY : adapt_thr, tr2_init, tr2_multi, gamma_only
   !
   USE paw_variables,        ONLY : okpaw, ddd_paw, total_core_energy, only_paw
   USE paw_onecenter,        ONLY : PAW_potential
   USE paw_symmetry,         ONLY : PAW_symmetrize_ddd
+  USE ions_base,            ONLY : nat
+  USE loc_scdm,             ONLY : use_scdm, localize_orbitals
   !
   !
   IMPLICIT NONE
@@ -68,14 +70,17 @@ SUBROUTINE electrons()
       tr2_final     ! final threshold for exx minimization 
                     ! when using adaptive thresholds.
   LOGICAL :: first, exst
+  REAL(DP) :: etot_cmp_paw(nat,2,2)
+  LOGICAL :: DoLoc
   !
   !
+  DoLoc = local_thr.gt.0.0d0
   exxen = 0.0d0
   iter = 0
   first = .true.
   tr2_final = tr2
   IF ( dft_is_hybrid() ) THEN
-     ! printout = 0  : do not print etot and energy components at each scf step
+     !printout = 0  ! do not print etot and energy components at each scf step
      printout = 1  ! print etot, not energy components at each scf step
   ELSE IF ( lmd ) THEN
      printout = 1  ! print etot, not energy components at each scf step
@@ -85,6 +90,7 @@ SUBROUTINE electrons()
   IF (dft_is_hybrid() .AND. adapt_thr ) tr2= tr2_init
   fock0 = 0.D0
   fock1 = 0.D0
+  fock3 = 0.D0
   IF (.NOT. exx_is_active () ) fock2 = 0.D0
   !
   !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -101,7 +107,7 @@ SUBROUTINE electrons()
         ELSE IF ( iter < 0 .OR. iter > niter ) THEN
            iter = 0
         ELSE 
-           READ (iunres, *) fock0, fock1, fock2
+           READ (iunres, *) exxen, fock0, fock1, fock2
            ! FIXME: et and wg should be read from xml file
            READ (iunres, *) (wg(1:nbnd,ik),ik=1,nks)
            READ (iunres, *) (et(1:nbnd,ik),ik=1,nks)
@@ -109,15 +115,17 @@ SUBROUTINE electrons()
            ! ... if restarting here, exx was already active
            ! ... initialize stuff for exx
            first = .false.
-           CALL exxinit()
+           CALL exxinit(DoLoc)
+           IF( DoLoc ) CALL localize_orbitals( )
            ! FIXME: ugly hack, overwrites exxbuffer from exxinit
            CALL seqopn (iunres, 'restart_exx', 'unformatted', exst)
            IF (exst) READ (iunres, iostat=ios) exxbuff
            IF (ios /= 0) WRITE(stdout,'(5x,"Error in EXX restart!")')
+           IF (use_ace) CALL aceinit ( )
            !
            CALL v_of_rho( rho, rho_core, rhog_core, &
                ehart, etxc, vtxc, eth, etotefield, charge, v)
-           IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw)
+           IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw,etot_cmp_paw)
            CALL set_vrs( vrs, vltot, v%of_r, kedtau, v%kin_r, dfftp%nnr, &
                          nspin, doublegrid )
            !
@@ -148,7 +156,7 @@ SUBROUTINE electrons()
                         & i6)') iter
            CALL seqopn (iunres, 'restart_e', 'formatted', exst)
            WRITE (iunres, *) iter-1, tr2, dexx
-           WRITE (iunres, *) fock0, fock1, fock2
+           WRITE (iunres, *) exxen, fock0, fock1, fock2
            WRITE (iunres, *) (wg(1:nbnd,ik),ik=1,nks)
            WRITE (iunres, *) (et(1:nbnd,ik),ik=1,nks)
            CLOSE (unit=iunres, status='keep')
@@ -170,12 +178,14 @@ SUBROUTINE electrons()
         ! Activate exact exchange, set orbitals used in its calculation,
         ! then calculate exchange energy (will be useful at next step)
         !
-        CALL exxinit()
-#if defined(__EXX_ACE) 
-        fock2 = exxenergyace()
-#else  
-        fock2 = exxenergy2()
-#endif
+        CALL exxinit(DoLoc)
+        IF( DoLoc ) CALL localize_orbitals( )
+        IF (use_ace) THEN
+           CALL aceinit ( ) 
+           fock2 = exxenergyace()
+        ELSE
+           fock2 = exxenergy2()
+        ENDIF
         exxen = 0.50d0*fock2 
         etot = etot - etxc 
         !
@@ -186,7 +196,7 @@ SUBROUTINE electrons()
              ehart, etxc, vtxc, eth, etotefield, charge, v)
         etot = etot + etxc + exxen
         !
-        IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw)
+        IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw,etot_cmp_paw)
         CALL set_vrs( vrs, vltot, v%of_r, kedtau, v%kin_r, dfftp%nnr, &
              nspin, doublegrid )
         !
@@ -195,48 +205,76 @@ SUBROUTINE electrons()
         ! fock1 is the exchange energy calculated for orbitals at step n,
         !       using orbitals at step n-1 in the expression of exchange
         !
-#if defined(__EXX_ACE)
-        fock1 = exxenergyace()
-#else  
-        fock1 = exxenergy2()
-#endif
+        IF (use_ace) THEN
+           fock1 = exxenergyace()
+        ELSE
+           fock1 = exxenergy2()
+        ENDIF
         !
         ! Set new orbitals for the calculation of the exchange term
         !
-        CALL exxinit()
+        CALL exxinit(DoLoc)
+        IF( DoLoc ) CALL localize_orbitals( )
+        IF (use_ace) CALL aceinit ( fock3 )
         !
         ! fock2 is the exchange energy calculated for orbitals at step n,
         !       using orbitals at step n in the expression of exchange 
         ! fock0 is fock2 at previous step
         !
         fock0 = fock2
-#if defined(__EXX_ACE) 
-        fock2 = exxenergyace()
-#else  
-        fock2 = exxenergy2()
-#endif
+        IF (use_ace) THEN
+           fock2 = exxenergyace()
+        ELSE
+           fock2 = exxenergy2()
+        ENDIF
         !
         ! check for convergence. dexx is positive definite: if it isn't,
         ! there is some numerical problem. One such cause could be that
         ! the treatment of the divergence in exact exchange has failed. 
+        ! FIXME: to be properly implemented for all cases
         !
-        dexx = fock1 - 0.5D0*(fock0+fock2)
-        IF ( dexx < 0d0 ) CALL errore( 'electrons', 'dexx is negative! &
-           &  Check that exxdiv_treatment is appropriate for the system', 1 )
+!civn 
+!       IF (use_ace .AND. (nspin == 1) .AND. gamma_only) THEN
+        IF ( DoLoc ) THEN
+          dexx =  0.5D0 * ((fock1-fock0)+(fock3-fock2)) 
+        ELSE
+          dexx = fock1 - 0.5D0*(fock0+fock2)
+        END IF 
+        !
+        IF ( dexx < 0.0_dp ) THEN
+           IF( Doloc ) THEN
+              WRITE(stdout,'(5x,a,1e12.3)') "BEWARE: negative dexx:", dexx
+              dexx = ABS ( dexx )
+           ELSE
+              CALL errore( 'electrons', 'dexx is negative! &
+                   & Check that exxdiv_treatment is appropriate for the system,&
+                   & or ecutfock may be too low', 1 )
+           ENDIF
+        ENDIF
         !
         !   remove the estimate exchange energy exxen used in the inner SCF
         !
         etot = etot + exxen + 0.5D0*fock2 - fock1
+        hwf_energy = hwf_energy + exxen + 0.5D0*fock2 - fock1 ! [LP]
         exxen = 0.5D0*fock2 
-        ! write(*,*) '@chken', etot
-        hwf_energy = hwf_energy + 0.5D0*fock2 - fock1
+        !
         IF ( dexx < tr2_final ) THEN
-           WRITE( stdout, 9066 ) '!', etot, hwf_energy, dexx
+           WRITE( stdout, 9066 ) '!!', etot, hwf_energy
         ELSE
-           WRITE( stdout, 9066 ) ' ', etot, hwf_energy, dexx
+           WRITE( stdout, 9066 ) '  ', etot, hwf_energy
         END IF
+        IF ( dexx>1.d-8 ) THEN
+          WRITE( stdout, 9067 ) dexx
+        ELSE
+          WRITE( stdout, 9068 ) dexx
+        ENDIF
+        
         WRITE( stdout, 9062 ) - fock1
-        WRITE( stdout, 9064 ) 0.5D0*fock2
+        IF (use_ace) THEN
+           WRITE( stdout, 9063 ) 0.5D0*fock2
+        ELSE
+           WRITE( stdout, 9064 ) 0.5D0*fock2
+        ENDIF
         !
         IF ( dexx < tr2_final ) THEN
            IF ( do_makov_payne ) CALL makov_payne( etot )
@@ -258,7 +296,7 @@ SUBROUTINE electrons()
         conv_elec=.FALSE.
         CALL seqopn (iunres, 'restart_e', 'formatted', exst)
         WRITE (iunres, *) iter, tr2, dexx
-        WRITE (iunres, *) fock0, fock1, fock2
+        WRITE (iunres, *) exxen, fock0, fock1, fock2
         ! FIXME: et and wg are written to xml file
         WRITE (iunres, *) (wg(1:nbnd,ik),ik=1,nks)
         WRITE (iunres, *) (et(1:nbnd,ik),ik=1,nks)
@@ -276,10 +314,12 @@ SUBROUTINE electrons()
   ! ... formats
   !
 9062 FORMAT( '     - averaged Fock potential =',0PF17.8,' Ry' )
-9064 FORMAT( '     + Fock energy             =',0PF17.8,' Ry' )
-9066 FORMAT(/,A1,'    total energy              =',0PF17.8,' Ry' &
-            /'     Harris-Foulkes estimate   =',0PF17.8,' Ry' &
-            /'     est. exchange err (dexx)  =',0PF17.8,' Ry' )
+9063 FORMAT( '     + Fock energy (ACE)       =',0PF17.8,' Ry' )
+9064 FORMAT( '     + Fock energy (full)      =',0PF17.8,' Ry' )
+9066 FORMAT(/,A2,'   total energy              =',0PF17.8,' Ry' &
+            /'     Harris-Foulkes estimate   =',0PF17.8,' Ry' )
+9067 FORMAT('     est. exchange err (dexx)  =',0PF17.8,' Ry' )
+9068 FORMAT('     est. exchange err (dexx)  =',1PE17.1,' Ry' )
 9101 FORMAT(/'     EXX self-consistency reached' )
 9120 FORMAT(/'     EXX convergence NOT achieved after ',i3,' iterations: stopping' )
 9121 FORMAT(/'     scf convergence threshold =',1PE17.1,' Ry' )
@@ -302,11 +342,11 @@ SUBROUTINE electrons_scf ( printout, exxen )
   USE check_stop,           ONLY : check_stop_now, stopped_by_user
   USE io_global,            ONLY : stdout, ionode
   USE cell_base,            ONLY : at, bg, alat, omega, tpiba2
-  USE ions_base,            ONLY : zv, nat, nsp, ityp, tau, compute_eextfor
+  USE ions_base,            ONLY : zv, nat, nsp, ityp, tau, compute_eextfor, atm
   USE basis,                ONLY : starting_pot
   USE bp,                   ONLY : lelfield
   USE fft_base,             ONLY : dfftp
-  USE gvect,                ONLY : ngm, gstart, nl, nlm, g, gg, gcutm
+  USE gvect,                ONLY : ngm, gstart, g, gg, gcutm
   USE gvecs,                ONLY : doublegrid, ngms
   USE klist,                ONLY : xk, wk, nelec, ngk, nks, nkstot, lgauss, &
                                    two_fermi_energies, tot_charge
@@ -316,7 +356,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
   USE gvecw,                ONLY : ecutwfc
   USE ener,                 ONLY : etot, hwf_energy, eband, deband, ehart, &
                                    vtxc, etxc, etxcc, ewld, demet, epaw, &
-                                   elondon, ef_up, ef_dw, exdm, ef
+                                   elondon, edftd3, ef_up, ef_dw, exdm, ef
   USE scf,                  ONLY : scf_type, scf_type_COPY, bcast_scf_type,&
                                    create_scf_type, destroy_scf_type, &
                                    open_mix_file, close_mix_file, &
@@ -326,24 +366,29 @@ SUBROUTINE electrons_scf ( printout, exxen )
                                    iprint, conv_elec, &
                                    restart, io_level, do_makov_payne,  &
                                    gamma_only, iverbosity, textfor,     &
-                                   llondon, scf_must_converge, lxdm, ts_vdw
+                                   llondon, ldftd3, scf_must_converge, lxdm, ts_vdw
   USE control_flags,        ONLY : n_scf_steps, scf_error
 
   USE io_files,             ONLY : iunmix, output_drho, &
                                    iunres, iunefield, seqopn
   USE ldaU,                 ONLY : eth, Hubbard_U, Hubbard_lmax, &
                                    niter_with_fixed_ns, lda_plus_u
-  USE extfield,             ONLY : tefield, etotefield, monopole, etotmonofield !TB
+  USE extfield,             ONLY : tefield, etotefield, gate, etotgatefield !TB
   USE noncollin_module,     ONLY : noncolin, magtot_nc, i_cons,  bfield, &
                                    lambda, report
   USE spin_orb,             ONLY : domag
-  USE io_rho_xml,           ONLY : write_rho
+  USE io_rho_xml,           ONLY : write_scf
   USE uspp,                 ONLY : okvan
   USE mp_bands,             ONLY : intra_bgrp_comm
   USE mp_pools,             ONLY : root_pool, my_pool_id, inter_pool_comm
   USE mp,                   ONLY : mp_sum, mp_bcast
   !
   USE london_module,        ONLY : energy_london
+  USE dftd3_api,            ONLY : dftd3_pbc_dispersion, &
+                                   dftd3_init, dftd3_set_functional, &
+                                   get_atomic_number, dftd3_input, &
+                                   dftd3_calc
+  USE dftd3_qe,             ONLY : dftd3, dftd3_in, energy_dftd3
   USE xdm_module,           ONLY : energy_xdm
   USE tsvdw_module,         ONLY : EtsvdW
   !
@@ -353,14 +398,14 @@ SUBROUTINE electrons_scf ( printout, exxen )
   USE dfunct,               ONLY : newd
   USE esm,                  ONLY : do_comp_esm, esm_printpot, esm_ewald
   USE fcp_variables,        ONLY : lfcpopt, lfcpdyn
-  USE iso_c_binding,        ONLY : c_int
+  USE wrappers,             ONLY : memstat
   !
   USE plugin_variables,     ONLY : plugin_etot
   !
   IMPLICIT NONE
   !
   INTEGER, INTENT (IN) :: printout
-  REAL(DP),INTENT (IN) :: exxen    ! current estimate of the echange energy
+  REAL(DP),INTENT (IN) :: exxen    ! current estimate of the exchange energy
   !
   ! ... a few local variables
   !
@@ -373,8 +418,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
       i,            &! counter on polarization
       idum,         &! dummy counter on iterations
       iter,         &! counter on iterations
-      ios
-  INTEGER(kind=c_int) :: kilobytes
+      ios, kilobytes
   REAL(DP) :: &
       tr2_min,     &! estimated error on energy coming from diagonalization
       descf,       &! correction for variational energy
@@ -391,6 +435,12 @@ SUBROUTINE electrons_scf ( printout, exxen )
   ! ... external functions
   !
   REAL(DP), EXTERNAL :: ewald, get_clock
+  REAL(DP) :: etot_cmp_paw(nat,2,2)
+  !
+  ! auxiliary variables for grimme-d3
+  !
+  REAL(DP) :: latvecs(3,3)
+  INTEGER:: atnum(1:nat), na
   !
   iter = 0
   dr2  = 0.0_dp
@@ -420,6 +470,22 @@ SUBROUTINE electrons_scf ( printout, exxen )
      elondon = 0.d0
   END IF
   !
+  ! Grimme-D3 correction to the energy
+  !
+  IF(ldftd3) THEN
+     latvecs(:,:)=at(:,:)*alat
+     tau(:,:)=tau(:,:)*alat
+     do na=1, nat
+        atnum(na) = get_atomic_number(trim(atm(ityp(na))))
+     end do
+     call dftd3_pbc_dispersion(dftd3,tau,atnum,latvecs,energy_dftd3)
+     edftd3=energy_dftd3*2.d0
+     tau(:,:)=tau(:,:)/alat
+  ELSE
+     edftd3= 0.0
+  END IF
+  !
+  !
   call create_scf_type ( rhoin )
   !
   WRITE( stdout, 9002 )
@@ -607,7 +673,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
            CALL v_of_rho( rhoin, rho_core, rhog_core, &
                           ehart, etxc, vtxc, eth, etotefield, charge, v)
            IF (okpaw) THEN
-              CALL PAW_potential(rhoin%bec, ddd_paw, epaw)
+              CALL PAW_potential(rhoin%bec, ddd_paw, epaw,etot_cmp_paw)
               CALL PAW_symmetrize_ddd(ddd_paw)
            ENDIF
            !
@@ -636,7 +702,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
            vnew%of_r(:,:) = v%of_r(:,:) - vnew%of_r(:,:)
            !
            IF (okpaw) THEN
-              CALL PAW_potential(rho%bec, ddd_paw, epaw)
+              CALL PAW_potential(rho%bec, ddd_paw, epaw,etot_cmp_paw)
               CALL PAW_symmetrize_ddd(ddd_paw)
            ENDIF
            !
@@ -657,7 +723,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
      !
      CALL plugin_scf_energy(plugin_etot,rhoin)
      !
-     CALL plugin_scf_potential(rhoin,conv_elec,dr2)
+     CALL plugin_scf_potential(rhoin,conv_elec,dr2,vltot)
      !
      ! ... define the total local potential (external + scf)
      !
@@ -718,8 +784,9 @@ SUBROUTINE electrons_scf ( printout, exxen )
      !
      etot = eband + ( etxc - etxcc ) + ewld + ehart + deband + demet + descf
      ! for hybrid calculations, add the current estimate of exchange energy
-     etot = etot - exxen  
-     ! write(*,*) '@chk', etot 
+     ! (it will subtracted later if exx_is_active to be replaced with a better estimate)
+     etot = etot - exxen
+     hwf_energy = hwf_energy - exxen ! [LP]
      !
      IF (okpaw) etot = etot + epaw
      IF ( lda_plus_u ) etot = etot + eth
@@ -735,6 +802,13 @@ SUBROUTINE electrons_scf ( printout, exxen )
         etot = etot + elondon
         hwf_energy = hwf_energy + elondon
      END IF
+     !
+     ! grimme-d3 dispersion energy
+     IF (ldftd3) THEN
+        etot = etot + edftd3
+        hwf_energy = hwf_energy + edftd3
+     END IF
+     !
      ! calculate the xdm energy contribution with converged density
      if (lxdm .and. conv_elec) then
         exdm = energy_xdm()
@@ -751,10 +825,10 @@ SUBROUTINE electrons_scf ( printout, exxen )
         etot = etot + etotefield
         hwf_energy = hwf_energy + etotefield
      END IF
-     ! TB monopole energy
-     IF ( monopole) THEN
-        etot = etot + etotmonofield
-        hwf_energy = hwf_energy + etotmonofield
+     ! TB gate energy
+     IF ( gate) THEN
+        etot = etot + etotgatefield
+        hwf_energy = hwf_energy + etotgatefield
      END IF
      !
      IF ( lfcpopt .or. lfcpdyn ) THEN
@@ -777,7 +851,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
         !
         ! ... print out ESM potentials if desired
         !
-        IF ( do_comp_esm ) CALL esm_printpot()
+        IF ( do_comp_esm ) CALL esm_printpot( rho%of_g )
         !
         WRITE( stdout, 9110 ) iter
         !
@@ -808,7 +882,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
   ! ... exiting: write (unless disabled) the charge density to file
   ! ... (also write ldaU ns coefficients and PAW becsum)
   !
-  IF ( io_level > -1 ) CALL write_rho( rho, nspin )
+  IF ( io_level > -1 ) CALL write_scf( rho, nspin )
   !
   ! ... delete mixing info if converged, keep it if not
   !
@@ -829,7 +903,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
 9000 FORMAT(/'     total cpu time spent up to now is ',F10.1,' secs' )
 9001 FORMAT(/'     per-process dynamical memory: ',f7.1,' Mb' )
 9002 FORMAT(/'     Self-consistent Calculation' )
-9010 FORMAT(/'     iteration #',I3,'     ecut=', F9.2,' Ry',5X,'beta=',F4.2 )
+9010 FORMAT(/'     iteration #',I3,'     ecut=', F9.2,' Ry',5X,'beta=',F5.2 )
 9050 FORMAT(/'     WARNING: integrated charge=',F15.8,', expected=',F15.8 )
 9101 FORMAT(/'     End of self-consistent calculation' )
 9110 FORMAT(/'     convergence has been achieved in ',i3,' iterations' )
@@ -1097,14 +1171,26 @@ SUBROUTINE electrons_scf ( printout, exxen )
                ( eband + deband ), ehart, ( etxc - etxcc ), ewld
           !
           IF ( llondon ) WRITE ( stdout , 9074 ) elondon
+          IF ( ldftd3 )  WRITE ( stdout , 9078 ) edftd3
           IF ( lxdm )    WRITE ( stdout , 9075 ) exdm
           IF ( ts_vdw )  WRITE ( stdout , 9076 ) 2.0d0*EtsvdW
           IF ( textfor)  WRITE ( stdout , 9077 ) eext
           IF ( tefield )            WRITE( stdout, 9061 ) etotefield
-          IF ( monopole )           WRITE( stdout, 9062 ) etotmonofield ! TB
+          IF ( gate )               WRITE( stdout, 9062 ) etotgatefield ! TB
           IF ( lda_plus_u )         WRITE( stdout, 9065 ) eth
           IF ( ABS (descf) > eps8 ) WRITE( stdout, 9069 ) descf
-          IF ( okpaw )              WRITE( stdout, 9067 ) epaw
+          IF ( okpaw ) THEN
+            WRITE( stdout, 9067 ) epaw
+            ! Detailed printout of PAW energy components, if verbosity is high
+            IF(iverbosity>0)THEN
+            WRITE( stdout, 9068) SUM(etot_cmp_paw(:,1,1)), &
+                                 SUM(etot_cmp_paw(:,1,2)), &
+                                 SUM(etot_cmp_paw(:,2,1)), &
+                                 SUM(etot_cmp_paw(:,2,2)), &
+            SUM(etot_cmp_paw(:,1,1))+SUM(etot_cmp_paw(:,1,2))+ehart, &
+            SUM(etot_cmp_paw(:,2,1))+SUM(etot_cmp_paw(:,2,2))+etxc-etxcc
+            ENDIF
+          ENDIF
           !
           ! ... With Fermi-Dirac population factor, etot is the electronic
           ! ... free energy F = E - TS , demet is the -TS contribution
@@ -1160,9 +1246,15 @@ SUBROUTINE electrons_scf ( printout, exxen )
             /'     xc contribution           =',F17.8,' Ry' &
             /'     ewald contribution        =',F17.8,' Ry' )
 9061 FORMAT( '     electric field correction =',F17.8,' Ry' )
-9062 FORMAT( '     monopole field correction =',F17.8,' Ry' ) ! TB
+9062 FORMAT( '     gate field correction     =',F17.8,' Ry' ) ! TB
 9065 FORMAT( '     Hubbard energy            =',F17.8,' Ry' )
 9067 FORMAT( '     one-center paw contrib.   =',F17.8,' Ry' )
+9068 FORMAT( '      -> PAW hartree energy AE =',F17.8,' Ry' &
+            /'      -> PAW hartree energy PS =',F17.8,' Ry' &
+            /'      -> PAW xc energy AE      =',F17.8,' Ry' &
+            /'      -> PAW xc energy PS      =',F17.8,' Ry' &
+            /'      -> total E_H with PAW    =',F17.8,' Ry'& 
+            /'      -> total E_XC with PAW   =',F17.8,' Ry' )
 9069 FORMAT( '     scf correction            =',F17.8,' Ry' )
 9070 FORMAT( '     smearing contrib. (-TS)   =',F17.8,' Ry' )
 9071 FORMAT( '     Magnetic field            =',3F12.7,' Ry' )
@@ -1172,6 +1264,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
 9075 FORMAT( '     Dispersion XDM Correction =',F17.8,' Ry' )
 9076 FORMAT( '     Dispersion T-S Correction =',F17.8,' Ry' )
 9077 FORMAT( '     External forces energy    =',F17.8,' Ry' )
+9078 FORMAT( '     DFT-D3 Dispersion         =',F17.8,' Ry' )
 9080 FORMAT(/'     total energy              =',0PF17.8,' Ry' &
             /'     Harris-Foulkes estimate   =',0PF17.8,' Ry' &
             /'     estimated scf accuracy    <',0PF17.8,' Ry' )
@@ -1199,7 +1292,7 @@ FUNCTION exxenergyace ( )
   USE kinds,    ONLY : DP
   USE buffers,  ONLY : get_buffer
   USE exx,      ONLY : vexxace_gamma, vexxace_k, domat
-  USE klist,    ONLY : nks, ngk, igk_k
+  USE klist,    ONLY : nks, ngk
   USE wvfct,    ONLY : nbnd, npwx, current_k
   USE lsda_mod, ONLY : lsda, isk, current_spin
   USE io_files, ONLY : iunwfc, nwordwfc
