@@ -6,11 +6,6 @@
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
 !----------------------------------------------------------------------------
-! TB
-! included monopole related stuff, search for 'TB'
-!----------------------------------------------------------------------------
-!
-!----------------------------------------------------------------------------
 SUBROUTINE iosys()
   !-----------------------------------------------------------------------------
   !
@@ -21,7 +16,7 @@ SUBROUTINE iosys()
   !
   USE kinds,         ONLY : DP
   USE funct,         ONLY : dft_is_hybrid, dft_has_finite_size_correction, &
-                            set_finite_size_volume, get_inlc 
+                            set_finite_size_volume, get_inlc, get_dft_short
   USE funct,         ONLY: set_exx_fraction, set_screening_parameter
   USE control_flags, ONLY: adapt_thr, tr2_init, tr2_multi
   USE constants,     ONLY : autoev, eV_to_kelvin, pi, rytoev, &
@@ -70,8 +65,11 @@ SUBROUTINE iosys()
                             fcp_mu_ => fcp_mu, &
                             fcp_mass_ => fcp_mass, &
                             fcp_temperature, &
+                            fcp_relax_ => fcp_relax, &
                             fcp_relax_step_ => fcp_relax_step, &
-                            fcp_relax_crit_ => fcp_relax_crit
+                            fcp_relax_crit_ => fcp_relax_crit, &
+                            fcp_mdiis_size_ => fcp_mdiis_size, &
+                            fcp_mdiis_step_ => fcp_mdiis_step
   !
   USE extfield,      ONLY : tefield_  => tefield, &
                             dipfield_ => dipfield, &
@@ -79,34 +77,34 @@ SUBROUTINE iosys()
                             emaxpos_  => emaxpos, &
                             eopreg_   => eopreg, &
                             eamp_     => eamp, &
-  ! TB added monopole related variables
-                            zmon_     => zmon, &
-                            monopole_ => monopole, &
+  ! TB added gate related variables
+                            zgate_    => zgate, &
+                            gate_     => gate, &
                             relaxz_   => relaxz, &
-                            block_   => block, &
+                            block_    => block, &
                             block_1_   => block_1, &
                             block_2_   => block_2, &
                             block_height_   => block_height, &
                             forcefield, &
-                            forcemono
+                            forcegate
   !
   USE io_files,      ONLY : input_drho, output_drho, &
                             psfile, tmp_dir, wfc_dir, &
                             prefix_     => prefix, &
-                            pseudo_dir_ => pseudo_dir
+                            pseudo_dir_ => pseudo_dir, &
+                            check_tempdir, clean_tempdir
   !
   USE force_mod,     ONLY : lforce, lstres, force
   !
   USE fft_base, ONLY : dfftp
   USE fft_base, ONLY : dffts
   !
-  USE klist,         ONLY : lgauss, ngauss, two_fermi_energies, &
+  USE klist,         ONLY : ltetra, lgauss, ngauss, two_fermi_energies, &
                             smearing_          => smearing, &
                             degauss_           => degauss, &
                             tot_charge_        => tot_charge, &
                             tot_magnetization_ => tot_magnetization
-  !
-  USE ktetra,        ONLY : ltetra
+  USE ktetra,        ONLY : tetra_type
   USE start_k,       ONLY : init_start_k
   !
   USE ldaU,          ONLY : Hubbard_U_     => hubbard_u, &
@@ -129,15 +127,16 @@ SUBROUTINE iosys()
   !
   USE a2F,           ONLY : la2F_ => la2F
   !
-  USE exx,           ONLY : x_gamma_extrapolation_ => x_gamma_extrapolation, &
+  USE exx_base,      ONLY : x_gamma_extrapolation_ => x_gamma_extrapolation, &
                             nqx1_ => nq1, &
                             nqx2_ => nq2, &
                             nqx3_ => nq3, &
                             exxdiv_treatment_ => exxdiv_treatment, &
                             yukawa_           => yukawa, &
-                            ecutvcut_         => ecutvcut, &
-                            ecutfock_         => ecutfock
-  !
+                            ecutvcut_         => ecutvcut
+  USE exx,          ONLY:   ecutfock_         => ecutfock, &
+                            use_ace, nbndproj, local_thr 
+  USE loc_scdm,      ONLY : use_scdm, scdm_den, scdm_grd 
   !
   USE lsda_mod,      ONLY : nspin_                  => nspin, &
                             starting_magnetization_ => starting_magnetization, &
@@ -149,7 +148,10 @@ SUBROUTINE iosys()
   !
   USE extrapolation, ONLY : pot_order, wfc_order
   USE control_flags, ONLY : isolve, max_cg_iter, david, tr2, imix, gamma_only,&
-                            nmix, iverbosity, niter, &
+                            nmix, iverbosity, smallmem, niter, &
+                            io_level, ethr, lscf, lbfgs, lmd, &
+                            lbands, lconstrain, restart, twfcollect, &
+                            llondon, ldftd3, do_makov_payne, lxdm, &
                             remove_rigid_rot_ => remove_rigid_rot, &
                             diago_full_acc_   => diago_full_acc, &
                             tolp_             => tolp, &
@@ -162,13 +164,10 @@ SUBROUTINE iosys()
                             tqr_              => tqr, &
                             tq_smoothing_     => tq_smoothing, &
                             tbeta_smoothing_  => tbeta_smoothing, &
-                            io_level, ethr, lscf, lbfgs, lmd, &
-                            lbands, lconstrain, restart, twfcollect, &
-                            llondon, do_makov_payne, lxdm, &
                             ts_vdw_           => ts_vdw, &
                             lecrpa_           => lecrpa, &
-                            smallmem
-  USE control_flags, ONLY: scf_must_converge_ => scf_must_converge
+                            scf_must_converge_=> scf_must_converge
+  USE check_stop,    ONLY : max_seconds_ => max_seconds
   !
   USE wvfct,         ONLY : nbnd_ => nbnd
   USE gvecw,         ONLY : ecfixed_ => ecfixed, &
@@ -191,7 +190,7 @@ SUBROUTINE iosys()
 
   !
   USE symm_base, ONLY : no_t_rev_ => no_t_rev, nofrac, allfrac, &
-                        nosym_ => nosym, nosym_evc_=> nosym_evc
+                        nosym_ => nosym, nosym_evc_=> nosym_evc, spacegroup
   !
   USE bfgs_module,   ONLY : bfgs_ndim_        => bfgs_ndim, &
                             trust_radius_max_ => trust_radius_max, &
@@ -204,12 +203,13 @@ SUBROUTINE iosys()
                             nwan_             => nwan, &
                             print_wannier_coeff_    => print_wannier_coeff
 
+  USE Coul_cut_2D,  ONLY :  do_cutoff_2D 
+
   USE realus,                ONLY : real_space_ => real_space
 
   USE read_pseudo_mod,       ONLY : readpp
 
-  USE qmmm, ONLY : qmmm_config
-
+  USE qmmm,                  ONLY : qmmm_config
   !
   ! ... CONTROL namelist
   !
@@ -219,8 +219,8 @@ SUBROUTINE iosys()
                                pseudo_dir, disk_io, tefield, dipfield, lberry, &
                                gdir, nppstr, wf_collect,lelfield,lorbm,efield, &
                                nberrycyc, lkpoint_dir, efield_cart, lecrpa,    &
-                               vdw_table_name, memory, tqmmm,                  &
-                               efield_phase, monopole
+                               vdw_table_name, memory, max_seconds, tqmmm,     &
+                               efield_phase, gate
 
   !
   ! ... SYSTEM namelist
@@ -230,7 +230,7 @@ SUBROUTINE iosys()
                                ecutwfc, ecutrho, nr1, nr2, nr3, nr1s, nr2s, &
                                nr3s, noinv, nosym, nosym_evc, no_t_rev,     &
                                use_all_frac, force_symmorphic,              &
-                               starting_magnetization,                      &
+                               starting_charge, starting_magnetization,     &
                                occupations, degauss, smearing, nspin,       &
                                ecfixed, qcutz, q2sigma, lda_plus_U,         &
                                lda_plus_U_kind, Hubbard_U, Hubbard_J,       &
@@ -240,21 +240,23 @@ SUBROUTINE iosys()
                                x_gamma_extrapolation, nqx1, nqx2, nqx3,     &
                                exxdiv_treatment, yukawa, ecutvcut,          &
                                exx_fraction, screening_parameter, ecutfock, &
-                               gau_parameter,                               &
+                               gau_parameter, localization_thr, scdm, ace,    &
+                               scdmden, scdmgrd, n_proj,                      & 
                                edir, emaxpos, eopreg, eamp, noncolin, lambda, &
                                angle1, angle2, constrained_magnetization,     &
                                B_field, fixed_magnetization, report, lspinorb,&
                                starting_spin_angle, assume_isolated,spline_ps,&
                                vdw_corr, london, london_s6, london_rcut, london_c6, &
-                               london_rvdw, &
+                               london_rvdw, dftd3_threebody, dftd3_version,   &
                                ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,     &
                                xdm, xdm_a1, xdm_a2, lforcet,                  &
                                one_atom_occupations,                          &
                                esm_bc, esm_efield, esm_w, esm_nfit, esm_a,    &
                                lfcpopt, lfcpdyn, fcp_mu, fcp_mass, fcp_tempw, & 
-                               fcp_relax_step, fcp_relax_crit,                &
+                               fcp_relax, fcp_relax_step, fcp_relax_crit,     &
+                               fcp_mdiis_size, fcp_mdiis_step,                &
                                space_group, uniqueb, origin_choice,           &
-                               rhombohedral, zmon, relaxz, block, block_1,    &
+                               rhombohedral, zgate, relaxz, block, block_1,   &
                                block_2, block_height
   !
   ! ... ELECTRONS namelist
@@ -276,7 +278,7 @@ SUBROUTINE iosys()
                                pot_extrapolation,  wfc_extrapolation,          &
                                w_1, w_2, trust_radius_max, trust_radius_min,   &
                                trust_radius_ini, bfgs_ndim, rd_pos, sp_pos, &
-                               rd_for, rd_if_pos => if_pos, lsg
+                               rd_for, rd_if_pos, lsg
   !
   ! ... CELL namelist
   !
@@ -291,13 +293,17 @@ SUBROUTINE iosys()
   !
   ! ... CARDS
   !
-  USE input_parameters,   ONLY : k_points, xk, wk, nk1, nk2, nk3,  &
+  USE input_parameters,      ONLY : k_points, xk, wk, nk1, nk2, nk3,  &
                                  k1, k2, k3, nkstot
-  USE input_parameters, ONLY : nconstr_inp, trd_ht, rd_ht, cell_units
+  USE input_parameters,      ONLY : nconstr_inp, trd_ht, rd_ht, cell_units
   !
   USE constraints_module,    ONLY : init_constraint
   USE read_namelists_module, ONLY : read_namelists, sm_not_set
   USE london_module,         ONLY : init_london, lon_rcut, scal6, in_c6, in_rvdw
+  USE dftd3_api,             ONLY : dftd3_init, dftd3_set_params, &
+                                    dftd3_set_functional, dftd3_calc, &
+                                    dftd3_input
+  USE dftd3_qe,              ONLY : dftd3_printout, dftd3_xc, dftd3, dftd3_in
   USE xdm_module,            ONLY : init_xdm, a1i, a2i
   USE tsvdw_module,          ONLY : vdw_isolated, vdw_econv_thr
   USE us,                    ONLY : spline_ps_ => spline_ps
@@ -306,6 +312,8 @@ SUBROUTINE iosys()
   USE wyckoff,               ONLY : nattot, sup_spacegroup
   USE qexsd_module,          ONLY : qexsd_input_obj
   USE qes_types_module,      ONLY: input_type
+  !
+  USE vlocal,        ONLY : starting_charge_ => starting_charge
   ! 
   IMPLICIT NONE
   !
@@ -316,15 +324,16 @@ SUBROUTINE iosys()
      CHARACTER(LEN=*),INTENT(IN)  :: obj_tagname
      END SUBROUTINE
   END INTERFACE
-!!!!  
+  !
   CHARACTER(LEN=256), EXTERNAL :: trimcheck
+  CHARACTER(LEN=256):: dft_
+  !
   INTEGER, EXTERNAL :: read_config_from_file
   !
   INTEGER  :: ia, nt, inlc, ibrav_sg, ierr
   LOGICAL  :: exst, parallelfs
   REAL(DP) :: theta, phi, ecutwfc_pp, ecutrho_pp
   !
-  !
   ! ... various initializations of control variables
   !
   lforce    = tprnfor
@@ -388,12 +397,10 @@ SUBROUTINE iosys()
      CASE( 'langevin' )
         !
         calc        = 'ld'
-        temperature = tempw
         !
      CASE( 'langevin-smc', 'langevin+smc' )
         !
         calc        = 'ls'
-        temperature = tempw
         !
         !
      CASE DEFAULT
@@ -494,30 +501,30 @@ SUBROUTINE iosys()
   !
   ! TB
   ! IF ( tefield .and. ( .not. nosym ) ) THEN
-  IF ( tefield .and. ( .not. nosym ) .and. ( .not. monopole)) THEN
+  IF ( tefield .and. ( .not. nosym ) .and. ( .not. gate )) THEN
      nosym = .true.
      WRITE( stdout, &
             '(5x,"Presently no symmetry can be used with electric field",/)' )
   ENDIF
   !TB begin some checks on input
-  IF ( (monopole) .AND. ( .NOT. nosym )) THEN
-     WRITE( stdout,'(/,5x,"Presently symmetry can be used with monopole field",/)' )
+  IF ( (gate) .AND. ( .NOT. nosym )) THEN
+     WRITE( stdout,'(/,5x,"Presently symmetry can be used with gate field",/)' )
      WRITE( stdout,'(5x,"setting verbosity to high",/)' )
      WRITE( stdout,'(5x,"CAREFULLY CHECK ALL SYMMETRIES",/)' )
      verbosity='high'
   ENDIF
-  IF ((zmon>1.0).OR.(zmon<0.0)) &
-     CALL errore( 'iosys', 'Position of the monopole has to be between within ]0,1[' , 1 )
-  IF ( (monopole) .AND. ((tefield).OR.(dipfield)) ) THEN
+  IF ((zgate>1.0).OR.(zgate<0.0)) &
+     CALL errore( 'iosys', 'Position of the charged plate representing the gate has to be between within ]0,1[' , 1 )
+  IF ( (gate) .AND. ((tefield).OR.(dipfield)) ) THEN
      IF (edir .ne. 3) &
-        CALL errore( 'iosys','Using monopole and tefield/dipfield, edir must be 3', 1)
-     IF ((zmon>=emaxpos).AND.(zmon<=(emaxpos+eopreg))) &
-        CALL errore( 'iosys', 'Monopole between the 2 dipole planes not allowed' , 1 )
+        CALL errore( 'iosys','Using gate and tefield/dipfield, edir must be 3', 1)
+     IF ((zgate>=emaxpos).AND.(zgate<=(emaxpos+eopreg))) &
+        CALL errore( 'iosys', 'Charged plate between the 2 dipole planes not allowed' , 1 )
      IF ((block).AND.(block_1.NE.emaxpos).AND.(block_2.NE.(emaxpos+eopreg))) THEN
         WRITE( stdout,'(/,5x,"Neither block_1=emaxpos, nor block_2=emaxpos+eopreg, CHECK IF THIS IS WHAT YOU WANT",/)' )
         WRITE( stdout,'(/,5x,"eopreg is nevertheless used for the smooth increase of the barrier",/)' )
   ENDIF
-  IF ( (monopole) .AND. (block) ) THEN
+  IF ( (gate) .AND. (block) ) THEN
      IF ((block_1<0.0) .OR. (block_1>1.0) .OR. (block_2<0.0) .OR. (block_2>1.0)) &
         CALL errore( 'iosys', 'Both block_1, block_2 have to be between wihtin ]0,1[' , 1 )
      IF (block_1>=block_2) &
@@ -525,14 +532,13 @@ SUBROUTINE iosys()
      ENDIF
   ENDIF
   !TB end
-  IF ( (tefield.or.monopole) .and. tstress ) THEN !TB no stress with monopole
+  IF ( (tefield.or.gate) .and. tstress ) THEN !TB no stress with gate
      lstres = .false.
      WRITE( stdout, &
-            '(5x,"Presently stress not available with electric field and monopole",/)' )
+            '(5x,"Presently stress not available with electric field and gates",/)' )
   ENDIF
   !TB Why no E-field with SOC?
-  !IF ( tefield .and. ( nspin > 2 ) ) THEN
-  IF ( (tefield .and. ( nspin > 2 )) .and. (.not.monopole) ) THEN
+  IF ( (tefield .and. ( nspin > 2 )) .and. (.not.gate) ) THEN
      CALL errore( 'iosys', 'LSDA not available with electric field' , 1 )
   ENDIF
   !
@@ -546,11 +552,11 @@ SUBROUTINE iosys()
   tfixed_occ = .false.
   ltetra     = .false.
   lgauss     = .false.
+  ngauss     = 0
   !
   SELECT CASE( trim( occupations ) )
   CASE( 'fixed' )
      !
-     ngauss = 0
      IF ( degauss /= 0.D0 ) THEN
         CALL errore( ' iosys ', &
                    & ' fixed occupations, gauss. broadening ignored', -1 )
@@ -583,18 +589,18 @@ SUBROUTINE iosys()
      !
   CASE( 'tetrahedra' )
      !
-     ! replace "errore" with "infomsg" in the next line if you really want
-     ! to perform a calculation with forces using tetrahedra 
-     !
-     IF( lforce ) CALL errore( 'iosys', &
-        'force calculation with tetrahedra not recommanded: use smearing',1)
-     !
-     ! as above, for stress
-     !
-     IF( lstres ) CALL errore( 'iosys', &
-        'stress calculation with tetrahedra not recommanded: use smearing',1)
-     ngauss = 0
      ltetra = .true.
+     tetra_type = 0
+     !
+  CASE( 'tetrahedra_lin', 'tetrahedra-lin')
+     !
+     ltetra = .true.
+     tetra_type = 1
+     !
+  CASE('tetrahedra_opt', 'tetrahedra-opt')
+     !
+     ltetra = .true.
+     tetra_type = 2
      !
   CASE( 'from_input' )
      !
@@ -604,10 +610,16 @@ SUBROUTINE iosys()
   CASE DEFAULT
      !
      CALL errore( 'iosys','occupations ' // trim( occupations ) // &
-                & 'not implemented', 1 )
+                & ' not implemented', 1 )
      !
   END SELECT
   !
+  IF( ltetra ) THEN
+     IF( lforce ) CALL infomsg( 'iosys', &
+       'BEWARE:  force calculation with tetrahedra (not recommanded)')
+     IF( lstres ) CALL infomsg( 'iosys', &
+       'BEWARE: stress calculation with tetrahedra (not recommanded)')
+  END IF
   IF( nbnd < 1 ) &
      CALL errore( 'iosys', 'nbnd less than 1', nbnd )
   !
@@ -1004,6 +1016,11 @@ SUBROUTINE iosys()
   CASE( 'not_controlled', 'not-controlled', 'not controlled' )
      !
      control_temp = .false.
+     IF ( ion_dynamics(1:8) == 'langevin' ) THEN
+        temperature  = tempw
+     ELSE
+        temperature  = 0.0_dp
+     END IF
      !
   CASE( 'initial' )
      !
@@ -1134,8 +1151,8 @@ SUBROUTINE iosys()
   tefield_    = tefield
   dipfield_   = dipfield
   !TB start
-  monopole_   = monopole
-  zmon_    = zmon
+  gate_   = gate
+  zgate_    = zgate
   relaxz_  = relaxz
   block_   = block
   block_1_ = block_1
@@ -1193,6 +1210,7 @@ SUBROUTINE iosys()
   lda_plus_u_kind_        = lda_plus_u_kind
   la2F_                   = la2F
   nspin_                  = nspin
+  starting_charge_        = starting_charge
   starting_magnetization_ = starting_magnetization
   starting_ns             = starting_ns_eigenvalue
   U_projection            = U_projection_type
@@ -1240,41 +1258,60 @@ SUBROUTINE iosys()
     CASE( 'grimme-d2', 'Grimme-D2', 'DFT-D', 'dft-d' )
       !
       llondon= .TRUE.
+      ldftd3 = .FALSE.
       ts_vdw_= .FALSE.
       lxdm   = .FALSE.
       !
+    CASE( 'grimme-d3', 'Grimme-D3', 'DFT-D3', 'dft-d3' )
+      !
+      ldftd3 = .TRUE.
+      llondon= .FALSE.
+      ts_vdw_= .FALSE.
+      lxdm   = .FALSE.
+      !
+
     CASE( 'TS', 'ts', 'ts-vdw', 'ts-vdW', 'tkatchenko-scheffler' )
       !
       llondon= .FALSE.
+      ldftd3 = .FALSE.
       ts_vdw_= .TRUE.
       lxdm   = .FALSE.
       !
     CASE( 'XDM', 'xdm' )
        !
       llondon= .FALSE.
+      ldftd3 = .FALSE.
       ts_vdw_= .FALSE.
       lxdm   = .TRUE.
       !
     CASE DEFAULT
       !
       llondon= .FALSE.
+      ldftd3 = .FALSE.
       ts_vdw_= .FALSE.
       lxdm   = .FALSE.
       !
   END SELECT
   IF ( london ) THEN
      CALL infomsg("iosys","london is obsolete, use ""vdw_corr='grimme-d2'"" instead")
+     vdw_corr='grimme-d2'
      llondon = .TRUE.
   END IF
+  IF ( ldftd3 ) THEN
+     vdw_corr='grimme-d3'
+  ENDIF
   IF ( xdm ) THEN
      CALL infomsg("iosys","xdm is obsolete, use ""vdw_corr='xdm'"" instead")
+     vdw_corr='xdm'
      lxdm = .TRUE.
   END IF
   IF ( ts_vdw ) THEN
      CALL infomsg("iosys","ts_vdw is obsolete, use ""vdw_corr='TS'"" instead")
+     vdw_corr='TS'
      ts_vdw_ = .TRUE.
   END IF
-  IF ( llondon.AND.lxdm .OR. llondon.AND.ts_vdw_ .OR. lxdm.AND.ts_vdw_ ) &
+  IF ( llondon.AND.lxdm .OR. llondon.AND.ts_vdw_ .OR. lxdm.AND.ts_vdw_ .OR. &
+           ldftd3.AND.llondon .OR. ldftd3.AND.lxdm .OR. ldftd3.AND.ts_vdw ) &
      CALL errore("iosys","must choose a unique vdW correction!", 1)
   !
   IF ( llondon) THEN
@@ -1305,6 +1342,7 @@ SUBROUTINE iosys()
                                                    &number',1 )
      CALL sup_spacegroup(rd_pos,sp_pos,rd_for,rd_if_pos,space_group,nat,&
               uniqueb,rhombohedral,origin_choice,ibrav_sg)
+     spacegroup = space_group
      IF (ibrav==-1) THEN
         ibrav=ibrav_sg
      ELSEIF (ibrav /= ibrav_sg) THEN
@@ -1321,6 +1359,7 @@ SUBROUTINE iosys()
   do_makov_payne  = .false.
   do_comp_mt      = .false.
   do_comp_esm     = .false.
+  do_cutoff_2D    = .false.
   !
   SELECT CASE( trim( assume_isolated ) )
       !
@@ -1342,6 +1381,11 @@ SUBROUTINE iosys()
       !
       do_comp_esm    = .true.
       !
+    CASE( '2D' )
+      !
+      do_cutoff_2D   = .true.
+      !
+
   END SELECT
   !
   IF ( do_comp_mt .AND. lstres ) THEN
@@ -1390,10 +1434,11 @@ SUBROUTINE iosys()
   !
   IF ( fcp_temperature == 0.0_DP ) &
      fcp_temperature = temperature
+  fcp_relax_      = fcp_relax
   fcp_relax_step_ = fcp_relax_step
   fcp_relax_crit_ = fcp_relax_crit
-  !
-  CALL plugin_read_input()
+  fcp_mdiis_size_ = fcp_mdiis_size
+  fcp_mdiis_step_ = fcp_mdiis_step
   !
   ! ... read following cards
   !
@@ -1413,7 +1458,7 @@ SUBROUTINE iosys()
   ENDIF
   !
   IF ( tefield ) ALLOCATE( forcefield( 3, nat_ ) )
-  IF ( monopole ) ALLOCATE( forcemono( 3, nat_ ) ) !TB monopole forces
+  IF ( gate ) ALLOCATE( forcegate( 3, nat_ ) ) !TB gate forces
   !
   ! ... note that read_cards_pw no longer reads cards!
   !
@@ -1423,7 +1468,10 @@ SUBROUTINE iosys()
   !
   call cell_base_init ( ibrav, celldm, a, b, c, cosab, cosac, cosbc, &
                         trd_ht, rd_ht, cell_units )
-
+  !
+  ! ... once input variables have been stored, read optional plugin input files
+  !
+  CALL plugin_read_input("PW")
   !
   ! ... Files (for compatibility) and directories
   !     This stuff must be done before calling read_config_from_file!
@@ -1501,6 +1549,23 @@ SUBROUTINE iosys()
   !
   CALL init_dofree ( cell_dofree )
   !
+  !
+  ! ... Initialize temporary directory(-ies)
+  !
+  CALL check_tempdir ( tmp_dir, exst, parallelfs )
+  IF ( .NOT. exst .AND. restart ) THEN
+     CALL infomsg('iosys', 'restart disabled: needed files not found')
+     restart = .false.
+  ELSE IF ( .NOT. exst .AND. (lbands .OR. .NOT. lscf) ) THEN
+     CALL errore('iosys', 'bands or non-scf calculation not possible: ' // &
+                          'needed files are missing', 1)
+  ELSE IF ( exst .AND. .NOT.restart ) THEN
+     CALL clean_tempdir ( tmp_dir )
+  END IF
+  IF ( TRIM(wfc_dir) /= TRIM(tmp_dir) ) &
+     CALL check_tempdir( wfc_dir, exst, parallelfs )
+  !
+
   ! ... read pseudopotentials (also sets DFT and a few more variables)
   ! ... returns values read from PP files into ecutwfc_pp, ecutrho_pp
   !
@@ -1518,6 +1583,19 @@ SUBROUTINE iosys()
   exxdiv_treatment_ = trim(exxdiv_treatment)
   yukawa_   = yukawa
   ecutvcut_ = ecutvcut
+  use_ace   = ace
+  nbndproj  = n_proj
+  local_thr = localization_thr
+  use_scdm  = scdm
+  scdm_den = scdmden
+  scdm_grd = scdmgrd
+  IF ( local_thr > 0.0_dp .AND. .NOT. gamma_only) &
+     CALL errore('input','localization for k-points not implemented',1)
+  IF ( local_thr > 0.0_dp .AND. .NOT. use_ace ) &
+     CALL errore('input','localization without ACE not implemented',1)
+! IF ( local_thr > 0.0_dp .AND. nspin > 1 ) &
+!    CALL errore('input','spin-polarized localization not implemented',1)
+  IF ( use_scdm ) CALL errore('input','use_scdm not yet implemented',1)
   !
   IF(ecutfock <= 0.0_DP) THEN
      ! default case
@@ -1529,8 +1607,8 @@ SUBROUTINE iosys()
   END IF
   IF ( lstres .AND. dft_is_hybrid() .AND. npool > 1 )  CALL errore('iosys', &
          'stress for hybrid functionals not available with pools', 1)
-  IF ( lmovecell.AND. dft_is_hybrid() ) CALL errore('iosys',&
-         'Variable cell and hybrid XC not tested',1)
+  IF ( lmovecell.AND. dft_is_hybrid() ) CALL infomsg('iosys',&
+         'Variable cell and hybrid XC little tested')
   !
   ! ... must be done AFTER dft is read from PP files and initialized
   ! ... or else the two following parameters will be overwritten
@@ -1557,7 +1635,7 @@ SUBROUTINE iosys()
      ! The next two lines have been moved before the call to read_config_from_file:
      !      at_old    = at
      !      omega_old = omega
-     IF ( cell_factor_ <= 0.D0 ) cell_factor_ = 1.2D0
+     IF ( cell_factor_ <= 0.0_dp ) cell_factor_ = 2.0_dp
      !
      IF ( cmass <= 0.D0 ) &
         CALL errore( 'iosys', &
@@ -1569,9 +1647,22 @@ SUBROUTINE iosys()
      !
   ENDIF
   !
-  ! ... allocate arrays for dispersion correction
+  ! ... allocate arrays for DFT-D2 dispersion correction
   !
   IF ( llondon) CALL init_london ( )
+  !
+  ! Setting DFT-D3 functional dependent parameters
+  !
+  IF ( ldftd3)  THEN
+      if (dftd3_version==2) dftd3_threebody=.false.
+      dftd3_in%threebody = dftd3_threebody
+      CALL dftd3_init(dftd3, dftd3_in)
+      CALL dftd3_printout(dftd3, dftd3_in)
+      dft_ = get_dft_short( )
+      dft_ = dftd3_xc ( dft_ )
+      CALL dftd3_set_functional(dftd3, func=dft_, version=dftd3_version,tz=.false.)
+  END IF
+  !
   IF ( lxdm) CALL init_xdm ( )
   !
   ! ... variables for constrained dynamics are set here
@@ -1589,21 +1680,8 @@ SUBROUTINE iosys()
   CALL pw_init_qexsd_input(qexsd_input_obj, obj_tagname="input")
   CALL deallocate_input_parameters ()  
   !
-  ! ... Initialize temporary directory(-ies)
+  max_seconds_ = max_seconds
   !
-  CALL check_tempdir ( tmp_dir, exst, parallelfs )
-  IF ( .NOT. exst .AND. restart ) THEN
-     CALL infomsg('iosys', 'restart disabled: needed files not found')
-     restart = .false.
-  ELSE IF ( .NOT. exst .AND. (lbands .OR. .NOT. lscf) ) THEN
-     CALL errore('iosys', 'bands or non-scf calculation not possible: ' // &
-                          'needed files are missing', 1)
-  ELSE IF ( exst .AND. .NOT.restart ) THEN
-     CALL clean_tempdir ( tmp_dir )
-  END IF
-  IF ( TRIM(wfc_dir) /= TRIM(tmp_dir) ) &
-     CALL check_tempdir( wfc_dir, exst, parallelfs )
-
   RETURN
   !
 END SUBROUTINE iosys
@@ -1655,14 +1733,14 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
   !
   USE kinds,              ONLY : DP
   USE input_parameters,   ONLY : atom_label, atom_pfile, atom_mass, taspc, &
-                                 tapos, rd_pos, atomic_positions, if_pos,  &
+                                 tapos, rd_pos, atomic_positions, rd_if_pos,  &
                                  sp_pos, f_inp, rd_for, tavel, sp_vel, rd_vel, &
                                  lsg
   USE dynamics_module,    ONLY : vel
   USE cell_base,          ONLY : at, ibrav
   USE ions_base,          ONLY : nat, ntyp => nsp, ityp, tau, atm, extfor
   USE fixed_occ,          ONLY : tfixed_occ, f_inp_ => f_inp
-  USE ions_base,          ONLY : if_pos_ =>  if_pos, amass, fixatom
+  USE ions_base,          ONLY : if_pos, amass, fixatom
   USE control_flags,      ONLY : textfor, tv0rd
   USE wyckoff,            ONLY : nattot, tautot, ityptot, extfortot, &
                                  if_postot, clean_spacegroup
@@ -1704,7 +1782,7 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
      tau(:,:)=tautot(:,:)
      ityp(:) = ityptot(:)
      extfor(:,:) = extfortot(:,:)
-     if_pos_(:,:) = if_postot(:,:)
+     if_pos(:,:) = if_postot(:,:)
      CALL clean_spacegroup()
   ELSE 
      DO ia = 1, nat
@@ -1712,7 +1790,7 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
         tau(:,ia) = rd_pos(:,ia)
         ityp(ia)  = sp_pos(ia)
         extfor(:,ia) = rd_for(:,ia)
-        if_pos_(:,ia) = if_pos(:,ia)
+        if_pos(:,ia) = rd_if_pos(:,ia)
         !
      ENDDO
   ENDIF
@@ -1734,7 +1812,7 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
   ! ... if_pos whose value is 0 when the coordinate is to be kept fixed, 1
   ! ... otherwise. 
   !
-  fixatom = COUNT( if_pos_(1,:)==0 .AND. if_pos_(2,:)==0 .AND. if_pos_(3,:)==0 )
+  fixatom = COUNT( if_pos(1,:)==0 .AND. if_pos(2,:)==0 .AND. if_pos(3,:)==0 )
   !
   tau_format = trim( atomic_positions )
   !
@@ -1796,72 +1874,3 @@ SUBROUTINE convert_tau (tau_format, nat_, tau)
   END SELECT
   !
 END SUBROUTINE convert_tau
-!-----------------------------------------------------------------------
-SUBROUTINE check_tempdir ( tmp_dir, exst, pfs )
-  !-----------------------------------------------------------------------
-  !
-  ! ... Verify if tmp_dir exists, creates it if not
-  ! ... On output:
-  ! ...    exst= .t. if tmp_dir exists
-  ! ...    pfs = .t. if tmp_dir visible from all procs of an image
-  !
-  USE wrappers,      ONLY : f_mkdir_safe
-  USE io_global,     ONLY : ionode, ionode_id
-  USE mp_images,     ONLY : intra_image_comm, nproc_image, me_image
-  USE mp,            ONLY : mp_barrier, mp_bcast, mp_sum
-  !
-  IMPLICIT NONE
-  !
-  CHARACTER(len=*), INTENT(in) :: tmp_dir
-  LOGICAL, INTENT(out)         :: exst, pfs
-  !
-  INTEGER             :: ios, image, proc, nofi
-  CHARACTER (len=256) :: file_path, filename
-  CHARACTER(len=6), EXTERNAL :: int_to_char
-  !
-  ! ... create tmp_dir on ionode
-  ! ... f_mkdir_safe returns -1 if tmp_dir already exists
-  ! ...                       0 if         created
-  ! ...                       1 if         cannot be created
-  !
-  IF ( ionode ) ios = f_mkdir_safe( TRIM(tmp_dir) )
-  CALL mp_bcast ( ios, ionode_id, intra_image_comm )
-  exst = ( ios == -1 )
-  IF ( ios > 0 ) CALL errore ('check_tempdir','tmp_dir cannot be opened',1)
-  !
-  ! ... let us check now if tmp_dir is visible on all nodes
-  ! ... if not, a local tmp_dir is created on each node
-  !
-  ios = f_mkdir_safe( TRIM(tmp_dir) )
-  CALL mp_sum ( ios, intra_image_comm )
-  pfs = ( ios == -nproc_image ) ! actually this is true only if .not.exst 
-  !
-  RETURN
-  !
-END SUBROUTINE check_tempdir
-!
-!-----------------------------------------------------------------------
-SUBROUTINE clean_tempdir( tmp_dir )
-  !-----------------------------------------------------------------------
-  !
-  USE io_files,         ONLY : prefix, delete_if_present
-  USE io_global,        ONLY : ionode
-  !
-  IMPLICIT NONE
-  !
-  CHARACTER(len=*), INTENT(in) :: tmp_dir
-  !
-  CHARACTER (len=256) :: file_path, filename
-  !
-  ! ... remove temporary files from tmp_dir ( only by the master node )
-  !
-  file_path = trim( tmp_dir ) // trim( prefix )
-  IF ( ionode ) THEN
-     CALL delete_if_present( trim( file_path ) // '.update' )
-     CALL delete_if_present( trim( file_path ) // '.md' )
-     CALL delete_if_present( trim( file_path ) // '.bfgs' )
-  ENDIF
-  !
-  RETURN
-  !
-END SUBROUTINE clean_tempdir
