@@ -11,7 +11,7 @@
 ! This macro force the normalization of betamix matrix, usually not necessary
 !#define __NORMALIZE_BETAMIX
 !
-#if defined(__GFORTRAN)
+#if defined(__GFORTRAN__)
 #if (__GNUC__<4) || ((__GNUC__==4) && (__GNUC_MINOR__<8))
 #define __GFORTRAN_HACK
 #endif
@@ -326,7 +326,7 @@ SUBROUTINE approx_screening( drho )
   USE kinds,         ONLY : DP
   USE constants,     ONLY : e2, pi, fpi
   USE cell_base,     ONLY : omega, tpiba2
-  USE gvect,         ONLY : gg, ngm, nl, nlm
+  USE gvect,         ONLY : gg, ngm
   USE klist,         ONLY : nelec
   USE lsda_mod,      ONLY : nspin
   USE control_flags, ONLY : ngm0
@@ -375,8 +375,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
   USE kinds,                ONLY : DP
   USE constants,            ONLY : e2, pi, tpi, fpi, eps8, eps32
   USE cell_base,            ONLY : omega, tpiba2
-  USE gvecs,                ONLY : nls, nlsm
-  USE gvect,                ONLY : gg, ngm, nl, nlm
+  USE gvect,                ONLY : gg, ngm
   USE wavefunctions_module, ONLY : psic
   USE klist,                ONLY : nelec
   USE lsda_mod,             ONLY : nspin
@@ -397,7 +396,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
   INTEGER :: &
     iwork(mmx), i, j, m, info, is
   REAL(DP) :: &
-    rs, avg_rsm1, target, dr2_best
+    avg_rsm1, target, dr2_best
   REAL(DP) :: &
     aa(mmx,mmx), invaa(mmx,mmx), bb(mmx), work(mmx), vec(mmx), agg0
   COMPLEX(DP), ALLOCATABLE :: &
@@ -416,6 +415,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
   !
   IF ( nspin == 2 ) THEN
      !
+     !$omp parallel do private(rrho,rmag)
      DO ig = 1, ngm0
         !
         rrho = drho%of_g(ig,1) + drho%of_g(ig,2)
@@ -425,6 +425,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
         drho%of_g(ig,2) = rmag
         !
      END DO
+     !$omp end parallel do
      !
   END IF
   !
@@ -436,50 +437,54 @@ SUBROUTINE approx_screening2( drho, rhobest )
   ALLOCATE( v( ngm0, mmx ), &
             w( ngm0, mmx ), dv( ngm0 ), vbest( ngm0 ), wbest( ngm0 ) )
   !
-  v(:,:)   = ZERO
-  w(:,:)   = ZERO
-  dv(:)    = ZERO
-  vbest(:) = ZERO
-  wbest(:) = ZERO
+  !$omp parallel
+     !
+     CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
+     IF ( nspin == 2 ) THEN
+        !$omp do
+        DO ig = 1, ngm0
+           psic(dffts%nl(ig)) = ( rhobest%of_g(ig,1) + rhobest%of_g(ig,2) )
+        ENDDO
+        !$omp end do nowait
+     ELSE
+        !$omp do
+        DO ig = 1, ngm0
+           psic(dffts%nl(ig)) = rhobest%of_g(ig,1)
+        ENDDO
+        !$omp end do nowait
+     END IF
+     !
+     IF ( gamma_only ) THEN
+        !$omp do
+        DO ig = 1, ngm0
+           psic(dffts%nlm(ig)) = CONJG( psic(dffts%nl(ig)) )
+        ENDDO
+        !$omp end do nowait
+     ENDIF
+     !
+  !$omp end parallel
   !
   ! ... calculate alpha from density
   !
-  psic(:) = ZERO
-  !
-  IF ( nspin == 2 ) THEN
-     !
-     psic(nls(:ngm0)) = ( rhobest%of_g(:ngm0,1) + rhobest%of_g(:ngm0,2) )
-     !
-  ELSE
-     !
-     psic(nls(:ngm0)) = rhobest%of_g(:ngm0,1)
-     !
-  END IF
-  !
-  IF ( gamma_only ) psic(nlsm(:ngm0)) = CONJG( psic(nls(:ngm0)) )
-  !
-  CALL invfft ('Smooth', psic, dffts)
-  !
-  alpha(:) = REAL( psic(1:dffts%nnr) )
+  CALL invfft ('Rho', psic, dffts)
   !
   avg_rsm1 = 0.D0
   !
+  !$omp parallel do reduction(+:avg_rsm1)
   DO ir = 1, dffts%nnr
-     !
-     alpha(ir) = ABS( alpha(ir) )
+     alpha(ir) = ABS( REAL( psic(ir) ) )
      !
      IF ( alpha(ir) > eps32 ) THEN
         !
-        rs        = ( 3.D0 / fpi / alpha(ir) )**one_third
-        avg_rsm1  = avg_rsm1 + 1.D0 / rs
-        alpha(ir) = rs
+        alpha(ir) = ( 3.D0 / fpi / alpha(ir) )**one_third
+        avg_rsm1  = avg_rsm1 + 1.D0 / alpha(ir)
         !
      END IF   
      !
+     alpha(ir) = 3.D0 * ( tpi / 3.D0 )**( 5.D0 / 3.D0 ) * alpha(ir)
+     !
   END DO
-  !
-  alpha = 3.D0 * ( tpi / 3.D0 )**( 5.D0 / 3.D0 ) * alpha
-  rs       = ( 3.D0 * omega / fpi / nelec )**one_third
+  !$omp end parallel do
   !
   CALL mp_sum( avg_rsm1 , intra_bgrp_comm )
   avg_rsm1 = ( dffts%nr1*dffts%nr2*dffts%nr3 ) / avg_rsm1
@@ -487,20 +492,39 @@ SUBROUTINE approx_screening2( drho, rhobest )
   !
   ! ... calculate deltaV and the first correction vector
   !
-  psic(:) = ZERO
+  !$omp parallel
+     CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
+     !$omp do
+     DO ig = 1, ngm0
+        psic(dffts%nl(ig)) = drho%of_g(ig,1)
+     ENDDO
+     !$omp end do nowait
+     !
+     IF ( gamma_only ) THEN
+        !$omp do
+        DO ig = 1, ngm0
+           psic(dffts%nlm(ig)) = CONJG( psic(dffts%nl(ig)) )
+        ENDDO
+        !$omp end do nowait
+     ENDIF
+  !$omp end parallel
   !
-  psic(nls(:ngm0)) = drho%of_g(:ngm0,1)
+  CALL invfft ('Rho', psic, dffts)
   !
-  IF ( gamma_only ) psic(nlsm(:ngm0)) = CONJG( psic(nls(:ngm0)) )
+  !$omp parallel do
+  DO ir = 1, dffts%nnr
+     psic(ir) = psic(ir) * alpha(ir)
+  ENDDO
+  !$omp end parallel do
   !
-  CALL invfft ('Smooth', psic, dffts)
+  CALL fwfft ('Rho', psic, dffts)
   !
-  psic(:dffts%nnr) = psic(:dffts%nnr) * alpha(:)
-  !
-  CALL fwfft ('Smooth', psic, dffts)
-  !
-  dv(:) = psic(nls(:ngm0)) * gg(:ngm0) * tpiba2
-  v(:,1)= psic(nls(:ngm0)) * gg(:ngm0) / ( gg(:ngm0) + agg0 )
+  !$omp parallel do
+  DO ig = 1, ngm0
+     dv(ig) = psic(dffts%nl(ig)) * gg(ig) * tpiba2
+     v(ig,1)= psic(dffts%nl(ig)) * gg(ig) / ( gg(ig) + agg0 )
+  ENDDO
+  !$omp end parallel do
   !
   m       = 1
   aa(:,:) = 0.D0
@@ -510,21 +534,41 @@ SUBROUTINE approx_screening2( drho, rhobest )
      !
      ! ... generate the vector w
      !     
-     w(:,m) = fpi * e2 * v(:,m)
+     !$omp parallel
+        CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
+        !$omp do
+        DO ig = 1, ngm0
+           !
+           w(ig,m) = fpi * e2 * v(ig,m)
+           !
+           psic(dffts%nl(ig)) = v(ig,m)
+        ENDDO
+        !$omp end do nowait
+        !
+        IF ( gamma_only ) THEN
+           !$omp do
+           DO ig = 1, ngm0
+              psic(dffts%nlm(ig)) = CONJG( psic(dffts%nl(ig)) )
+           ENDDO
+           !$omp end do nowait
+        ENDIF
+     !$omp end parallel
      !
-     psic(:) = ZERO
+     CALL invfft ('Rho', psic, dffts)
      !
-     psic(nls(:ngm0)) = v(:,m)
+     !$omp parallel do
+     DO ir = 1, dffts%nnr
+        psic(ir) = psic(ir) * alpha(ir)
+     ENDDO
+     !$omp end parallel do
      !
-     IF ( gamma_only ) psic(nlsm(:ngm0)) = CONJG( psic(nls(:ngm0)) )
+     CALL fwfft ('Rho', psic, dffts)
      !
-     CALL invfft ('Smooth', psic, dffts)
-     !
-     psic(:dffts%nnr) = psic(:dffts%nnr) * alpha(:)
-     !
-     CALL fwfft ('Smooth', psic, dffts)
-     !
-     w(:,m) = w(:,m) + gg(:ngm0) * tpiba2 * psic(nls(:ngm0))
+     !$omp parallel do
+     DO ig = 1, ngm0
+        w(ig,m) = w(ig,m) + gg(ig) * tpiba2 * psic(dffts%nl(ig))
+     ENDDO
+     !$omp end parallel do
      !
      ! ... build the linear system
      !
@@ -552,15 +596,23 @@ SUBROUTINE approx_screening2( drho, rhobest )
      !
      FORALL( i = 1:m ) vec(i) = SUM( invaa(i,:)*bb(:) )
      !
-     vbest(:) = ZERO
-     wbest(:) = dv(:)
-     !
-     DO i = 1, m
+     !$omp parallel
+        !$omp do
+        DO ig = 1, ngm0
+           vbest(ig) = ZERO
+           wbest(ig) = dv(ig)
+        ENDDO
+        !$omp end do nowait
         !
-        vbest = vbest + vec(i) * v(:,i)
-        wbest = wbest - vec(i) * w(:,i)
-        !
-     END DO
+        DO i = 1, m
+           !$omp do
+           DO ig = 1, ngm0
+              vbest(ig) = vbest(ig) + vec(i) * v(ig,i)
+              wbest(ig) = wbest(ig) - vec(i) * w(ig,i)
+           ENDDO
+           !$omp end do nowait
+        END DO
+     !$omp end parallel
      !
      dr2_best = local_tf_ddot( wbest, wbest, ngm0 )
      !
@@ -568,21 +620,29 @@ SUBROUTINE approx_screening2( drho, rhobest )
      !
      IF ( dr2_best < target ) THEN
         !
-        drho%of_g(:ngm0,1) = vbest(:)
-        !
-        IF ( nspin == 2 ) THEN
-           !
+        !$omp parallel private(rrho,rmag)
+           !$omp do
            DO ig = 1, ngm0
-              !
-              rrho = drho%of_g(ig,1)
-              rmag = drho%of_g(ig,2)
-              !
-              drho%of_g(ig,1) = 0.5D0 * ( rrho + rmag )
-              drho%of_g(ig,2) = 0.5D0 * ( rrho - rmag )
-              !
-           END DO
+              drho%of_g(ig,1) = vbest(ig)
+           ENDDO
+           !$omp end do nowait
            !
-        END IF
+           IF ( nspin == 2 ) THEN
+              !
+              !$omp do
+              DO ig = 1, ngm0
+                 !
+                 rrho = drho%of_g(ig,1)
+                 rmag = drho%of_g(ig,2)
+                 !
+                 drho%of_g(ig,1) = 0.5D0 * ( rrho + rmag )
+                 drho%of_g(ig,2) = 0.5D0 * ( rrho - rmag )
+                 !
+              END DO
+              !$omp end do nowait
+              !
+           END IF
+        !$omp end parallel
         !
         DEALLOCATE( alpha, v, w, dv, vbest, wbest )
         !
@@ -592,7 +652,11 @@ SUBROUTINE approx_screening2( drho, rhobest )
         !
         m = 1
         !
-        v(:,m)  = vbest(:)
+        !$omp parallel do
+        DO ig = 1, ngm0
+           v(ig,m)  = vbest(ig)
+        ENDDO
+        !$omp end parallel do
         aa(:,:) = 0.D0
         bb(:)   = 0.D0
         !
@@ -602,7 +666,11 @@ SUBROUTINE approx_screening2( drho, rhobest )
      !
      m = m + 1
      !
-     v(:,m) = wbest(:) / ( gg(:ngm0) + agg0 )
+     !$omp parallel do
+     DO ig = 1, ngm0
+        v(ig,m) = wbest(ig) / ( gg(ig) + agg0 )
+     ENDDO
+     !$omp end parallel do
      !
   END DO repeat_loop
   !
