@@ -33,11 +33,10 @@ SUBROUTINE setup()
   ! ...    electric-field, LDA+U calculations, and for parallelism
   !
   USE kinds,              ONLY : DP
-  USE constants,          ONLY : eps8, rytoev, fpi
+  USE constants,          ONLY : eps8, rytoev, fpi, pi, degspin
   USE parameters,         ONLY : npk
   USE io_global,          ONLY : stdout
   USE io_files,           ONLY : tmp_dir, prefix, xmlpun, delete_if_present
-  USE constants,          ONLY : pi, degspin
   USE cell_base,          ONLY : at, bg, alat, tpiba, tpiba2, ibrav, omega
   USE ions_base,          ONLY : nat, tau, ntyp => nsp, ityp, zv
   USE basis,              ONLY : starting_pot, natomwfc
@@ -49,7 +48,7 @@ SUBROUTINE setup()
   USE fft_base,           ONLY : smap
   USE gvecs,              ONLY : doublegrid, gcutms, dual
   USE klist,              ONLY : xk, wk, nks, nelec, degauss, lgauss, &
-                                 lxkcry, nkstot, &
+                                 ltetra, lxkcry, nkstot, &
                                  nelup, neldw, two_fermi_energies, &
                                  tot_charge, tot_magnetization
   USE lsda_mod,           ONLY : lsda, nspin, current_spin, isk, &
@@ -58,10 +57,11 @@ SUBROUTINE setup()
   USE electrons_base,     ONLY : set_nelup_neldw
   USE start_k,            ONLY : nks_start, xk_start, wk_start, &
                                  nk1, nk2, nk3, k1, k2, k3
-  USE ktetra,             ONLY : tetra, ntetra, ltetra
+  USE ktetra,             ONLY : tetra_type, opt_tetra_init, tetra_init
   USE symm_base,          ONLY : s, t_rev, irt, nrot, nsym, invsym, nosym, &
                                  d1,d2,d3, time_reversal, sname, set_sym_bl, &
-                                 find_sym, inverse_s, no_t_rev, allfrac
+                                 find_sym, inverse_s, no_t_rev &
+                                 , allfrac, remove_sym
   USE wvfct,              ONLY : nbnd, nbndx
   USE control_flags,      ONLY : tr2, ethr, lscf, lmd, david, lecrpa,  &
                                  isolve, niter, noinv, ts_vdw, &
@@ -76,23 +76,24 @@ SUBROUTINE setup()
   USE fixed_occ,          ONLY : f_inp, tfixed_occ, one_atom_occupations
   USE funct,              ONLY : set_dft_from_name
   USE mp_pools,           ONLY : kunit
-  USE mp_bands,           ONLY : intra_bgrp_comm
+  USE mp_bands,           ONLY : intra_bgrp_comm, nyfft
   USE spin_orb,           ONLY : lspinorb, domag
   USE noncollin_module,   ONLY : noncolin, npol, m_loc, i_cons, &
                                  angle1, angle2, bfield, ux, nspin_lsda, &
                                  nspin_gga, nspin_mag
-!
-! 
-  USE pw_restart_new,     ONLY : pw_readschema_file, init_vars_from_schema 
-  USE qes_libs_module,    ONLY : qes_reset_output, qes_reset_input, qes_reset_parallel_info, qes_reset_general_info
-  USE qes_types_module,   ONLY : output_type, input_type, parallel_info_type, general_info_type 
-!
+#if defined(__OLDXML) 
   USE pw_restart,         ONLY : pw_readfile
-!
-  USE exx,                ONLY : ecutfock, exx_grid_init, exx_div_check
+#else
+  USE pw_restart_new,     ONLY : pw_readschema_file, init_vars_from_schema 
+  USE qes_libs_module,    ONLY : qes_reset_output, qes_reset_parallel_info, qes_reset_general_info
+  USE qes_types_module,   ONLY : output_type, parallel_info_type, general_info_type 
+#endif
+  USE exx,                ONLY : ecutfock, nbndproj
+  USE exx_base,           ONLY : exx_grid_init, exx_mp_init, exx_div_check
   USE funct,              ONLY : dft_is_meta, dft_is_hybrid, dft_is_gradient
   USE paw_variables,      ONLY : okpaw
   USE fcp_variables,      ONLY : lfcpopt, lfcpdyn
+  USE extfield,           ONLY : gate
   !
   IMPLICIT NONE
   !
@@ -101,14 +102,13 @@ SUBROUTINE setup()
   REAL(DP) :: iocc, ionic_charge, one
   !
   LOGICAL, EXTERNAL  :: check_para_diag
-!
-#if defined(__XSD)
-  TYPE(output_type),ALLOCATABLE             :: output_obj 
-  TYPE(input_type),ALLOCATABLE              :: input_obj
-  TYPE(parallel_info_type),ALLOCATABLE      :: parinfo_obj
-  TYPE(general_info_type),ALLOCATABLE       :: geninfo_obj
+  !
+#if !defined(__OLDXML)
+  TYPE(output_type)                         :: output_obj 
+  TYPE(parallel_info_type)                  :: parinfo_obj
+  TYPE(general_info_type)                   :: geninfo_obj
 #endif
-!  
+  !  
 #if defined(__MPI)
   LOGICAL :: lpara = .true.
 #else
@@ -141,7 +141,7 @@ SUBROUTINE setup()
                          'hybrid XC not allowed in non-scf calculations', 1 )
      IF ( ANY (upf(1:ntyp)%nlcc) ) CALL infomsg( 'setup ', 'BEWARE:' // &
                & ' nonlinear core correction is not consistent with hybrid XC')
-     IF (okpaw) CALL errore('setup','PAW and hybrid XC not tested',1)
+     !IF (okpaw) CALL errore('setup','PAW and hybrid XC not tested',1)
      IF (okvan) THEN
         IF (ecutfock /= 4*ecutwfc) CALL infomsg &
            ('setup','Warning: US/PAW use ecutfock=4*ecutwfc, ecutfock ignored')
@@ -168,32 +168,22 @@ SUBROUTINE setup()
   !
   nelec = ionic_charge - tot_charge
   !
-#if defined (__XSD)
+#if defined (__OLDXML)
+  IF ( (lfcpopt .OR. lfcpdyn) .AND. restart ) THEN
+     CALL pw_readfile( 'ef', ierr )
+     tot_charge = ionic_charge - nelec
+  END IF
+  !
+#else 
   IF ( lbands .OR. ( (lfcpopt .OR. lfcpdyn ) .AND. restart )) THEN 
-     ALLOCATE ( output_obj, input_obj, parinfo_obj, geninfo_obj )
-     CALL pw_readschema_file( ierr , output_obj, input_obj, parinfo_obj, geninfo_obj )
+     CALL pw_readschema_file( ierr , output_obj, parinfo_obj, geninfo_obj )
   END IF
   !
   ! 
-  IF (lfcpopt .AND. restart ) THEN  
-     CALL init_vars_from_schema( 'fcpopt', ierr,  output_obj, input_obj, parinfo_obj, geninfo_obj)
+  IF ( (lfcpopt .OR. lfcpdyn) .AND. restart ) THEN  
+     CALL init_vars_from_schema( 'ef', ierr,  output_obj, parinfo_obj, geninfo_obj)
      tot_charge = ionic_charge - nelec
   END IF 
-  IF (lfcpdyn .AND. restart ) THEN    
-     CALL init_vars_from_schema( 'fcpdyn', ierr,  output_obj, input_obj, parinfo_obj, geninfo_obj ) 
-     tot_charge = ionic_charge - nelec 
-  END IF
-#else 
-  IF ( lfcpopt .AND. restart ) THEN
-     CALL pw_readfile( 'fcpopt', ierr )
-     tot_charge = ionic_charge - nelec
-  END IF
-  !
-  IF ( lfcpdyn .AND. restart ) THEN
-
-     CALL pw_readfile( 'fcpdyn', ierr )
-     tot_charge = ionic_charge - nelec
-  END IF
 #endif
   !
   ! ... magnetism-related quantities
@@ -414,11 +404,7 @@ SUBROUTINE setup()
   nbndx = nbnd
   IF ( isolve == 0 ) nbndx = david * nbnd
   !
-#if defined(__MPI)
   use_para_diag = check_para_diag( nbnd )
-#else
-  use_para_diag = .FALSE.
-#endif
   !
   ! ... Set the units in real and reciprocal space
   !
@@ -427,9 +413,10 @@ SUBROUTINE setup()
   !
   ! ... Compute the cut-off of the G vectors
   !
-  doublegrid = ( dual > 4.D0 )
-  IF ( doublegrid .AND. (.NOT.okvan .AND. .not.okpaw) ) &
-     CALL infomsg ( 'setup', 'no reason to have ecutrho>4*ecutwfc' )
+  doublegrid = ( dual > 4.0_dp + eps8 )
+  IF ( doublegrid .AND. ( .NOT.okvan .AND. .NOT.okpaw .AND. &
+                          .NOT. ANY (upf(1:ntyp)%nlcc)      ) ) &
+       CALL infomsg ( 'setup', 'no reason to have ecutrho>4*ecutwfc' )
   gcutm = dual * ecutwfc / tpiba2
   gcutw = ecutwfc / tpiba2
   !
@@ -459,8 +446,8 @@ SUBROUTINE setup()
      dffts%nr2 = dfftp%nr2
      dffts%nr3 = dfftp%nr3
   END IF
-  CALL fft_type_allocate ( dfftp, at, bg, gcutm, intra_bgrp_comm )
-  CALL fft_type_allocate ( dffts, at, bg, gcutms, intra_bgrp_comm)
+  CALL fft_type_allocate ( dfftp, at, bg, gcutm, intra_bgrp_comm, nyfft=nyfft )
+  CALL fft_type_allocate ( dffts, at, bg, gcutms, intra_bgrp_comm, nyfft=nyfft)
   !
   !  ... generate transformation matrices for the crystal point group
   !  ... First we generate all the symmetry matrices of the Bravais lattice
@@ -551,8 +538,9 @@ SUBROUTINE setup()
      !
      ! ... eliminate rotations that are not symmetry operations
      !
-     CALL find_sym ( nat, tau, ityp, dfftp%nr1, dfftp%nr2, dfftp%nr3, &
-                  magnetic_sym, m_loc )
+     CALL find_sym ( nat, tau, ityp, magnetic_sym, m_loc, gate )
+     !
+     IF ( .NOT. allfrac ) CALL remove_sym ( dfftp%nr1, dfftp%nr2, dfftp%nr3 )
      !
   END IF
   !
@@ -576,41 +564,39 @@ SUBROUTINE setup()
            .AND. .NOT. ( calc == 'mm' .OR. calc == 'nm' ) ) &
        CALL infomsg( 'setup', 'Dynamics, you should have no symmetries' )
   !
-  ntetra = 0
-  !
   IF ( lbands ) THEN
      !
      ! ... if calculating bands, we read the Fermi energy
      !
-#if defined (__XSD)
-     CALL init_vars_from_schema( 'ef',   ierr , output_obj, input_obj,parinfo_obj, geninfo_obj)
-#else
+#if defined (__OLDXML)
      CALL pw_readfile( 'reset', ierr )
      CALL pw_readfile( 'ef',   ierr )
+#else
+     CALL init_vars_from_schema( 'ef',   ierr , output_obj, parinfo_obj, geninfo_obj)
 #endif 
      CALL errore( 'setup ', 'problem reading ef from file ' // &
              & TRIM( tmp_dir ) // TRIM( prefix ) // '.save', ierr )
-
      !
   ELSE IF ( ltetra ) THEN
      !
      ! ... Calculate quantities used in tetrahedra method
      !
-     ntetra = 6 * nk1 * nk2 * nk3
+     IF (nks_start /= 0) CALL errore( 'setup ', 'tetrahedra need automatic k-point grid',1)
      !
-     ALLOCATE( tetra( 4, ntetra ) )
-     !
-     CALL tetrahedra( nsym, s, time_reversal, t_rev, at, bg, npk, k1, k2, k3, &
-          nk1, nk2, nk3, nkstot, xk, wk, ntetra, tetra )
+     IF (tetra_type == 0) then
+        CALL tetra_init( nsym, s, time_reversal, t_rev, at, bg, npk, k1,k2,k3, &
+             nk1, nk2, nk3, nkstot, xk )
+     ELSE 
+        CALL opt_tetra_init(nsym, s, time_reversal, t_rev, at, bg, npk, &
+             k1, k2, k3, nk1, nk2, nk3, nkstot, xk, 1)
+     END IF
      !
   END IF
-#if defined(__XSD) 
+#if !defined(__OLDXML) 
   IF ( lbands .OR. ( (lfcpopt .OR. lfcpdyn ) .AND. restart ) ) THEN 
      CALL qes_reset_output ( output_obj ) 
-     CALL qes_reset_input ( input_obj ) 
      CALL qes_reset_parallel_info ( parinfo_obj ) 
      CALL qes_reset_general_info ( geninfo_obj ) 
-     DEALLOCATE ( output_obj, input_obj, parinfo_obj, geninfo_obj ) 
   END IF 
 #endif
   !
@@ -654,6 +640,7 @@ SUBROUTINE setup()
   !
   IF ( dft_is_hybrid() ) THEN
      CALL exx_grid_init()
+     CALL exx_mp_init()
      CALL exx_div_check()
   ENDIF
 
@@ -672,7 +659,7 @@ SUBROUTINE setup()
   !
   ! ... initialize d1 and d2 to rotate the spherical harmonics
   !
-  IF (lda_plus_u .or. okpaw ) CALL d_matrix( d1, d2, d3 )
+  IF (lda_plus_u .or. okpaw .or. (okvan.and.dft_is_hybrid()) ) CALL d_matrix( d1, d2, d3 )
   !
   RETURN
   !
@@ -693,10 +680,11 @@ LOGICAL FUNCTION check_para_diag( nbnd )
   LOGICAL, SAVE :: first = .TRUE.
   LOGICAL, SAVE :: saved_value = .FALSE.
 
-  IF( .NOT. first ) then
+#if defined(__MPI)
+  IF( .NOT. first ) THEN
       check_para_diag = saved_value
       RETURN
-  end if
+  END IF
   first = .FALSE.
   !
   IF( np_ortho(1) > nbnd ) &
@@ -736,5 +724,8 @@ LOGICAL FUNCTION check_para_diag( nbnd )
      !
   END IF
   !
+#else
+  check_para_diag = .FALSE.
+#endif
   RETURN
 END FUNCTION check_para_diag
