@@ -17,6 +17,7 @@ subroutine stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
   USE fft_base,         ONLY : dfftp
   USE vdW_DF,           ONLY : stress_vdW_DF
   USE rVV10,            ONLY : stress_rVV10
+  USE io_global,        ONLY : stdout
   !
   IMPLICIT NONE
   !
@@ -31,7 +32,6 @@ subroutine stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
   inlc = get_inlc()
 
   if ( inlc==1 .or. inlc==2 .or. inlc==4 .or. inlc==5 .or. inlc==6 ) then
-     if ( nspin>1 ) call errore ('stres_vdW_DF',   'vdW stress not implemented for nspin > 1',1)
      CALL stress_vdW_DF(rho, rho_core, nspin, sigma_nonloc_dft)
   elseif ( inlc == 3 ) then
      if ( nspin>2 ) call errore ('stress_rVV10', 'rVV10 stress not implemented for nspin > 2',1)
