@@ -1,25 +1,16 @@
 !
-! Copyright (C) 2010-2015 Quantum ESPRESSO group
+! Copyright (C) 2010-2017 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
 !--------------------------------------------------------------------------
-! TB
-! included test in subroutine sgam_at to disable symmetries which
-! involve the z-axis, search for 'TB' or 'TS'
-! (TS as Thibault Sohier had the idea)
-!
-! note that verbosity is still set to high in input.f90 in order
-! to check if everything works as intended
-!--------------------------------------------------------------------------
-!
-!--------------------------------------------------------------------------
 !
 MODULE symm_base
 
   USE kinds,      ONLY : DP
+  USE io_global,  ONLY : stdout
   USE cell_base,  ONLY : at, bg
   !
   ! ... The variables needed to describe the symmetry properties
@@ -37,12 +28,14 @@ MODULE symm_base
   !
   PUBLIC :: s, sr, sname, ft, ftau, nrot, nsym, nsym_ns, nsym_na, t_rev, &
             no_t_rev, time_reversal, irt, invs, invsym, d1, d2, d3, &
-            allfrac, nofrac, nosym, nosym_evc
+            allfrac, nofrac, nosym, nosym_evc, fft_fact, spacegroup
   INTEGER :: &
        s(3,3,48),            &! symmetry matrices, in crystal axis
        invs(48),             &! index of inverse operation: S^{-1}_i=S(invs(i))
        ftau(3,48),           &! fractional translations, in FFT coordinates
+       fft_fact(3),          &! FFT dimensions must be multiple of fft_fact
        nrot,                 &! number of bravais lattice symmetries
+       spacegroup = 0,       &! space group index, as read from input
        nsym = 1,             &! total number of crystal symmetries
        nsym_ns = 0,          &! nonsymmorphic (fractional translation) symms
        nsym_na = 0            ! excluded nonsymmorphic symmetries because
@@ -53,9 +46,6 @@ MODULE symm_base
        accep = 1.0d-5         ! initial value of the acceptance threshold
                               ! for position comparison by eqvect in checksym
   !
-  ! ... note: ftau are used for symmetrization in real space (phonon, exx)
-  ! ... in which case they must be commensurated with the FFT grid
-  !
   CHARACTER(len=45) ::  sname(48)   ! name of the symmetries
   INTEGER :: &
        t_rev(48) = 0          ! time reversal flag, for noncolinear magnetism
@@ -80,7 +70,7 @@ MODULE symm_base
   ! ... Exported routines
   !
   PUBLIC ::  find_sym, inverse_s, copy_sym, checkallsym, &
-             s_axis_to_cart, set_sym, set_sym_bl, find_sym_ifc
+             s_axis_to_cart, set_sym, set_sym_bl, find_sym_ifc, remove_sym 
   !
 CONTAINS
    !
@@ -122,17 +112,19 @@ SUBROUTINE set_sym_bl ( )
   USE matrix_inversion
   IMPLICIT NONE
   !
+  CHARACTER(LEN=6), EXTERNAL :: int_to_char
+  !
   ! sin3 = sin(pi/3), cos3 = cos(pi/3), msin3 = -sin(pi/3), mcos3 = -cos(pi/3)
   !
-  real(DP), PARAMETER :: sin3 = 0.866025403784438597d0, cos3 = 0.5d0, &
+  REAL(DP), PARAMETER :: sin3 = 0.866025403784438597d0, cos3 = 0.5d0, &
                         msin3 =-0.866025403784438597d0, mcos3 = -0.5d0
-  real(DP) :: s0(3, 3, 32), overlap (3, 3), rat (3), rot (3, 3), value
+  REAL(DP) :: s0(3, 3, 32), overlap (3, 3), rat (3), rot (3, 3), value
   ! s0: the s matrices in cartesian axis
   ! overlap: inverse overlap matrix between direct lattice
   ! rat: the rotated of a direct vector ( cartesian )
   ! rot: the rotated of a direct vector ( crystal axis )
   ! value: component of the s matrix in axis basis
-  INTEGER :: jpol, kpol, mpol, irot, imat(24)
+  INTEGER :: jpol, kpol, mpol, irot, imat(32)
   ! counters over the polarizations and the rotations
 
   CHARACTER (len=45) :: s0name (64)
@@ -295,13 +287,15 @@ SUBROUTINE set_sym_bl ( )
      sname(nrot)=s0name(irot)
      imat(nrot)=irot
      nrot = nrot+1
-     IF (nrot > 25) CALL errore('set_sym_bl','some problem with symmetries',1)
 10   CONTINUE
   ENDDO
   nrot = nrot-1
   IF ( nrot /= 1 .AND. nrot /= 2 .AND. nrot /= 4 .AND. nrot /= 6 .AND. &
-       nrot /= 8 .AND. nrot /=12 .AND. nrot /=24 ) CALL errore('set_sym_bl',&
-         'wrong number of symmetries! Use standard orientations for axis',nrot)
+       nrot /= 8 .AND. nrot /=12 .AND. nrot /=24 ) THEN
+       WRITE (stdout, '(80("-"),/,"NOTICE: Bravais lattice has wrong number (",&
+      & i2,") of symmetries - symmetries are disabled",/,80("-"))' ) nrot
+      nrot = 1
+  END IF
   !
   !     set the inversion symmetry ( Bravais lattices have always inversion
   !     symmetry )
@@ -322,8 +316,10 @@ SUBROUTINE set_sym_bl ( )
   IF ( .not. is_group ( nrot ) ) THEN
   !    This happens for instance for an hexagonal lattice with one axis 
   !    oriented at 15 degrees from the x axis, the other along (-1,1,0)
-     CALL errore ('set_sym_bl', &
-         'Symmetry group not a group! Use standard orientations for axis',1)
+      CALL infomsg('set_sym_bl', 'NOTICE: Symmetry group for Bravais lattice &
+      &is not a group (' // TRIM(int_to_char(nrot)) // &
+      &') - symmetries are disabled')
+      nrot = 1
   ENDIF
   !
   RETURN
@@ -331,7 +327,7 @@ SUBROUTINE set_sym_bl ( )
 END SUBROUTINE set_sym_bl
 !
 !-----------------------------------------------------------------------
-SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
+SUBROUTINE find_sym ( nat, tau, ityp, magnetic_sym, m_loc, no_z_inv )
   !-----------------------------------------------------------------------
   !
   !     This routine finds the point group of the crystal, by eliminating
@@ -340,9 +336,12 @@ SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
   !
   IMPLICIT NONE
   !
-  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
-  real(DP), INTENT(in) :: tau (3,nat), m_loc(3,nat)
+  INTEGER, INTENT(in) :: nat, ityp (nat)
+  REAL(DP), INTENT(in) :: tau (3,nat), m_loc(3,nat)
   LOGICAL, INTENT(in) :: magnetic_sym
+  LOGICAL, INTENT(IN), OPTIONAL :: no_z_inv
+  ! no_z_inv: if .true., disable symmetries sending z into -z.
+  !           Some calculations (e.g. gate fields) require this
   !
   INTEGER :: i
   LOGICAL :: sym (48)
@@ -354,7 +353,11 @@ SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
   !    Here we find the true symmetries of the crystal
   !
   symm: DO i=1,3 !emine: if it is not resolved in 3 steps it is sth else?
-    CALL sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
+    IF ( PRESENT(no_z_inv) ) THEN
+       CALL sgam_at ( nat, tau, ityp, sym, no_z_inv )
+    ELSE
+       CALL sgam_at ( nat, tau, ityp, sym )
+    ENDIF
     !
     !    Here we check for magnetic symmetries
     !
@@ -401,45 +404,43 @@ SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
 END SUBROUTINE find_sym
 !
 !-----------------------------------------------------------------------
-SUBROUTINE sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
+SUBROUTINE sgam_at ( nat, tau, ityp, sym, no_z_inv)
   !-----------------------------------------------------------------------
   !
   !     Given the point group of the Bravais lattice, this routine finds
   !     the subgroup which is the point group of the considered crystal.
   !     Non symmorphic groups are allowed, provided that fractional
-  !     translations are allowed (nofrac=.false), that the unit cell is
-  !     not a supercell, and that they are commensurate with the FFT grid
+  !     translations are allowed (nofrac=.false) and that the unit cell
+  !     is not a supercell
   !
   !     On output, the array sym is set to .true.. for each operation
   !     of the original point group that is also a symmetry operation
   !     of the crystal symmetry point group
   !
-  USE io_global,  ONLY : stdout
-  USE extfield,   ONLY : monopole ! TS & TB
-  USE kinds
   IMPLICIT NONE
   !
-  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
+  INTEGER, INTENT(in) :: nat, ityp (nat)
   ! nat  : number of atoms in the unit cell
   ! ityp : species of each atom in the unit cell
-  ! nr*  : dimensions of the FFT mesh
-  !
-  real(DP), INTENT(in) :: tau (3, nat)
   !
+  REAL(DP), INTENT(in) :: tau (3, nat)
   ! tau  : cartesian coordinates of the atoms
   !
-  !     output variables
+  LOGICAL, INTENT(in), OPTIONAL :: no_z_inv
+  ! no_z_inv: if .true., disable symmetry operations sending z into -z.
+  !           Some calculations (e.g. gate fields) require this
   !
   LOGICAL, INTENT(out) :: sym (48)
   ! sym(isym)    : flag indicating if sym.op. isym in the parent group
   !                is a true symmetry operation of the crystal
   !
-  INTEGER :: na, kpol, nb, irot, i, j, isymTS ! TS & TB
+  INTEGER :: na, kpol, nb, irot, i, j
   ! counters
-  real(DP) , ALLOCATABLE :: xau (:,:), rau (:,:)
+  REAL(DP) , ALLOCATABLE :: xau (:,:), rau (:,:)
   ! atomic coordinates in crystal axis
-  LOGICAL :: fractional_translations
-  real(DP) :: ft_(3), ftaux(3)
+  LOGICAL :: fractional_translations, no_z
+  INTEGER :: nfrac
+  REAL(DP) :: ft_(3), ftaux(3)
   !
   ALLOCATE(xau(3,nat))
   ALLOCATE(rau(3,nat))
@@ -481,23 +482,9 @@ SUBROUTINE sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
   !
 10 CONTINUE
   nsym_ns = 0
+  fft_fact(:)= 1
   DO irot = 1, nrot
      !
-     ! check that the grid is compatible with the S rotation
-     !
-     IF ( mod (s (2, 1, irot) * nr1, nr2) /= 0 .or. &
-          mod (s (3, 1, irot) * nr1, nr3) /= 0 .or. &
-          mod (s (1, 2, irot) * nr2, nr1) /= 0 .or. &
-          mod (s (3, 2, irot) * nr2, nr3) /= 0 .or. &
-          mod (s (1, 3, irot) * nr3, nr1) /= 0 .or. &
-          mod (s (2, 3, irot) * nr3, nr2) /= 0 ) THEN
-        sym (irot) = .false.
-        WRITE( stdout, '(5x,"warning: symmetry operation # ",i2, &
-             &         " not compatible with FFT grid. ")') irot
-        WRITE( stdout, '(3i4)') ( (s (i, j, irot) , j = 1, 3) , i = 1, 3)
-        GOTO 20
-     ENDIF
-
      DO na = 1, nat
         ! rau = rotated atom coordinates
         rau (:, na) = s (1,:, irot) * xau (1, na) + &
@@ -540,6 +527,19 @@ SUBROUTINE sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
               IF (sym (irot) ) THEN
                  nsym_ns = nsym_ns + 1
                  ft (:,irot) = ft_(:)
+                 !
+                 ! Find factors that must be present in FFT grid dimensions
+                 ! in order to ensure that fractional translations are
+                 ! commensurate with FFT grids 
+                 DO i = 1, 3
+                    IF ( ABS (ft_(i)) > eps2 ) THEN
+                       nfrac = NINT(1.0_dp/ft_(i)) 
+                    ELSE
+                       nfrac = 0
+                    END IF
+                    fft_fact(i) = mcm ( fft_fact(i), nfrac )
+                 END DO
+                 !
                  GOTO 20
               ENDIF
            ENDIF
@@ -549,39 +549,14 @@ SUBROUTINE sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
 20   CONTINUE
   ENDDO
   !
-  ! convert ft to FFT coordinates, check if compatible with FFT grid
-  ! for real-space symmetrization (if done: currently, exx, phonon)
-  !
-  nsym_na = 0
-  DO irot =1, nrot
-     IF ( sym(irot) .and. .not. allfrac ) THEN
-        ftaux(1) = ft(1,irot) * nr1
-        ftaux(2) = ft(2,irot) * nr2
-        ftaux(3) = ft(3,irot) * nr3
-        ! check if the fractional translations are commensurate
-        ! with the FFT grid, discard sym.op. if not
-        ! (needed because ph.x symmetrizes in real space)
-        IF (abs (ftaux(1) - nint (ftaux(1)) ) / nr1 > eps2 .or. &
-            abs (ftaux(2) - nint (ftaux(2)) ) / nr2 > eps2 .or. &
-            abs (ftaux(3) - nint (ftaux(3)) ) / nr3 > eps2 ) THEN
-            !     WRITE( stdout, '(5x,"warning: symmetry operation", &
-            !          &     " # ",i2," not allowed.   fractional ", &
-            !          &     "translation:"/5x,3f11.7,"  in crystal", &
-            !          &     " coordinates")') irot, ft_
-            sym (irot) = .false.
-            nsym_na = nsym_na + 1
-            nsym_ns = nsym_ns - 1
-         ENDIF
-         ftau (:, irot) = nint (ftaux(:))
-      ENDIF
-  ENDDO
-  ! TS & TB disable all symmetries z -> -z when monopole is present
-  IF (monopole) THEN
-    do isymTS=1,48
-      if (s(3,3,isymTS).eq.-1) sym(isymTS)=.false.
-    enddo
+  ! disable all symmetries z -> -z
+  IF ( PRESENT(no_z_inv) ) THEN
+     IF ( no_z_inv ) THEN
+        DO irot=1,nrot
+           IF (s(3,3,irot) == -1) sym(irot)=.false.
+        END DO
+     ENDIF
   ENDIF
-  ! TS & TB
   !
   !   deallocate work space
   !
@@ -602,7 +577,7 @@ SUBROUTINE sgam_at_mag ( nat, m_loc, sym )
   IMPLICIT NONE
   !
   INTEGER, INTENT(in) :: nat
-  real(DP), INTENT(in) :: m_loc(3, nat)
+  REAL(DP), INTENT(in) :: m_loc(3, nat)
   !
   ! m_loc: local magnetization, must be invariant under the sym.op.
   !
@@ -613,7 +588,7 @@ SUBROUTINE sgam_at_mag ( nat, m_loc, sym )
   !
   INTEGER :: na, nb, irot
   LOGICAL :: t1, t2
-  real(DP) , ALLOCATABLE ::  mxau(:,:), mrau(:,:)
+  REAL(DP) , ALLOCATABLE ::  mxau(:,:), mrau(:,:)
   ! magnetization and rotated magnetization in crystal axis
   !
   ALLOCATE ( mxau(3,nat), mrau(3,nat) )
@@ -686,26 +661,26 @@ SUBROUTINE sgam_at_mag ( nat, m_loc, sym )
   RETURN
 END SUBROUTINE sgam_at_mag
 !
-SUBROUTINE set_sym(nat, tau, ityp, nspin_mag, m_loc, nr1, nr2, nr3)
+SUBROUTINE set_sym(nat, tau, ityp, nspin_mag, m_loc)
   !
   ! This routine receives as input atomic types and positions, if there
-  ! is noncollinear magnetism and the initial magnetic moments, the fft
-  ! dimensions nr1, nr2, nr3; it sets the symmetry elements of this module.
+  ! is noncollinear magnetism and the initial magnetic moments;
+  ! it sets the symmetry elements of this module.
   ! Note that at and bg are those in cell_base. It sets nrot, nsym, s,
   ! sname, sr, invs, ftau, irt, t_rev,  time_reversal, and invsym
   !
   !-----------------------------------------------------------------------
   !
   IMPLICIT NONE
-  ! input
-  INTEGER, INTENT(in)  :: nat, ityp(nat), nspin_mag, nr1, nr2, nr3
+  !
+  INTEGER, INTENT(in)  :: nat, ityp(nat), nspin_mag
   REAL(DP), INTENT(in) :: tau(3,nat)
   REAL(DP), INTENT(in) :: m_loc(3,nat)
   !
   time_reversal = (nspin_mag /= 4)
   t_rev(:) = 0
   CALL set_sym_bl ( )
-  CALL find_sym ( nat, tau, ityp, nr1, nr2, nr3, .not.time_reversal, m_loc )
+  CALL find_sym ( nat, tau, ityp, .not.time_reversal, m_loc)
   !
   RETURN
   END SUBROUTINE set_sym
@@ -830,7 +805,7 @@ LOGICAL FUNCTION checksym ( irot, nat, ityp, xau, rau, ft_ )
   INTEGER, INTENT(in) :: nat, ityp (nat), irot
   ! nat : number of atoms
   ! ityp: the type of each atom
-  real(DP), INTENT(in) :: xau (3, nat), rau (3, nat), ft_(3)
+  REAL(DP), INTENT(in) :: xau (3, nat), rau (3, nat), ft_(3)
   ! xau: the initial vectors (in crystal coordinates)
   ! rau: the rotated vectors (as above)
   ! ft_: fractionary translation (as above)
@@ -868,22 +843,22 @@ LOGICAL FUNCTION checksym ( irot, nat, ityp, xau, rau, ft_ )
 END FUNCTION checksym
 !
 !-----------------------------------------------------------------------
-SUBROUTINE checkallsym ( nat, tau, ityp, nr1, nr2, nr3 )
+SUBROUTINE checkallsym ( nat, tau, ityp )
   !-----------------------------------------------------------------------
-  !     given a crystal group this routine checks that the actual
-  !     atomic positions and bravais lattice vectors are compatible with
-  !     it. Used in relaxation/MD runs to check that atomic motion is
+  !     given a crystal group this routine checks that the actual atomic
+  !     positions and bravais lattice vectors are compatible with it.
+  !     Used in relaxation/MD runs to check that atomic motion is
   !     consistent with assumed symmetry.
   !
   IMPLICIT NONE
   !
-  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
-  real(DP), INTENT(in) :: tau (3, nat)
+  INTEGER, INTENT(in) :: nat, ityp (nat)
+  REAL(DP), INTENT(in) :: tau (3, nat)
   !
   INTEGER :: na, kpol, isym, i, j, k, l
   LOGICAL :: loksym (48)
-  real(DP) :: sx (3, 3), sy(3,3)
-  real(DP) , ALLOCATABLE :: xau(:,:), rau(:,:)
+  REAL(DP) :: sx (3, 3), sy(3,3)
+  REAL(DP) , ALLOCATABLE :: xau(:,:), rau(:,:)
   !
   ALLOCATE (xau( 3 , nat))
   ALLOCATE (rau( 3 , nat))
@@ -955,11 +930,10 @@ SUBROUTINE s_axis_to_cart ( )
   !     This routine transforms symmetry matrices expressed in the
   !     basis of the crystal axis into rotations in cartesian axis
   !
-  USE kinds
   IMPLICIT NONE
   !
   INTEGER :: isym
-  real(dp):: sa(3,3), sb(3,3)
+  REAL(dp):: sa(3,3), sb(3,3)
   !
   DO isym = 1,nsym
      sa (:,:) = dble ( s(:,:,isym) )
@@ -980,7 +954,7 @@ SUBROUTINE find_sym_ifc ( nat, tau, ityp)
   IMPLICIT NONE
   !
   INTEGER, INTENT(in) :: nat, ityp (nat)
-  real(DP), INTENT(in) :: tau (3,nat)
+  REAL(DP), INTENT(in) :: tau (3,nat)
   !
   INTEGER :: i
   LOGICAL :: sym (48)
@@ -991,28 +965,12 @@ SUBROUTINE find_sym_ifc ( nat, tau, ityp)
   !
   !    Here we find the true symmetries of the crystal
   !
-  !symm: DO i=1,3 !emine: if it is not resolved in 3 steps it is sth else?
-    CALL sgam_at_ifc ( nat, tau, ityp, sym )
-    !
-    !    Here we re-order all rotations in such a way that true sym.ops
-    !    are the first nsym; rotations that are not sym.ops. follow
-    !
-    nsym = copy_sym ( nrot, sym )
-    !
-  !  IF ( .not. is_group ( nsym ) ) THEN
-  !     IF (i == 1) CALL infomsg ('find_sym', &
-  !                    'Not a group! Trying with lower acceptance parameter...')
-  !     accep = accep * 0.5d0
-  !     IF (i == 3) THEN
-  !       CALL infomsg ('find_sym', 'Still not a group! symmetry disabled')
-  !       nsym = 1
-  !     ENDIF
-  !     CYCLE symm
-  !  ELSE
-  !     IF (i > 1) CALL infomsg ('find_sym', 'Symmetry operations form a group')
-  !     exit symm
-  !  ENDIF
-  !ENDDO symm
+  CALL sgam_at_ifc ( nat, tau, ityp, sym )
+  !
+  !    Here we re-order all rotations in such a way that true sym.ops
+  !    are the first nsym; rotations that are not sym.ops. follow
+  !
+  nsym = copy_sym ( nrot, sym )
   !
   ! check if inversion (I) is a symmetry.
   ! If so, it should be the (nsym/2+1)-th operation of the group
@@ -1041,15 +999,13 @@ SUBROUTINE sgam_at_ifc ( nat, tau, ityp, sym )
   !     of the original point group that is also a symmetry operation
   !     of the crystal symmetry point group
   !
-  USE io_global,  ONLY : stdout
-  USE kinds
   IMPLICIT NONE
   !
   INTEGER, INTENT(in) :: nat, ityp (nat)
   ! nat  : number of atoms in the unit cell
   ! ityp : species of each atom in the unit cell
   !
-  real(DP), INTENT(in) :: tau (3, nat)
+  REAL(DP), INTENT(in) :: tau (3, nat)
   !
   ! tau  : cartesian coordinates of the atoms
   !
@@ -1061,10 +1017,10 @@ SUBROUTINE sgam_at_ifc ( nat, tau, ityp, sym )
   !
   INTEGER :: na, kpol, nb, irot, i, j
   ! counters
-  real(DP) , ALLOCATABLE :: xau (:,:), rau (:,:)
+  REAL(DP) , ALLOCATABLE :: xau (:,:), rau (:,:)
   ! atomic coordinates in crystal axis
   LOGICAL :: fractional_translations
-  real(DP) :: ft_(3)
+  REAL(DP) :: ft_(3)
   !
   ALLOCATE(xau(3,nat))
   ALLOCATE(rau(3,nat))
@@ -1146,4 +1102,98 @@ SUBROUTINE sgam_at_ifc ( nat, tau, ityp, sym )
   RETURN
 END SUBROUTINE sgam_at_ifc
 
+!-----------------------------------------------------------------------
+SUBROUTINE remove_sym ( nr1, nr2, nr3 )
+  !
+  ! ... ensure that ftau used for symmetrization in real space (phonon, exx)
+  ! ... are commensurated with the FFT grid
+  !
+  IMPLICIT NONE
+  !
+  INTEGER, INTENT(in) :: nr1, nr2, nr3
+  !
+  LOGICAL :: sym(48)
+  INTEGER :: isym, nsym_, i, j
+  REAL(dp) :: ftaux(3)
+  !
+  nsym_ = nsym
+  sym(1:nsym_) = .TRUE.
+  nsym_na = 0
+  !
+  DO isym = 1, nsym_
+     !
+     ! check that the grid is compatible with the S rotation
+     !
+     IF ( mod (s (2, 1, isym) * nr1, nr2) /= 0 .or. &
+          mod (s (3, 1, isym) * nr1, nr3) /= 0 .or. &
+          mod (s (1, 2, isym) * nr2, nr1) /= 0 .or. &
+          mod (s (3, 2, isym) * nr2, nr3) /= 0 .or. &
+          mod (s (1, 3, isym) * nr3, nr1) /= 0 .or. &
+          mod (s (2, 3, isym) * nr3, nr2) /= 0 ) THEN
+        sym (isym) = .false.
+        WRITE( stdout, '(5x,"warning: symmetry operation # ",i2, &
+             &         " not compatible with FFT grid. ")') isym
+        WRITE( stdout, '(3i4)') ( (s (i, j, isym) , j = 1, 3) , i = 1, 3)
+        sym(isym) = .FALSE.
+        IF ( abs (ft(1,isym)) > eps2 .or. &
+             abs (ft(2,isym)) > eps2 .or. &
+             abs (ft(3,isym)) > eps2 ) nsym_ns = nsym_ns - 1
+     ENDIF
+     !
+     ! convert ft to FFT coordinates, check if compatible with FFT grid
+     ! for real-space symmetrization 
+     !
+     ftaux(1) = ft(1,isym) * nr1
+     ftaux(2) = ft(2,isym) * nr2
+     ftaux(3) = ft(3,isym) * nr3
+     ! check if the fractional translations are commensurate
+     ! with the FFT grid, discard sym.op. if not
+     ! (needed because ph.x symmetrizes in real space)
+     IF ( abs (ftaux(1) - nint (ftaux(1)) ) / nr1 > eps2 .or. &
+          abs (ftaux(2) - nint (ftaux(2)) ) / nr2 > eps2 .or. &
+          abs (ftaux(3) - nint (ftaux(3)) ) / nr3 > eps2 ) THEN
+        !     WRITE( stdout, '(5x,"warning: symmetry operation", &
+        !          &     " # ",i2," not allowed.   fractional ", &
+        !          &     "translation:"/5x,3f11.7,"  in crystal", &
+        !          &     " coordinates")') isym, ft_
+        sym (isym) = .FALSE.
+        nsym_na = nsym_na + 1
+        nsym_ns = nsym_ns - 1
+     ENDIF
+     ftau (:, isym) = nint (ftaux(:))
+     !
+  ENDDO
+  !
+  ! ... count symmetries, reorder them exactly as in "find_sym" 
+  !
+  nsym = copy_sym ( nsym_, sym )
+  invsym = all ( s(:,:,nsym/2+1) == -s(:,:,1) )
+  CALL inverse_s ( )
+  CALL s_axis_to_cart ( )
+  !
+END SUBROUTINE remove_sym
+!
+
+INTEGER FUNCTION mcm(i,j)
+  ! returns minimum common multiple of two integers
+  ! if i=0, returns j, and vice versa; if i<0 or j<0, returns -1
+  INTEGER, INTENT(IN) :: i,j
+  INTEGER :: n1,n2,k
+  
+  IF ( i < 0 .OR. j < 0 ) THEN
+     mcm = -1
+  ELSE IF ( i == 0 .AND. j == 0 ) THEN
+     mcm = 0
+  ELSE
+     n1 = MIN (i,j)
+     n2 = MAX (i,j)
+     DO k=1,n1
+        mcm = k*n2 
+        IF ( MOD( mcm, n1 ) == 0 ) RETURN
+     END DO
+     mcm = n2
+  END IF
+  
+END FUNCTION mcm
+
 END MODULE symm_base
