@@ -237,7 +237,8 @@ SUBROUTINE update_pot()
   ! ...                     + beta0*( tau(t-dt) -tau(t-2*dt) )
   !
   !
-  USE io_files,      ONLY : prefix, iunupdate, tmp_dir, wfc_dir, nd_nmbr, seqopn
+  USE io_files,      ONLY : prefix, iunupdate, tmp_dir, wfc_dir, nd_nmbr, &
+                            postfix, seqopn
   USE io_global,     ONLY : ionode, ionode_id
   USE cell_base,     ONLY : bg
   USE ions_base,     ONLY : nat, tau, nsp, ityp
@@ -249,6 +250,7 @@ SUBROUTINE update_pot()
   IMPLICIT NONE
   !
   REAL(DP), ALLOCATABLE :: tauold(:,:,:)
+  CHARACTER(LEN=256)    :: dirname
   INTEGER               :: rho_extr, wfc_extr
   LOGICAL               :: exists
   !
@@ -342,29 +344,20 @@ SUBROUTINE update_pot()
   ! ... determines the maximum effective order of the extrapolation on the
   ! ... basis of the files that are really available (for the charge density)
   !
-  IF ( ionode ) THEN
+  rho_extr = MIN( 1, history, pot_order )
+  !
+  IF ( pot_order > 1 .AND. ionode ) THEN
      !
-     rho_extr = MIN( 1, history, pot_order )
-     !
-     INQUIRE( FILE = TRIM( tmp_dir ) // TRIM( prefix ) // &
-            & '.save/charge-density.old.dat', EXIST = exists )
-     !
-     IF ( .NOT. exists ) &
-        !
-        INQUIRE( FILE = TRIM( tmp_dir ) // TRIM( prefix ) // &
-               & '.save/charge-density.old.xml', EXIST = exists )
+     dirname =  TRIM( tmp_dir ) // TRIM( prefix ) // postfix
+     INQUIRE( FILE = TRIM( dirname ) // 'charge-density.old.dat', &
+          EXIST = exists )
      !
      IF ( exists ) THEN
         !
         rho_extr = MIN( 2, history, pot_order )
         !
-        INQUIRE( FILE = TRIM( tmp_dir ) // TRIM( prefix ) // &
-               & '.save/charge-density.old2.dat', EXIST = exists )
-        !
-        IF ( .NOT. exists ) &
-           !
-           INQUIRE( FILE = TRIM( tmp_dir ) // TRIM( prefix ) // &
-                  & '.save/charge-density.old2.xml', EXIST = exists )
+        INQUIRE( FILE = TRIM( dirname ) //'charge-density.old2.dat', &
+             EXIST = exists )
         !
         IF ( exists ) rho_extr = MIN( 3, history, pot_order )
         !
@@ -374,7 +367,7 @@ SUBROUTINE update_pot()
   !
   CALL mp_bcast( rho_extr, ionode_id, intra_image_comm )
   !
-  CALL extrapolate_charge( rho_extr )
+  CALL extrapolate_charge( dirname, rho_extr )
   !
   CALL stop_clock( 'update_pot' )
   !
@@ -383,7 +376,7 @@ SUBROUTINE update_pot()
 END SUBROUTINE update_pot
 !
 !----------------------------------------------------------------------------
-SUBROUTINE extrapolate_charge( rho_extr )
+SUBROUTINE extrapolate_charge( dirname, rho_extr )
   !----------------------------------------------------------------------------
   !
   USE constants,            ONLY : eps32
@@ -392,7 +385,7 @@ SUBROUTINE extrapolate_charge( rho_extr )
   USE ions_base,            ONLY : nat, tau, nsp, ityp
   USE fft_base,             ONLY : dfftp, dffts
   USE fft_interfaces,       ONLY : fwfft, invfft
-  USE gvect,                ONLY : ngm, g, gg, gstart, nl, eigts1, eigts2, eigts3
+  USE gvect,                ONLY : ngm, g, gg, gstart, eigts1, eigts2, eigts3
   USE lsda_mod,             ONLY : lsda, nspin
   USE scf,                  ONLY : rho, rho_core, rhog_core, v
   USE ldaU,                 ONLY : eth
@@ -403,13 +396,14 @@ SUBROUTINE extrapolate_charge( rho_extr )
   USE vlocal,               ONLY : strf
   USE noncollin_module,     ONLY : noncolin
   USE klist,                ONLY : nelec
-  USE io_rho_xml,           ONLY : write_rho, read_rho
+  USE xml_io_base,          ONLY : write_rho, read_rho
   USE paw_variables,        ONLY : okpaw, ddd_paw
   USE paw_onecenter,        ONLY : PAW_potential
   !
   IMPLICIT NONE
   !
   INTEGER, INTENT(IN) :: rho_extr
+  CHARACTER(LEN=*), INTENT(IN) :: dirname
   !
   REAL(DP), ALLOCATABLE :: work(:,:), work1(:,:)
     ! work  is the difference between rho and atomic rho at time t
@@ -495,7 +489,9 @@ SUBROUTINE extrapolate_charge( rho_extr )
         WRITE( UNIT = stdout, FMT = '(5X, &
              & "NEW-OLD atomic charge density approx. for the potential")' )
         !
-        CALL write_rho( rho%of_r, 1, 'old' )
+        ! ... no need to save old charge density if no extrapolation is done
+        !
+        IF ( pot_order > 1 ) CALL write_rho( dirname, rho%of_r, 1, 'old' )
         !
      ELSE IF ( rho_extr == 2 ) THEN
         !
@@ -504,13 +500,13 @@ SUBROUTINE extrapolate_charge( rho_extr )
         !
         ! ...   oldrho  ->  work
         !
-        CALL read_rho( work, 1, 'old' )
+        CALL read_rho( dirname, work, 1, 'old' )
         !
         ! ...   rho%of_r   ->  oldrho
         ! ...   work  ->  oldrho2
         !
-        CALL write_rho( rho%of_r,  1, 'old' )
-        CALL write_rho( work, 1, 'old2' )
+        CALL write_rho( dirname, rho%of_r,  1, 'old' )
+        CALL write_rho( dirname, work, 1, 'old2' )
         !
         ! ... extrapolation
         !
@@ -528,14 +524,14 @@ SUBROUTINE extrapolate_charge( rho_extr )
         ! ...   oldrho2  ->  work1
         ! ...   oldrho   ->  work
         !
-        CALL read_rho( work1, 1, 'old2' )
-        CALL read_rho( work,  1, 'old' )
+        CALL read_rho( dirname, work1, 1, 'old2' )
+        CALL read_rho( dirname, work,  1, 'old' )
         !
         ! ...   rho%of_r   ->  oldrho
         ! ...   work  ->  oldrho2
         !
-        CALL write_rho( rho%of_r,  1, 'old' )
-        CALL write_rho( work, 1, 'old2' )
+        CALL write_rho( dirname, rho%of_r,  1, 'old' )
+        CALL write_rho( dirname, work, 1, 'old2' )
         !
         rho%of_r(:,1) = rho%of_r(:,1) + alpha0*( rho%of_r(:,1) - work(:,1) ) + &
                                beta0*( work(:,1) - work1(:,1) )
@@ -593,9 +589,9 @@ SUBROUTINE extrapolate_charge( rho_extr )
      !
      psic(:) = rho%of_r(:,is)
      !
-     CALL fwfft ('Dense', psic, dfftp)
+     CALL fwfft ('Rho', psic, dfftp)
      !
-     rho%of_g(:,is) = psic(nl(:))
+     rho%of_g(:,is) = psic(dfftp%nl(:))
      !
   END DO
   !
