@@ -40,9 +40,17 @@ module xdm_module
   ! have moments been computed before?
   LOGICAL :: saved = .FALSE.
 
-  ! a1 and a2 coefficients, with defaults for pw86pbe
-  REAL(DP) :: a1i = 0.6836_DP
-  REAL(DP) :: a2i = 1.5045_DP
+  ! a1 and a2 coefficients
+  REAL(DP) :: a1i = 0.0_DP
+  REAL(DP) :: a2i = 0.0_DP
+
+  ! iexch icorr igcx igcc
+  ! SLA PW B86B PBC ( 1  4 22  4 0 0)
+  ! USE funct, ONLY: get_iexch, get_icorr, get_igcx, get_igcc
+  ! IF (get_iexch().EQ.1.AND.get_icorr().EQ.4.AND.get_igcx().EQ.3.AND.get_igcc().EQ.4) THEN
+  !   !
+  !   sR=0.94_DP !PBE=sla+pw+pbx+pbc
+  !   !
 
   ! radial atomic densities
   REAL(DP), ALLOCATABLE :: rfree(:,:), w2free(:,:), rmaxg2(:)
@@ -171,6 +179,7 @@ CONTAINS
     USE scf, ONLY: rho
     USE io_global, ONLY: stdout, ionode
     USE fft_base, ONLY : dfftp
+    USE funct, ONLY : get_iexch, get_icorr, get_igcx, get_igcc
     USE cell_base, ONLY : at, alat, omega
     USE ions_base, ONLY: nat, tau, atm, ityp, ntyp => nsp
     USE constants, ONLY: au_gpa
@@ -178,8 +187,8 @@ CONTAINS
     USE atom, ONLY: msh, rgrid
     USE splinelib, ONLY : splint
     USE mp_images, ONLY : me_image, nproc_image, intra_image_comm
-    USE mp_pools,  ONLY : me_pool
     USE mp, ONLY : mp_sum
+    USE mp_bands, ONLY : intra_bgrp_comm
 
     REAL(DP) :: evdw
 
@@ -187,7 +196,7 @@ CONTAINS
     REAL(DP), PARAMETER :: ecut = 1e-11_DP
 
     INTEGER :: ialloc
-    INTEGER :: i, iat, n, ix, iy, iz, j, jj
+    INTEGER :: i, iat, n, ix, iy, iz, j, jj, iy0, iz0
     REAL(DP), ALLOCATABLE :: gaux(:,:), ggaux(:,:,:), rhoat(:), rhocor(:), rhoae(:)
     REAL(DP), ALLOCATABLE :: lapr(:), gmod(:), avol(:), b(:)
     REAL(DP) :: taus, rhos, ds, qs, rhs, xroot, xshift, xold, expx, gx, fx, ffx
@@ -202,10 +211,9 @@ CONTAINS
     INTEGER :: i3, nn
     REAL(DP) :: for(3,nat), sigma(3,3), sat(3,3)
     INTEGER :: resto, divid, first, last, it
-    INTEGER :: idx0, idx, ispin
+    INTEGER :: idx, ispin, iexch, icorr, igcx, igcc
     INTEGER, EXTERNAL :: atomic_number
-
-    real*8 :: iix, iiy, iiz
+    REAL(DP) :: iix, iiy, iiz
 
     ! initialize
     IF (nspin > 2) CALL errore('energy_xdm','nspin > 2 not implemented',1)
@@ -217,6 +225,41 @@ CONTAINS
     ! do we need to recalculate the coefficients?
     docalc = .NOT.saved .OR. .NOT.(lbfgs .OR. lmd)
 
+    ! Set the coefficients if none are given in the input
+    ! See: http://schooner.chem.dal.ca/wiki/XDM#Quantum_ESPRESSO
+    ! For functionals not in the list, please contact aoterodelaroza@gmail.com
+    IF (a1i==0._DP .AND. a2i==0._DP) THEN
+       iexch = get_iexch()
+       icorr = get_icorr()
+       igcx = get_igcx()
+       igcc = get_igcc()
+       IF (iexch==1 .AND. icorr==4 .AND. igcx==22 .AND. igcc==4) THEN
+          ! B86bPBE
+          a1i = 0.6512_DP
+          a2i = 1.4633_DP
+       ELSE IF (iexch==1 .AND. icorr==4 .AND. igcx==21 .AND. igcc==4) THEN
+          ! PW86PBE
+          a1i = 0.6836_DP
+          a2i = 1.5045_DP
+       ELSE IF (iexch==1 .AND. icorr==4 .AND. igcx==3 .AND. igcc==4) THEN
+          ! PBE
+          a1i = 0.3275_DP
+          a2i = 2.7673_DP
+       ELSE IF (iexch==1 .AND. icorr==3 .AND. igcx==1 .AND. igcc==3) THEN
+          ! BLYP
+          a1i = 0.4502_DP
+          a2i = 1.6210_DP
+       ELSE
+          IF (ionode) THEN
+             WRITE (stdout,'(/"Error: XDM not parametrized for this functional and XDM parameters not given.")')
+             WRITE (stdout,'("For the XDM parametrization list, please visit")')
+             WRITE (stdout,'("  http://schooner.chem.dal.ca/wiki/XDM#Quantum_ESPRESSO")')
+             WRITE (stdout,'("For functionals not in the list, please contact aoterodelaroza@gmail.com"/)')
+          ENDIF
+          CALL errore('energy_xdm','XDM not parametrized for this functional and XDM parameters not given.',1)
+       END IF
+    ENDIF
+
     ! Define damping coefficients
     a1 = a1i
     a2 = a2i / bohr_radius_angs
@@ -248,9 +291,6 @@ CONTAINS
        ! don't need the core anymore
        DEALLOCATE(rhocor)
 
-       ! define the starting index for each processor
-       idx0 = dfftp%nr1x * dfftp%nr2x * dfftp%ipp(me_pool+1)
-
        ! allocate arrays and initialize
        ALLOCATE(b(dfftp%nnr),STAT=ialloc)
        if (ialloc /= 0) CALL alloc_failed("b")
@@ -270,7 +310,7 @@ CONTAINS
           IF (ialloc /= 0) CALL alloc_failed("gaux, ggaux")
 
           ! valence gradient and laplacian
-          CALL external_ggradient(rho%of_r(:,ispin),gaux,ggaux)
+          CALL external_hessian(rho%of_r(:,ispin),gaux,ggaux)
           lapr = ggaux(1,1,:) + ggaux(2,2,:) + ggaux(3,3,:)
           DEALLOCATE(ggaux)
           gmod = sqrt(gaux(1,:)**2 + gaux(2,:)**2 + gaux(3,:)**2)
@@ -332,14 +372,21 @@ CONTAINS
              it = ityp(iat)
              nn = msh(it)
              taub = tau(:,iat) * alat
-             DO n = 1, dfftp%nr1x*dfftp%nr2x * dfftp%npl
-                idx = idx0 + n - 1
 
-                iz = idx / (dfftp%nr1x*dfftp%nr2x)
-                idx = idx - (dfftp%nr1x*dfftp%nr2x)*iz
-                iy = idx / dfftp%nr1x
-                idx = idx - dfftp%nr1x*iy
-                ix = idx
+             iy0 = dfftp%my_i0r2p ; iz0 = dfftp%my_i0r3p
+             DO n = 1, dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p
+                !
+                ! ... three dimensional indexes
+                !
+                idx = n -1
+                iz  = idx / (dfftp%nr1x*dfftp%my_nr2p)
+                idx = idx - (dfftp%nr1x*dfftp%my_nr2p)*iz
+                iz  = iz + iz0
+                iy  = idx / dfftp%nr1x
+                idx = idx - dfftp%nr1x * iy
+                iy  = iy + iy0
+                ix  = idx
+
                 iix = ix / REAL(dfftp%nr1,DP)
                 iiy = iy / REAL(dfftp%nr2,DP)
                 iiz = iz / REAL(dfftp%nr3,DP)
@@ -369,10 +416,9 @@ CONTAINS
              END DO ! n
           END DO ! iat
        END DO ! ispin
-#if defined(__MPI)
-       CALL mp_sum(avol,intra_image_comm)
-       CALL mp_sum(ml,intra_image_comm)
-#endif
+       CALL mp_sum(avol,intra_bgrp_comm)
+       CALL mp_sum(ml,intra_bgrp_comm)
+
        avol = avol * omega / (dfftp%nr1*dfftp%nr2*dfftp%nr3)
        ml = ml * omega / (dfftp%nr1*dfftp%nr2*dfftp%nr3)
 
@@ -519,14 +565,12 @@ CONTAINS
     sigma = -0.5_DP * sigma / omega
     ehadd = -0.5_DP * ehadd
 
-#if defined(__MPI)
     CALL mp_sum(evdw,intra_image_comm)
     CALL mp_sum(for,intra_image_comm)
     CALL mp_sum(sigma,intra_image_comm)
     DO nn = 6, 10
        CALL mp_sum(ehadd(nn),intra_image_comm)
     ENDDO
-#endif
 
     ! Convert to Ry
     evdw = evdw * 2
@@ -701,7 +745,6 @@ CONTAINS
     USE scf,           ONLY : scf_type
     USE fft_base,      ONLY : dfftp
     USE mp,            ONLY : mp_bcast, mp_sum
-    USE mp_pools,      ONLY : me_pool
     USE mp_images,     ONLY : intra_image_comm
     USE io_global,     ONLY : ionode_id
     USE splinelib,     ONLY : spline, splint
@@ -712,7 +755,7 @@ CONTAINS
 
     TYPE(paw_info)          :: i
     INTEGER                 :: ipol, ir, is, it, lm
-    INTEGER                 :: j, k, l, idx, idx0
+    INTEGER                 :: j, k, l, j0, k0, idx
     INTEGER                 :: ia, il, im, ml, mm
     REAL(DP),ALLOCATABLE    :: wsp_lm(:,:), ylm_posi(:,:), d1y(:,:), d2y(:,:)
     REAL(DP),ALLOCATABLE    :: rho_lm(:,:,:), rho_lm_ae(:,:,:), rho_lm_ps(:,:,:)
@@ -802,18 +845,18 @@ CONTAINS
        ENDDO
        DEALLOCATE(d1y,d2y)
 
-       ! define the starting index for each processor
-       idx0 = dfftp%nr1x * dfftp%nr2x * dfftp%ipp(me_pool+1)
-
        ALLOCATE(ylm_posi(1,i%l**2))
-       rsp_point : DO ir = 1, dfftp%nr1x*dfftp%nr2x * dfftp%npl
+       j0 = dfftp%my_i0r2p ; k0 = dfftp%my_i0r3p
+       rsp_point : DO ir = 1, dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p
           ! three dimensional indices (i,j,k)
-          idx   = idx0 + ir - 1
-          k     = idx / ( dfftp%nr1x* dfftp%nr2x)
-          idx   = idx - ( dfftp%nr1x* dfftp%nr2x)*k
-          j     = idx /  dfftp%nr1x
-          idx   = idx -  dfftp%nr1x*j
-          l     = idx
+          idx = ir -1
+          k   = idx / (dfftp%nr1x*dfftp%my_nr2p)
+          idx = idx - (dfftp%nr1x*dfftp%my_nr2p)*k
+          k   = k + k0
+          j   = idx / dfftp%nr1x
+          idx = idx - dfftp%nr1x * j
+          j   = j + j0
+          l   = idx
 
           ! ... do not include points outside the physical range!
           IF ( l >=  dfftp%nr1 .OR. j >=  dfftp%nr2 .OR. k >=  dfftp%nr3 ) CYCLE rsp_point
@@ -870,31 +913,30 @@ CONTAINS
     USE fft_base,  ONLY : dfftp
     USE splinelib, ONLY : splint
     use cell_base, ONLY : alat
-    USE mp_pools,  ONLY : me_pool
     implicit none
 
     real(DP), intent(out) :: rhoc(dfftp%nnr) ! core density in the real-space grid
     real(DP), intent(out) :: rhot(dfftp%nnr) ! core density in the real-space grid
 
     integer :: i, it, nn
-    integer :: n, idx0, idx, ix, iy, iz
+    integer :: n, idx, ix, iy, iz, iy0, iz0
     real(DP) :: x(3), xx(3), r, r2, rrho
 
-    ! define the starting index for each processor
-    idx0 = dfftp%nr1x * dfftp%nr2x * dfftp%ipp(me_pool+1)
-
     rhot = 0._DP
     rhoc = 0._DP
 
+    iy0 = dfftp%my_i0r2p ; iz0 = dfftp%my_i0r3p
     ! run over the real-space density grid
     DO n = 1, dfftp%nnr
-       idx = idx0 + n - 1
-
-       iz = idx / (dfftp%nr1x*dfftp%nr2x)
-       idx = idx - (dfftp%nr1x*dfftp%nr2x)*iz
-       iy = idx / dfftp%nr1x
-       idx = idx - dfftp%nr1x*iy
-       ix = idx
+       ! three dimensional indices (i,j,k)
+       idx  = n -1
+       iz   = idx / (dfftp%nr1x*dfftp%my_nr2p)
+       idx  = idx - (dfftp%nr1x*dfftp%my_nr2p)*iz
+       iz   = iz + iz0
+       iy   = idx / dfftp%nr1x
+       idx  = idx - dfftp%nr1x * iy
+       iy   = iy + iy0
+       ix   = idx
 
        x = ix / REAL(dfftp%nr1,DP) * at(:,1) + iy / REAL(dfftp%nr2,DP) * at(:,2) + &
           iz / REAL(dfftp%nr3,DP) * at(:,3)
