@@ -27,14 +27,15 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
   ! Modified by Simone Binnie in 2012 (EXX)
   ! Modified by Xiaochuan Ge  in 2013 (Davidson)
   ! Modified by Iurii Timrov  in 2014 (Environ)
+  ! Modified by Oliviero Andreussi in 2018 (Environ-->Plugins)
   !
   USE kinds,                ONLY : DP
   USE ions_base,            ONLY : ityp, nat, ntyp=>nsp
   USE cell_base,            ONLY : tpiba2
-  USE fft_base,             ONLY : dffts, dfftp, dtgs
-  USE fft_interfaces,       ONLY : fwfft
-  USE gvecs,                ONLY : nls, nlsm
-  USE gvect,                ONLY : nl, ngm, gstart, g, gg
+  USE fft_base,             ONLY : dffts, dfftp
+  USE fft_interfaces,       ONLY : fwfft, fft_interpolate
+  USE fft_helper_subroutines
+  USE gvect,                ONLY : ngm, gstart, g, gg
   USE io_global,            ONLY : stdout
   USE klist,                ONLY : nks, xk, ngk, igk_k
   USE lr_variables,         ONLY : evc0, sevc0, revc0, rho_1, rho_1c, &
@@ -61,11 +62,6 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
   USE becmod,               ONLY : bec_type, becp, calbec
   USE lr_exx_kernel
   USE dv_of_drho_lr
-#if defined(__ENVIRON)
-  USE plugin_flags,         ONLY : use_environ
-  USE scf,                  ONLY : rho
-  USE solvent_tddfpt,       ONLY : calc_vsolvent_tddfpt
-#endif
   !
   IMPLICIT NONE
   !
@@ -82,12 +78,6 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
   COMPLEX(DP), ALLOCATABLE :: dvrs_temp(:,:), spsi1(:,:), dvrsc(:,:), &
                               & dvrssc(:), sevc1_new(:,:,:)
   !
-  ! Environ related arrays
-  !
-  REAL(DP), ALLOCATABLE :: &
-          dv_pol(:), &  ! response polarization potential
-          dv_epsilon(:) ! response dielectric potential
-  !
   IF (lr_verbosity > 5) THEN
      WRITE(stdout,'("<lr_apply_liouvillian>")')
   ENDIF
@@ -120,8 +110,8 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
      ELSE
         ALLOCATE( dvrsc(dfftp%nnr, nspin) )
         ALLOCATE( dvrssc(dffts%nnr) )
-        dvrsc(:,:)=0.0d0
-        dvrssc(:) =0.0d0
+        dvrsc(:,:)=(0.0d0,0.0d0)
+        dvrssc(:) =(0.0d0,0.0d0)
      ENDIF
      !
      ! Calculation of the charge density response
@@ -134,15 +124,15 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
      !
      IF (no_hxc) THEN
         !
-        ! With no_hxc=.true. we recover the independent electron 
+        ! With no_hxc=.true. we recover the independent electron
         ! approximation, so we zero the interation.
         !
         IF (gamma_only) THEN
            dvrs(:,1) = 0.0d0
-           CALL interpolate (dvrs(:,1),dvrss,-1)
+           CALL fft_interpolate (dfftp, dvrs(:,1), dffts, dvrss)
         ELSE
-           dvrsc(:,1) = 0.0d0
-           CALL cinterpolate (dvrsc(:,1),dvrssc,-1)
+           dvrsc(:,1) = (0.0d0,0.0d0)
+           CALL fft_interpolate (dfftp, dvrsc(:,1), dffts, dvrssc)
         ENDIF
         !
      ELSE
@@ -156,45 +146,19 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
            !
            ALLOCATE( dvrs_temp(dfftp%nnr, nspin) )
            !
-           dvrs_temp = CMPLX( dvrs, 0.0d0, kind=DP )         
+           dvrs_temp = CMPLX( dvrs, 0.0d0, kind=DP )
            !
            DEALLOCATE ( dvrs )  ! to save memory
            !
            CALL dv_of_drho(dvrs_temp,.FALSE.)
            !
            ALLOCATE ( dvrs(dfftp%nnr, nspin) )
-           ! 
+           !
            dvrs = DBLE(dvrs_temp)
            !
            DEALLOCATE(dvrs_temp)
            !
-#if defined(__ENVIRON)
-           !
-           IF ( use_environ ) THEN
-              !
-              ALLOCATE( dv_pol(dfftp%nnr) )
-              ALLOCATE( dv_epsilon(dfftp%nnr) )
-              dv_pol(:) = 0.0d0
-              dv_epsilon(:) = 0.0d0
-              !
-              IF (.not.davidson) THEN
-                 WRITE( stdout, '(5x,"ENVIRON: Calculate the response &
-                             & polarization and dielectric potentials")' )
-              ENDIF
-              !
-              CALL calc_vsolvent_tddfpt(dfftp%nnr, nspin, rho%of_r(:,1), &
-                                       & rho_1(:,1), dv_pol, dv_epsilon)
-              !
-              ! Add the response polarization and dielectric potentials
-              ! to the response HXC potential.
-              !
-              dvrs(:,1) = dvrs(:,1) + dv_pol(:) + dv_epsilon(:)
-              !
-              DEALLOCATE( dv_pol )
-              DEALLOCATE( dv_epsilon )
-              !
-           ENDIF
-#endif           
+           CALL plugin_tddfpt_potential(rho_1,dvrs)
            !
         ELSE
            !
@@ -204,16 +168,24 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
            !
         ENDIF
         !
+        ! USPP case: compute the integral of the response HXC potential with
+        ! the Q function
+        !
         IF ( okvan )  THEN
-           IF ( tqr ) THEN
-              CALL newq_r(dvrs,d_deeq,.TRUE.)
+           IF (gamma_only) THEN
+              IF ( tqr ) THEN
+                 CALL newq_r(dvrs,d_deeq,.TRUE.)
+              ELSE
+                 ALLOCATE( psic(dfftp%nnr) )
+                 psic(:) = (0.0d0,0.0d0)
+                 CALL newq(dvrs,d_deeq,.TRUE.)
+                 DEALLOCATE( psic )
+              ENDIF
            ELSE
-              ALLOCATE( psic(dfftp%nnr) )
-              psic(:)=(0.0d0,0.0d0)
-              !
-              CALL newq(dvrs,d_deeq,.TRUE.)
-              !
-              DEALLOCATE( psic )
+              ! IT: Here we need to compute the integral of dvrsc and Q function
+              ! The issue is that newq wants as input an array of type REAL while
+              ! dvrsc is of type COMPLEX.
+              CALL errore( 'lr_apply_liouvillian', 'The integral of Q and dV is not implemented', 1 )
            ENDIF
         ENDIF
         !
@@ -222,9 +194,9 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
         ! Put the interaction on the smooth grid.
         !
         IF (gamma_only) THEN
-           CALL interpolate (dvrs(:,1),dvrss,-1)
+           CALL fft_interpolate (dfftp, dvrs(:,1), dffts, dvrss)
         ELSE
-           CALL cinterpolate (dvrsc(:,1),dvrssc,-1)
+           CALL fft_interpolate (dfftp, dvrsc(:,1), dffts, dvrssc)
         ENDIF
         !
      ENDIF
@@ -350,7 +322,6 @@ CONTAINS
     USE lr_variables,             ONLY : becp_1, tg_revc0
     USE realus,                   ONLY : tg_psic
     USE mp_global,                ONLY : me_bgrp
-    USE fft_parallel,             ONLY : tg_gather
     USE mp_global,                ONLY : ibnd_start, ibnd_end, inter_bgrp_comm
     USE mp,                       ONLY : mp_sum
     USE lr_exx_kernel,            ONLY : lr_exx_sum_int
@@ -418,16 +389,16 @@ CONTAINS
           !end: calculation of becp2
        ENDIF
 
-      IF ( dtgs%have_task_groups ) THEN
+      IF ( dffts%has_task_groups ) THEN
          !
-         v_siz =  dtgs%tg_nnr * dtgs%nogrp
+         v_siz =  dffts%nnr_tg
          !
-         incr = 2 * dtgs%nogrp
+         incr = 2 * fftx_ntgrp(dffts)
          !
          ALLOCATE( tg_dvrss(1:v_siz) )
          tg_dvrss=0.0d0
          !
-         CALL tg_gather(dffts, dtgs, dvrss, tg_dvrss)
+         CALL tg_gather(dffts, dvrss, tg_dvrss)
          !
       ENDIF
        !
@@ -447,9 +418,9 @@ CONTAINS
           ! Product with the potential vrs = (vltot+vr)
           ! revc0 is on smooth grid. psic is used up to smooth grid
           !
-          IF (dtgs%have_task_groups) THEN
+          IF (dffts%has_task_groups) THEN
              !
-             DO ir=1, dffts%nr1x*dffts%nr2x*dtgs%tg_npp( me_bgrp + 1 )
+             DO ir=1, dffts%nr1x*dffts%nr2x*dffts%my_nr3p
                 !
                 tg_psic(ir) = tg_revc0(ir,ibnd,1)*CMPLX(tg_dvrss(ir),0.0d0,DP)
                 !
@@ -513,7 +484,7 @@ CONTAINS
                            psic( box_beta(ir,ia) ) = &
                                 &psic(  box_beta(ir,ia) ) + &
                                 &betasave(ir,ih,ia)*&
-                                &CMPLX( w1(ih), w2(ih) )
+                                &CMPLX( w1(ih), w2(ih), KIND=dp )
                           !
                           ENDDO
                         !
@@ -540,7 +511,7 @@ CONTAINS
 #if defined(__MPI)
        CALL mp_sum( evc1_new(:,:,1), inter_bgrp_comm )
 #endif
-       IF (dtgs%have_task_groups) DEALLOCATE (tg_dvrss)
+       IF (dffts%has_task_groups) DEALLOCATE (tg_dvrss)
        !
        IF( nkb > 0 .and. okvan .and. real_space_debug <= 7) THEN
           !The non real_space part
@@ -642,7 +613,7 @@ SUBROUTINE lr_apply_liouvillian_k()
              !
              DO ig = 1,ngk(ik)
                 !
-                evc1_new(ig,ibnd,ik) = psic(nls(igk_k(ig,ik)))
+                evc1_new(ig,ibnd,ik) = psic(dffts%nl(igk_k(ig,ik)))
                 !
              ENDDO
              !
