@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -25,9 +25,9 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   USE ions_base,              ONLY : ityp, nat, ntyp=>nsp
   USE cell_base,              ONLY : omega
   USE ener,                   ONLY : ef
-  USE gvecs,                  ONLY : nls, nlsm, doublegrid
+  USE gvecs,                  ONLY : doublegrid
   USE fft_base,               ONLY : dffts, dfftp
-  USE fft_interfaces,         ONLY : invfft
+  USE fft_interfaces,         ONLY : invfft, fft_interpolate
   USE io_global,              ONLY : stdout
   USE kinds,                  ONLY : dp
   USE klist,                  ONLY : nks, xk, wk, ngk, igk_k
@@ -42,14 +42,13 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   USE wavefunctions_module,   ONLY : psic
   USE wvfct,                  ONLY : nbnd, et, wg, npwx
   USE control_flags,          ONLY : gamma_only
-  USE uspp,                   ONLY : vkb, nkb, okvan, qq, becsum
+  USE uspp,                   ONLY : vkb, nkb, okvan, qq_nt, becsum
   USE uspp_param,             ONLY : upf, nh
   USE io_global,              ONLY : ionode, stdout
   USE io_files,               ONLY : tmp_dir, prefix
   USE mp,                     ONLY : mp_sum
   USE mp_global,              ONLY : inter_pool_comm, intra_bgrp_comm,&
                                      inter_bgrp_comm 
-  USE realus,                 ONLY : addusdens_r
   USE charg_resp,             ONLY : w_T, lr_dump_rho_tot_cube,&
                                      & lr_dump_rho_tot_xyzd, &
                                      & lr_dump_rho_tot_xcrys,&
@@ -61,6 +60,7 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   USE lr_exx_kernel,          ONLY : lr_exx_kernel_int, revc_int,&
                                      & revc_int_c
   USE constants,              ONLY : eps12
+  USE fft_helper_subroutines
   !
   IMPLICIT NONE
   !
@@ -73,12 +73,13 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   ! Local variables
   !
   INTEGER       :: ir, ik, ibnd, jbnd, ig, ijkb0, np, na
-  INTEGER       :: ijh,ih,jh,ikb,jkb ,ispin 
+  INTEGER       :: ijh,ih,jh,ikb,jkb,is
   INTEGER       :: i, j, k, l
   REAL(kind=dp) :: w1, w2, scal, rho_sum
   ! These are temporary buffers for the response 
   REAL(kind=dp), ALLOCATABLE :: rho_sum_resp_x(:), rho_sum_resp_y(:),&
                               & rho_sum_resp_z(:)  
+  COMPLEX(kind=dp), ALLOCATABLE :: rhoaux(:,:)
   CHARACTER(len=256) :: tempfile, filename
   !
   IF (lr_verbosity > 5) THEN
@@ -88,12 +89,12 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   CALL start_clock('lr_calc_dens')
   !
   ALLOCATE( psic(dfftp%nnr) )
-  psic(:)    = (0.0d0,0.0d0)
+  psic(:) = (0.0d0, 0.0d0)
   !
   IF (gamma_only) THEN
-     rho_1(:,:) =  0.0d0
+     rho_1(:,:) = 0.0d0
   ELSE
-     rho_1c(:,:) =  0.0d0
+     rho_1c(:,:) = (0.0d0, 0.0d0)
   ENDIF
   !
   IF (gamma_only) THEN
@@ -104,7 +105,7 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
      !
      ! If a double grid is used, interpolate onto the fine grid
      !
-     IF ( doublegrid ) CALL interpolate(rho_1,rho_1,1)
+     IF ( doublegrid ) CALL fft_interpolate(dffts, rho_1(:,1), dfftp, rho_1(:,1))
      !
 #if defined(__MPI)
      CALL mp_sum(rho_1, inter_bgrp_comm)
@@ -118,19 +119,45 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
      !
      ! If a double grid is used, interpolate onto the fine grid
      !
-     IF ( doublegrid ) CALL cinterpolate(rho_1c,rho_1c,1)
+     IF ( doublegrid ) CALL fft_interpolate(dffts, rho_1c(:,1), dfftp, rho_1c(:,1))
      !
   ENDIF
   !
-  ! Here we add the Ultrasoft contribution to the charge density
-  ! response. 
+  ! Here we add the ultrasoft contribution to the charge density response. 
   !
   IF (okvan) THEN
-     IF (tqr) THEN
-        CALL addusdens_r(rho_1,.FALSE.)
-     ELSE
-        CALL addusdens(rho_1)
-     ENDIF
+     ! 
+     ALLOCATE(rhoaux(dfftp%nnr,nspin_mag))
+     !
+     ! Compute the US part of the response charge density in G-space
+     ! and put the result in rhoaux
+     !
+     rhoaux(:,:) = (0.0d0, 0.0d0)
+     CALL addusdens(rhoaux)
+     !
+     DO is = 1, nspin_mag
+        !
+        ! FFT of the US part of the response charge density 
+        ! from G-space to R-space
+        !
+        psic(:) = (0.d0, 0.d0)
+        psic( dfftp%nl(:) ) = rhoaux(:,is)
+        IF ( gamma_only ) psic( dfftp%nlm(:) ) = CONJG(rhoaux(:,is))
+        CALL invfft ('Rho', psic, dfftp)
+        !
+        ! Sum up the normal and the US parts of the response charge density
+        ! in R-space
+        !
+        IF (gamma_only) THEN
+           rho_1(:,is)  = rho_1(:,is) + DBLE(psic(:))
+        ELSE
+           rho_1c(:,is) = rho_1c(:,is) + psic(:)
+        ENDIF
+        !
+     ENDDO
+     !
+     DEALLOCATE(rhoaux)
+     !
   ENDIF
   !
   ! The psic workspace can present a memory bottleneck
@@ -149,10 +176,10 @@ SUBROUTINE lr_calc_dens( evc1, response_calc )
   !
   IF (lr_verbosity > 0) THEN
      ! 
-     DO ispin = 1, nspin_mag
+     DO is = 1, nspin_mag
         !
         rho_sum = 0.0d0
-        rho_sum = SUM(rho_1(:,ispin))
+        rho_sum = SUM(rho_1(:,is))
         !
 #if defined(__MPI)
         CALL mp_sum(rho_sum, intra_bgrp_comm )
@@ -330,13 +357,12 @@ CONTAINS
                                     me_bgrp, me_pool
     USE mp,                  ONLY : mp_sum
     USE realus,              ONLY : tg_psic
-    USE fft_base,            ONLY : dffts, dtgs
-    USE fft_parallel,        ONLY : tg_gather
+    USE fft_base,            ONLY : dffts
 
     IMPLICIT NONE
     !
     INTEGER :: ibnd_start_gamma, ibnd_end_gamma
-    INTEGER :: v_siz, incr, ioff, idx
+    INTEGER :: v_siz, incr, ir3, ioff, ioff_tg, nxyp, idx
     REAL(DP), ALLOCATABLE :: tg_rho(:)
     !
     ibnd_start_gamma = ibnd_start
@@ -345,11 +371,11 @@ CONTAINS
     !
     incr = 2
     !
-    IF ( dtgs%have_task_groups ) THEN
+    IF ( dffts%has_task_groups ) THEN
        !
-       v_siz =  dtgs%tg_nnr * dtgs%nogrp
+       v_siz =  dffts%nnr_tg
        !
-       incr = 2 * dtgs%nogrp
+       incr = 2 * fftx_ntgrp(dffts)
        !
        ALLOCATE( tg_rho( v_siz ) )
        tg_rho= 0.0_DP
@@ -362,18 +388,16 @@ CONTAINS
        !
        CALL invfft_orbital_gamma(evc1(:,:,1),ibnd,nbnd)
        !
-       IF (dtgs%have_task_groups) THEN
+       IF (dffts%has_task_groups) THEN
           !
           ! Now the first proc of the group holds the first two bands
-          ! of the 2*dtgs%nogrp bands that we are processing at the same time,
+          ! of the 2*ntgrp bands that we are processing at the same time,
           ! the second proc. holds the third and fourth band
           ! and so on.
           !
           ! Compute the proper factor for each band
           !
-          DO idx = 1, dtgs%nogrp
-             IF( dtgs%nolist( idx ) == me_bgrp ) EXIT
-          ENDDO
+          idx = fftx_tgpe(dffts) + 1
           !
           ! Remember two bands are packed in a single array :
           ! proc 0 has bands ibnd   and ibnd+1
@@ -395,10 +419,10 @@ CONTAINS
              w2 = w1
           END IF
           !
-          DO ir = 1, dtgs%tg_npp( me_bgrp + 1 ) * dffts%nr1x * dffts%nr2x
+          DO ir = 1, dffts%nr1x * dffts%nr2x * dffts%my_nr3p
              tg_rho(ir) = tg_rho(ir) &
                   + 2.0d0*(w1*real(tg_revc0(ir,ibnd,1),dp)*real(tg_psic(ir),dp)&
-                  + w2*aimag(tg_revc0(ir,ibnd,1))*aimag(tg_psic(ir)))
+                  +       w2*aimag(tg_revc0(ir,ibnd,1))*aimag(tg_psic(ir)))
           ENDDO
           !
        ELSE
@@ -426,7 +450,7 @@ CONTAINS
           DO ir = 1, dffts%nnr
              rho_1(ir,1) = rho_1(ir,1) &
                   + 2.0d0*(w1*real(revc0(ir,ibnd,1),dp)*real(psic(ir),dp)&
-                  + w2*aimag(revc0(ir,ibnd,1))*aimag(psic(ir)))
+                  +       w2*aimag(revc0(ir,ibnd,1))*aimag(psic(ir)))
           ENDDO
           !
           ! OBM - psic now contains the response functions in real space.
@@ -449,23 +473,11 @@ CONTAINS
        !
     ENDDO
     !
-    IF (dtgs%have_task_groups) THEN
+    IF (dffts%has_task_groups) THEN
        !
        ! reduce the group charge
        !
-       CALL mp_sum( tg_rho, gid = dtgs%ogrp_comm )
-       !
-       ioff = 0
-       DO idx = 1, dtgs%nogrp
-          IF ( me_bgrp == dtgs%nolist( idx ) ) EXIT
-          ioff = ioff + dffts%nr1x * dffts%nr2x * dffts%npp( dtgs%nolist( idx ) + 1 )
-       END DO
-       !
-       ! copy the charge back to the processor location
-       !
-       DO ir = 1, dffts%nnr
-          rho_1(ir,1) = rho_1(ir,1) + tg_rho(ir+ioff)
-       ENDDO
+       CALL tg_reduce_rho( rho_1, tg_rho, 1, dffts )
        !
     ENDIF
     !
@@ -514,7 +526,7 @@ CONTAINS
                               2.d0 * w1 * becp%r(ikb,ibnd) *&
                               & becp_1(ikb,ibnd)
                          !
-                         scal = scal + qq(ih,ih,np) *1.d0 *&
+                         scal = scal + qq_nt(ih,ih,np) *1.d0 *&
                               &  becp%r(ikb,ibnd) * becp_1(ikb,ibnd)
                          !
                          ijh = ijh + 1
@@ -529,7 +541,7 @@ CONTAINS
                                  &becp%r(jkb,ibnd) + & 
                                  becp_1(jkb,ibnd) * becp%r(ikb,ibnd))
                             !
-                            scal = scal + qq(ih,jh,np) * 1.d0 *&
+                            scal = scal + qq_nt(ih,jh,np) * 1.d0 *&
                                  & (becp%r(ikb,ibnd) * &
                                  &becp_1(jkb, ibnd) + &
                                  &becp%r(jkb,ibnd) * becp_1(ikb,ibnd))
@@ -564,7 +576,7 @@ CONTAINS
        !
     ENDIF
     !
-    IF ( dtgs%have_task_groups ) THEN
+    IF ( dffts%has_task_groups ) THEN
        DEALLOCATE( tg_rho )
     END IF
     !   
@@ -591,7 +603,7 @@ CONTAINS
           psic(:) = (0.0d0,0.0d0)
           !
           DO ig = 1, ngk(ik)
-             psic(nls(igk_k(ig,ik)))=evc1(ig,ibnd,ik)
+             psic(dffts%nl(igk_k(ig,ik)))=evc1(ig,ibnd,ik)
           ENDDO
           !
           CALL invfft ('Wave', psic, dffts)
@@ -659,7 +671,7 @@ CONTAINS
                                  &DBLE(CONJG(becp%k(ikb,ibnd)) *&
                                  & becp1_c(ikb,ibnd,ik)) 
                             !
-                            scal = scal + qq(ih,ih,np) * 1.d0 *&
+                            scal = scal + qq_nt(ih,ih,np) * 1.d0 *&
                                  &  DBLE(CONJG(becp%k(ikb,ibnd)) *&
                                  & becp1_c(ikb,ibnd,ik))
                             !
@@ -677,7 +689,7 @@ CONTAINS
                                     becp1_c(jkb,ibnd,ik) *&
                                     & CONJG(becp%k(ikb,ibnd)))
                                !
-                               scal = scal + qq(ih,jh,np) * 1.d0 * &
+                               scal = scal + qq_nt(ih,jh,np) * 1.d0 * &
                                     & DBLE(CONJG(becp%k(ikb,ibnd)) *&
                                     & becp1_c(jkb,ibnd,ik)+&
                                     & becp%k(jkb,ibnd) * &
