@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2017 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -16,24 +16,23 @@ SUBROUTINE lr_readin
   USE lr_variables
   USE lr_dav_variables
   USE kinds,               ONLY : DP
-  USE io_files,            ONLY : tmp_dir, prefix, wfc_dir
+  USE io_files,            ONLY : tmp_dir, prefix, wfc_dir, create_directory
   USE lsda_mod,            ONLY : current_spin, nspin, isk, lsda
   USE control_flags,       ONLY : twfcollect,use_para_diag, &
                                   & tqr, lkpoint_dir, gamma_only, &
                                   & do_makov_payne
   USE scf,                 ONLY : vltot, v, vrs, vnew, &
                                   & destroy_scf_type, rho
-  USE fft_base,            ONLY : dfftp, dffts, dtgs
+  USE fft_base,            ONLY : dfftp, dffts
   USE gvecs,               ONLY : doublegrid
   USE wvfct,               ONLY : nbnd, et, wg, current_k
   USE lsda_mod,            ONLY : isk
   USE ener,                ONLY : ef
   USE io_global,           ONLY : ionode, ionode_id, stdout
-  USE klist,               ONLY : nks, wk, nelec, lgauss
+  USE klist,               ONLY : nks, wk, nelec, lgauss, ltetra
   USE fixed_occ,           ONLY : tfixed_occ
-  USE input_parameters,    ONLY : degauss, nosym, wfcdir, outdir,&
-                                  & max_seconds
-  USE ktetra,              ONLY : ltetra
+  USE input_parameters,    ONLY : degauss, nosym, wfcdir, outdir
+  USE check_stop,          ONLY : max_seconds
   USE realus,              ONLY : real_space, real_space_debug,&
                                   & init_realspace_vars, qpointlist,&
                                   & betapointlist
@@ -53,27 +52,11 @@ SUBROUTINE lr_readin
   USE martyna_tuckerman,   ONLY : do_comp_mt
   USE esm,                 ONLY : do_comp_esm
   USE qpoint,              ONLY : xq
-  USE xml_io_base,         ONLY : create_directory
-  USE io_rho_xml,          ONLY : write_rho
+  USE io_rho_xml,          ONLY : write_scf
   USE noncollin_module,    ONLY : noncolin
   USE mp_bands,            ONLY : ntask_groups
   USE constants,           ONLY : eps4
   USE control_lr,          ONLY : lrpa
-#if defined(__ENVIRON)
-  USE environ_base,        ONLY : environ_base_init, ir_end
-  USE environ_input,       ONLY : read_environ
-  USE environ_base,        ONLY : ifdtype, nfdpoint
-  USE ions_base,           ONLY : nsp, ityp, zv, tau, nat
-  USE cell_base,           ONLY : at, alat, omega, ibrav
-  USE solvent_tddfpt,      ONLY : solvent_initbase_tddfpt
-  USE environ_init,        ONLY : environ_initions, environ_initcell,      &
-                                  environ_clean, environ_initbase,         &
-                                  environ_initions_allocate
-  USE environ_main,        ONLY : calc_venviron
-  USE mp_bands,            ONLY : me_bgrp
-  USE plugin_flags,        ONLY : use_environ
-#endif
-  
 
   IMPLICIT NONE
   !
@@ -83,23 +66,24 @@ SUBROUTINE lr_readin
   ! Fine control of beta_gamma_z file
   CHARACTER(LEN=80) :: disk_io
   ! Specify the amount of I/O activities
+  CHARACTER(LEN=6) :: int_to_char
   INTEGER :: ios, iunout, ierr, ipol
   LOGICAL :: auto_rs
-  CHARACTER(LEN=6) :: int_to_char
+  LOGICAL, EXTERNAL  :: check_para_diag
   !
   NAMELIST / lr_input /   restart, restart_step ,lr_verbosity, prefix, outdir, &
                         & test_case_no, wfcdir, disk_io, max_seconds
   NAMELIST / lr_control / itermax, ipol, ltammd, real_space, real_space_debug, lrpa,   &
                         & charge_response, tqr, auto_rs, no_hxc, n_ipol, project,      &
                         & scissor, ecutfock, pseudo_hermitian, d0psi_rs, lshift_d0psi, &
-                        & q1, q2, q3, approximation 
+                        & q1, q2, q3, approximation
   NAMELIST / lr_post /    omeg, beta_gamma_z_prefix, w_T_npol, plot_type, epsil, itermax_int,sum_rule
   namelist / lr_dav /     num_eign, num_init, num_basis_max, residue_conv_thr, precondition,         &
                         & dav_debug, reference,single_pole, sort_contr, diag_of_h, close_pre,        &
                         & broadening,print_spectrum,start,finish,step,if_check_orth, if_random_init, &
                         & if_check_her,p_nbnd_occ,p_nbnd_virt,poor_of_ram,poor_of_ram2,max_iter,     &
                         & ecutfock, conv_assistant,if_dft_spectrum,no_hxc,d0psi_rs,lshift_d0psi,     &
-                        & lplot_drho, vccouple_shift
+                        & lplot_drho, vccouple_shift, ltammd
   !
   auto_rs = .TRUE.
   !
@@ -126,8 +110,8 @@ SUBROUTINE lr_readin
      pseudo_hermitian=.true.
      ipol = 1
      n_ipol = 1
-     no_hxc = .FALSE.      
-     lrpa = .false.         
+     no_hxc = .FALSE.
+     lrpa = .false.
      real_space = .FALSE.
      real_space_debug = 0
      charge_response = 0
@@ -147,8 +131,8 @@ SUBROUTINE lr_readin
      !
      ! For EELS
      !
-     q1 = 1.0d0         
-     q2 = 1.0d0         
+     q1 = 1.0d0
+     q2 = 1.0d0
      q3 = 1.0d0
      approximation = 'TDDFT'
      !
@@ -183,8 +167,7 @@ SUBROUTINE lr_readin
      if_dft_spectrum=.false.
      lplot_drho=.false.
      !
-     ! 
-     !   Reading possible plugin arguments (-environ).
+     !   Reading possible plugin arguments
      !
      CALL plugin_arguments()
      !
@@ -240,7 +223,7 @@ SUBROUTINE lr_readin
      !
      IF (.NOT.eels) THEN
         w_T_prefix = TRIM( tmp_dir ) // &
-                   & TRIM( beta_gamma_z_prefix ) // ".beta_gamma_z." 
+                   & TRIM( beta_gamma_z_prefix ) // ".beta_gamma_z."
      ENDIF
      !
      ierr = 0
@@ -248,20 +231,20 @@ SUBROUTINE lr_readin
      IF (eels) THEN
         !
         IF (n_ipol /= 1) THEN
-           WRITE(stdout,'(5X,"n_ipol /= 1 is not allowed for EELS.")') 
-           WRITE(stdout,'(5X,"Setting n_ipol = 1 ...")') 
+           WRITE(stdout,'(5X,"n_ipol /= 1 is not allowed for EELS.")')
+           WRITE(stdout,'(5X,"Setting n_ipol = 1 ...")')
            n_ipol = 1
         ENDIF
         IF (ipol /= 1) THEN
-           WRITE(stdout,'(5X,"ipol /= 1 is not allowed for EELS.")') 
-           WRITE(stdout,'(5X,"Setting ipol = 1 ...")')                
+           WRITE(stdout,'(5X,"ipol /= 1 is not allowed for EELS.")')
+           WRITE(stdout,'(5X,"Setting ipol = 1 ...")')
            ipol = 1
         ENDIF
         LR_polarization = 1
         !
      ELSE
         !
-        ! Optics: set up polarization direction(s) 
+        ! Optics: set up polarization direction(s)
         !
         IF ( ipol==4 ) THEN
            !
@@ -342,7 +325,7 @@ SUBROUTINE lr_readin
   !
   ! Required for restart runs as this never gets initialized.
   !
-  current_k = 1     
+  current_k = 1
   !
   outdir = TRIM( tmp_dir ) // TRIM( prefix ) // '.save'
   !
@@ -352,7 +335,7 @@ SUBROUTINE lr_readin
      IF (real_space) real_space_debug=99
      IF (real_space_debug > 0) real_space=.TRUE.
      IF (lr_verbosity > 1) THEN
-        WRITE(stdout,'(5x,"Status of real space flags: TQR=", L5 ,& 
+        WRITE(stdout,'(5x,"Status of real space flags: TQR=", L5 ,&
                       &"  REAL_SPACE=", L5)') tqr, real_space
      ENDIF
      !
@@ -378,28 +361,28 @@ SUBROUTINE lr_readin
   !
   ! Optical case: the variables igk_k and ngk are set up through this path:
   ! read_file -> init_igk.
-  ! EELS: the variables igk_k and ngk will be re-set up later (because there 
+  ! EELS: the variables igk_k and ngk will be re-set up later (because there
   ! will be not only poins k but also points k+q) through the path:
-  ! lr_run_nscf -> init_run -> hinit0 -> init_igk 
+  ! lr_run_nscf -> init_run -> hinit0 -> init_igk
   !
   CALL read_file()
   !
   !   Set wfc_dir - this is done here because read_file sets wfc_dir = tmp_dir
   !   FIXME:,if wfcdir is not present in input, wfc_dir is set to "undefined"
-  !   instead of tmp_dir, because of the logic used in the rest of TDDFPT   
+  !   instead of tmp_dir, because of the logic used in the rest of TDDFPT
   !
   wfc_dir = trimcheck ( wfcdir )
   !
   IF (eels) THEN
      !
-     ! Specify the temporary derictory.
+     ! Specify the temporary directory.
      !
      tmp_dir = tmp_dir_lr
      !
      ! Copy the scf-charge-density to the tmp_dir (PH/check_initial_status.f90).
      ! Needed for the nscf calculation.
      !
-     IF (.NOT.restart) CALL write_rho( rho, nspin )
+     IF (.NOT.restart) CALL write_scf( rho, nspin )
      !
      ! If a band structure calculation needs to be done, do not open a file
      ! for k point (PH/phq_readin.f90)
@@ -408,69 +391,26 @@ SUBROUTINE lr_readin
      !
   ENDIF
   !
-  ! Make sure all the features used in the PWscf calculation 
+  ! Make sure all the features used in the PWscf calculation
   ! are actually supported by TDDFPT.
   !
   CALL input_sanity()
   !
-#if defined(__ENVIRON)
+  ! Compute and add plugin contributions to SCF potential
   !
-  ! Self-consistent continuum solvation model
+  CALL plugin_read_input("TD")
   !
-  IF ( use_environ ) THEN
-     !
-     ! Periodic boundary corrections, which were possibly activated in PW
-     !
-#if defined(__MPI)
-  IF (ionode) THEN
-#endif
-     if (do_makov_payne) then
-        assume_isolated = 'makov-payne'
-     elseif (do_comp_mt) then
-        assume_isolated = 'martyna-tuckerman'
-     elseif (do_comp_esm) then
-        assume_isolated = 'esm'
-     else
-        assume_isolated = 'none'
-     endif
-#if defined(__MPI)
-  ENDIF
-  CALL mp_bcast(assume_isolated, ionode_id, world_comm)
-#endif
-     !
-     ! Copied from PW/src/input.f90
-     !
-     CALL read_environ( nat, nsp, assume_isolated, ibrav )
-     !
-     ! Taken from PW/src/init_run.f90
-     !
-     ir_end = MIN(dfftp%nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
-     CALL environ_initbase( dfftp%nnr )
-     !
-     ! Taken from PW/src/electrons.f90
-     !
-     CALL environ_initions( dfftp%nnr, nat, nsp, ityp, zv, tau, alat )
-     CALL environ_initcell( dfftp%nnr, dfftp%nr1, dfftp%nr2, dfftp%nr3, ibrav, omega, alat, at )
-     !
-     ! Compute additional unperturbed potentials due to the presence of the
-     ! environment and add them to the SCF (HXC) potential.
-     !
-     WRITE( stdout, '(/5x,"Computing and adding the polarization potentials to HXC potential")' )
-     !
-     CALL calc_venviron( .TRUE., dfftp%nnr, nspin, 0.0d0, rho%of_r, v%of_r(:,1))
-     !
-     ! Now, once the Environ potentials were computed we can deallocate numerous 
-     ! Environ arrays because they are not needed any longer.
-     !
-     CALL environ_clean( .TRUE. )
-     !
-     ! Allocations for TDDFPT
-     !
-     CALL solvent_initbase_tddfpt(ifdtype, nfdpoint, dfftp%nnr)
-     !
-  ENDIF
+  CALL plugin_initbase()
   !
-#endif
+  CALL plugin_init_ions()
+  !
+  CALL plugin_init_cell()
+  !
+  CALL plugin_init_potential( v%of_r(:,1) )
+  !
+  CALL plugin_scf_potential( rho, .FALSE., -1.D0, v%of_r(:,1) )
+  !
+  CALL plugin_clean( 'TD', .FALSE. )
   !
   !  Deallocate some variables created with read_file but not used in TDDFPT
   !
@@ -499,8 +439,8 @@ SUBROUTINE lr_readin
   !
   ! Now put the potential calculated in read_file into the correct place
   ! and deallocate the redundant associated variables.
-  ! Set the total local potential vrs on the smooth mesh 
-  ! adding the scf (Hartree + XC) part and the sum of 
+  ! Set the total local potential vrs on the smooth mesh
+  ! adding the scf (Hartree + XC) part and the sum of
   ! all the local pseudopotential contributions.
   ! vrs = vltot + v%of_r
   !
@@ -510,8 +450,8 @@ SUBROUTINE lr_readin
   CALL destroy_scf_type(v)
   !
   ! Recalculate the weights of the Kohn-Sham orbitals.
-  ! (Should this not be a call to weights() to make this 
-  ! less insulator specific?)                            
+  ! (Should this not be a call to weights() to make this
+  ! less insulator specific?)
   !
   IF (.NOT.eels) CALL iweights( nks, wk, nbnd, nelec, et, ef, wg, 0, isk)
   !
@@ -519,20 +459,15 @@ SUBROUTINE lr_readin
   !
   ! Scalapack related stuff.
   !
-#if defined(__MPI)
-  use_para_diag = .TRUE.
-  CALL check_para_diag( nbnd )
-#else
-  use_para_diag = .FALSE.
-#endif
+  use_para_diag = check_para_diag( nbnd )
   !
   RETURN
   !
 CONTAINS
   !
   SUBROUTINE input_sanity()
-    !-------------------------------------------------------------------------- 
-    ! 
+    !--------------------------------------------------------------------------
+    !
     ! This subroutine aims to gather all of the input sanity checks
     ! (features enabled in PWscf which are unsupported in TDDFPT).
     ! Written by Simone Binnie 2011
@@ -592,7 +527,7 @@ CONTAINS
        !
     ENDIF
     !
-    !  Check that either we have the same number of procs as the initial PWscf run 
+    !  Check that either we have the same number of procs as the initial PWscf run
     !  OR that the wavefunctions were gathered into one file at the end of
     !  the PWscf run.
     !
@@ -611,15 +546,17 @@ CONTAINS
     !
     ! No taskgroups and EXX.
     !
-    IF (dtgs%have_task_groups .AND. dft_is_hybrid()) &
+    IF (dffts%has_task_groups .AND. dft_is_hybrid()) &
          & CALL errore( 'lr_readin', ' Linear response calculation ' // &
          & 'not implemented for EXX+Task groups', 1 )
     !
     ! Experimental task groups warning.
     !
-    IF (dtgs%have_task_groups) &
+    IF (dffts%has_task_groups) &
          & CALL infomsg( 'lr_readin','Usage of task &
-         &groups with TDDFPT is still experimental. Use at your own risk.' )
+         & groups with TDDFPT is still experimental. Use at your own risk.' )
+    !      & CALL errore( 'lr_readin', ' Linear response calculation ' // &
+    !      & 'not implemented for task groups', 1 )
     !
     ! No PAW support.
     !
@@ -660,12 +597,9 @@ CONTAINS
        !
        ! Note, all variables of the turboDavidson code cannot be used by turboEELS.
        !
-#if defined(__ENVIRON)
+       ! EELS + plugins is not supported.
        !
-       ! EELS + implicit solvent model is not supported.
-       !
-       IF ( use_environ ) CALL errore( 'lr_readin', 'Implicit solvent model cannot be used.', 1)
-#endif
+       CALL plugin_check('lr_readin')
        !
     ENDIF
     !
@@ -726,5 +660,5 @@ CONTAINS
       RETURN
       !
     END SUBROUTINE read_rs_status
- 
+
 END SUBROUTINE lr_readin
