@@ -14,7 +14,6 @@ PROGRAM X_Spectra
   USE cell_base,       ONLY : bg, at, celldm
   USE parameters,      ONLY : ntypx,lmaxx,lqmax
   USE ions_base,       ONLY : nat, ntyp => nsp, ityp, tau
-  USE ktetra,          ONLY : ltetra, ntetra, tetra
   USE start_k,         ONLY : nk1, nk2, nk3, k1, k2, k3
   USE wvfct,           ONLY : npwx ,nbnd, et, wg ! et(nbnd,nkstot)
   USE radial_grids,    ONLY : ndmx
@@ -145,6 +144,7 @@ PROGRAM X_Spectra
   ! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
 
      i=0
+     IF(xiabs<1 .or. xiabs>ntyp) CALL errore("xspectra","xiabs < 1 or xiabs > ntyp",1)
      DO na=1,nat
         IF(ityp(na).EQ.xiabs) i=i+1
      ENDDO
@@ -383,7 +383,7 @@ PROGRAM X_Spectra
      ENDIF
      !
      WRITE(stdout,'(5x,"Cross-section successfully written in ",a,/)') &
-      'xanes.dat'
+      xanes_file
      IF (.NOT. xonly_plot) WRITE(stdout,'(5x,"... End STEP 2 ...",/)')
 
   ELSEIF(TRIM(calculation).EQ.'rxes') THEN
@@ -437,7 +437,6 @@ SUBROUTINE stop_xspectra
   USE control_flags, ONLY: twfcollect
   USE io_files, ONLY: iunwfc
   USE mp_global, ONLY: mp_global_end
-  USE parallel_include
   !
 #if defined(__MPI)
 
@@ -956,7 +955,7 @@ SUBROUTINE initialize_gamma_tab
   REAL(dp) :: e, x, y, dx
   INTEGER  :: i, j, n
 
-  dx = (xemax-xemin)/dfloat(xnepoint)
+  dx = (xemax-xemin)/DBLE(xnepoint)
 
   DO n = 1, xnepoint
      x = xemin + (n-1)*dx
