@@ -372,10 +372,10 @@ CONTAINS
      !     ----------------------------------------------------------
      SUBROUTINE convert_casino(upf_out)
        !     ----------------------------------------------------------
-       USE kinds, ONLY : dp
-       USE upf_module
-       USE radial_grids, ONLY: radial_grid_type, deallocate_radial_grid
-       USE funct, ONLY : set_dft_from_name, get_iexch, get_icorr, &
+       USE kinds,        ONLY : dp
+       USE pseudo_types, ONLY : pseudo_upf
+       USE radial_grids, ONLY : radial_grid_type, deallocate_radial_grid
+       USE funct,        ONLY : set_dft_from_name, get_iexch, get_icorr, &
                          get_igcx, get_igcc
 
        IMPLICIT NONE
@@ -575,10 +575,9 @@ CONTAINS
 
      SUBROUTINE write_casino_tab(upf_in, grid)
 
-       USE upf_module
+       USE pseudo_types, ONLY : pseudo_upf
        USE radial_grids, ONLY: radial_grid_type, deallocate_radial_grid
 
-
        IMPLICIT NONE
 
        TYPE(pseudo_upf), INTENT(in)       :: upf_in
@@ -612,7 +611,7 @@ CONTAINS
 
      SUBROUTINE conv_upf2casino(upf_in,grid)
 
-       USE upf_module
+       USE pseudo_types, ONLY : pseudo_upf
        USE radial_grids, ONLY: radial_grid_type, deallocate_radial_grid
 
 
